#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# All actions for the RBT project should be registered via the toplevel
# method called RBT.action().
# =========================================================================== #
# require 'rbt/actions/actions.rb'
# RBT.action(:map_this_input_to_that_registered_program, _)
# =========================================================================== #
module RBT

  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/colours/rev_simp_sdir_sfancy_sfile_and_swarn.rb'

  # ========================================================================= #
  # === RBT.action                                  (action tag, actions tag)
  #
  # The second argument to this method defaults to ARGV since as of
  # May 2022.
  #
  # If you want to require all actions then use this line:
  #
  #   require 'rbt/requires/require_actions.rb'
  #
  # ========================================================================= #
  def self.action(
      action_that_is_desired = nil,
      optional_argument1     = ARGV, # This is typically the main input to this method.
      options_hash           = {},
      &block
    )
    action_that_is_desired = action_that_is_desired.to_s.delete('?').to_sym
    _ = optional_argument1 # Easier "pointer" here.
    # ======================================================================= #
    # Before September 2022 this method turned the variable optional_argument1
    # into a String, if it was an Array. This is no longer done, as we lose
    # information that way.
    # ======================================================================= #
    case action_that_is_desired # (case tag)
    # ======================================================================= #
    # === :cmake
    # ======================================================================= #
    when :cmake,
         :run_cmake
      require 'rbt/cmake/cmake.rb'
      RBT.cmake(_, &block)
    # ======================================================================= #
    # === :run_meson_configure_step
    # ======================================================================= #
    when :run_meson_configure_step
      require 'rbt/toplevel_methods/meson.rb'
      RBT.run_meson_configure_step(_, &block)
    # ======================================================================= #
    # === :run_meson
    #
    # This entry point runs the whole meson-stack, aka "meson", "ninja"
    # and then "ninja install".
    # ======================================================================= #
    when :run_meson
      use_this_prefix = '/usr/'
      if _ and !_.empty?
        use_this_prefix = _
      end
      esystem 'meson --prefix='+use_this_prefix+' --libdir='+use_this_prefix+'/lib BUILD_DIRECTORY'
      cd('BUILD_DIRECTORY')
      esystem 'ninja'
      esystem 'ninja install'
    # ======================================================================= #
    # === :meson_appdir_prefix
    # ======================================================================= #
    when :meson_appdir_prefix,
         :MesonAppdirPrefix
      require 'rbt/toplevel_methods/meson.rb'
      RBT.meson_appdir_prefix
    # ======================================================================= #
    # === :map_this_input_to_that_registered_program
    #
    # Usage examples from within a .rb file:
    #
    #   action(:map_this_input_to_that_registered_program, _)
    #   RBT.action(:map_this_input_to_that_registered_program, _)
    #
    # ======================================================================= #
    when :map_this_input_to_that_registered_program,
         :MapThisInputToThatRegisteredProgram
      require 'rbt/actions/individual_actions/map_this_input_to_that_registered_program/map_this_input_to_that_registered_program.rb'
      RBT::Action::MapThisInputToThatRegisteredProgram[_, &block] # This also is automatically { :be_quiet }
    # ======================================================================= #
    # === :check_for_available_versions
    #
    # Commandline usage example:
    #
    #   raction check_for_available_versions
    #
    # ======================================================================= #
    when :check_for_available_versions,
         :CheckForAvailableVersions
      require 'rbt/utility_scripts/check_for_available_versions.rb'
      RBT.check_for_available_versions(_, &block)
    # ======================================================================= #
    # === :cookbooks
    # ======================================================================= #
    when :cookbooks,
         :Cookbooks,
         :cookbook,
         :Cookbook,
         :sanitize_cookbook,
         :SanitizeCookbook,
         :sanitized_cookbook,
         :sanitize_data,
         :SanitizedData,
         :cookbook_for,
         :CookbookFor,
         :cookbook_dataset,
         :CookbookDataset
      require 'rbt/requires/require_the_cookbook_class.rb'
      RBT.sanitize_cookbook(_, options_hash, &block)
    # ======================================================================= #
    # === :create_app_dir_skeleton
    #
    # This entry point must be similar to:
    #
    #   rcp htop-1.2
    #   rcp htop-1.2 --blank
    #   rcppwd
    #
    # Commandline invocation example:
    #
    #   raction create_app_dir_skeleton wesnoth-1.2.3
    #
    # ======================================================================= #
    when :create_app_dir_skeleton,
         :create_appdir_skeleton,
         :CreateAppDirSkeleton,
         :create_program_skeleton,
         :create_program,
         :CreateProgram,
         :appdir_skeleton,
         :rcp
      require 'rbt/actions/individual_actions/create_app_dir_skeleton/create_app_dir_skeleton.rb'
      RBT.create_app_dir_skeleton(_, &block)
    # ======================================================================= #
    # === :save_the_available_programs_versions
    # ======================================================================= #
    when :save_the_available_programs_versions,
         :SaveTheAvailableProgramsVersions
      require 'rbt/utility_scripts/save_the_available_programs_versions.rb'
      RBT::SaveTheAvailableProgramsVersions.new
    # ======================================================================= #
    # === :backup_program
    # ======================================================================= #
    when :backup_program,
         :BackupProgram
      require 'rbt/utility_scripts/backup_program.rb'
      RBT.backup_program(_)
    # ======================================================================= #
    # === :software_manager
    #
    # This is the general action that can be used to install or otherwise
    # compile a program from its source code. The class that will handle
    # this step is RBT::Action::Installer. In other words, this entry point
    # can be used to compile a program.
    #
    # It is just about the most important entry point for RBT.actions()
    # in general.
    #
    # Usage examples:
    #
    #   RBT.action(:compile_program, 'gcc')
    #   RBT.action(:compile_program, :lfs1)
    #   RBT.action(:compile_program, :lfs2)
    #   RBT.action(:compile_program, :lfs3) { :use_sound }
    #   RBT.action(:installer, :lfs2)
    #   RBT.installer(:htop)
    #
    # Commandline usage examples:
    #
    #   raction software_manager lfs2
    #   raction installer lfs2
    #
    # ======================================================================= #
    when :software_manager,
         :SoftwareManager,
         :sm,
         :SM,
         :installer,
         :Installer,
         :install, # ← Not 100% sure about this alias here yet, but we'll see (February 2024)
         :compile_program,
         :compile
      require 'rbt/actions/individual_actions/software_manager/software_manager.rb'
      # ===================================================================== #
      # === Add support for "lfs1", "lfs2", "lfs3" as input here:
      # ===================================================================== #
      if _.is_a?(Array) and _.first.to_s.start_with?('lfs')
        _ = _.first.to_s
      end
      if _.to_s.start_with?('lfs') # Assume: :lfs1
        # =================================================================== #
        # Load the dataset from the file lfs_steps.yml next:
        # =================================================================== #
        dataset = YAML.load_file("#{project_yaml_directory?}lfs/lfs_steps.yml")
        _ = dataset[
          _.to_s.delete_prefix('lfs').to_i - 1
        ]
      end
      RBT.software_manager(_, &block)    
    # ======================================================================= #
    # === :registered_binaries
    #
    # This action will always return how many registered binaries are
    # available for the RBT project.
    #
    # Usage example from within ruby code:
    #
    #   action(:registered_binaries)
    #
    # ======================================================================= #
    when :registered_binaries,
         :RegisteredBinaries
      require 'rbt/registered/registered_binaries.rb'
      return RBT.registered_binaries?
    # ======================================================================= #
    # === :generate_homepage
    # ======================================================================= #
    when :generate_homepage,
         :GenerateHomepage
      require 'rbt/cookbooks/generate_homepage/generate_homepage.rb'
      RBT.generate_homepage(_)
    # ======================================================================= #
    # === :check_for_duplicate_binaries
    # ======================================================================= #
    when :check_for_duplicate_binaries,
         :CheckForDuplicateBinaries
      require 'rbt/utility_scripts/check_for_duplicate_binaries.rb'
      RBT::CheckForDuplicateBinaries.new(_, &block)
    # ======================================================================= #
    # === :disable_md5sum_calculation
    # ======================================================================= #
    when :disable_md5sum_calculation
      require 'rbt/requires/require_the_cookbook_class.rb'
      RBT::Action::Cookbooks::SanitizeCookbook.disable_md5sum_calculation # Cookbooks::Cookbook must provide this method.
    # ======================================================================= #
    # === :run_make_then_make_install
    # ======================================================================= #
    when :run_make_then_make_install
      require 'rbt/toplevel_methods/run_make_then_make_install.rb'
      RBT.run_make_then_make_install
    # ======================================================================= #
    # === :return_chained_programs
    # ======================================================================= #
    when :return_chained_programs,
         :ReturnChainedPrograms
      require 'rbt/constants/constants.rb'
      return RBT.return_chained_programs
    # ======================================================================= #
    # === :change_tab_title
    #
    # This entry point can be used to rename a KDE konsole tab.
    # ======================================================================= #
    when :change_tab_title,
         :ChangeTabTitle,
         :consider_changing_the_tab_title,
         :ConsiderChangingTheTabTitle,
         :rename_konsole_tab,
         :rename_shell,
         :set_title,
         :consider_renaming_the_kde_konsole_tab
      require 'rbt/toplevel_methods/toplevel_methods.rb'
      RBT.consider_changing_the_tab_title(_[0], _[1])
    # ======================================================================= #
    # === :check_for_inclusion
    # ======================================================================= #
    when :check_for_inclusion,
         :CheckForInclusion
      require 'rbt/cookbooks/check_for_inclusion/check_for_inclusion.rb'
      RBT::Cookbooks::CheckForInclusion.new(_)
    # ======================================================================= #
    # === :cookbook_aliases
    # ======================================================================= #
    when :cookbook_aliases,
         :CookbookAliases,
         :cookbooks_aliases,
         :CookbooksAliases
      require 'rbt/aliases/aliases.rb'
      RBT.cookbook_aliases
    # ======================================================================= #
    # === :ntrad
    # ======================================================================= #
    when :ntrad,
         :non_traditional,
         :appdir
      require 'rbt/actions/individual_actions/ntrad/ntrad.rb'
      _ = [_].flatten.compact
      work_on_these_programs = [_].flatten.compact
      RBT.ntrad(work_on_these_programs, &block)
    # ======================================================================= #
    # === :suggest_cookbook_for
    #
    # Usage example from within ruby code:
    #
    #   action(:suggest_cookbook_for, :htop)
    #
    # ======================================================================= #
    # bl $RBT/actions/individual_actions/cookbooks/suggest_cookbook_for/suggest_cookbook_for.rb
    # ======================================================================= #
    when :suggest_cookbook_for
      require 'rbt/actions/individual_actions/cookbooks/suggest_cookbook_for/suggest_cookbook_for.rb'
      RBT.suggest_cookbook_for(_, &block)
    # ======================================================================= #
    # === :symlink_into_the_usr_bin_hierarchy
    #
    # Symlink the given Array into the /usr/bin/ hierarchy. Optionally a
    # directory can be given as main input.
    #
    # Invocation example:
    #
    #   RBT.action(:symlink_into_the_usr_bin_hierarchy, :ninja)
    #
    # ======================================================================= #
    when :symlink_into_the_usr_bin_hierarchy
      require 'rbt/utility_scripts/symlink_into_the_usr_bin_hierarchy.rb'
      RBT::SymlinkIntoTheUsrBinHierarchy.new(_, &block)
    # ======================================================================= #
    # === :to_current
    #
    # You can pass the program name and program version into this entry
    # point. Passing the full path is also possible.
    #
    # Usage example:
    #
    #   RBT.action(:to_current, 'htop-1.2')
    #
    # ======================================================================= #
    when :to_current,
         :ToCurrent
      require 'rbt/actions/individual_actions/to_current/to_current.rb'
      RBT.to_current(_, &block)
    # ======================================================================= #
    # === :symlink_from_to_current
    #
    # Usage example from within ruby files:
    #
    #   action(:symlink_from_to_current, _)
    #
    # ======================================================================= #
    when :symlink_from_to_current,
         :SymlinkFromToCurrent,
         :symlink_this_appdir,
         :SymlinkThisAppdir
      require 'rbt/utility_scripts/symlink_from_to_current/symlink_from_to_current.rb'
      RBT::SymlinkFromToCurrent.new(_)
    # ======================================================================= #
    # === :force
    #
    # This variant forcues the symlink-action.
    # ======================================================================= #
    when :force,
         :symlink_from_to_current2,
         :SymlinkFromToCurrent2
      require 'rbt/utility_scripts/symlink_from_to_current/symlink_from_to_current.rb'
      _ = RBT::SymlinkFromToCurrent.new(_, :do_not_run_yet)
      _.do_force_symlink
      _.run
    # ======================================================================= #
    # === :infobox_for
    # ======================================================================= #
    when :infobox_for,
         :InfoboxFor,
         :infobox,
         :Infobox,
         :flexbox,
         :Flexbox
      begin # Try to pull in the sexy flexbox.
        require 'ascii_paradise/requires/require_flexbox.rb'
        AsciiParadise::Flexbox.new(_, &block)
      rescue LoadError
        :flexbox_is_not_available
      end
    # ======================================================================= #
    # === :rbt_version
    # ======================================================================= #
    when :rbt_version
      require 'rbt/version/version.rb'
      return RBT.version?
    # ======================================================================= #
    # === :ensure_that_the_log_directory_exists
    #
    # Usage example:
    #
    #   action(:ensure_that_the_log_directory_exists)
    #
    # ======================================================================= #
    when :ensure_that_the_log_directory_exists
      _ = log_directory?
      mkdir(_) unless File.directory? _
    # ======================================================================= #
    # === :generate_version_file
    #
    # Usage example:
    #
    #   action(:generate_version_file)
    #
    # ======================================================================= #
    when :generate_version_file,
         :GenerateVersionFile
      require 'rbt/toplevel_methods/generate_programs_version_yaml_file.rb'
      RBT.generate_programs_version_yaml_file
    # ======================================================================= #
    # === :test_this_alias
    # ======================================================================= #
    when :test_this_alias,
         :rbt_test_alias
      require 'rbt/toplevel_methods/available_programs.rb'
      require 'rbt/aliases/aliases.rb'
      RBT.test_this_alias(_)
    # ======================================================================= #
    # === :prefix_object
    # ======================================================================= #
    when :prefix_object,
         :PrefixObject,
         :prefix,
         :Prefix
      require 'rbt/prefix/prefix.rb'
      _ = _.first if _.is_a? Array
      RBT.prefix_object(_, &block)
    # ======================================================================= #
    # === :return_appdir_prefix
    #
    # Usage example:
    #
    #   action(:return_appdir_prefix, 'htop')
    #
    # ======================================================================= #
    when :return_appdir_prefix,
         :appdir_prefix
      RBT.return_appdir_prefix(_)
    # ======================================================================= #
    # === :determine_appdir_prefix_from_this_input
    #
    # Usage example:
    #
    #   RBT.installer(:determine_appdir_prefix_from_this_input)
    #
    # ======================================================================= #
    when :determine_appdir_prefix_from_this_input
      RBT.determine_appdir_prefix_from_this_input(_)
    # ======================================================================= #
    # === :extract
    #
    # This entry point allows the user to extract an archive.
    #
    # It is a special entry point, because it can also handle pseudo
    # symbols, such as in the following case:
    #
    #   RBT.action(:extract, ':htop') { :be_silent }
    #
    # This will take htop from its default position on the local filesystem.
    #
    # Usage example:
    #
    #   RBT.action(:extract, '/home/x/src/htop/htop-3.1.2.tar.xz', extract_to: '/Depot/jjjj/')
    #    
    # Commandline invocation example:
    #
    #   raction mate_desktop
    #
    # ======================================================================= #
    when :extract,
         :Extract,
         :action_extracter,
         :extract_what_to
      require 'rbt/actions/individual_actions/extract/extract.rb'
      extract_to = options_hash[:extract_to] if options_hash.is_a?(Hash)
      unless _.is_a?(Symbol)
        _ = _.to_s
      end
      if extract_to.nil?
        extract_to = RBT.log_directory?
      end
      RBT.extract(_, extract_to, &block)
    # ======================================================================= #
    # === :program_version
    #
    # This entry point will specifically make use of
    # ProgramInformation.program_version? to find out the program-version
    # from a given String.
    #
    # Commandline usage example:
    #
    #   raction program_version gtkmm-3.24.7
    #
    # ======================================================================= #
    when :program_version,
         :ProgramVersion
      require 'program_information'
      return ProgramInformation.program_version?(_, &block)
    # ======================================================================= #
    # === :program_name
    #
    # This entry point will specifically make use of
    # ProgramInformation.program_name? to find out the program-name from
    # a given String.
    #
    # Commandline usage example:
    #
    #   raction program_name gtkmm-3.24.7
    #
    # ======================================================================= #
    when :program_name,
         :ProgramName
      require 'program_information'
      return ProgramInformation.program_name?(_, &block)
    # ======================================================================= #
    # === :increment_program_version
    #
    # Commandline usage example:
    #
    #   raction increment_program_version
    #
    # ======================================================================= #
    when :increment_program_version,
         :IncrementProgramVersion,
         :increment,
         :Increment
      require 'rbt/utility_scripts/increment_program_version/increment_program_version.rb'
      RBT.increment_program_version
    # ======================================================================= #
    # === report_mate_desktop_version
    #
    # Commandline invocation example:
    #
    #   raction mate_desktop
    #
    # ======================================================================= #
    when :report_mate_desktop_version,
         :report_the_mate_desktop_version,
         :report_mate_desktop,
         :mate_desktop_version,
         :mate_version,
         :mate_desktop,
         :mate?
      require 'rbt/actions/individual_actions/information/report_mate_desktop_version.rb'
      RBT.report_mate_desktop_version
    # ======================================================================= #
    # === :create_program_version_url_file
    #
    # Usage example:
    #
    #   RBT.action(:CreateProgramVersionUrlFile, :ruby)
    #
    # Commandline invocation example:
    #
    #   raction create_program_version_url_file
    #
    # ======================================================================= #
    when :create_program_version_url_file,
         :CreateProgramVersionUrlFile,
         :program_version_url_file
      require 'rbt/actions/individual_actions/information/create_program_version_url_file.rb'
      RBT.create_program_version_url_file(_)
    # ======================================================================= #
    # === :compile_the_python_addons
    # ======================================================================= #
    when :compile_the_python_addons
      require 'rbt/requires/require_the_installer_class.rb'
      compile_object = RBT::Action::Installer.new { :dont_run_yet }
      compile_object.compile_the_python_addons
      compile_object.do_initiate_batch_compilation
    # ======================================================================= #
    # === :gitty
    # ======================================================================= #
    when :gitty,
         :Gitty
      require 'rbt/utility_scripts/gitty.rb'
      RBT.gitty(_)
    # ======================================================================= #
    # === :simple_appdir_configure
    #
    #   bl $RBT/actions/individual_actions/simple_appdir_configure/simple_appdir_configure.rb
    #
    # ======================================================================= #
    when :simple_appdir_configure,
         :SimpleAppdirConfigure
      require 'rbt/actions/individual_actions/simple_appdir_configure/simple_appdir_configure.rb'
      RBT.simple_appdir_configure(_)
    # ======================================================================= #
    # === :file_ldd_dataset
    # ======================================================================= #
    when :file_ldd_dataset,
         :ldd_file
      require 'rbt/utility_scripts/simplified_ldd_output.rb'
      RBT.file_ldd_dataset?
    # ======================================================================= #
    # === :ldd_dataset
    #
    # Usage example:
    #
    #   dataset = RBT.action(:ldd_dataset)
    #
    # ======================================================================= #
    when :ldd_dataset,
         :LddDataset
      require 'rbt/utility_scripts/simplified_ldd_output.rb'
      RBT.ldd_dataset?
    # ======================================================================= #
    # === :create_ldd_dataset
    #
    # This will create a new ldd-dependencies file, containing all
    # local libraries.
    #
    # Commandline invocation example:
    #
    #   rbt_action create_ldd_dataset
    #
    # ======================================================================= #
    when :create_ldd_dataset,
         :create_dependencies_of_libraries_file,
         :CreateDependenciesOfLibrariesFile
      require 'rbt/utility_scripts/simplified_ldd_output.rb'
      RBT.create_ldd_dataset
    # ======================================================================= #
    # === :show_configuration_options
    #
    # This entry point allows the user to query the configuration
    # options used by a given program. On my home computer I have
    # aliased this to "sco", so I can just do "sco gcc" to query
    # the configuration options used by gcc.
    # ======================================================================= #
    when :show_configuration_options,
         :ShowConfigurationOptions,
         :sco
      require 'rbt/requires/require_show_configuration_options.rb'
      RBT.show_configuration_options(_, &block)
    # ======================================================================= #
    # === :scan_for_blfs_programs
    #
    # Commandline invocation example:
    #
    #   raction scan_for_blfs_programs
    #
    # ======================================================================= #
    when :scan_for_blfs_programs,
         :ScanForBlfsPrograms,
         :missing_blfs,
         :MissingBlfs
      require 'rbt/actions/individual_actions/cookbooks/scan_for_blfs_programs/scan_for_blfs_programs.rb'
      RBT.scan_for_blfs_programs
    # ======================================================================= #
    # === :return_blfs
    #
    # This variant, although a slight misnomer, is for commandline use.
    #
    # It will simply print the BLFS of the program at hand.
    #
    # Commandline invocation example:
    #
    #   rbt_action return_blfs
    #
    # ======================================================================= #
    when :return_blfs
      require 'rbt/actions/individual_actions/cookbooks/blfs/blfs.rb' # bl $RBT/actions/individual_actions/cookbooks/blfs/blfs.rb
      e RBT.return_blfs_page_of_this_program(_, &block)
    # ======================================================================= #
    # === :copy_these_archives
    # ======================================================================= #
    when :copy_these_archives,
         :CopyTheseArchives,
         :copy,
         :Copy
      require 'rbt/utility_scripts/copy_these_archives.rb'
      RBT.copy_these_archives(_)
    # ======================================================================= #
    # === :not_found_libraries
    #
    # This entry point allows the user to show libraries that are missing,
    # aka "not found libraries".
    # ======================================================================= #
    when :not_found_libraries,
         :NotFoundLibraries
      require 'rbt/utility_scripts/not_found_libraries.rb'
      RBT.not_found_libraries
    # ======================================================================= #
    # === :remove_empty_directories
    #
    # Commandline usage example:
    #
    #   raction remove_empty_directories
    #
    # ======================================================================= #
    when :remove_empty_directories,
         :RemoveEmptyDirectories
      require 'rbt/utility_scripts/remove_empty_directories.rb'
      RBT.remove_empty_directories(_)
    # ======================================================================= #
    # === :show_manual_steps
    #
    # Commandline usage example:
    #
    #   raction show_manual_steps glibc
    #
    # ======================================================================= #
    when :show_manual_steps
      require 'rbt/utility_scripts/show_manual_steps.rb'
      RBT.show_manual_steps(_)
    # ======================================================================= #
    # === :chain_compile
    #
    # Invocation example:
    #
    #   raction chain_compile kde5_plasma
    #
    # ======================================================================= #
    when :chain_compile,
         :ChainCompile
      require 'rbt/utility_scripts/chain_compile.rb'
      RBT.chain_compile(_, &block)
    # ======================================================================= #
    # === :show_compile_chain
    #
    # Invocation example:
    #
    #   raction show_compile_chain kde5_plasma
    #
    # ======================================================================= #
    when :show_compile_chain,
         :ShowCompileChain,
         :display_compile_chain,
         :DisplayCompileChain,
         :show_compile_chain_of,
         :ShowCompileChainOf
      require 'rbt/actions/individual_actions/show_compile_chain/show_compile_chain.rb'
      RBT.show_compile_chain(_)
    # ======================================================================= #
    # === :download_all_source_archives
    # ======================================================================= #
    when :download_all_source_archives,
         :DownloadAllSourceArchives,
         :obtain_source_archives,
         :ObtainSourceArchives,
         :batch_download_all_source_archives,
         :BatchDownloadAllSourceArchives
      require 'rbt/cookbooks/download_all_source_archives/download_all_source_archives.rb'
      RBT::Cookbooks::DownloadAllSourceArchives.new(_)
    # ======================================================================= #
    # === :batch_validate_the_cookbook_recipes
    # ======================================================================= #
    when :batch_validate_the_cookbook_recipes,
         :BatchValidateTheCookbookRecipes,
         :validate_entries,
         :ValidateEntries
      require 'rbt/checks_and_validations/batch_validate_the_cookbook_recipes.rb'
      RBT.validate_entries
    # ======================================================================= #
    # === :homepage
    # ======================================================================= #
    when :homepage,
         :report_the_homepage,
         :report_homepage
      require 'rbt/cookbooks/homepage/homepage.rb'
      RBT.homepage(_)
    # ======================================================================= #
    # === :compile_in_traditional_manner
    #
    # This entry point will compile via the /usr/ prefix.
    # ======================================================================= #
    when :compile_in_traditional_manner
      require 'rbt/utility_scripts/compile_in_traditional_manner.rb'
      RBT::CompileInTraditionalManner.new(_)
    # ======================================================================= #
    # === :store_abbreviations
    #
    # Commandline usage example:
    #
    #   raction abbrev
    #
    # ======================================================================= #
    when :store_abbreviations,
         :StoreAbbreviations,
         :abbrev
      require 'rbt/toplevel_methods/available_programs.rb'
      RBT.store_abbreviations
    # ======================================================================= #
    # === :update_available_programs
    #
    # Commandline usage example:
    #
    #   raction update_available_programs
    #
    # ======================================================================= #
    when :update_available_programs,
         :UpdateAvailablePrograms,
         :save_available_programs,
         :SaveAvailablePrograms
      require 'rbt/toplevel_methods/available_programs.rb'
      RBT.store_available_programs_into_a_yaml_file
    # ======================================================================= #
    # === :url1
    #
    # This will display the url1 content on the commandline.
    #
    # Commandline invocation example:
    #
    #   raction url1 openssl
    #   raction url1 cairo
    #   raction url1 pango
    #   raction url1 sdl2
    #   raction url1 sdl2image
    #   raction url1 sdl2mixer
    #
    # ======================================================================= #
    when :url1
      result = action(:SanitizeCookbook, _, &block)
      e result.url?
    # ======================================================================= #
    # === :url2
    #
    # This will display the url2 content on the commandline.
    #
    # Commandline invocation example:
    #
    #   raction url2 ruby 
    #
    # ======================================================================= #
    when :url2
      result = action(:SanitizeCookbook, _, &block)
      e result.url2?
    # ======================================================================= #
    # === :return_blfs_page_of_this_program
    #
    # Usage examples:
    #
    #   RBT.action(:blfs, 'glibc')
    #   action(:blfs, 'abc') { :def }
    #   action(:return_blfs_page_of_this_program, 'abc') { :def }
    #
    # Commandline invocation example:
    #
    #   rbt_action return_blfs_page_of_this_program htop
    #
    # ======================================================================= #
    when :return_blfs_page_of_this_program,
         :return_blfs_entry_for,
         :blfs_page_of_this_program,
         :blfs_page,
         :report_all_blfs_entries
      require 'rbt/actions/individual_actions/cookbooks/blfs/blfs.rb'
      RBT.return_blfs_page_of_this_program(_, &block)
    # ======================================================================= #
    # === :blfs_instance
    #
    # This will return an instance of class BLFS.
    #
    # Usage example:
    #
    #   action(:blfs_instance, 'abc') { :def }
    #
    # ======================================================================= #
    when :blfs_instance,
         :BlfsInstance,
         :blfs,
         :Blfs
      require 'rbt/actions/individual_actions/cookbooks/blfs/blfs.rb'
      RBT.blfs_instance(_, &block)
    # ======================================================================= #
    # === :validate_blfs_entries
    #
    # Commandline usage example:
    #
    #   raction validate_blfs_entries
    #
    # ======================================================================= #
    when :validate_blfs_entries
      require 'rbt/checks_and_validations/validate_blfs_entries.rb'
      RBT.validate_blfs_entries
    # ======================================================================= #
    # === :paste_blfs
    # ======================================================================= #
    when :paste_blfs
      require 'rbt/requires/require_the_installer_class.rb'
      _ = RBT::Action::Installer.new(_, :do_not_run_yet) 
      _.try_to_show_the_BLFS_content_of_this_program(_)
    # ======================================================================= #
    # === :cleanup_system_tags
    #
    # Usage example:
    #
    #   action(:CleanupSystemTags) { :be_quiet }
    #
    # ======================================================================= #
    when :cleanup_system_tags,
         :CleanupSystemTags
      require 'rbt/utility_scripts/cleanup_system_tags.rb'
      RBT::CleanupSystemTags.new(_, &block)
    # ======================================================================= #
    # === :allowed_cookbook_entries
    # ======================================================================= #
    when :allowed_cookbook_entries,
         :AllowedCookbookEntries
      require 'rbt/checks_and_validations/allowed_cookbook_entries.rb'
      RBT::Cookbooks::AllowedCookbookEntries.new
    # ======================================================================= #
    # === :static_overview_of_the_available_binaries_on_this_computer_system
    # ======================================================================= #
    when :static_overview_of_the_available_binaries_on_this_computer_system,
         :StaticOverviewOfTheAvailableBinariesOnThisComputerSystem
      require 'rbt/utility_scripts/static_overview_of_the_available_binaries_on_this_computer_system.rb'
      RBT.static_overview_of_the_available_binaries_on_this_computer_system
    # ======================================================================= #
    # === :validate_cookbook_aliases
    #
    # Usage example:
    #
    #   RBT.action(:validate_cookbook_aliases)
    #
    # Commandline usage example:
    #
    #   raction validate_cookbook_aliases
    #
    # ======================================================================= #
    when :validate_cookbook_aliases,
         :ValidateCookbookAliases,
         :validate_the_cookbook_aliases,
         :ValidateTheCookbookAliases
      require 'rbt/checks_and_validations/validate_cookbook_aliases.rb'
      RBT.validate_cookbook_aliases
    # ======================================================================= #
    # === :expand_cookbooks
    #
    # Commandline invocation example:
    #
    #   raction expand_cookbooks
    #
    # ======================================================================= #
    when :expand_cookbooks,
         :ExpandCookbooks,
         :expand,
         :EXPAND,
         :expand_this_cookbook,
         :ExpandThisCookbook
      require 'rbt/actions/individual_actions/cookbooks/expand_cookbooks/expand_cookbooks.rb'
      RBT.expand_cookbooks(_)
    # ======================================================================= #
    # === :show_last_updated
    #
    # Usage example:
    #
    #   action(:ShowLastUpdated)
    #
    # Commandline usage example:
    #
    #   raction show_last_updated
    #
    # ======================================================================= #
    when :show_last_updated,
         :ShowLastUpdated
      require 'rbt/cookbooks/show_last_updated.rb'
      RBT.show_last_updated(_)
    # ======================================================================= #
    # === :purge_incorrect_yaml_directories_in_the_programs_hierarchy
    #
    # Commandline usage example:
    #
    #   raction purge_incorrect_yaml_directories_in_the_programs_hierarchy
    #
    # ======================================================================= #
    when :purge_incorrect_yaml_directories_in_the_programs_hierarchy,
         :PurgeIncorrectYamlDirectoriesInTheProgramsHierarchy
      require 'rbt/utility_scripts/purge_incorrect_yaml_directories_in_the_programs_hierarchy.rb'
      RBT::PurgeIncorrectYamlDirectoriesInTheProgramsHierarchy.new(_, &block)
    # ======================================================================= #
    # === :binary_name_belongs_to_which_program
    # ======================================================================= #
    when :binary_name_belongs_to_which_program,
         :BinaryNameBelongsToWhichProgram
      require 'rbt/utility_scripts/binary_name_belongs_to_which_program.rb'
      RBT::BinaryNameBelongsToWhichProgram.new(_, &block)
    # ======================================================================= #
    # === :check_for_gnome_updates
    # ======================================================================= #
    when :check_for_gnome_updates,
         :CheckForGnomeUpdates
      require 'rbt/check_for_updates/check_for_gnome_updates.rb'
      RBT::Cookbooks::CheckForGnomeUpdates.new(_, &block)
    # ======================================================================= #
    # === :register_program_files_into_yaml_database
    # ======================================================================= #
    when :register_program_files_into_yaml_database,
         :RegisterProgramFilesIntoYamlDatabase
      require 'rbt/utility_scripts/register_program_files_into_yaml_database.rb'
      RBT.register_program_files_into_yaml_database(_, &block)
    # ======================================================================= #
    # === :colour_make_install
    # ======================================================================= #
    when :colour_make_install,
         :ColourMakeInstall
      require 'rbt/make/colour_make_install.rb'
      RBT.colour_make_install(_)
    # ======================================================================= #
    # === :filter_for_valid_program_names
    #
    # Here we will modify e. g. "htop.yml" towards "htop" or "hto" towards
    # "htop".
    # ======================================================================= #
    when :filter_for_valid_program_names,
         :main_filter,
         :sanitize_this_program_name
      require 'rbt/actions/individual_actions/filter_for_valid_program_names/filter_for_valid_program_names.rb'
      return RBT.filter_for_valid_program_names(_)
    # ======================================================================= #
    # === :show_how_many_files_are_tracked
    #
    # Invocation example from the commandline:
    #
    #   raction show_how_many_files_are_tracked
    #
    # ======================================================================= #
    when :show_how_many_files_are_tracked,
         :ShowHowManyFilesAreTracked
      require 'rbt/actions/individual_actions/statistics/show_how_many_files_are_tracked.rb'
      RBT.show_how_many_files_are_tracked
    # ======================================================================= #
    # === :compile_strategies
    #
    # Invocation example from the commandline:
    #
    #   raction compile_strategies
    #
    # ======================================================================= #
    when :compile_strategies,
         :CompileStrategies,
         :report_the_compile_strategies,
         :ReportTheCompileStrategies
      require 'rbt/actions/individual_actions/statistics/compile_strategies.rb'
      RBT.report_the_compile_strategies
    # ======================================================================= #
    # === :show_compile_time_statistics
    #
    # Invocation example from the commandline:
    #
    #   raction show_compile_time_statistics
    #
    # ======================================================================= #
    when :show_compile_time_statistics,
         :ShowCompileTimeStatistics
      require 'rbt/actions/individual_actions/statistics/show_compile_time_statistics.rb'
      RBT.show_compile_time_statistics
    # ======================================================================= #
    # === :return_compile_time_statistics
    #
    # Commandline usage example:
    #
    #   raction return_compile_time_statistics
    #
    # ======================================================================= #
    when :return_compile_time_statistics,
         :ReturnCompileTimeStatistics
      require 'rbt/actions/individual_actions/statistics/return_compile_time_statistics.rb'
      pp RBT.return_compile_time_statistics
    # ======================================================================= #
    # === :apply_sed_operations
    # ======================================================================= #
    when :apply_sed_operations,
         :ApplySedOperations
      require 'rbt/utility_scripts/apply_sed_operations.rb'
      RBT.apply_sed_operations(_)
    # ======================================================================= #
    # === :local_check_for_latest
    #
    # Commandline usage example:
    #
    #   raction local_check_for_latest
    #
    # ======================================================================= #
    when :local_check_for_latest,
         :LocalCheckForLatest
      require 'rbt/cookbooks/local_check_for_latest/local_check_for_latest.rb'
      RBT.local_check_for_latest
    # ======================================================================= #
    # === :create_and_populate_pkgconfig_directory
    #
    # Commandline invocation example:
    #
    #   raction create_and_populate_pkgconfig_directory
    #
    # ======================================================================= #
    when :create_and_populate_pkgconfig_directory,
         :CreateAndPopulatePkgconfigDirectory,
         :consider_creating_and_populating_the_pkgconfig_directory,
         :ConsiderCreatingAndPopulatingThePkgconfigDirectory
      require 'rbt/misc/create_and_populate_pkgconfig_directory.rb'
      CreateAndPopulatePkgconfigDirectory.new(_)
    # ======================================================================= #
    # === :update_entry
    # ======================================================================= #
    when :update_entry,
         :UpdateEntry
      require 'rbt/requires/require_class_update_entry.rb'
      RBT::UpdateEntry.new(_, &block)
    # ======================================================================= #
    # === :remove_program
    # ======================================================================= #
    when :remove_program,
         :RemoveProgram
      require 'rbt/utility_scripts/remove_program.rb'
      RBT.remove_program(_)
    # ======================================================================= #
    # === :remove_programs
    #
    # This is the plurarl form.
    # ======================================================================= #
    when :remove_programs,
         :RemovePrograms
      require 'rbt/utility_scripts/remove_programs/remove_programs.rb'
      RBT.remove_programs
    # ======================================================================= #
    # === :return_xfce_version
    # ======================================================================= #
    when :return_xfce_version
      require 'rbt/toplevel_methods/try_to_return_a_special_compile_component.rb'
    # ======================================================================= #
    # === :report_xfce_version
    #
    # Commandline invocation example:
    #
    #   raction xfce_version
    #   raction xfce?
    #
    # ======================================================================= #
    when :report_xfce_version,
         :xfce_version,
         :xfce?
      require 'rbt/actions/individual_actions/information/report_xfce_version.rb'
      RBT.report_xfce_version
    # ======================================================================= #
    # === :highest
    #
    # Usage example:
    #
    #   action(:highest)
    #
    # Commandline invocation example:
    #
    #   raction highest 55
    #
    # ======================================================================= #
    when :highest
      require 'rbt/actions/individual_actions/information/highest.rb'
      RBT.highest(_)
    # ======================================================================= #
    # === :create_dependencies_file
    #
    # This entry point is specifically for GoboLinux, allowing the user
    # to create a dependencies file.
    # ======================================================================= #
    when :create_dependencies_file,
         :CreateDependenciesFile
      require 'rbt/linux/gobolinux/create_dependencies_file.rb'
      create_dependencies_file = RBT::Linux::Gobolinux::CreateDependenciesFile.new(
        _[0], 
        _[1],
        :dont_run_yet
      )
      create_dependencies_file.allow_creation_of_resources
      create_dependencies_file.run # Finally, run it.
    # ======================================================================= #
    # === :show_dependencies_of
    # ======================================================================= #
    when :show_dependencies_of
      require 'rbt/cookbooks/show_dependencies_of.rb'
      RBT.show_dependencies_of(_)
    # ======================================================================= #
    # === :update_kde_porting_aids
    # ======================================================================= #
    when :update_kde_porting_aids,
         :UpdateKdePortingAids
      require 'rbt/check_for_updates/update_kde_porting_aids.rb'
      RBT.update_kde_porting_aids
    # ======================================================================= #
    # === :update_kde_apps
    # ======================================================================= #
    when :update_kde_apps,
         :UpdateKdeApps,
         :update_kde_application,
         :UpdateKdeApplication,
         :update_kde_applications,
         :UpdateKdeApplications
      require 'rbt/check_for_updates/update_kde_applications.rb'
      RBT.update_kde_applications
    # ======================================================================= #
    # === :update_kde_framework
    # ======================================================================= #
    when :update_kde_framework
      require 'rbt/check_for_updates/update_kde_framework.rb'
      RBT.update_kde_framework
    # ======================================================================= #
    # === :missing_use_this_build_system_entry
    #
    # Commandline usage example:
    #
    #   raction missing_use_this_build_system_entry
    #
    # ======================================================================= #
    when :missing_use_this_build_system_entry,
         :missing_build_system,
         :MissingBuildSystem
      require 'rbt/checks_and_validations/missing_use_this_build_system_entry.rb'
      RBT.missing_use_this_build_system_entry
    # ======================================================================= #
    # === :update_all_ruby_gems
    # ======================================================================= #
    when :update_all_ruby_gems,
         :check_for_new_release_on_rubygems,
         :check_for_new_releases_on_rubygems
      require 'rbt/check_for_updates/check_for_new_release_on_rubygems.rb'
      RBT.update_all_ruby_gems
    # ======================================================================= #
    # === :check_for_existing_source_directories
    # ======================================================================= #
    when :check_for_existing_source_directories,
         :CheckForExistingSourceDirectories
      require 'rbt/cookbooks/check_for_existing_source_directories.rb'
      RBT.check_if_source_directories_exist
    # ======================================================================= #
    # === :try_to_return_a_special_compile_component
    #
    #   rbt_action try_to_return_a_special_compile_component
    #
    # ======================================================================= #
    when :try_to_return_a_special_compile_component,
         :TryToReturnASpecialCompileComponent
      require 'rbt/toplevel_methods/try_to_return_a_special_compile_component.rb'
      RBT.try_to_return_a_special_compile_component(_)
    # ======================================================================= #
    # === :generate_shell_completion
    # ======================================================================= #
    when :generate_shell_completion,
         :GenerateShellCompletion
      require 'rbt/generate_files/generate_shell_completion.rb'
      RBT.generate_shell_completion(_)
    # ======================================================================= #
    # === :symlink_all_upcased_programs_to_downcased_variants
    # ======================================================================= #
    when :symlink_all_upcased_programs_to_downcased_variants,
         :SymlinkAllUpcasedProgramsToDowncasedVariants
      require 'rbt/utility_scripts/symlink_all_upcased_programs_to_downcased_variants.rb'
      RBT::SymlinkAllUpcasedProgramsToDowncasedVariants.new
    # ======================================================================= #
    # === :package_rbt_scripts
    # ======================================================================= #
    when :package_rbt_scripts,
         :PackageRbtScripts
      require 'rbt/utility_scripts/package_rbt_scripts.rb'
      RBT::PackageRbtScripts.new
    # ======================================================================= #
    # === :sed_wrapper
    # ======================================================================= #
    when :sed_wrapper,
         :SedWrapper
      require 'rbt/actions/individual_actions/sed_wrapper/sed_wrapper.rb'
      RBT.sed_wrapper(_)
    # ======================================================================= #
    # === :report_how_many_binaries_are_registered
    #
    # Commandline invocation example:
    #
    #   raction report_how_many_binaries_are_registered
    #
    # ======================================================================= #
    when :report_how_many_binaries_are_registered
      require 'rbt/toplevel_methods/report_how_many_binaries_are_registered.rb'
      RBT.report_how_many_binaries_are_registered
    # ======================================================================= #
    # === :search_for_tags
    # ======================================================================= #
    when :search_for_tags,
         :SearchForTags,
         :stag
      require 'rbt/requires/require_search_for_tags.rb'
      RBT.search_for_tags(_)
    # ======================================================================= #
    # === :remove_all_symlinks
    # ======================================================================= #
    when :remove_all_symlinks,
         :RemoveAllSymlinks
      require 'rbt/utility_scripts/remove_all_symlinks.rb'
      RemoveAllSymlinks.new
    # ======================================================================= #
    # === :url_new
    # ======================================================================= #
    when :url_new
      require 'rbt/cookbooks/url/url.rb'
      RBT::Cookbooks::Url.new(_)
    # ======================================================================= #
    # === :url
    # ======================================================================= #
    when :url
      require 'rbt/cookbooks/url/url.rb'
      RBT.return_url1_of_this_program(_)
    # ======================================================================= #
    # === :create_shell_script_containing_these_program_versions
    # ======================================================================= #
    when :create_shell_script_containing_these_program_versions,
         :generate_a_big_shell_script
      require 'rbt/shell/create_shell_script_containing_these_program_versions.rb'
      RBT.create_shell_script_containing_these_program_versions
    # ======================================================================= #
    # === :create_shell_scripts
    # ======================================================================= #
    when :create_shell_scripts,
         :create_bash_scripts,
         :to_bash,
         :CreateBashScripts
      require 'rbt/generate_files/create_bash_scripts.rb'
      RBT::CreateBashScripts.new # bl $RBT/create_bash_scripts.rb
    # ======================================================================= #
    # === :register_everything
    #
    # Commandline invocation example:
    #
    #   raction register_everything
    #
    # ======================================================================= #
    when :register_everything,
         :RegisterEverything
      require 'rbt/registered/register_everything.rb'
      RBT.register_everything
    # ======================================================================= #
    # === :show_all_about
    #
    # Usage example:
    #
    #   RBT.action(:show_all_about, 'ruby')
    #
    # Commandline invocation example:
    #
    #   raction show_all_about ruby
    #
    # ======================================================================= #
    when :show_all_about,
         :ShowAllAbout
      require 'rbt/actions/individual_actions/information/show_all_about.rb'
      RBT.show_all_about(_)
    # ======================================================================= #
    # === :remove_libtool_files
    #
    # Invocation example:
    #
    #   raction remove_libtool_files kde5_plasma
    #
    # ======================================================================= #
    when :remove_libtool_files,
         :delete_libtool_files
      require 'rbt/libtool/remove_libtool_files/remove_libtool_files.rb'
      RBT.remove_libtool_files(_)
    # ======================================================================= #
    # === :expanded_cookbooks
    #
    # raction expanded_cookbooks htop
    #
    # ======================================================================= #
    when :expanded_cookbooks,
         :ExpandedCookbooks,
         :expanded
      require 'rbt/actions/individual_actions/cookbooks/expanded_cookbook/expanded_cookbook.rb'
      pp RBT.expanded_cookbook(_).dataset?
    # ======================================================================= #
    # === :aggregate_information_from_the_expanded_cookbooks
    #
    # Commandline invocation example:
    #
    #   raction aggregate
    #
    # ======================================================================= #
    when :aggregate_information_from_the_expanded_cookbooks,
         :aggregate
      require 'rbt/actions/individual_actions/information/aggregate_information_from_the_expanded_cookbooks.rb'
      RBT.aggregate_information_from_the_expanded_cookbooks(_)
    # ======================================================================= #
    # === :scan_source_archive
    # ======================================================================= #
    when :scan_source_archive,
         :ScanSourceArchive
      require 'rbt/cookbooks/scan_source_archive/scan_source_archive.rb'
      RBT.scan_source_archive      
    # ======================================================================= #
    # === :report_the_registered_programs
    #
    # Invocation example from the commandline:
    #
    #   raction report_the_registered_programs
    #
    # ======================================================================= #
    when :report_the_registered_programs,
         :ReportTheRegisteredPrograms
      require 'rbt/utility_scripts/report_the_registered_programs.rb'
      RBT.report_the_registered_programs
    # ======================================================================= #
    # === :feedback_licenses
    #
    # Commandline usage example:
    #
    #   raction feedback_licenses
    #
    # ======================================================================= #
    when :feedback_licenses,
         :feedback_licences,
         :FeedbackLicenses,
         :FeedbackLicences
      require 'rbt/cookbooks/feedback_licenses.rb'
      RBT.feedback_licenses(_) # bl $RUBY_COOKBOOKS/lib/cookbooks/feedback_licenses.rb
    # ======================================================================= #
    # === :scan_for_missing_last_update_entries
    # ======================================================================= #
    when :scan_for_missing_last_update_entries,
         :missing_last_update_entries
      require 'rbt/checks_and_validations/scan_for_missing_last_update_entries.rb:class'
      RBT::Cookbooks::ScanForMissingLastUpdateEntries.new
    # ======================================================================= #
    # === check_for_mate_desktop_updates
    #
    # Commandline invocation example:
    #
    #   raction check_for_mate_desktop_updates
    #
    # ======================================================================= #
    when :check_for_mate_desktop_updates,
         :CheckForMateDesktopUpdates,
         :mate_updates
      require 'rbt/check_for_updates/check_for_mate_desktop_updates.rb'
      RBT.check_for_mate_desktop_updates
    # ======================================================================= #
    # === :show_all_statistics
    #
    # Usage example:
    #
    #   RBT.action(:show_all_statistics)
    #
    # Commandline invocation example:
    #
    #   raction show_all_statistics
    #
    # ======================================================================= #
    when :show_all_statistics,
         :ShowAllStatistics,
         :show_statistics,
         :ShowStatistics
      require 'rbt/actions/individual_actions/statistics/show_all_statistics.rb'
      RBT.show_all_statistics
    # ======================================================================= #
    # === :display_barplot_statistics
    #
    # Usage example:
    #
    #   action(:display_barplot_statistics)
    #
    # Invocation example from the commandline:
    #
    #   raction display_barplot_statistics
    #
    # ======================================================================= #
    when :display_barplot_statistics
      require 'rbt/actions/individual_actions/statistics/display_barplot_statistics.rb'
      RBT.display_barplot_statistics
    # ======================================================================= #
    # === :uninstall
    # ======================================================================= #
    when :uninstall
      require 'rbt/utility_scripts/uninstall.rb'
      RBT.uninstall(_)
    # ======================================================================= #
    # === :what_could_become_an_app_dir
    # ======================================================================= #
    when :what_could_become_an_app_dir
      require 'rbt/utility_scripts/what_could_become_an_app_dir.rb'
      RBT.what_could_become_an_app_dir
    # ======================================================================= #
    # === :run_configure
    #
    # Usage example:
    #
    #   RBT.action(:run_configure, :htop)
    #
    # ======================================================================= #
    when :run_configure,
         :configure
      require 'rbt/actions/individual_actions/configure/configure.rb'
      RBT::Actions::Configure.new(_)
    # ======================================================================= #
    # === :run_configure
    #
    # This will really just run "./configure" basically.
    #
    # Usage example:
    #
    #   RBT.installer(:htop)
    #
    # ======================================================================= #
    when :simple_run_configure,
         :simple_configure
      yielded = nil
      if block_given?
        yielded = yield
      end
      cmd = "./configure #{_}"
      case yielded
      # ===================================================================== #
      # === :colourize_and_padded
      # ===================================================================== #
      when :colourize_and_padded,
           :colourized_and_padded
        e
        e steelblue(cmd)
        e
        system cmd
      # ===================================================================== #
      # === :colourized
      # ===================================================================== #
      when :colourized,
           :colourize
        e steelblue(cmd)
        system cmd
      else # else without colours.
        esystem cmd
      end
    # ======================================================================= #
    # === :all_urls
    # ======================================================================= #
    when :all_urls,
         :all_URLs
      require 'rbt/cookbooks/all_urls/all_urls.rb'
      RBT.all_urls(_)
    # ======================================================================= #
    # === :create_registered_tags
    #
    # Commandline usage example:
    #
    #   raction create_registered_tags
    #
    # ======================================================================= #
    when :create_registered_tags,
         :CreateRegisteredTags
      require 'rbt/registered/registered_tags.rb'
      RBT.create_registered_tags
    # ======================================================================= #
    # === :feedback_description_of
    #
    # Commandline usage example:
    #
    #   raction feedback_description_of ruby
    #
    # ======================================================================= #
    when :feedback_description_of
      require 'rbt/actions/individual_actions/information/feedback_description_of.rb'
      RBT.feedback_description_of(_)
    # ======================================================================= #
    # === :feedback_program_description
    #
    # This entry point will quickly show the description of the given
    # program, onto the commandline.
    #
    # Commandline usage example:
    #
    #   raction feedback_program_description ruby
    #
    # ======================================================================= #
    when :feedback_program_description
      require 'rbt/toplevel_methods/feedback_program_description.rb'
      RBT.feedback_program_description(_)
    # ======================================================================= #
    # === :create_cookbook_yaml_file
    #
    # Commandline usage example:
    #
    #   raction create_cookbook_yaml_file libxml2
    #
    # ======================================================================= #
    when :create_cookbook_yaml_file,
         :create_new_cookbook,
         :create_a_new_cookbook
      require 'rbt/cookbooks/create_cookbook_yaml_file/create_cookbook_yaml_file.rb'
      RBT.create_cookbook_yaml_file(_)
    # ======================================================================= #
    # === :update_tags
    #
    # This entry point will update the registered tags.
    # ======================================================================= #
    when :update_tags,
         :registered_tags
      require 'rbt/registered/registered_tags.rb'
      RBT::RegisteredTags.new
    # ======================================================================= #
    # === :chained_programs
    # ======================================================================= #
    when :chained_programs
      require 'rbt/constants/constants.rb'
      return RBT.chained_programs?
    # ======================================================================= #
    # === :url_action
    # ======================================================================= #
    when :url_action
      require 'rbt/url_action/url_action.rb'
      RBT::UrlAction[_]
    # ======================================================================= #
    # === :dual_compile
    # ======================================================================= #
    when :dual_compile
      require 'rbt/toplevel_methods/dual_compile.rb'
      RBT.dual_compile(_)
    # ======================================================================= #
    # === :custom_toolchain
    # ======================================================================= #
    when :custom_toolchain
      require 'rbt/requires/require_the_installer_class.rb'
      RBT::Action::Installer.new(_)
    # ======================================================================= #
    # === :chroot_compile
    # ======================================================================= #
    when :chroot_compile
      require 'rbt/toplevel_methods/chroot_and_chown_related_code.rb'
      RBT.chroot_compile(_)
    # ======================================================================= #
    # === :uchroot_compile
    # ======================================================================= #
    when :uchroot_compile
      require 'rbt/toplevel_methods/toplevel_methods.rb'
      RBT.uchroot_compile(_) # uchroot nasm bison
    # ======================================================================= #
    # === :generate_all_slack_desc_files
    #
    # This action allows the user to batch-create all slack-desc files.
    # ======================================================================= #
    when :generate_all_slack_desc_files
      require 'rbt/requires/require_the_installer_class.rb'
      _ = RBT::Action::Installer.new(:dont_run_yet)
      target = "#{rbt_log_directory?}slackware_desc_files/"
      opnn; mkdir(target, :be_verbose)
      cd target
      RBT.all_programs?.each {|entry|
        _.generate_slack_desc_file_for_this_program(entry)
      }
    # ======================================================================= #
    # === :trad
    # ======================================================================= #
    when :trad,
         :traditional
      require 'rbt/utility_scripts/compile_in_traditional_manner.rb'
      RBT::CompileInTraditionalManner.new(_)
    # ======================================================================= #
    # === :report_where_the_source_directory_can_be_found
    # ======================================================================= #
    when :report_where_the_source_directory_can_be_found,
         :report_base_directory,
         :report_archive_directory
      e "#{rev}The directory for source archives can be found at:"
      e
      e "  #{sdir(RBT.source_directory?)}"
      e
    # ======================================================================= #
    # === :report_host_cpu
    # ======================================================================= #
    when :report_host_cpu
      e "#{rev}The host system (CPU) in use is: "\
        "#{sfancy(host_cpu?)}"
    # ======================================================================= #
    # === :manual_steps
    # ======================================================================= #
    when :manual_steps
      require 'rbt/requires/require_the_installer_class.rb'
      compile_this_program = _.first
      compile_object = RBT::Action::Installer.new(compile_this_program, :do_not_run_yet)
      compile_object.set_compile_this_program(compile_this_program)
      compile_object.menu('--manual-steps')
      compile_object.run
    # ======================================================================= #
    # === :find_all_archive_types
    # ======================================================================= #
    when :find_all_archive_types
      require 'rbt/cookbooks/find_all_archive_types.rb'
      RBT::Cookbooks::FindAllArchiveTypes.new(_).show_results
    # ======================================================================= #
    # === :beautify_configure_help_output
    # ======================================================================= #
    when :beautify_configure_help_output
      require 'rbt/utility_scripts/beautify_configure_help_output.rb'
      RBT::BeautifyConfigureHelpOutput.new(_)
    # ======================================================================= #
    # === :colour_make
    # ======================================================================= #
    when :colour_make
      require 'rbt/make/make.rb'
      RBT::Make.new(_)
    # ======================================================================= #
    # === :compile_via_appdir_prefix_without_symlinking
    # ======================================================================= #
    when :compile_via_appdir_prefix_without_symlinking
      require 'rbt/toplevel_methods/compile_via_appdir_prefix_without_symlinking.rb'
      RBT.compile_via_appdir_prefix_without_symlinking(_)
    # ======================================================================= #
    # === :feedback_all_available_formats
    # ======================================================================= #
    when :feedback_all_available_formats
      require 'rbt/cookbooks/sanitize_cookbook/misc.rb'
      RBT::Cookbooks::SanitizeCookbook.feedback_all_available_formats
    # ======================================================================= #
    # === :create_snapcraft_file
    # ======================================================================= #
    when :create_snapcraft_file
      require 'rbt/cookbooks/create_snapcraft_file.rb'
      RBT::Cookbooks::CreateSnapcraftFile.new(_)
    # ======================================================================= #
    # === :extra_information
    # ======================================================================= #
    when :extra_information
      require 'rbt/utility_scripts/extra_information.rb'
      RBT::ExtraInformation.show(_)
    # ======================================================================= #
    # === :autoupdate_this_program
    # ======================================================================= #
    when :autoupdate_this_program
      require 'rbt/utility_scripts/autoupdate_this_program.rb'
      RBT::AutoupdateThisProgram.new(_)
    # ======================================================================= #
    # === :autosymlink
    # ======================================================================= #
    when :autosymlink
      require 'rbt/utility_scripts/autosymlink.rb'
      RBT::Autosymlink.new(_)
    # ======================================================================= #
    # === :compile_these_programs
    #
    # Invocation example:
    #
    #   compile_these_programs php python
    #
    # ======================================================================= #
    when :compile_these_programs
      require 'rbt/requires/require_the_installer_class.rb'
      if _.is_a?(String) and _.include?(' ')
        _ = optional_argument1.split(' ')
      end
      _.each {|entry|
        RBT::Action::Installer.new(entry)
      }
    # ======================================================================= #
    # === :compile_into_home_dir
    # ======================================================================= #
    when :compile_into_home_dir,
         :home_dir
      require 'rbt/utility_scripts/compile_into_home_dir.rb'
      RBT::CompileIntoHomeDir.new(_)
    # ======================================================================= #
    # === :home_dir_without_symlinking
    # ======================================================================= #
    when :home_dir_without_symlinking,
         :home_dir_without_symlink
      require 'rbt/utility_scripts/compile_into_home_dir_without_symlinking.rb'
      RBT::CompileIntoHomeDirWithoutSymlinking.new(_)
    # ======================================================================= #
    # === :parse_help
    # ======================================================================= #
    when :parse_help,
         :parse_configure_help
      require 'rbt/utility_scripts/parse_configure_help.rb'
      RBT::ParseConfigureHelp.new(_)
    # ======================================================================= #
    # === :query_file_association
    # ======================================================================= #
    when :query_file_association
      require 'rbt/utility_scripts/query_file_association.rb'
      RBT::QueryFileAssociation.new(_)
    # ======================================================================= #
    # === :make
    # ======================================================================= #
    when :make
      require 'rbt/make/make.rb'
      RBT::Make.new(_).run
    # ======================================================================= #
    # === :show_versions_of_these_programs
    #
    # Usage example:
    #
    #   show_versions_of_these_programs htop php ruby
    #
    # ======================================================================= #
    when :show_versions_of_these_programs
      require 'rbt/utility_scripts/show_versions_of_these_programs.rb'
      RBT::ShowVersionsOfThesePrograms.new(_)
    # ======================================================================= #
    # === :toolchain
    # ======================================================================= #
    when :toolchain
      require 'rbt/utility_scripts/toolchain.rb'
      RBT::Toolchain.new(_)
    # ======================================================================= #
    # === :chainer
    # ======================================================================= #
    when :chainer
      require 'rbt/utility_scripts/chainer.rb'
      RBT::Chainer.new(_)
    # ======================================================================= #
    # === :check_for_binary_duplicates
    # ======================================================================= #
    when :check_for_binary_duplicates
      require 'rbt/toplevel_methods/check_for_binary_duplicates.rb'
      RBT.check_for_binary_duplicates    
    # ======================================================================= #
    # === :create_programs_url_file
    # ======================================================================= #
    when :create_programs_url_file
      require 'rbt/actions/individual_actions/information/create_program_version_url_file.rb'
      RBT.create_programs_url_file
    # ======================================================================= #
    # === :create_log_directory
    #
    # This entry point can be used to create a new default log
    # directory for the RBT gem.
    # ======================================================================= #
    when :create_log_directory,
         :log_directory
      require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'
      RBT.create_log_directory
    # ======================================================================= #
    # === :report_total_size_of_all_archives
    #
    # Invocation example:
    #
    #   raction report_total_size_of_all_archives
    #
    # ======================================================================= #
    when :report_total_size_of_all_archives
      require 'rbt/toplevel_methods/archive_related_code.rb'
      RBT.report_total_size_of_all_archives
    # ======================================================================= #
    # === :create_html_page_for_these_programs
    #
    # Invocation example:
    #
    #   create_html_page_for_these_programs php ruby libx11 zlib
    #
    # ======================================================================= #
    when :create_html_page_for_these_programs
      require 'rbt/toplevel_methods/html.rb'
      _ = _
      if _.is_a?(String) and _.include?(' ')
        _ = _.split(' ')
      end
      RBT.create_html_page_for_these_programs(_)
    # ======================================================================= #
    # === :create_programs_version_html_file
    #
    # Commandline invocation example:
    #
    #   raction create_programs_version_html_file
    #
    # ======================================================================= #
    when :create_programs_version_html_file
      require 'rbt/toplevel_methods/html.rb'
      RBT.create_programs_version_html_file
    # ======================================================================= #
    # === :find_url_for
    # ======================================================================= #
    when :find_url_for
      require 'rbt/toplevel_methods/url.rb'
      RBT.find_url_for(_)
    # ======================================================================= #
    # === :find_likely_directory
    # ======================================================================= #
    when :find_likely_directory
      require 'rbt/actions/individual_actions/find_likely_directory/find_likely_directory.rb'
      RBT.find_likely_directory(_)
    # ======================================================================= #
    # === :binary_of
    # ======================================================================= #
    when :binary_of
      require 'rbt/utility_scripts/binary_name_belongs_to_which_program.rb'
      RBT::BinaryNameBelongsToWhichProgram.new(_)
    # ======================================================================= #
    # === :headers
    # ======================================================================= #
    when :headers
      require 'rbt/actions/individual_actions/headers/headers.rb'
      RBT.headers(ARGV).report
    # ======================================================================= #
    # === :remove_symlinks
    # ======================================================================= #
    when :remove_symlinks
      require 'rbt/utility_scripts/remove_symlinks.rb'
      RBT::RemoveSymlinks.new(_)
    # ======================================================================= #
    # === :show_available_tags
    # ======================================================================= #
    when :show_available_tags
      require 'rbt/toplevel_methods/show_available_tags.rb'
      RBT.show_available_tags(:default, :be_quiet)
    # ======================================================================= #
    # === :download
    # ======================================================================= #
    when :download,
         :rbt_download # A necessary "alias".
      require 'rbt/toplevel_methods/download.rb'
      RBT.download(_)
    # ======================================================================= #
    # === :libtool
    # ======================================================================= #
    when :libtool
      require 'rbt/libtool/libtool.rb'
      RBT.libtool(_)
    # ======================================================================= #
    # === :gather_information
    # ======================================================================= #
    when :gather_information
      require 'rbt/actions/individual_actions/information/gather_information.rb'
      RBT.gather_information(_)
    # ======================================================================= #
    # === :multi_url_displayer
    # ======================================================================= #
    when :multi_url_displayer,
         :multi_URL_displayer
      require 'rbt/cookbooks/multi_url_displayer/multi_url_displayer.rb'
      RBT::Cookbooks::MultiUrlDisplayer.new(_)
    # ======================================================================= #
    # === :create_pkgconfig_file
    # ======================================================================= #
    when :create_pkgconfig_file
      require 'rbt/generate_files/create_pkgconfig_file.rb'
      RBT::CreatePkgconfigFile.new(_)
    # ======================================================================= #
    # === :install_this_rubygem
    #
    # Usage example:
    #
    #   RBT.action(:install_this_rubygem, 'path_to_the.gem')
    #
    # Commandline invocation example:
    #
    #   raction install_gem /home/x/src/rails/rails-7.0.1.gem
    #
    # ======================================================================= #
    when :install_this_rubygem,
         :install_this_gem,
         :install_gem
      RBT.install_this_rubygem(_)
    # ======================================================================= #
    # === :build_detector
    #
    # Commandline invocation example:
    #
    #   raction build_detector
    #
    # ======================================================================= #
    when :build_detector
      require 'rbt/utility_scripts/build_detector.rb'
      RBT::BuildDetector.new(_).report
    # ======================================================================= #
    # === :raw_cookbook
    #
    # Usage example:
    #
    #   RBT.action(:raw_cookbook, 'glib')
    #
    # ======================================================================= #
    when :raw_cookbook,
         :raw
      require 'rbt/actions/individual_actions/cookbooks/raw_cookbook/raw_cookbook.rb'
      RBT.raw_cookbook(_, options_hash, &block)
    # ======================================================================= #
    # === :check_this_cookbook_file_for_errors
    #    
    # Commandline invocation example:
    #
    #   raction check_this_cookbook_file_for_errors ruby
    #
    # ======================================================================= #
    when :check_this_cookbook_file_for_errors
      require 'rbt/toplevel_methods/check_this_cookbook_file_for_errors.rb'
      RBT.check_this_cookbook_file_for_errors(_)
    # ======================================================================= #
    # === :remove_outdated_archives
    #
    # Commandline invocation example:
    #
    #   raction remove_outdated_archives
    #
    # ======================================================================= #
    when :remove_outdated_archives
      require 'rbt/utility_scripts/remove_outdated_archives.rb'
      RBT::RemoveOutdatedArchives.new(_)
    # ======================================================================= #
    # === :scookie
    # ======================================================================= #
    when :scookie
      require 'rbt/toplevel_methods/report_extended_information_about_this_program.rb'
      RBT.report_extended_information_about_this_program(_)
    # ======================================================================= #
    # === :find_alternative_archive
    #
    # Usage example:
    #
    #   RBT.action(:find_alternative_archive, '$MY_SRC/gdkpixbuf/gdk-pixbuf')
    #
    # Commandline invocation example:
    #
    #   raction find_alternative_archive $MY_SRC/gdkpixbuf/gdk-pixbuf
    #
    # ======================================================================= #
    when :find_alternative_archive,
         :alternative_archive,
         :return_alternative_archive
      require 'rbt/actions/individual_actions/find_alternative_archive/find_alternative_archive.rb'
      return RBT.return_alternative_archive(_)
    # ======================================================================= #
    # === :generate_programs_version_yaml_file
    #
    # Usage example:
    #
    #   RBT.action(:generate_programs_version_yaml_file)
    #
    # Commandline invocation example:
    #
    #   raction generate_programs_version_yaml_file
    #
    # ======================================================================= #
    when :generate_programs_version_yaml_file
      require 'rbt/toplevel_methods/generate_programs_version_yaml_file.rb'
      RBT.generate_programs_version_yaml_file
    # ======================================================================= #
    # === :try_to_find_shortcut_for_a_chain_compiled_program
    #
    # Usage example:
    #
    #   RBT.action(:try_to_find_shortcut_for_a_chain_compiled_program, 'kde1')
    #
    # Commandline invocation example:
    #
    #   raction shortcut_to_chain_compiled_program kde1
    #
    # ======================================================================= #
    when :try_to_find_shortcut_for_a_chain_compiled_program,
         :shortcut_to_chain_compiled_program
      require 'rbt/toplevel_methods/try_to_find_shortcut_for_a_chain_compiled_program.rb'
      e RBT.try_to_find_shortcut_for_a_chain_compiled_program(_)
    # ======================================================================= #
    # === :get_all_files_from
    #
    # Usage example:
    #
    #   RBT.action(:get_all_files_from, '/home/x/src/htop/')
    #
    # ======================================================================= #
    when :get_all_files_from
      RBT.get_all_files_from(_)
    # ======================================================================= #
    # === :infer_build_system
    #
    # Usage example:
    #
    #   RBT.action(:infer_build_system)
    #
    # Commandline invocation example:
    #
    #   raction installer php
    #
    # ======================================================================= #
    when :infer_build_system,
         :infer
      require 'rbt/actions/individual_actions/infer_build_system/infer_build_system.rb'
      RBT.infer_then_report_the_build_system(_)
    # ======================================================================= #
    # === :fast_return_file_size_of_this_program
    #
    # Usage example:
    #
    #   RBT.action(:fast_return_file_size_of_this_program, 'glib')
    #
    # ======================================================================= #
    when :fast_return_file_size_of_this_program
      require 'rbt/toplevel_methods/fast_return_file_size_of_this_program.rb'
      RBT.fast_return_file_size_of_this_program(_, &block)
    # ======================================================================= #
    # === :make_app_prefix
    # ======================================================================= #
    when :make_app_prefix
      require 'rbt/toplevel_methods/make_app_prefix.rb'
      RBT.make_app_prefix(_)
    # ======================================================================= #
    # === :check_for_invalid_entries_in_this_cookbook
    # ======================================================================= #
    when :check_for_invalid_entries_in_this_cookbook
      require 'rbt/actions/individual_actions/checks_and_validations/check_for_invalid_entries_in_this_cookbook.rb'
      RBT::Action::Cookbooks::CheckForInvalidEntriesInThisCookbook.new(_)
    # ======================================================================= #
    # === nil
    # ======================================================================= #
    when nil
      e 'Please provide a valid action to the RBT.action() '\
        'method (nil was supplied).'
    else # else tag
      e "#{rev}Not a registered action: "\
        "#{action_that_is_desired} "\
        "(class: #{action_that_is_desired.class})"
    end
  end; self.instance_eval { alias actions action } # === RBT.actions

  # ========================================================================= #
  # === RBT.action_compile
  #
  # This provides a simplified "compile-action".
  #
  # Invocation example:
  #
  #   RBT.acompile(:htop)
  #
  # ========================================================================= #
  def self.action_compile(
      compile_this_program,
      &block
    )
    RBT.action(:Installer, compile_this_program.to_s, &block)
  end; self.instance_eval { alias acompile action_compile } # === RBT.acompile

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  first_argument = :extract
  if ARGV and !ARGV[0].nil?
    first_argument = ARGV[0].to_sym
  end
  RBT.action(first_argument, :php)
  # RBT.action(:create_program_skeleton, ARGV)
  e RBT.action(:filter_for_valid_program_names, ARGV)
  RBT.action(:generate_all_slack_desc_files)
end # raction
    # raction htop-1.2
    # raction htop.yml
