#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Cookbooks::ExpandCookbooks
#
# This class is able to expand the dataset stored in the various individual
# .yml files, from the RBT project, and sanitize this dataset into one
# big Hash. This Hash will contain all the information about the various
# different programs, in a proper way.
#
# Note that this class can also expand the dataset of a single .yml file,
# such as if you pass in, as argument, "ruby" or "php". See the bottom
# of the file for this.
#
# Thus, the class has to support two different primary use cases; to work
# in a batch-manner, and in a per-yaml file specific way.
#
# Usage example:
#
#   RBT::Action::Cookbooks::ExpandCookbooks.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/cookbooks/expand_cookbooks/expand_cookbooks.rb'
# RBT::Action::Cookbooks::ExpandCookbooks.new(ARGV)
# RBT.expand_cookbooks(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Cookbooks

class ExpandCookbooks < RBT::Action # === RBT::Action::Cookbooks::ExpandCookbooks

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      expand_which_programs = :all_of_them,
      run_already           = true,
      &block
    )
    reset
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    set_expand_these_programs(
      expand_which_programs
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @simple_or_complex
    # ======================================================================= #
    @simple_or_complex = :simple
    # ======================================================================= #
    # === @expand_these_programs
    # ======================================================================= #
    @expand_these_programs = [] # Initialize it properly.
    obtain_available_cookbooks
  end

  # ========================================================================= #
  # === simple_or_complex?
  # ========================================================================= #
  def simple_or_complex?
    @simple_or_complex
  end

  # ========================================================================= #
  # === consider_updating_program_version_and_url_file
  # ========================================================================= #
  def consider_updating_program_version_and_url_file
    if is_on_roebe?
      # ===================================================================== #
      # On my home system I will auto-update that file as well.
      # ===================================================================== #
      action(:create_program_version_url_file)
    end
  end

  # ========================================================================= #
  # === set_expand_these_programs
  # ========================================================================= #
  def set_expand_these_programs(
      i = :all_of_them
    )
    if i.is_a? Array
      i = i.first
    end
    i = :all_of_them if i.nil? # Assign to the default value, in this case.
    case i
    # ======================================================================= #
    # === expand_cookbooks --help
    #
    # We must be careful here - some programs such as "help2man" exist,
    # so we can not merely scan for the "help" substring.
    # ======================================================================= #
    when /^-?-?help$/i
      show_help(:then_exit)
    # ======================================================================= #
    # === :all_of_them
    # ======================================================================= #
    when :all_of_them,
         '' # Empty Strings are treated the same since as of May 2022.
      i = @available_programs
    end
    unless i.is_a? Array
      i = [i] # Turn it into an Array.
    end
    i.map! {|entry|
      entry = entry.dup if entry.frozen?
      entry.delete!('-') if entry.include?('-')
      if entry and entry.end_with?('.yml')
        entry.delete_suffix!('.yml')
      end
      entry
    }
    @expand_these_programs = i
  end

  # ========================================================================= #
  # === main_array?
  # ========================================================================= #
  def main_array?
    @expand_these_programs
  end

  # ========================================================================= #
  # === return_default_program_name_as_string
  # ========================================================================= #
  def return_default_program_name_as_string
    remove_file_archive_at_the_end(
      File.basename(main_array?.first.to_s)
    )
  end

  # ========================================================================= #
  # === obtain_available_cookbooks
  # ========================================================================= #
  def obtain_available_cookbooks
    @available_programs = available_programs?
  end

  # ========================================================================= #
  # === ensure_that_the_base_directory_exists
  # ========================================================================= #
  def ensure_that_the_base_directory_exists
    unless File.directory? directory_expanded_cookbooks?
      mkdir(directory_expanded_cookbooks?)
    end
  end

  # ========================================================================= #
  # === inform_the_user_what_we_will_do_next
  # ========================================================================= #
  def inform_the_user_what_we_will_do_next
    if main_array?.size < 2
      opne "The program called `"\
            "#{sfancy(return_default_program_name_as_string.downcase)}#{rev}`"\
            " will now be expanded,"
      opne 'towards its individual yaml file.'
    else
      opne "#{rev}We will now expand the whole dataset stored into "\
           "the individual yaml files."
    end
  end

  # ========================================================================= #
  # === work_on_each_cookbook
  #
  # Work on each cookbook and save the dataset as a hash.
  # ========================================================================= #
  def work_on_each_cookbook
    is_on_roebe = is_on_roebe?
    if @expand_these_programs.size > 1
      opne "#{rev}Various programs will be expanded now, using the "\
           "#{steelblue(simple_or_complex?)}"\
           "#{rev} mode."
    end
    @expand_these_programs.each {|program|
      program = remove_archive_at_the_end(File.basename(program)).downcase
      # if program =~ /\d+/
      #   program = ProgramInformation.return_program_name(program)
      # end
      # ^^^ This is not ideal, unfortunately, due to programs such
      #     as "help2man".
      _ = action(:SanitizeCookbook, program, autogenerate_expanded_dataset: false) { @simple_or_complex }
      hash = _.internal_dataset? # Obtain the full dataset here - our main Hash.
      # ===================================================================== #
      # We will sanitize this Hash a little bit - in particular we will
      # remove empty URL entries, such as "url4: ''", as this does not
      # appear to be too terribly useful. For more sanitize-operation
      # see the method that does the job.
      # ===================================================================== #
      hash = sanitize_this_hash(hash)
      dataset_to_save = YAML.dump(hash)
      what = dataset_to_save
      
      # ===================================================================== #
      # Get the target, aka where to store, next:
      # ===================================================================== #
      into = rds("#{directory_expanded_cookbooks?}#{program}.yml")
      RBT.store_this_expanded_dataset_into_that_yaml_file(what, into)
      opne "Now storing the #{royalblue('Hash')} #{rev}from "\
           "`#{slateblue(program)}#{rev}"\
           "` into `#{sfile(into)}#{rev}`."
      if is_on_roebe
        into = "#{RUBY_SRC_DIR_RBT_YAML}expanded_cookbooks/#{File.basename(into)}"
        opne "Also storing the #{royalblue('Hash')} #{rev}from "\
             "`#{slateblue(program)}"\
             "#{rev}` into `#{sfile(into)}#{rev}`."
      end
    }
  end

  # ========================================================================= #
  # === sanitize_this_hash                                     (sanitize tag)
  # ========================================================================= #
  def sanitize_this_hash(i)
    if i.is_a? Hash # Only work on Hashes.
      # ===================================================================== #
      # Next, sort this Hash alphabetically:
      # ===================================================================== #
      array = i.sort_by {|key, value| key }
      i = array.to_h
    end
    i
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help(also_exit = false)
    case also_exit
    # ======================================================================= #
    # === :then_exit
    # ======================================================================= #
    when :then_exit
      also_exit = true
    end
    opne 'This class will expand the given arguments '\
         '(program names)  into .yml files.'
    e
    e 'Invocation example:'
    e
    e sfancy('  expand_cookbooks rubyprogressbar')
    e
    exit if also_exit
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    ensure_that_the_base_directory_exists
    inform_the_user_what_we_will_do_next
    work_on_each_cookbook
    consider_updating_program_version_and_url_file
  end

  # ========================================================================= #
  # === RBT::Action::Cookbooks::ExpandCookbooks[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

# =========================================================================== #
# === RBT.expand_cookbooks
#
# The first argument to this method should be the program-name whose .yml
# file we wish to autogenerate, such as 'subversion' or 'htop' or 'ruby'.
#
# It will then be expanded to its full, correct dataset.
# =========================================================================== #
def self.expand_cookbooks(
    i = ARGV,
    &block
  )
  return RBT::Action::Cookbooks::ExpandCookbooks.new(i, &block)
end; self.instance_eval { alias expand_this_cookbook expand_cookbooks } # === RBT.expand_this_cookbook

end

if __FILE__ == $PROGRAM_NAME
  RBT.expand_cookbooks(ARGV)
end # expandcookbooks
    # expandcookbooks htop
    # expandcookbooks m2crypto
    # expandcookbooks subversion
    # expandcookbooks rdesktop
    # expandcookbooks tiff