# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/actions/individual_actions/cookbooks/sanitize_cookbook/sanitize_related_code.rb'
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

module Cookbooks

class SanitizeCookbook < RBT::Action # === RBT::Action::Cookbooks::SanitizeCookbook

  # ========================================================================= #
  # === sanitize_everything (sanitize tag, sanitize everything tag, sync tag, sani tag)
  #
  # This is the method that will sanitize the whole internal dataset,
  # which is typically derived from a yaml-based dataset, via class
  # RawCookbook.
  #
  # Some entries have to come in a specific order, since they depend on
  # prior values that were already set correctly.
  # ========================================================================= #
  def sanitize_everything(
      url1 = raw_cookbook?.url1?,
      url2 = raw_cookbook?.url2?
    )
    sanitize_url1(url1) # This must come before we sync program-name-and-program-version.
    sanitize_url2(url2)
    sanitize_program_name_and_program_version # The program name and program version should come quite early.
    sanitize_homepage
    sanitize_wikipedia
    sanitize_md5sum
    sanitize_required_deps_on
    sanitize_BLFS_entry
    sanitize_short_name
    sanitize_description
    sanitize_extra_information
    sanitize_last_update_entry
    sanitize_env_variables
    sanitize_has_to_be_compiled
    sanitize_short_description
    sanitize_pre_configure_steps
    sanitize_is_an_active_project
    sanitize_build_static
    sanitize_can_be_compiled_statically
    sanitize_keep_extracted
    sanitize_program_version
    sanitize_apply_patch
    # ======================================================================= #
    # === Sanitize the Prefix
    #
    # The prefix setting can only be sanitized after the program_version
    # has been sanitized, since the actual prefix may depend on the
    # program version in use, such as for AppDirs.
    # ======================================================================= #
    sanitize_the_prefix # Must come BEFORE we sanitize the post-installation steps.
    # ======================================================================= #
    # === Clean up post-install related activities
    #
    # The clean-up of post-install related activities should only be
    # called after the prefix was already defined, since a few
    # internal-parts of the method sync_postinstall
    # may need to know the real prefix in use.
    # ======================================================================= #
    sanitize_postinstall
    sanitize_sed_entry
    sanitize_the_binaries
    sanitize_the_headers
    sanitize_symlink_headers
    sanitize_the_libraries
    sanitize_configure_command_to_use
    sanitize_configure_options
    sanitize_configure_options_explained
    sanitize_may_we_modify_the_configure_options
    sanitize_cmake_configure_options
    sanitize_meson_configure_options
    sanitize_installation_steps
    sanitize_gir_files
    sanitize_tags
    sanitize_licence
    sanitize_autogen
    sanitize_use_autoconf
    sanitize_pre_make_commands
    sanitize_parameters_to_make
    sanitize_preinstall
    sanitize_git_url
    sanitize_use_this_make_command
    sanitize_use_this_program_name_on_gobolinux
    sanitize_pkgconfig_files
    sanitize_symlink_pkgconfig_files
    sanitize_symlink_headers
    sanitize_run_configure
    sanitize_autosymlink_lib64
    sanitize_do_not_symlink
    sanitize_echo_yes
    sanitize_enable_shared
    sanitize_enable_static
    sanitize_flatpak_url
    sanitize_libexec
    sanitize_m4_files
    sanitize_mirror
    sanitize_symlink_glib_schema
    sanitize_use_this_make_install_command
    sanitize_configure_base_dir
    sanitize_base_dir
    sanitize_svn_url
    sanitize_use_build_directory
    sanitize_use_this_build_directory
    sanitize_use_this_build_system
    sanitize_localstatedir
    sanitize_do_not_download
    sanitize_modify_the_makefile
    sanitize_manual_steps
    sanitize_sha256
    # ======================================================================= #
    # Last but not least, find out the file size of the program in question.
    # ======================================================================= #
    calculate_archive_size
    calculate_md5sum if shall_we_display_the_md5sum?
  end; alias sync_everything sanitize_everything # === sync_everything

  # ========================================================================= #
  # === sanitize_sha256
  # ========================================================================= #
  def sanitize_sha256(
      i = program_path?
    )
    if File.file?(i) # The file ought to exist.
      _ = RBT.shasum(i)
      set_sha256(_)
    end
  end

  # ========================================================================= #
  # === sanitize_program_path
  # ========================================================================= #
  def sanitize_program_path
    reassemble_program_path
  end; alias rebuild_path sanitize_program_path # === rebuild_path

  # ========================================================================= #
  # === sanitize_env_variables
  #  
  # This will sanitize ENV variables. The ENV variables must be a
  # Hash at all times.
  # ========================================================================= #
  def sanitize_env_variables
    # ======================================================================= #
    # Sync it first.
    # ======================================================================= #
    dataset?[:set_env_variables] = raw_cookbook?.set_env_variables?
    _ = set_env_variables?
    if _ and _.is_a? Array # Must also check for nil.
      # ===================================================================== #
      # If it is an Array, then it must be wrong, so correct this here
      # silently. We need a Hash, as specified in the specification.
      # ===================================================================== #
      hash = {}
      _.each {|inner_hash|
        if inner_hash.is_a? Hash
          key   = inner_hash.keys.first
          value = inner_hash.values.first
          hash[key] = value
        end
      }
      dataset?[:set_env_variables] = hash
    end 
  end; alias sanitize_set_env_variables sanitize_env_variables # === sanitize_set_env_variables

  # ========================================================================= #
  # === sanitize_configure_options                            (configure tag)
  #
  # Work on the configure options, i.e. @dataset['configure_options'].
  #
  # Note that this depends on program_version? having been set properly.
  #
  # Some options should not be sanitized, such as "-Wno-dev -G Ninja .."
  # in applications that use the cmake build system, e. g. LLVM.
  # ========================================================================= #
  def sanitize_configure_options
    # ======================================================================= #
    # We must first check if we are allowed to modify the configure options
    # at all. Some programs, such as openssl, will not allow such
    # modification due to their strange way how they mix different
    # configure-options.
    #
    # There are some exceptions to this, though. Newlines for example
    # can always be removed.
    # ======================================================================= #
    _ = raw_cookbook?.configure_options?.to_s.strip # We don't want newlines.
    if may_we_modify_the_configure_options?
      # ===================================================================== #
      # Get rid of multi-blanks next.
      # ===================================================================== #
      if _
        # =================================================================== #
        # We will assume this to be a comment, so we have to remove these
        # comments.
        # =================================================================== #
        if _.include? ' # '
          _ = remove_comments_from_each_line(_)
        end
        _.squeeze!('/')
        _.gsub!(/#{N}/,' ')
      end
      # ===================================================================== #
      # Treat 'VERSION' as a macro, for expansion.
      # ===================================================================== #
      if _.include? 'PROGRAM_VERSION'
        _.gsub!(/PROGRAM_VERSION/, program_version?.to_s) if _
      elsif _.include? 'VERSION'
        _.gsub!(/VERSION/, program_version?.to_s) if _
      end
      # ===================================================================== #
      # Treat 'CURRENT' as a macro, for expansion.
      # ===================================================================== #
      if _.include? 'CURRENT'
        _.gsub!(/CURRENT/, program_version?.to_s) if _
      end
      # ===================================================================== #
      # === NAME_OF_THE_DISTRIBUTION
      # ===================================================================== #
      if _.include? 'NAME_OF_THE_DISTRIBUTION'
        name = load_yaml(
          "#{project_yaml_directory?}configuration/name_of_the_distribution.yml"
        )
        _.gsub!(/NAME_OF_THE_DISTRIBUTION/, name)
      end
      unless _.empty?
        # =================================================================== #
        # Finally, run through the configure options. If they start with
        # the Strings "enable" or "disable", with or without, we prepend --
        # if this is not yet the case.
        #
        # Since as of September 2011, we will always prepend --, unless
        # the first character already is a '-' (this is done to 
        # =================================================================== #
        splitted = _.split(' ')
        _ = splitted.map {|configure_option|
          # If configure_option =~ /^disable|enable|with|without/
          # ================================================================= #
          # Only add when not yet first char is NOT a '-'.
          # ================================================================= #
          unless configure_option.start_with?('-') or
                 configure_option.start_with?('$') or
                 configure_option.start_with?('.')
            unless (configure_option == 'Ninja') and
                   splitted.include?('-G') # For cmake variants like: "-G Ninja"
              configure_option.prepend('--')
            end
          end
          # end
          configure_option
        }.join(' ')
      end
      # ===================================================================== #
      # Try to handle $ input.
      # ===================================================================== #
      if _.include? '$'
        unless Object.const_defined? :ConvertGlobalEnv
          require 'convert_global_env'
        end
        _ = ConvertGlobalEnv.convert(_)
      end
    end
    _.tr!(N, ' ') # Replace all newlines again.
    set_configure_options(_) # Last but not least, reassign.
  end

  # ========================================================================= #
  # === sanitize_base_dir
  # ========================================================================= #
  def sanitize_base_dir
    _ = base_dir?.to_s
    set_base_dir(_)
  end

  # ========================================================================= #
  # === sanitize_configure_base_dir
  #
  # We must ensure that this variable has a trailing / character.
  #
  # There are several reasons for this, but one important reason is
  # that class RBT::Compile will do a calculation based on the
  # amount of '/' in the configure_base_dir string. So if we have
  # one more or less '/', the calculation's result will vary -
  # thus, we must ensure that the '/' is trailing here.
  # ========================================================================= #
  def sanitize_configure_base_dir
    _ = raw_cookbook?.configure_base_dir?
    if _.nil?
      _ = configure_base_dir?
    end
    set_configure_base_dir(rds("#{_}/"))
  end

  # ========================================================================= #
  # === sanitize_symlink_glib_schema
  # ========================================================================= #
  def sanitize_symlink_glib_schema
    _ = raw_cookbook?.symlink_glib_schema?.to_s
    if _ and _.empty?
      _ = symlink_glib_schema?.to_s
    end
    set_symlink_glib_schema(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_use_this_make_command
  #
  # This will default to whatever is stored in the configuration file.
  #
  # Should be 'make' usually, but could be "cmake" or another make-like
  # command.
  # ========================================================================= #
  def sanitize_use_this_make_command
    _ = use_this_make_command?
    if _ and _.empty?
      _ = raw_cookbook?.use_this_make_command?
      if _.nil? or _.empty?
        _ = MAKE_COMMAND_TO_USE # MAKE_COMMAND_TO_USE is defined in constants.rb
      end
    end
    set_use_this_make_command(_)
  end

  # ========================================================================= #
  # === sanitize_extra_information
  #
  # We must not lose information from the original dataset, in regards
  # to extra_information.
  # ========================================================================= #
  def sanitize_extra_information
    _ = raw_cookbook?.extra_information?
    if _
      _.chomp!  if _.respond_to? :chomp!
      _.rstrip! if _.respond_to? :rstrip!
      _ = word_wrap(_) if use_word_wrap?
      set_extra_information(_)
    end
  end

  # ========================================================================= #
  # === sanitize_configure_options_explained
  # ========================================================================= #
  def sanitize_configure_options_explained
    _ = raw_cookbook?.configure_options_explained?.to_s
    set_configure_options_explained(_)
  end

  # ========================================================================= #
  # === sanitize_enable_static
  # ========================================================================= #
  def sanitize_enable_static
    _ = raw_cookbook?.enable_static?.to_s
    if _ and _.empty?
      _ = enable_static?.to_s
    end
    set_do_not_symlink(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_use_autoconf
  # ========================================================================= #
  def sanitize_use_autoconf
    _ = raw_cookbook?.use_autoconf?
    if _ and _.is_a?(String) and _.empty?
      _ = use_autoconf?.to_s
    end
    _ = to_bool(_)
    _ = false if _.nil?
    set_use_autoconf(_)
  end

  # ========================================================================= #
  # === sanitize_wikipedia
  # ========================================================================= #
  def sanitize_wikipedia
    _ = raw_cookbook?.wikipedia?
    set_wikipedia(_)
  end

  # ========================================================================= #
  # === sanitize_git_url
  # ========================================================================= #
  def sanitize_git_url
    _ = git_url?
    _ = raw_cookbook?.git_url? if _.nil?
    set_git_url(_) if _
  end

  # ========================================================================= #
  # === sanitize_use_this_program_name_on_gobolinux
  # ========================================================================= #
  def sanitize_use_this_program_name_on_gobolinux(
      are_we_on_gobolinux = are_we_on_gobolinux?
    )
    _ = program_compile_name?
    if are_we_on_gobolinux and
      !use_this_program_name_on_gobolinux?.to_s.empty?
      _ = use_this_program_name_on_gobolinux?.to_s
      set_program_compile_name(_) # Will always be at the least an empty String.
    end
  end; alias sanitize_program_compile_name sanitize_use_this_program_name_on_gobolinux # === sync_use_this_program_name_on_gobolinux

  # ========================================================================= #
  # === sanitize_preinstall
  # ========================================================================= #
  def sanitize_preinstall
    _ = preinstall?
    if _ and _.empty?
      _ = raw_cookbook?.preinstall?
    end
    if _.is_a?(String) and (_.count(N) > 1)
      _ = _.split(N)
    end
    if _.nil? # Safeguard to ensure Array.
      set_preinstall []
    else
      _ = [_] if _.is_a? String
      if _.is_a? Array
        _.compact!
        _.map!(&:strip)
        _.reject! {|entry| entry.empty? } # Weed out empty entries.
        _.flatten!
      end
      set_preinstall _
    end
  end

  # ========================================================================= #
  # === sanitize_cmake_configure_options
  #
  # This method is similar to the method that will sanitize the configure
  # options, but it is only valid for cmake, not for GNU autoconfigure.
  #
  # A similar method exists for meson-configure-options by the way.
  # ========================================================================= #
  def sanitize_cmake_configure_options
    _ = cmake_configure_options?
    if _.nil?
      if raw_cookbook?.cmake_configure_options?
        _ = raw_cookbook?.cmake_configure_options?
      end
    end
    if _ and !_.empty?
      _ = _.to_s.strip
      # ===================================================================== #
      # We will assume this to be a comment, so we have to remove these
      # comments.
      # ===================================================================== #
      if _.include? ' # '
        _ = remove_comments_from_each_line(_)
      end
      _.tr!(N,' ')
      _.strip!
      set_cmake_configure_options(_)
    end
  end

  # ========================================================================= #
  # === sanitize_meson_configure_options
  #
  # This method is similar to the method sanitize_configure_options()
  # but it is only valid for meson, not for GNU autoconfigure.
  #
  # The configure options for meson must be kept as a String, at all times.
  # ========================================================================= #
  def sanitize_meson_configure_options
    _ = raw_cookbook?.meson_configure_options?
    _ = meson_configure_options? unless meson_configure_options?.nil?
    # ======================================================================= #
    # We sanitize it a bit, if the option exists.
    # ======================================================================= #
    if _
      _ = _.to_s.strip
      # ===================================================================== #
      # We will assume this to be a comment, so we have to remove these
      # comments.
      # ===================================================================== #
      if _.include? ' # '
        _ = remove_comments_from_each_line(_)
      end
      if _.include? N
        _.tr!(N,' ')
      end
      _.strip!
      _.squeeze!(' ')
      set_meson_configure_options(_)
    end
  end

  # ========================================================================= #
  # === sanitize_pre_configure_steps
  # ========================================================================= #
  def sanitize_pre_configure_steps
    _ = pre_configure_steps?
    _ = raw_cookbook?.pre_configure_steps? if _ and _.empty?
    set_pre_configure_steps(_) if _
  end

  # ========================================================================= #
  # === sanitize_pkgconfig_files
  # ========================================================================= #
  def sanitize_pkgconfig_files
    _ = pkgconfig_files?
    if _ and _.empty?
      _ = raw_cookbook?.pkgconfig_files? if raw_cookbook?.pkgconfig_files?
    end
    set_pkgconfig_files(_)
  end

  # ========================================================================= #
  # === sanitize_echo_yes
  # ========================================================================= #
  def sanitize_echo_yes
    _ = raw_cookbook?.echo_yes?.to_s
    set_echo_yes(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_svn_url
  # ========================================================================= #
  def sanitize_svn_url
    _ = svn_url?
    if _.nil?
      _ = raw_cookbook?.svn_url?
    end
    set_svn_url(_.to_s)
  end

  # ========================================================================= #
  # === sanitize_autogen
  # ========================================================================= #
  def sanitize_autogen
    _ = raw_cookbook?.use_autogen?
    if _.nil?
      _ = use_autogen?
    end
    set_use_autogen(
      to_bool(_)
    )
  end

  # ========================================================================= #
  # === sanitize_postinstall
  #
  # This will sanitize the postinstall stuff, by assigning to the
  # respective entry in the main Hash.
  #
  # Note that we have to invoke this method only after short_name
  # and program_version was set, because one entry requires this
  # specifically.
  #
  # As of 20.09.2014, the specification requires that postinstall
  # must be an Array at all times.
  #
  # Invocation example:
  #
  #   scookie libraw1394
  #
  # ========================================================================= #
  def sanitize_postinstall
    _ = [postinstall?].flatten.compact # Get rid of nil values.
    if _ and _.is_a?(Array) and _.empty?
      _ = raw_cookbook_dataset?['postinstall']
    end
    if _.is_a? Array
      _.map! {|entry| substitute_macros(entry) }
    end
    # ===================================================================== #
    # Turn it into an Array for sure:
    # ===================================================================== #
    _ = [_].flatten.compact
    # ===================================================================== #
    # Sanitize the @postinstall variable next.
    # ===================================================================== #
    if _.is_a? Array
      point_to_binaries    = sysbin_directory?
      point_to_libraries   = syslib_directory?
      point_to_includes    = sysinclude_directory?
      point_to_program_dir = program_directory?
      _.map! {|entry|
        if entry.include? 'UNIVERSAL_BINARY_DIR'
          entry.gsub!(/UNIVERSAL_BINARY_DIR/,  point_to_binaries)
        end
        entry.gsub!(/UNIVERSAL_BINARIES_DIR/,  point_to_binaries)
        entry.sub!(/UBD/,                      point_to_binaries)
        entry.gsub!(/UNIVERSAL_LIBRARIES_DIR/, point_to_libraries)
        entry.sub!(/ULD/,                      point_to_libraries)
        entry.gsub!(/UNIVERSAL_INCLUDE_DIR/,   point_to_includes)
        entry.gsub!(/PROGRAM_DIR/,             point_to_program_dir)
        entry.gsub!(/PROGRAMS_DIR/,            point_to_program_dir)
        entry.gsub!(/,cp/,                     ',copy_file')
        # ================================================================= #
        # Handle the MIDDLE_VERSION macro next. This one is, for
        # python, the value "8", from version "3.8.3" - thus the
        # middle part.
        # ================================================================= #
        if entry.include? 'MIDDLE_VERSION'
          entry.sub!(/MIDDLE_VERSION/, return_middle_part_of_the_program_version)
        end
        entry.strip! # Strip it.
        entry = rds(entry) # Replace // with // here.
        entry
      }
      set_postinstall _ # Reassign.
    end
    unless postinstall?.is_a? Array
      _ = rds(postinstall?)
      _ = _.split(',') if _.include? ','
      set_postinstall [_].flatten
    end
  end

  # ========================================================================= #
  # === sanitize_the_binaries                                  (binaries tag)
  #
  # This method will sanitize the binaries entry.
  #
  # The binaries must be kept in Array format at all times.
  # ========================================================================= #
  def sanitize_the_binaries
    _ = raw_cookbook?.binaries?
    _ = [] if _.nil?
    if _ and !_.empty?
      if _.is_a?(String) and !_.empty?
        _ = _.split(',')
      elsif _.is_a? String
        _ = [_]
      end
      if _.is_a? Array
        if _.first.include? ',' # Assume that we must split then.
          _.map! {|entry| entry.split(',') }
        end if _.first
      end
    end
    _ = _.flatten.compact.map(&:strip)
    set_binaries(_)
  end; alias sanitize_binaries sanitize_the_binaries # === sanitize_binaries

  # ========================================================================= #
  # === sanitize_has_to_be_compiled
  # ========================================================================= #
  def sanitize_has_to_be_compiled
    _ = raw_cookbook?.has_to_be_compiled?
    if _.nil? or (_.is_a?(String) and _.empty?) # Empty Strings will be "sanitized" here.
      _ = has_to_be_compiled?
    end
    set_has_to_be_compiled(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_pre_make_commands
  #
  # We must keep the pre_make_commands in Array format. The method
  # here will make sure of this.
  #
  # We can only invoke this after having set program_version before,
  # because some macro-substitutions require the program version to
  # be properly set before the make-commands can be sanitized.
  # ========================================================================= #
  def sanitize_pre_make_commands
    _ = pre_make_commands?
    if _.empty?
      _ = raw_cookbook?.pre_make_commands?
    end
    _.compact! if _.is_a? Array # Get rid of nil values.
    case _ # Do a bit free-form sanitizing next.
    when 'handle configure.py'
      _ = 'python configure.py'
    end
    array = []
    if _
      unless _.empty?
        _ = substitute_macros(_)
        array << _
      end
    end
    array.flatten!
    array.compact!
    set_pre_make_commands(array) # Turn it into an Array here.
  end

  # ========================================================================= #
  # === sanitize_description                      (description tag, desc tag)
  #
  # Get rid of trailing newlines in our description, and apply
  # some other operations. 
  # ========================================================================= #
  def sanitize_description
    _ = description?.to_s.chomp
    _.chop! if _.end_with? '"' # Get rid of leading and trailing ".
    _[0,1] = '' if _.start_with? '"'
    # ======================================================================= #
    # Some macros have to be expanded before we invoke substitute_macros().
    # SELF_NAME is such an example.
    # ======================================================================= #
    if _.include? 'SELF_NAME'
      _.gsub!(/SELF_NAME/, short_name?)
    end
    _ = substitute_macros(_)
    begin # Replace newlines with ' ' characters.
      _ = _.gsub(/#{N}/,' ').squeeze(' ')
    rescue Exception => error
      cliner {
        opne 'An error happened for the description:'
        pp @_
        pp error
      }
    end
    # ======================================================================= #
    # Next, if this is empty, but short_description? is not, then
    # simply copy-paste the content of short_description? onto
    # the main program description.
    # ======================================================================= #
    if _.to_s.empty? and !short_description?.to_s.strip.empty?
      _ = short_description?.dup
    end
    set_description(_)
  end

  # ========================================================================= #
  # === sanitize_gir_files
  # ========================================================================= #
  def sanitize_gir_files
    _ = gir_files?
    if _.empty?
      _ = raw_cookbook?.gir_files? if raw_cookbook?.gir_files?
    end
    set_gir_files(_)
  end

  # ========================================================================= #
  # === sanitize_configure_command_to_use
  # ========================================================================= #
  def sanitize_configure_command_to_use
    _ = configure_command_to_use?
    if _ and _.empty?
      _ = raw_cookbook?.configure_command_to_use?.to_s # It must be a String.
    end
    if _
      if _.empty? and deps_on? and deps_on?.include?('waf')
        _ = 'waf'
      end
      if _.empty?
        _ = 'configure' # set default, reset every time.
      else
        _ = convert_global_env(_) if _.include? '$'
      end
      set_configure_command_to_use(_)
    end
  end; alias sanitize_configure_command sanitize_configure_command_to_use # === sanitize_configure_command

  # ========================================================================= #
  # === sanitize_autosymlink_lib64
  # ========================================================================= #
  def sanitize_autosymlink_lib64
    _ = raw_cookbook?.autosymlink_lib64?
    set_autosymlink_lib64(_)
  end

  # ========================================================================= #
  # === sanitize_can_be_compiled_statically
  # ========================================================================= #
  def sanitize_can_be_compiled_statically
    _ = raw_cookbook?.can_be_compiled_statically?
    set_can_be_compiled_statically(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_the_libraries
  #
  # This sanitizes the libraries entry, @_[:libraries].
  #
  # We must keep this entry in Array format.
  # ========================================================================= #
  def sanitize_the_libraries
    _ = raw_cookbook?.libraries?
    unless _.is_a? Array
      _ = [_].compact
    end
    # ======================================================================= #
    # Next, we will assume something such as:
    #   ["libdevkit-power-gobject.{so,a},libupower-glib.{so,a}"] 
    # ======================================================================= #
    unless _.empty? # safeguard against nil entries.
      if _ and _.first and _.first.include?('}') and _.first.include?('{')
        splitted = _.first.split(/\},/)
        splitted.map! {|entry|
          entry =~ REGEX_TO_SANITIZE_LIBRARIES
          [$1+'.'+$2, $1+'.'+$3]
        }
        splitted.flatten!
        _ = splitted
      end
    end
    if _.is_a? Array
      if _.first.include? ','
        _ = _.first.split(',')
        _.map!(&:strip) # It should not include empty whitespace.
        # =================================================================== #
        # Some libraries may have components such as foo{so.a}
        # We need to split these up into foo.so and foo.a.
        # =================================================================== #
      end if _.first
    end
    new_value = [_].flatten.compact
    set_libraries(new_value)
  end; alias sanitize_libraries sanitize_the_libraries # === sanitize_libraries

  # ========================================================================= #
  # === sanitize_flatpak_url
  # ========================================================================= #
  def sanitize_flatpak_url
    _ = raw_cookbook?.flatpak_url?.to_s
    if _ and _.empty?
      _ = flatpak_url?.to_s
    end
    set_flatpak_url(_)
  end

  # ========================================================================= #
  # === sanitize_last_update
  #
  # The only sanitize-operation we do here is if it is an valid date,
  # such as "32 Aug 2017".
  # ========================================================================= #
  def sanitize_last_update
    _ = last_update?
    if _ and _.include? ' '
      splitted = _.split(' ')
      day = splitted.first # Fetch the day-entry.
      if day.to_i > 31
        opne swarn('The last_update entry for ')+
             steelblue(program_name?)+
             swarn(' is wrong (too high).')
        opne swarn('Please correct this entry.')
        opne swarn('The faulty entry currently looks like this:')
        e
        e "  #{_}"
        e
        # =================================================================== #
        # We could exit here but this is mostly an annoyance rather than
        # a bug, so we continue. As of April 2021, we will exit.
        # =================================================================== #
        exit
      end
    elsif _ and (_.count('.') == 2) # Input such as '24.09.2017'
      splitted = _.split('.')
      splitted[1] = return_month_based_on_this_number(splitted[1])[0,3] # This is the month but only the first 3 characters.
      _ = splitted.join(' ')
      set_last_update(_)
    end
  end; alias sanitize_last_update_entry sanitize_last_update # === sanitize_last_update_entry

  # ========================================================================= #
  # === sanitize_mirror
  # ========================================================================= #
  def sanitize_mirror
    _ = raw_cookbook?.mirror?.to_s
    set_mirror(_)
  end

  # ========================================================================= #
  # === sanitize_use_this_build_directory
  # ========================================================================= #
  def sanitize_use_this_build_directory
    _ = raw_cookbook?.use_this_build_directory?
    set_use_this_build_directory(_)
  end

  # ========================================================================= #
  # === sanitize_short_name
  # ========================================================================= #
  def sanitize_short_name
    _ = raw_cookbook?.short_name?
    if _.nil?
      _ = short_name?
    end
    set_short_name(_)
  end

  # ========================================================================= #
  # === sanitize_homepage
  # ========================================================================= #
  def sanitize_homepage
    _ = raw_cookbook?.homepage?
    unless _.is_a? String # Make sure we have a String.
      _ = _.to_s
    end
    set_homepage(_)
  end

  # ========================================================================= #
  # === sanitize_required_deps_on
  #
  # The entry required_deps_on will be sanitized here - it must be an 
  # Array at all times.
  # ========================================================================= #
  def sanitize_required_deps_on
    _ = raw_cookbook?.required_deps_on?
    if _.nil?
      _ = return_proper_type_for(:required_deps_on)
    end
    if _.is_a? String # Input such as 'perl cmake'. Has to become an Array.
      if _.include? ' '
        unless _.include?('>') or _.include?('<') # But not when we have comparison-operators.
          _ = _.split(' ')
        end
      end
    end
    array = []
    _ = [_].flatten.compact # Must be an Array.
    _.map! {|entry|
      if entry.include? ','
        entry = entry.split(',').map(&:strip)
      end 
      entry
    } unless _.empty?
    array << _
    array.flatten!
    array.compact!
    # ======================================================================= #
    # At this point the entry is a sanitized Array. But the cookbook
    # file may still have made a mistake, such as when an entry
    # was accidentally listed twice. (I had this problem in Feb
    # 2017, which was why I added the following code.)
    # So we need to check whether the entries are all unique - and
    # if they are not, we will notify the user that something may
    # be wrong with the yaml file in question.
    # ======================================================================= #
    if array.uniq.size != array.size
      path = RBT.individual_cookbooks_dir?+
             program?+
             '.yml'
      non_unique_entries = array.uniq.map { |e|
        [array.count(e), e]
      }.select {| c, _ | c > 1 }.sort.reverse.first.last
      unless non_unique_entries.is_a? Array
        non_unique_entries = [non_unique_entries] # We need an Array.
      end
      opnwarn('There may be one (or several) duplicate dependencies '\
             'listed in the yaml file.')
      opne swarn('  Path: ')+sfile(path)
      opne swarn('Here are the duplicate entries:')
      non_unique_entries.each {|duplicate_entry|
        e "  #{simp(duplicate_entry)}"
      }
      e swarn('If you wish to check for that on your own, have a look at the')
      e swarn('yaml file at `')+sfile(path)+swarn('`.')
      e swarn('Check the "required_deps_on:" entry of the .yml file.')
    end
    set_required_deps_on(array)
  end; alias sanitize_required_deps sanitize_required_deps_on # === sanitize_required_deps

  # ========================================================================= #
  # === sanitize_sed_entry                                          (sed tag)
  #
  # The sed entry allows us to make some very simple replacements in files.
  #
  # We will guarantee that this setting will be an Array.
  # ========================================================================= #
  def sanitize_sed_entry
    # ======================================================================= #
    # Only sanitize the sed entry if we have that key.
    # ======================================================================= #
    if has_key? :sed
      raw_data = raw_cookbook_dataset?
      _ = sed?
      if (_ == false) and raw_data.has_key?('sed')
        _ = raw_data['sed']
      end
      if _
        _ = _.split(N) if _.is_a? String
        if _
          # ================================================================= #
          # We need an Array past this point.
          # ================================================================= #
          _ = [_].compact # We don't want to store nil values here.
          set_sed(_)
        end
      end
    end
  end; alias try_to_sanitize_the_sed_entry sanitize_sed_entry # === try_to_sanitize_the_sed_entry

  # ========================================================================= #
  # === sanitize_blfs_entry
  #
  # This will be only synced if we actually have an entry.
  #
  # The BLFS entry must be guaranteed to be an Array.
  # ========================================================================= #
  def sanitize_blfs_entry
    _ = raw_cookbook?.blfs?
    if _
      unless _.is_a? Array
        if _.nil?
          _ = []
        elsif _.empty?
          _ = [] # Empty Array then.
        else
          _ = [_]
        end
      end
      set_blfs(_)
    else # Set it to be an empty Array by default, in this case.
      set_blfs []
    end
  end; alias sanitize_BLFS_entry sanitize_blfs_entry # === sanitize_BLFS_entry

  # ========================================================================= #
  # === sanitize_licence
  #
  # Empty license should default to "unknown" license. This method will
  # ensure that this is the case.
  # ========================================================================= #
  def sanitize_licence
    _ = licence?
    begin
      if _.nil? or _.empty?
        _ = 'unknown'
      else
        if _.is_a?(String) and _.include?(',') # ← In this case, turn it into an Array.
          _ = _.split(',').map(&:strip)
        else
          _ = 'unknown' if _.nil? or _.empty?
        end
      end
      set_licence(_)
    rescue Exception => error
      opne 'An error happened in sanitize_licence().'
      pp data?
      pp error
      pp error.class
      opne 'Exiting now - this error (in sanitize_licence()) '\
           'should be fixed though.'
      exit
    end
  end

  # ========================================================================= #
  # === sanitize_short_description
  #
  # Right now this method will only get rid of newlines and do a .strip()
  # call.
  # ========================================================================= #
  def sanitize_short_description
    short_description = short_description?
    if short_description.nil?
      short_description = raw_cookbook?.short_description?
    end
    if short_description and short_description.include? N
      set_short_description(
        short_description.to_s.delete(N).strip
      )
    end
  end

  # ========================================================================= #
  # === sanitize_do_not_symlink
  # ========================================================================= #
  def sanitize_do_not_symlink
    _ = raw_cookbook?.do_not_symlink?.to_s
    if _ and _.empty?
      _ = do_not_symlink?.to_s
    end
    set_do_not_symlink(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_keep_extracted
  # ========================================================================= #
  def sanitize_keep_extracted
    _ = raw_cookbook?.keep_extracted?.to_s
    if _ and _.empty?
      _ = keep_extracted?.to_s
    end
    set_keep_extracted(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_libexec
  # ========================================================================= #
  def sanitize_libexec
    _ = raw_cookbook?.libexec?
    if _ and _.empty?
      _ = libexec?
    end
    set_libexec(_)
  end

  # ========================================================================= #
  # === sanitize_use_this_make_install_command
  # ========================================================================= #
  def sanitize_use_this_make_install_command
    _ = raw_cookbook?.use_this_make_install_command?
    set_use_this_make_install_command(_)
  end

  # ========================================================================= #
  # === sanitize_program_name_and_program_version
  # ========================================================================= #
  def sanitize_program_name_and_program_version
    _ = raw_cookbook?.program_name_and_program_version?
    if _.nil?
      _ = program_name_and_program_version?
    end
    _ = return_sanitized_program_name_and_program_version(_, url1?)
    unless _.nil?
      set_program_name_and_program_version(_)
    end
  end

  # ========================================================================= #
  # === sanitize_symlink_pkgconfig_files
  # ========================================================================= #
  def sanitize_symlink_pkgconfig_files
    _ = raw_cookbook?.symlink_pkgconfig_files?
    if _.nil?
      _ = symlink_pkgconfig_files?
    end
    case _
    when 't'
      _ = 'yes'
    when 'f'
      _ = 'no'
    else
      _ = to_bool(_)
    end
    set_symlink_pkgconfig_files()
  end

  # ========================================================================= #
  # === sanitize_installation_steps
  #
  # Installation steps are steps that we have to perform in order to
  # get different programs to install on a given computer system.
  #
  # Special macros can be used as part of the installation steps.
  #
  # For instance:
  #
  #   MY_PROGRAMS/Procps/PROGRAM_VERSION
  #
  # Will be expanded to whatever value MY_PROGRAMS was set to; usually
  # it is, or should be, pointing towards "/Programs/" or
  # "/home/Programs/".
  #
  # Do note that this means that the value for MY_PROGRAMS must
  # exist, otherwise this can not possibly work. Of course you
  # can omit using macros and just hardcode values there anyway;
  # the macros only exist to make it simpler to handle different
  # programs.
  #
  # Another example for macro-conversion would be the String 'RINSTALL1',
  # which will become expanded into the following three actions:
  #
  #   ruby install.rb config
  #   ruby install.rb setup
  #   ruby install.rb install
  #
  # The latter expansions are part of the file:
  #
  #   predefined_installation_instructions.yml
  #
  # So, if you want to add new macros, you can do so by modifying that
  # very yaml file.
  #
  # Keep in mind that the installation steps are stored as Array at all times.
  # ========================================================================= #
  def sanitize_installation_steps
    _ = installation_steps?
    if _ and _.empty?
      if raw_cookbook?.installation_steps? and
        !raw_cookbook?.installation_steps?.empty?
        _ = raw_cookbook?.installation_steps?
      end
    end
    unless _.is_a? Array
      _ = [_].flatten.compact # ← .compact to avoid retaining nil values.
    end
    if _.empty?
      _ << 'make' # ← This is the hardcoded default value at all times.
    end
    # ======================================================================= #
    # bl $RBT/yaml/predefined_installation_instructions.yml
    # ======================================================================= #
    dataset = load_yaml(
      file_predefined_installation_instructions?
    )
    _.map! {|entry|
      if dataset.has_key? entry
        entry = dataset[entry]
        if entry.is_a? String
          # ================================================================= #
          # The String may look like this:
          #
          #   copy_ruby; ruby setup.rb config; ruby setup.rb setup; ruby setup.rb install
          #
          # ================================================================= #
          if entry.include? ';'
            entry = entry.split(';').map(&:strip)
          end
        end
      end
    }
    _.flatten!
    _.compact!
    _.reject!(&:empty?)
    # ========================================================================= #
    # Past this point here we have an Array.
    # ========================================================================= #
    unless _.empty?
      _.map! {|line|
        line = line.dup if line.frozen?
        line.strip!
        # =================================================================== #
        # Next, replace all ',' with ';' characters.
        # =================================================================== #
        line.tr!(',',';') if line.include? ','
        if Object.const_defined? :ConvertGlobalEnv
          line = ConvertGlobalEnv[line] if line.include? '$'
        end
        # =================================================================== #
        # === Replace the predefined installation instruction next
        # =================================================================== #
        if predefined_installation_instructions?.include? line
          line = predefined_installation_instructions?[line]
        end
        if line.include? 'VERSION'
          # ================================================================= #
          # Need to set @program_version first, hence we update some stuff.
          # ================================================================= #
          set_program_name(program_name?) # This equals, for example, to "gnome-mahjonng-5.08"
          if line.include? 'PROGRAM_VERSION' 
            line.gsub!(/PROGRAM_VERSION/, program_version?) if program_version?
          elsif line.include? 'VERSION'
            line.gsub!(/VERSION/, program_version?) if program_version?
          end
        end
        line = line.split(';') if line.include? ';' # This may turn it into an Array.
        line
      }
    end
    _.flatten!
    _.map!(&:strip)
    set_installation_steps(_)
  end

  # ========================================================================= #
  # === sanitize_localstatedir
  # ========================================================================= #
  def sanitize_localstatedir
    _ = raw_cookbook?.localstatedir?.to_s
    if _ and _.empty?
      _ = localstatedir?.to_s
    end
    set_localstatedir(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_do_not_download
  # ========================================================================= #
  def sanitize_do_not_download
    _ = raw_cookbook?.do_not_download?
    if _.nil?
      _ = do_not_download?
    end
    set_do_not_download(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_parameters_to_make
  #
  # This method will sanitize the parameters that can be given to the
  # program called "make" (on the commandline). "make" is typically
  # invoked for a given program.
  # ========================================================================= #
  def sanitize_parameters_to_make
    _ = parameters_to_make?
    if _ and _.empty?
       _ = raw_cookbook?.parameters_to_make?
    end
    _ = [_].flatten.compact.reject {|entry| entry.is_a?(String) and entry.empty? }
    set_parameters_to_make(_)
  end; alias sanitize_make_options       sanitize_parameters_to_make # === sanitize_make_options

  # ========================================================================= #
  # === sanitize_enable_shared
  # ========================================================================= #
  def sanitize_enable_shared
    _ = raw_cookbook?.enable_shared?.to_s
    if _ and _.empty?
      _ = enable_shared?.to_s
    end
    set_enable_shared(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_is_an_active_project
  # ========================================================================= #
  def sanitize_is_an_active_project
    _ = raw_cookbook?.is_an_active_project?
    if _.nil?
      _ = is_an_active_project?
    end
    set_is_an_active_project(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_m4_files
  # ========================================================================= #
  def sanitize_m4_files
    _ = raw_cookbook?.m4_files?
    if _ and _.empty?
      _ = m4_files?
    end
    set_m4_files(_)
  end

  # ========================================================================= #
  # === sanitize_use_build_directory
  # ========================================================================= #
  def sanitize_use_build_directory
    _ = raw_cookbook?.use_build_directory?
    if _.nil?
      _ = use_build_directory?
    end
    set_use_build_directory(_)
  end

  # ========================================================================= #
  # === sanitize_build_static
  # ========================================================================= #
  def sanitize_build_static
    _ = raw_cookbook?.build_static?
    if _.nil?
      _ = build_static?
    end
    set_build_static(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_run_configure
  # ========================================================================= #
  def sanitize_run_configure
    _ = raw_cookbook?.run_configure?
    if _.nil?
      _ = run_configure?
    end
    if _ and _.is_a?(String) and _.empty?
      _ = run_configure?.to_s
    end
    _ = to_bool(_)
    _ = false if _.nil?
    set_run_configure(_)
  end

  # ========================================================================= #
  # === sanitize_apply_patch
  #
  # Here we sanitize the value for apply_patch. tiff.yml may be an example
  # for how to use this setting.
  # ========================================================================= #
  def sanitize_apply_patch
    _ = raw_cookbook?.apply_patch?
    _ = apply_patch? unless _.is_a?(Array)
    if _
      if _.include? '#'
        _ = _[0 .. (_.index('#') - 1)]
        set_apply_patch _
      end
      if _.include? 'if PROGRAM_VERSION' # Ok, we specified a specific version.
        version_to_compare = _.split(' ').last
        if version_to_compare == program_version? # Ok, patch can be applied.
        else # Else do not apply this patch.
          _ = ''
        end
      end
    end
    set_apply_patch(_)
  end; alias sanitize_apply_patch sanitize_apply_patch # === sanitize_apply_patch

  # ========================================================================= #
  # === sanitize_may_we_modify_the_configure_options
  # ========================================================================= #
  def sanitize_may_we_modify_the_configure_options
    _ = raw_cookbook?.may_we_modify_the_configure_options?.to_s
    if _ and _.empty?
      _ = may_we_modify_the_configure_options?.to_s
    end
    set_may_we_modify_the_configure_options(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_modify_the_makefile
  # ========================================================================= #
  def sanitize_modify_the_makefile
    _ = raw_cookbook?.modify_the_makefile?
    if _.nil? or ( _.is_a?(String) and _.empty? )
      _ = false # This is the default value.
    end
    set_modify_the_makefile(to_bool(_))
  end

  # ========================================================================= #
  # === sanitize_md5sum
  # ========================================================================= #
  def sanitize_md5sum
    _ = raw_cookbook?.md5sum?
    if _.nil?
      _ = md5sum?
      if _.nil?
        _ = `md5sum #{program_path?}` if File.exist? program_path?
      end
    end
    set_md5sum(_)
  end; alias sanitize_md5sum_value sanitize_md5sum # === sanitize_md5sum_value

  # ========================================================================= #
  # === sanitize_url2
  # ========================================================================= #
  def sanitize_url2(
      url2 = raw_cookbook?.url2?
    ) 
    dataset?[:url2] = url2
  end

  # ========================================================================= #
  # === sanitize_then_set_boolean_value
  #
  # This method will turn 't' into true and 'f' into false.
  # ========================================================================= #
  def sanitize_then_set_boolean_value(i)
    _ = obtain(i)
    set(i, to_bool(_))
  end; alias sanitize_boolean_value sanitize_then_set_boolean_value # === sanitize_boolean_value

  # ========================================================================= #
  # === sanitize_manual_steps
  # ========================================================================= #
  def sanitize_manual_steps
    _ = raw_cookbook?.manual_steps?
    set_manual_steps(_)
  end

  # ========================================================================= #
  # === sanitize_symlink_headers
  # ========================================================================= #
  def sanitize_symlink_headers
    _ = raw_cookbook?.symlink_headers?
    if _.nil?
      _ = symlink_headers?
    end
    case _
    when 't'
      _ = 'yes'
    when 'f'
      _ = 'no'
    end
    set_symlink_headers(_)
  end

  # ========================================================================= #
  # === sanitize_the_headers                                    (headers tag)
  #
  # The headers entry must be kept as an Array at all times, according
  # to the specification.
  # ========================================================================= #
  def sanitize_the_headers
    _ = headers?
    if _ and _.is_a?(Array) and _.empty?
      _ = raw_cookbook_dataset?['headers']
    end
    if _.is_a? Array
      if _.first.include? ','
        _ = _.first.split(',')
      end if _.first
    end
    if _.is_a? String
      _ = _.to_s.strip
      _ = _.split(',') if _.include? ','
      _ = nil if _.empty?
    end
    _ = [_].flatten.compact
    # ======================================================================= #
    # The individual entries may have a newline or an empty String at
    # the beginning. The next line of code ensures that we will get
    # rid of these.
    # ======================================================================= #
    _.map! {|entry| entry.delete(N).strip }
    # ======================================================================= #
    # Must keep it sorted, since as of July 2017. It is easier to 
    # keep track of these entries on the commandline when they
    # are sorted.
    # ======================================================================= #
    _.sort!
    set_headers(_)
  end; alias sanitize_headers sanitize_the_headers # === sanitize_headers

  # ========================================================================= #
  # === sanitize_the_prefix                             (sanitize prefix tag)
  #
  # The value for the prefix of the program at hand will be sanitized
  # here, in this method.
  #
  # The prefix must be a String at all times according to the specification.
  #
  # Since as of May 2013, we will never use a shortcut anymore, we
  # will *always* store the full, absolute path to our entry.
  #
  # "false" as value, is an indication of using the path to an AppDir.
  #
  # For example, a value of false would be interpreted like 
  # /Programs/Htop/2.0.0 - in other words, a proper appdir.
  #
  # The --prefix is always a directory, so as of Jan 2014 this method
  # will append a / to this unless the last character is already a /.
  # ========================================================================= #
  def sanitize_the_prefix
    # ======================================================================= #
    # General way to overrule the prefix setting in this case.
    # ======================================================================= #
    if COMPILE_EVERYTHING_INTO_STANDALONE_DIRECTORY
      old_prefix = 't'
    else
      # ===================================================================== #
      # We need to use .to_s because the prefix can be a boolean value as
      # well, such as true or false.
      # ===================================================================== #
      old_prefix = prefix?.to_s.dup
    end
    # ======================================================================= #
    # As of January 2014, we will pass the String through the macro-check,
    # which has to be done because our string may include PROGRAM_NAME
    # as a prefix.
    # ======================================================================= #
    old_prefix = substitute_macros(old_prefix).dup
    case old_prefix
    # ======================================================================= #
    # === traditional
    # ======================================================================= #
    when 't',
         'true',
         'yes',
         'y',
         'traditional'
      set_internal_prefix('/usr/')
    # ======================================================================= #
    # === non-traditional
    # ======================================================================= #
    when 'f',
         'false',
         'no',
         'AppDir',
         'return_appdir_path',
         'appdir_path'
      # ===================================================================== #
      # This must use an appdir prefix.
      # ===================================================================== #
      set_internal_prefix(return_appdir_prefix.dup) # ← This is the AppDir variant.
    else
      old_prefix = old_prefix.dup
      # ===================================================================== #
      # Append '/' unless we already have a trailing '/'.
      # ===================================================================== #
      old_prefix << '/' unless old_prefix.end_with? '/'
      old_prefix.sub!(/ULOCAL/,'/usr/local') if old_prefix.include? 'ULOCAL' # ULOCAL gets expanded to /usr/local
      old_prefix.sub!(/ULOC/,  '/usr/local') if old_prefix.include? 'ULOC'   # same counts for ULOC
      # ======================================================================= #
      # We used to delete '-' here in the past, but as of September 2017
      # this is no longer the case. Some programs may have a '-' as part
      # of the program version, and it may be better to include this
      # rather than remove it.
      # ======================================================================= #
      # old_prefix.delete!('-') if _.include? '-'
      set_internal_prefix(old_prefix)
    end
  end; alias sanitize_prefix sanitize_the_prefix # === sanitize_prefix
  
  # ========================================================================= #
  # === sanitize_tags
  #
  # Keep in mind that the tags must always be in Array format.
  #
  # We will update the tags entry through this method here.
  # ========================================================================= #  
  def sanitize_tags
    _ = tags?
    if _ and _.empty? and
       raw_cookbook?.tags? and
      !raw_cookbook?.tags?.empty?
      _ = raw_cookbook?.tags?
    end
    if _.nil?
      set_tags [] # In this case it will be an empty Array.
      return
    end
    # ======================================================================= #
    # Clean up the Array next.
    # ======================================================================= #
    if _.is_a? Array
      _.map! {|line|
        if line
          begin
            line.tr!(' ','_')
            line.strip!
          rescue Exception => error
            opne 'Something went wrong with the variable (called '+steelblue(line)+').'
            opne 'The whole dataset was:'
            e
            pp _
            e
            e 'The program was:'
            e
            e steelblue("  #{program_name?}")
            e
            e 'The error was (of class '+error.class.to_s+'):'
            pp error
          end
        end
        line
      }
    end
    dataset_containing_the_specification_of_registered_cookbook_entries =
      file_specification_of_registered_cookbook_entries
    format = dataset_containing_the_specification_of_registered_cookbook_entries['tags']
    unless _.class.to_s == format
      case format
      when 'Array'
        if _.is_a? String
          if _.include? ','
            _ = _.split(',').map(&:strip)
          end
        end 
        _ = [_].compact.flatten # Convert the String into an Array here and remove nil values.
      end
    end
    if _.is_a? Array
      # ===================================================================== #
      # Before we do anything else, we will apply a .map-strip and a 
      # exchange of all ' ' with '_' operation:
      # ===================================================================== #
      _.map! {|line|
        if line
          line.tr!(' ','_')
          line.strip!
        end
        line
      }
      first = _.first
      if first
        if first.include? ' ' # We have a situation such as this one here: ["video gnome"]
          first = first.split(' ')
          _ = first
        elsif first.is_a?(String) and first.include?(',') # Some entries may have something like:  ruby,gnome  in the yaml file, we check for these here.
          first = first.split(',')
          _ = first
        end
      end
    end
    # ======================================================================= #
    # Since as of Feb 2017, we will also ensure that the tags that are
    # in use, conform to our specification at hand. If not then we will
    # notify the user, and exit.
    # ======================================================================= #
    allowed_tags = load_yaml(RBT.file_allowed_tags)
    allowed_tags.map! {|line|
      line.tr(' ','_').strip
    }
    _.each {|this_tag| # Iterate through to find improper tags.
      if this_tag.to_s.empty? or # Empty tags are skipped as well.
         allowed_tags.include?(this_tag)
      else
        this_program = short_name?
        e
        e "#{rev}The specified tag called `#{simp(this_tag)}` for the program "\
          "#{sfancy(this_program.downcase)}"\
          " has not"
        e 'been registered in the file:'
        e
        e "  `#{sfile(RBT.file_allowed_tags)}`."
        e
        e 'This is currently '+violet('disallowed')+'.'
        e
        e 'The faulty entry must be corrected. So either the cookbook file '
        e 'in question should be modified, or the tag should be added'
        e "to the registry of allowed tags in the specification "\
          "file. (#{sfile('allowed_tags.yml')})"
        e
        if is_on_roebe?
          open_in_editor(
            this_program.downcase.delete('-')
          )
        end
        # =================================================================== #
        # No longer exit as of November 2013.
        # And in April 2021 we will exit again.
        # =================================================================== #
        exit # And exit here.
      end
    }
    set_tags(_) # Finally set the Array that has our tags.
  end

  # ========================================================================= #
  # === sanitize_use_this_build_system
  #
  # The build_system entry must always be a String.
  # ========================================================================= #
  def sanitize_use_this_build_system
    _ = use_this_build_system?
    _ = '' if _.nil?
    if _ and _.is_a?(String) and _.empty?
      # ===================================================================== #
      # Must set a new default value in this case.
      # ===================================================================== #
      _ = :default # This means to infer the build system in use.
    end
    set_use_this_build_system(_)
  end

  # ========================================================================= #
  # === sanitize_program_version
  # ========================================================================= #
  def sanitize_program_version
    _ = raw_cookbook?.program_version?
    if _.nil? or (_ and _.empty?)
      _ = program_version?.to_s
    end
    set_program_version(_)
  end

  # ========================================================================= #
  # === sanitize_url1
  #
  # We will chop off some parts of the URL.
  # ========================================================================= #
  def sanitize_url1(
      i = raw_cookbook?.url1?
    )
    set_url1(
      return_sanitize_url1(i)
    )
  end

end; end; end; end
