#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::FindAlternativeArchive
#
# This class can be used to find a likely alternative to a given
# (locally existing) archive.
#
# Usage example:
#
#   RBT::Action::FindAlternativeArchive.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/find_alternative_archive/find_alternative_archive.rb'
# RBT::Action::FindAlternativeArchive.new(ARGV)
# RBT.return_alternative_archive(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class Action

class FindAlternativeArchive < Base # === RBT::Action::FindAlternativeArchive

  # ========================================================================= #
  # === DEFAULT_ARGUMENT
  # ========================================================================= #
  DEFAULT_ARGUMENT =
    "#{HOME_DIRECTORY_FOR_USER_X}src/glib/"\
    "glib-2.72.3.tar.xz" # '/home/x/src/glib/glib-2.72.3.tar.xz'

  # ========================================================================= #
  # === initialize
  #
  # The first argument should be the full, "expected" path to our archive
  # in question. We need this full path so that this class can replace
  # it with a proper new path, if necessary.
  # ========================================================================= #
  def initialize(
      commandline_arguments = DEFAULT_ARGUMENT,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    if commandline_arguments.empty?
      commandline_arguments? << DEFAULT_ARGUMENT
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
  end

  # ========================================================================= #
  # === archive_type?
  # ========================================================================= #
  def archive_type? # Return the archive type here.
    return return_archive_type(path?)
  end; alias archive_type archive_type? # === archive_type

  # ========================================================================= #
  # === file_path?
  #
  # This method will return the file path.
  # ========================================================================= #
  def file_path?
    @internal_hash[:file_path]
  end; alias path?    file_path? # === path?
       alias path     file_path? # === path
       alias results? file_path? # === results?
       alias result?  file_path? # === result?
       alias result   file_path? # === result

  # ========================================================================= #
  # === determine_file_path
  # ========================================================================= #
  def determine_file_path
    set_file_path(
      first_argument?
    )
    determine_directory_name
  end

  # ========================================================================= #
  # === determine_directory_name
  # ========================================================================= #
  def determine_directory_name
    @internal_hash[:directory_name] =
      rds("#{File.dirname(file_path?)}/")
  end

  # ========================================================================= #
  # === directory_name?
  # ========================================================================= #
  def directory_name?
    @internal_hash[:directory_name]
  end

  # ========================================================================= #
  # === set_file_path
  # ========================================================================= #
  def set_file_path(i)
    @internal_hash[:file_path] = i
  end

  # ========================================================================= #
  # === filename?
  # ========================================================================= #
  def filename?
    remove_file_suffix(
      file_path?
    )
  end

  # ========================================================================= #
  # === return_default_files
  #
  # Return the default files from the @directory_name entry. Note that we
  # will not traverse into subdirectories.
  # ========================================================================= #
  def return_default_files(
      from = "#{directory_name?}*"
    )
    Dir[from]
  end

  # ========================================================================= #
  # === find_other_archive
  #
  # The input to this method should be a version, as a number.
  #
  # The method will then try to find other archives.
  # ========================================================================= #
  def find_other_archive(i)
    i = i.to_f unless i.is_a? Float
    _ = return_alternative_archives.select {|entry| entry =~ /#{i}/ }
    return _.first unless _.empty?
    nil # else we will return nil.
  end

  # ========================================================================= #
  # === has_alternative?
  # ========================================================================= #
  def has_alternative?
    return true unless return_alternative_archives.empty?
  end; alias more_than_one_entry? has_alternative? # === more_than_one_entry?

  # ========================================================================= #
  # === return_alternative_archives
  # ========================================================================= #
  def return_alternative_archives
    return_default_files.select {|entry|
      is_an_archive? entry
    }.sort.reverse
  end; alias check_for_other_archive_types return_alternative_archives # === check_for_other_archive_types

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_file_path
  end

  # ========================================================================= #
  # === RBT::Action::FindAlternativeArchive[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.return_alternative_archive
#
# This variant will return an array.
# =========================================================================== #
def self.return_alternative_archive(i = ARGV)
  return RBT::Action::FindAlternativeArchive.new(i).return_alternative_archives
end

# =========================================================================== #
# === RBT.find_alternative_archive
# =========================================================================== #
def self.find_alternative_archive(i = ARGV)
  RBT::Action::FindAlternativeArchive.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  require 'colours/autoinclude'
  require 'rbt/toplevel_methods/verbose_truth.rb'
  _ = RBT.find_alternative_archive(ARGV)
  e '-' * 80
  e
  e "#{rev}The #{sfancy('alternative archives')} #{rev}may be:"
  e
  array = _.return_alternative_archives
  pp array
  e
  e "#{rev}The filename may be: #{sfancy(_.filename?)}"
  e
  e lightgreen('Are there alternatives in that directory?')+' '+
    steelblue(RBT.verbose_truth(_.has_alternative?))
  e
  # ========================================================================= #
  # Next, feedback all results:
  # ========================================================================= #
  e '-' * 80
  e "#{rev}The path to the archive in question is at:"
  e
  e sfile("  #{_.path?.to_s}")
  e
  pp _.return_alternative_archives
end # faa $USERS/x/src/glib/glib-2.58.3.tar.xz
    # faa $MY_SRC/gdkpixbuf/gdk-pixbuf-2.26.5.tar.xz
    # faa $MY_SRC/gdkpixbuf/gdk-pixbuf
    # faa $MY_SRC/ruby/ruby-