#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::Headers
#
# This class will "map" the available and registered headers of all
# programs, and be able to tell the user, through the commandline, to
# which program a particular .h file may belong. 
#
# Usage example:
#
#   RBT::Action::Headers.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/headers/headers.rb'
# RBT::Action::Headers.new(ARGV)
# RBT.headers(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class Headers < RBT::Action # === RBT::Action::Headers

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @match
    # ======================================================================= #
    @match = nil
    # ======================================================================= #
    # === @do_report
    #
    # By default this class will not report.
    # ======================================================================= #
    @do_report = false
  end

  # ========================================================================= #
  # === load_up_the_registered_headers
  #
  # This method will simply load the registered .h (header) files,
  # from the yaml file.
  # ========================================================================= #
  def load_up_the_registered_headers(
      i = RBT.file_registered_headers
    )
    @registered_headers = YAML.load_file(i)
  end

  # ========================================================================= #
  # === try_to_find_a_match_for_the_given_input
  # ========================================================================= #
  def try_to_find_a_match_for_the_given_input
    first_argument = first_argument?
    if first_argument.nil?
      e "#{rev}Please provide some input, such as #{steelblue('ao.h')}#{rev}."
    else
      # ===================================================================== #
      # First check whether such a file exists in the registered headers.
      # ===================================================================== #
      if @registered_headers.has_key? first_argument
        @match = @registered_headers[first_argument]
      else # else we need to find any .header file.
        if @registered_headers.keys.any? {|entry| entry.end_with?("/#{first_argument}") }
          selection = @registered_headers.select {|header_file, name_of_the_program_at_hand|
            header_file.end_with?("/#{first_argument}")
          }
          if selection.empty?
            e "#{rev}No match has been found for the given "\
              "input #{steelblue(first_argument)}#{rev}."
          else
            @match = selection[selection.keys.first]
          end
        end
      end
    end
  end

  # ========================================================================= #
  # === report                                                   (report tag)
  # ========================================================================= #
  def report
    if @match
      _ = first_argument?
      e "#{rev}The given input #{steelblue(_)} "\
        "#{rev}matches to the program #{lightblue(@match)}#{rev}. "\
        "(#{olivedrab(_)} #{tomato('→')} #{mediumpurple(@match)})"
    end
  end

  # ========================================================================= #
  # === check_whether_the_input_is_a_header_file
  # ========================================================================= #
  def check_whether_the_input_is_a_header_file
    first_argument = first_argument?
    unless first_argument.end_with? '.h'
      e "#{steelblue(first_argument)} #{rev}does not seem to be a .h file."
    end
  end

  # ========================================================================= #
  # === match?
  # ========================================================================= #
  def match?
    @match
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_whether_the_input_is_a_header_file
    load_up_the_registered_headers
    try_to_find_a_match_for_the_given_input
    if @do_report
      report
    end
  end

  # ========================================================================= #
  # === RBT::Action::RawCookbook[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.return_the_program_belonging_to_this_header_file
#
# Usage examples:
#
#   RBT.return_the_program_belonging_to_this_header_file('ao.h') # => "libao"
#   RBT.return_the_program_belonging_to_this_header_file('aoee.h') # => nil
#
# =========================================================================== #
def self.return_the_program_belonging_to_this_header_file(i)
  RBT::Action::Headers.new(i).match?
end

# =========================================================================== #
# === RBT.headers
# =========================================================================== #
def self.headers(
    i            = ARGV,
    options_hash = {},
    &block
  )
  return RBT::Action::Headers.new(i, &block)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.headers(ARGV).report
end # rbtheaders gbm.h