#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::AggregateInformationFromTheExpandedCookbooks
#
# This class will simply iterate over all expanded cookbook .yml files
# and then:
#
#   (a) store the information into a really huge hash
#   (b) dump this dataset into a single .yml file
#
# The idea here is that we can then use this information to
# generate SQL instructions, for example. It is simpler to
# work with a single file in such a case.
#
# Usage examples:
#
#   RBT::Action::AggregateInformationFromTheExpandedCookbooks.new(ARGV)
#   RBT.aggregate_information_from_the_expanded_cookbooks
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/information/aggregate_information_from_the_expanded_cookbooks.rb'
# RBT.aggregate_information_from_the_expanded_cookbooks
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class AggregateInformationFromTheExpandedCookbooks < RBT::Action # === RBT::Action::AggregateInformationFromTheExpandedCookbooks

  # ========================================================================= #
  # === STORE_THE_NEW_YAML_FILE_HERE
  #
  # Since as of August 2022 we use a dedicated, separate directory for
  # storing the .yml file. Before tha twe stored into the expanded
  # cookbook directory directly, and this caused subsequent scripts
  # to struggle as old assumptions such as "one .yml file per
  # expanded cookbook dataset" in the given directory was no longer
  # correct.
  # ========================================================================= #
  STORE_THE_NEW_YAML_FILE_HERE =
    "#{RBT.log_directory?}"\
    "expanded_cookbook_dataset_from_all_programs/expanded_cookbook_dataset_from_all_programs.yml"

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @hash
    #
    # This is our main Hash storing everything. \o/
    # ======================================================================= #
    @hash = {}
  end

  # ========================================================================= #
  # === hash?
  # ========================================================================= #
  def hash?
    @hash
  end

  # ========================================================================= #
  # === work_through_the_cookbook_dataset_found_in_this_directory
  # ========================================================================= # 
  def work_through_the_cookbook_dataset_found_in_this_directory(i)
    opnn { :no_trailing }
    e
    e
    e "#{rev}Obtaining all entries from"
    e "#{rev}  #{sdir(i)}"
    e "#{rev}next - this may take a while."
    all_entries = Dir["#{i}*.yml"]
    all_entries.each {|this_yaml_file|
      name_of_the_program = File.basename(this_yaml_file).
                                 sub(/#{File.extname(this_yaml_file)}$/,'')
      dataset = YAML.load_file(this_yaml_file)
      new_hash = {}
      new_hash[name_of_the_program] = dataset
      @hash.update(new_hash)
    }
  end

  # ========================================================================= #
  # === load_up_the_existing_dataset
  #
  # This method can be used to load the existing dataset from a .yml
  # file that has been generated before.
  # ========================================================================= #
  def load_up_the_existing_dataset(
      from_here = STORE_THE_NEW_YAML_FILE_HERE
    ) 
    @hash = YAML.load_file(from_here)
  end

  # ========================================================================= #
  # === is_the_main_hash_empty?
  # ========================================================================= #
  def is_the_main_hash_empty?
    @hash.empty?
  end

  # ========================================================================= #
  # === store_the_big_hash_into_a_new_yaml_file
  # ========================================================================= #
  def store_the_big_hash_into_a_new_yaml_file
    e "#{rev}Next storing this big hash ("+
      "#{@hash.keys.size.to_s}#{rev}) in a new .yml file."
    what = YAML.dump(@hash)
    into = STORE_THE_NEW_YAML_FILE_HERE
    unless File.dirname(into)
      mkdir(into) { :be_quiet }
    end
    e 'The specific file will stored here:'
    e
    e sfile("  #{into}")
    e
    write_what_into(what, into)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = directory_expanded_cookbooks?
    if File.directory? _
      work_through_the_cookbook_dataset_found_in_this_directory(_)
      if is_the_main_hash_empty?
        opne "Unfortunately no entries could be found."
        opne "#{rev}Is the directory at #{sdir(_)} #{rev}empty?"
      else
        store_the_big_hash_into_a_new_yaml_file
      end
    else
      opnn; no_file_exists_at(_)
    end
  end

  # ========================================================================= #
  # === RBT::Action::AggregateInformationFromTheExpandedCookbooks[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.aggregate_information_from_the_expanded_cookbooks
# =========================================================================== #
def self.aggregate_information_from_the_expanded_cookbooks(
    optional_argument1,
    extract_to = :default
  )
  RBT::Action::AggregateInformationFromTheExpandedCookbooks.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.aggregate_information_from_the_expanded_cookbooks(ARGV)
end # actionextract htop*