#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::CreateProgramVersionUrlFile
#
# This class will create a local ".md" file that will keep the
# name of a program, the version and the remote (first) URL
# to the program at hand.
#
# Usage examples:
#
#   RBT::Action::CreateProgramVersionUrlFile.new(ARGV)
#   RBT.aggregate_information_from_the_expanded_cookbooks
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/information/create_program_version_url_file.rb'
# RBT.create_program_version_url_file
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class CreateProgramVersionUrlFile < RBT::Action # === RBT::Action::CreateProgramVersionUrlFile

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      # ===================================================================== #
      # === Handle Hashes next
      # ===================================================================== #
      if yielded.is_a? Hash and
         yielded.has_key?(:show_last_update)
        @internal_hash[:show_last_update] = yielded.delete(:show_last_update)
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @directory_expanded_cookbooks
    # ======================================================================= #
    @directory_expanded_cookbooks = RBT.directory_expanded_cookbooks?
    # ======================================================================= #
    # === :all_expanded_cookbooks
    # ======================================================================= #
    @internal_hash[:all_expanded_cookbooks] = []
    # ======================================================================= #
    # === :show_last_update
    # ======================================================================= #
    @internal_hash[:show_last_update] = false
    # ======================================================================= #
    # === @array_to_be_stored
    # ======================================================================= #
    @array_to_be_stored = []
  end

  # ========================================================================= #
  # === all_expanded_cookbooks?
  # ========================================================================= #
  def all_expanded_cookbooks?
    @internal_hash[:all_expanded_cookbooks]
  end

  # ========================================================================= #
  # === obtain_the_expanded_cookbooks
  # ========================================================================= #
  def obtain_the_expanded_cookbooks
    newline = "\n".dup
    # ======================================================================= #
    # We read the individual, expanded .yml files from there and only
    # store the relevant parts into the Array called @array_to_be_stored.
    # ======================================================================= #
    if expanded_cookbooks_directory_exists?
      @internal_hash[:all_expanded_cookbooks] = Dir["#{@directory_expanded_cookbooks}*.yml"].sort
      all_expanded_cookbooks?.each {|yaml_file|
        use_this_for_url1 = :url1
        if File.exist? yaml_file
          dataset = YAML.load_file(yaml_file)
        else
          e 'No file exists at '+sfile(yaml_file)+'.'
        end
        program_name    = File.basename(
          yaml_file.delete_suffix('.yml')
        ).downcase.to_sym
        program_version = dataset[:program_version]
        if program_version.nil?
          # ================================================================= #
          # This safeguard was added in April 2021 because we had a file
          # called ModemManager.yml still.
          # ================================================================= #
          opne 'A '+steelblue('nil')+' entry for program_version.'
          opne tomato('Debug this please.')
          opne 'The yaml file was: '+sfile(yaml_file.to_s)
          exit
        end
        program_url     = dataset[use_this_for_url1]
        @array_to_be_stored << [
          program_name, program_version, program_url
        ]
        if show_last_update?
          # ================================================================= #
          # In this case, append more information.
          # ================================================================= #
          last_update = dataset[:last_update]
          @array_to_be_stored[-1] << last_update 
        end
      }
      sort_the_array
      # ===================================================================== #
      # Now that we have populated @array_to_be_stored, we must
      # use it to create a .md file.
      # ===================================================================== #
      what = ''.dup
      @array_to_be_stored.each {|entry|
        # =================================================================== #
        # entry may look like this:
        #
        #   ["modemmanager", "1.16.2",
        #    "https://www.freedesktop.org/software/ModemManager/ModemManager-1.16.2.tar.xz",
        #    "09 Apr 2021"]
        #
        # =================================================================== #
        program_name, program_version, program_url = entry[0], entry[1], entry[2]
        if show_last_update?
          last_update = entry[3] # It is the last element.
        end
        # =================================================================== #
        # Add what we need here - name of the program, the version and
        # the remote URL. This will be properly padded.
        # =================================================================== #
        what << program_name.to_s.ljust(28)+
                program_version.to_s.ljust(22) # Add the program version here.
        # =================================================================== #
        # === Last updated when
        #
        # Append when the last update happened, if we use that entry at
        # all.
        # =================================================================== #
        if show_last_update?
          what << last_update.to_s.strip.ljust(19) # Add when it was last updated.
        end
        # =================================================================== #
        # Next, append the remote URL to the line.
        # =================================================================== #
        what << program_url.to_s.rstrip.ljust(44).rstrip+
                newline
      }
      into = RBT.file_programs_version_url
      opne "#{rev}Now #{steelblue('storing')} #{rev}into the file"
      opne "#{rev}`#{sfile(into)}#{rev}`."
      create_directory_if_it_does_not_yet_exist(File.dirname(into))
      write_what_into(what, into)
    else
      notify_the_user_that_we_require_the_expanded_cookbooks_directory
    end
  end

  # ========================================================================= #
  # === notify_the_user_that_we_require_the_expanded_cookbooks_directory
  # ========================================================================= #
  def notify_the_user_that_we_require_the_expanded_cookbooks_directory
    opne 'Currently, the functionality to create '+teal('program')+
         ', '+teal('version')+' and '+teal('URL')+', into'
    opne 'a single .md file, depends on the expanded '\
         'cookbook dataset.'
    e
    opne "This dataset should normally reside at "\
         "#{sdir(@directory_expanded_cookbooks)}."
    opne 'This directory does not exist, so the dataset is unavailable.'
    e
    opne 'You can generate the dataset anew via:'
    e
    opne '  rbt --expand'
    e
  end

  # ========================================================================= #
  # === sort_the_array
  # ========================================================================= #
  def sort_the_array
    sorted = @array_to_be_stored.sort_by {|entry|
      entry.first
    }
    @array_to_be_stored = sorted
  end

  # ========================================================================= #
  # === show_last_update?
  # ========================================================================= #
  def show_last_update?
    @internal_hash[:show_last_update]
  end

  # ========================================================================= #
  # === array_to_be_stored?
  # ========================================================================= #
  def array_to_be_stored?
    @array_to_be_stored
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    obtain_the_expanded_cookbooks
  end

  # ========================================================================= #
  # === RBT::Action::CreateProgramVersionUrlFile[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.create_program_version_url_file
#
# The boolean argument that can be passed to this method determines
# as to whether we will also show the time when the program has
# been last updated. By default this is true, meaning that we
# WILL show when a program has been last updated.
#
# To invoke this method from the commandline, do:
#
#   rbt --create_program_version_url_file
#
# =========================================================================== #
def self.create_program_version_url_file(
    also_show_last_update = :default # Since Feb 2018, we will show the last update by default.
  )
  if also_show_last_update and also_show_last_update.is_a?(Array) and
     also_show_last_update.empty?
    also_show_last_update = :default
  end
  case also_show_last_update # case tag
  # ========================================================================= #
  # === :also_show_last_update
  # ========================================================================= #
  when :also_show_last_update,
       :include_last_update,
       :default
    also_show_last_update = true
  end
  RBT::Action::CreateProgramVersionUrlFile.new {{ show_last_update: also_show_last_update }}
end; self.instance_eval { alias generate_programs_version_url_yaml_file      create_program_version_url_file } # === RBT.generate_programs_version_url_yaml_file
     self.instance_eval { alias create_programs_url_file                     create_program_version_url_file } # === RBT.create_programs_url_file
     self.instance_eval { alias create_local_file_for_all_available_programs create_program_version_url_file } # === RBT.create_local_file_for_all_available_programs

end

if __FILE__ == $PROGRAM_NAME
  RBT.create_program_version_url_file(ARGV)
end # rbtaction create_program_version_url_file