#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::MapThisInputToThatRegisteredProgram
#
# This class can map any given input given to it, and "make sense" of the
# input. In the simplest way it can sanitize a shorter input to a
# registered program in the RBT project. This forms the basis of simpler
# user input as well, since the user can type just a few characters and
# perhaps obtain a correct result (a program that is registered in RBT).
#
# The class will also annotate some characteristics about the given input
# at hand, such as whether the input refers to a real program, or a
# binary-file, or a library-file, or a header-file. That way code written
# in other .rb files can make use of this for downstream code. In fact: this
# was exactly why this class was created in the first place, since I got
# tired of ad-hoc logic stored in separate .rb files. So now the logic
# is all gathered in a single .rb file - this class here.
#
# Usage example:
#
#   RBT::Action::MapThisInputToThatRegisteredProgram.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/map_this_input_to_that_registered_program/map_this_input_to_that_registered_program.rb'
# RBT::Action::MapThisInputToThatRegisteredProgram.new(ARGV)
# RBT.map_this_input_to_that_registered_program(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class MapThisInputToThatRegisteredProgram < RBT::Action # === RBT::Action::MapThisInputToThatRegisteredProgram

  require 'rbt/toplevel_methods/available_programs.rb'
  require 'rbt/toplevel_methods/all_libraries.rb'
  require 'rbt/toplevel_methods/all_binaries.rb'
  require 'rbt/toplevel_methods/does_include.rb'
  require 'rbt/registered/registered_headers.rb'
  require 'rbt/registered/registered_pkgconfig_files.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    # ======================================================================= #
    # === Handle blocks next:
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :be_quiet
      # ===================================================================== #
      when :be_quiet
        set_be_quiet
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === :be_verbose
    # ======================================================================= #
    set_be_verbose
    # ======================================================================= #
    # === @real_program_name
    #
    # This is always nil on startup.
    # ======================================================================= #
    @real_program_name = nil
    # ======================================================================= #
    # === @type
    #
    # @type can be :abbreviation, :library, :header, :pkgconfig_file,
    # :partial_header or a similar entry.
    # ======================================================================= #
    @type = nil
  end; alias clear reset # === clear

  # ========================================================================= #
  # === set_type
  #
  # Type can be at the least :abbreviation and :program, but probably
  # more than that.
  # ========================================================================= #
  def set_type(i = :abbreviation)
    @type = i
  end

  # ========================================================================= #
  # === type?
  # ========================================================================= #
  def type?
    @type.to_s
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    i = first_argument?.to_s # Must work on a String.
    # ======================================================================= #
    # === Handle the real names
    # ======================================================================= #
    if RBT.is_this_a_registered_program?(i)
      @real_program_name = i
      if be_verbose?
        e "#{i} is a registered program."
      end
    # ======================================================================= #
    # === Handle input that incorrectly starts with "lib".
    #
    # Next, we handle input given to this class like "libcryptsetup",
    # which should really just be "cryptsetup" instead.
    # ======================================================================= #
    elsif i.start_with?('lib') and RBT.is_this_a_registered_program?(i.sub(/^lib/,''))
      @real_program_name = i.sub(/^lib/,'')
      if be_verbose?
        e "#{i} is a registered program, if the leading 'lib' prefix is dropped."
      end
      set_type :program
    # ======================================================================= #
    # === Handle abbreviations next
    # ======================================================================= #
    elsif RBT.is_this_a_registered_abbreviation?(i)
      @real_program_name = RBT.hash_aliases_to_the_available_programs?[i]
      if be_verbose?
        e "#{i} is a registered abbreviation."
        e "It points to the program called `#{@real_program_name}`." 
      end
      set_type :abbreviation
    # ======================================================================= #
    # === Handle binaries next
    #
    # Commandline usage example:
    #
    #   map_this_input_to_that_registered_program soxi
    #
    # ======================================================================= #
    elsif RBT.is_this_a_registered_binary?(i)
      @real_program_name = RBT.hash_all_binaries[i]
      if be_verbose?
        e "#{i} is a registered binary (executable)."
        e "It points to the program called `#{@real_program_name}`." 
      end
      set_type :binary
    # ======================================================================= #
    # === Handle binaries next, downcasing the input
    #
    # This entry point will find all registered binaries for all
    # available programs.
    #
    # Commandline usage example:
    #
    #   map_this_input_to_that_registered_program soxi
    #
    # ======================================================================= #
    elsif RBT.is_this_a_registered_binary?(i.downcase)
      @real_program_name = RBT.hash_all_binaries[i.downcase]
      if be_verbose?
        e "#{i.downcase} is a registered binary (executable)."
        e "It points to the program called `#{@real_program_name}`." 
      end
      set_type :binary
    # ======================================================================= #
    # === Handle binaries next, downcasing the input
    #
    # This entry point will find all registered binaries for all
    # available programs.
    #
    # Commandline usage example:
    #
    #   map_this_input_to_that_registered_program soxi
    #
    # ======================================================================= #
    elsif RBT.is_this_a_registered_binary?(i.downcase, :deep_check)
      @real_program_name = RBT.hash_all_binaries[i.downcase]
      if be_verbose?
        e "#{i.downcase} is a registered binary (executable)."
        e "It points to the program called `#{@real_program_name}`." 
      end
      set_type :binary
    # ======================================================================= #
    # === Handle libraries next
    #
    # Example:
    #
    #   map_this_input_to_that_registered_program libc.so
    #
    # ======================================================================= #
    elsif ::RBT.is_this_library_included?(i)
      this_library = RBT.this_library_to_that_program(i)
      @real_program_name = this_library
      if be_verbose?
        e "#{i} is a registered library."
        e "It points to the program called `#{@real_program_name}`." 
      end
      set_type :library
    # ======================================================================= #
    # === Handle headers next
    #
    # Commandline usage example:
    #
    #    map_this_input_to_that_registered_program udev.h
    #    map_this_input_to_that_registered_program wrap_init.h
    #
    # ======================================================================= #
    elsif RBT.is_this_header_included?(i)
      @real_program_name = RBT.all_headers?[i]
      if be_verbose?
        e "#{i} is a registered header file."
        e "It points to the program called `#{@real_program_name}`." 
      end
      set_type :header
    # ======================================================================= #
    # === Handle partial headers next
    #
    # Commandline usage example:
    #
    #    map_this_input_to_that_registered_program SMlib.h
    #
    # ======================================================================= #
    elsif RBT.is_this_partial_header_included?(i)
      selection = RBT.all_headers?.transform_keys {|entry|
        File.basename(entry)
      }
      if selection.has_key? i
        @real_program_name = selection[i]
        if be_verbose?
          e "#{i} is a registered partial header file - this means it"
          e "is in a subdirectory."
          e "It points to the program called `#{@real_program_name}`." 
        end
        set_type :partial_header
      else
        opne "#{rev}The header #{steelblue(i)} was not found, oddly enough."
      end
    # ======================================================================= #
    # === Handle .pc pkgconfig-entries next
    #
    # Commandline usage example:
    #
    #    map_this_input_to_that_registered_program xau.pc
    #
    # ======================================================================= #
    elsif RBT.is_this_pkgconfig_file_included?(i)
      @real_program_name = RBT.return_the_associated_program_for_this_pkgconfig_file(i)
      if be_verbose?
        e "#{i} is a registered .pc pkg-config file."
        e "It points to the program called `#{@real_program_name}`." 
      end
      set_type :pkgconfig_file
    # ======================================================================= #
    # === Handle partial headers next
    #
    # Commandline usage example:
    #
    #    map_this_input_to_that_registered_program wrap_init.h
    #
    # ======================================================================= #
    elsif RBT.is_this_partial_header_included?(i)
      _ = RBT.all_headers?
      # ===================================================================== #
      # Must find any partial match next.
      # ===================================================================== #
      possible_matches = _.select {|key, value| key.include?(i) }
      @real_program_name = possible_matches.first.last
      if be_verbose?
        e "#{i} is a registered header file."
        e "It points to the program called `#{@real_program_name}`." 
      end
      set_type :header
    else
      if be_verbose?
        e "#{sfancy(i)} #{rev}is not registered."
      end
    end
  end

  # ========================================================================= #
  # === real_program_name?
  # ========================================================================= #
  def real_program_name?
    @real_program_name
  end; alias return_possible_match real_program_name? # === return_possible_match

  # ========================================================================= #
  # === RBT::Action::MapThisInputToThatRegisteredProgram[]
  #
  # This method will return nil if no registered program has been found
  # for the given input. Otherwise it will return the real name of the
  # program at hand.
  #
  # .h files work as input as well.
  #
  # Usage examples:
  #
  #   x = RBT::Action::MapThisInputToThatRegisteredProgram[:php]
  #   x = RBT::Action::MapThisInputToThatRegisteredProgram[:gee]
  #   x = RBT::Action::MapThisInputToThatRegisteredProgram[:foobar]
  #   x = RBT::Action::MapThisInputToThatRegisteredProgram['stdio.h']
  #   x = RBT::Action::MapThisInputToThatRegisteredProgram[:soxi] # => "sox"
  #
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i) { :be_quiet }.real_program_name?
  end

end; end

# =========================================================================== #
# === RBT.map_this_input_to_that_registered_program
#
# Usage example:
#
#   RBT.map_this_input_to_that_registered_program('mutool') # => "mupdf"
#
# =========================================================================== #
def self.map_this_input_to_that_registered_program(
    i = ARGV,
    &block
  )
  i = i.first if i.is_a? Array
  return RBT::Action::MapThisInputToThatRegisteredProgram.new(i, &block)
end

end

if __FILE__ == $PROGRAM_NAME
  require 'colours'
  require 'cliner'
  RBT.map_this_input_to_that_registered_program(ARGV)
end # mapthisinputtothatregisteredprogram php
    # mapthisinputtothatregisteredprogram gee
    # supermapper mate-screenshot
    # supermapper matescreenshot
    # supermapper libcryptsetup
    # supermapper cryptsetup