#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Action::ShowCompileChain
#
# This small class allows us to show the compile chain of a certain
# "meta"-package. That is, of programs that are registered in the
# yaml file called 'chained_programs.yml'.
#
# The class was created in November 2018, to make available functionality
# that already existed from within class RBT::Install.
#
# Usage example:
#
#   RBT::Action::ShowCompileChain.new(ARGV)
#
# =========================================================================== #
# require 'rbt/actions/individual_actions/show_compile_chain/show_compile_chain.rb'
# RBT::Action::ShowCompileChain.new(ARGV)
# RBT.show_compile_chain(ARGV)
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class ShowCompileChain < RBT::Action # === RBT::Action::ShowCompileChain

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true,
      &block
    )
    reset
    set_commandline_arguments(i)
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @dataset
    # ======================================================================= #
    @dataset = chained_programs?
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === chained_programs?
  # ========================================================================= #
  def chained_programs?
    RBT.return_chained_programs
  end

  # ========================================================================= #
  # === sanitize_commandline_arguments
  # ========================================================================= #
  def sanitize_commandline_arguments
    commandline_arguments?.map! {|entry|
      entry = entry.to_s unless entry.is_a? String
      case entry # case tag
      # ===================================================================== #
      # === kde_apps
      # ===================================================================== #
      when /^kde(_|-)?apps$/
        entry = 'kde5_applications'
      # ===================================================================== #
      # === mate
      # ===================================================================== #
      when 'mate' # <- ShowCompileChainOf mate
        entry = 'all_of_the_mate_desktop'
      # ===================================================================== #
      # === portingaids
      # ===================================================================== #
      when 'portingaids' # <- ShowCompileChainOf portingaids
        entry = 'kde5_porting_aids'
      end
      entry
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    sanitize_commandline_arguments
    i = first_argument?
    # ======================================================================= #
    # The next variable will have a format such as:
    #   {"kde5_plasma"=> ["bluedevil"
    # ======================================================================= #
    selection = @dataset.select {|key, value|
      key.downcase.include? i.downcase
    }
    if selection.empty?
      # ===================================================================== #
      # Enter this clause when no match was found.
      # ===================================================================== #
      opne "#{rev}No match was found for `#{slateblue(i)}#{rev}`."
      opne 'The available chained programs are:'
      e
      available_programs = "  #{chained_programs?.keys.join(' ').strip}"
      available_programs = wrap_at(available_programs, 54)
      array_available_programs = available_programs.split(N).map(&:strip)
      array_available_programs.each_with_index {|these_programs, index| index += 1
        padded_index = index.to_s.ljust(4)
        e "  #{padded_index}"+
          royalblue(these_programs.tr(' ',', '))
      }
      e
    else
      # ===================================================================== #
      # Else we have at the least one entry in the Array called "selection".
      # ===================================================================== #
      first_entry = selection.keys.first
      array_available_programs = selection[first_entry]
      if array_available_programs.is_a?(String) and
         array_available_programs.include?(',')
        array_available_programs = array_available_programs.split(',').map(&:strip)
      end
      if array_available_programs.is_a?(Array) and
        !array_available_programs.empty?
        opne "#{rev}The following entries belong to the group "\
             "`#{sfancy(first_entry)}#{rev}`:"
        e
        array_available_programs.each_with_index {|this_entry, index| index += 1
          padded_index = (index.to_s.rjust(3)).ljust(6)
          e "#{lightgreen(padded_index)}#{royalblue(this_entry)}"
        }
        e
      end
    end
  end

  # ========================================================================= #
  # === RBT::Action::ShowCompileChain.of
  # ========================================================================= #
  def self.of(i = ARGV)
    [i].flatten.compact.each {|this_program|
      RBT::Action::ShowCompileChain.new(this_program)
    }
  end; self.instance_eval { alias [] of } # === RBT::Action::ShowCompileChain[]

end; end

# =========================================================================== #
# === RBT.show_compile_chain
#
# Usage example:
#
#   RBT.show_compile_chain(:kde5_plasma)
#
# =========================================================================== #
def self.show_compile_chain(
    i            = ARGV,
    options_hash = {},
    &block
  )
  result = []
  [i].flatten.compact.each {|this_program|
    result << RBT::Action::ShowCompileChain.new(this_program, &block)
  }
  return result.flatten # Return the Array here.
end; self.instance_eval { alias show_compile_chain_of show_compile_chain } # === RBT.show_compile_chain_of

end

if __FILE__ == $PROGRAM_NAME
  RBT.show_compile_chain
end # showcompilechainof kde5_plasma