#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/actions/individual_actions/software_manager/initialize.rb'
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class SoftwareManager < RBT::Action # === RBT::Action::SoftwareManager

  # ========================================================================= #
  # === initialize
  #
  # The first argument to this method will contain the commandline
  # arguments for class RBT::Action::SoftwareManager. These are usually
  # the name of the program (or programs) that the uses wishes to
  # compile / install.
  #
  # So, for instance, "htop" could be passed as the first argument to
  # this method. You can also pass in an Array here, and this class
  # will then attempt to batch-compile these programs.
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV, # ← This should usually be an Array of programs that should be installed.
      run_already           = true,
      &block
    )
    register_sigint
    reset
    if commandline_arguments.is_a?(Array) and commandline_arguments[1] and
       commandline_arguments[1].is_a?(Symbol)
      run_already = commandline_arguments[1] # Handle :dont_run early.
    end
    if commandline_arguments.is_a?(Array)
      last = commandline_arguments.last
      case last
      # ===================================================================== #
      # === :do_not_run_yet
      # ===================================================================== #
      when :do_not_run_yet
        run_already = false
      end
    end
    # ======================================================================= #
    # === Handle Symbols given to the commandline arguments next
    #
    # Handle symbols given as first argument next. This should come before
    # we call set_commandline_arguments().
    # ======================================================================= #
    case commandline_arguments
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet,
         :do_not_yet_run,
         :dont_run_yet
      run_already = false
      commandline_arguments = []
    end
    # ======================================================================= #
    # Finally set the commandline arguments here:
    # ======================================================================= #
    set_commandline_arguments(
      commandline_arguments
    )
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      # ===================================================================== #
      # Handle Symbols first:
      # ===================================================================== #
      if yielded.is_a? Symbol
        case yielded
        # =================================================================== #
        # === :do_not_run_yet
        # =================================================================== #
        when :do_not_run_yet,
             :do_not_run,
             :dont_run_yet
          run_already = false
        # =================================================================== #
        # === :do_not_check_whether_the_program_exists
        # =================================================================== #
        when :do_not_check_whether_the_program_exists
          do_not_check_whether_the_program_exists # ← This method exists, so use it too.
        # =================================================================== #
        # === :use_root_prefix
        # =================================================================== #
        when :use_root_prefix
          do_use_root_prefix
        end
      # ===================================================================== #
      # === Handle Hashes next
      # ===================================================================== #
      elsif yielded.is_a? Hash
        # =================================================================== #
        # === :run_already
        #
        # This entry point should be checked last. 
        # =================================================================== #
        if yielded.has_key? :run_already
          case yielded[:run_already]
          when :do_not_run_yet
            run_already = false
          end
        end
        # =================================================================== #
        # === :use_this_prefix
        # =================================================================== #
        if yielded.has_key? :use_this_prefix
          set_use_this_prefix(yielded[:use_this_prefix])
        end
        # =================================================================== #
        # === :shall_we_check_whether_the_program_exists
        # =================================================================== #
        if yielded.has_key? :shall_we_check_whether_the_program_exists
          case yielded[:shall_we_check_whether_the_program_exists]
          # ================================================================= #
          # === nope
          #
          # In this case we will not check whether the program exists.
          # ================================================================= #
          when :nope, false
            do_not_check_whether_the_program_exists
          end
        end
        # =================================================================== #
        # === :prefix
        #
        # Handle prefix-related keys first.
        # =================================================================== #
        if yielded.has_key? :prefix
          set_user_prefix(yielded.delete(:prefix)) # ← Set the new prefix via a method call.
        # =================================================================== #
        # === :use_this_prefix
        # =================================================================== #
        elsif yielded.has_key? :use_this_prefix
          set_user_prefix(yielded.delete(:use_this_prefix))
        # =================================================================== #
        # === :use_this_as_prefix
        # =================================================================== #
        elsif yielded.has_key? :use_this_as_prefix
          set_user_prefix(yielded.delete(:use_this_as_prefix))
        end
        # =================================================================== #
        # === :symlink_after_compilation_has_finished
        # =================================================================== #
        if yielded.has_key? :symlink_after_compilation_has_finished
          value = yielded.delete(:symlink_after_compilation_has_finished)
          if value == false
            set_do_not_symlink(:be_verbose) # Disable symlinking after compilation.
          end
        end
        # =================================================================== #
        # Add commandline_arguments if the key is :commandline_arguments
        #
        # Such a Hash may have a key like this:
        #
        #   {:commandline_arguments=>["--dont-use-colours", "ntrad"]}
        # =================================================================== #
        if yielded.has_key? :commandline_arguments
          append_to_the_commandline(
            yielded.delete(:commandline_arguments)
          )
        end
      else
        append_to_the_commandline(yielded)
      end
    end
    # ======================================================================= #
    # Handle special symbols passed to run_already next:
    # ======================================================================= #
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet,
         :dont_run_yet,
         :do_not_run_already
      run_already = false
    end
    consider_entering_the_log_directory # This should come before we handle blocks.
    # ======================================================================= #
    # Next store the first snapshot, which is the start-mark. This should
    # come after we handled any block given to this method.
    # ======================================================================= #
    time_snapshot
    run if run_already
  end

end; end; end
