#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file will contain the most important logic-parts of
# class SoftwareManager.
# =========================================================================== #
# require 'rbt/actions/individual_actions/software_manager/logic_related_code.rb'
# =========================================================================== #
require 'rbt/actions/action.rb'

module RBT

class Action

class SoftwareManager < RBT::Action # === RBT::Action::SoftwareManager

  # ========================================================================= #
  # === run_make                                                   (make tag)
  # ========================================================================= #
  def run_make(
      parameters_to_make = parameters_to_make?
    )
    if run_make? and can_we_continue?
      _ = 'make -j$(nproc)'.dup # This variant uses the n CPU core variant.
      _ << " #{parameters_to_make}" unless parameters_to_make.empty?
      colourized_and_padded_esystem(_)
    end
  end; alias run_make_command run_make # === run_make_command

  # ========================================================================= #
  # === run_cmake_then_make_then_make_install
  #
  # Just a convenience method to run "cmake", then "make" and then
  # "make install".
  # ========================================================================= #
  def run_cmake_then_make_then_make_install
    run_cmake_configure_step
    run_premake_then_make_then_make_install
    # run_make
    # run_make_install
    # opne "#{rev}Next running"
    # run_this_system_command_in_a_coloured_and_padded_manner(cmd) # Using this since as of 28.08.2022.
    # ======================================================================= #
    # Now that the cmake, premake, make, make install was run we scan
    # through the registered problems/errors and act on it, before
    # resetting the error-line again.
    # ======================================================================= #
    # run_through_the_registered_problems_registered_errors_and_behaviour_changes
    # reset_error_map_line_to_a_specific_error
  end

  # ========================================================================= #
  # === run_cmake_configure_step                                  (cmake tag)
  #
  # This method essentially does something like:
  #
  #   cmake -DCMAKE_INSTALL_PREFIX=/usr ..
  #
  # Then it will perform:
  #
  #   - premake
  #   - make
  #   - make install
  #
  # ========================================================================= #
  def run_cmake_configure_step(
      use_this_as_build_directory = use_this_as_build_directory?
    )
    # ======================================================================= #
    # We should check whether cmake exists on the host system or whether
    # it does not.
    # ======================================================================= #
    if cmake_exists?
      _ = 'cmake'.dup
      # ===================================================================== #
      # Add the prefix next, if the user wants to use a prefix at all:
      # ===================================================================== #
      if do_we_use_any_prefix? # Check whether the user wants to use a prefix at all.
        _ << " -DCMAKE_INSTALL_PREFIX=#{prefix?}"
      end
      # ===================================================================== #
      # Obtain the cmake-configure options next and append them:
      # ===================================================================== #
      configure_options = cmake_configure_options?
      if configure_options and !configure_options.empty?
        _ << " #{configure_options}"
      end
      _ << ' .'
      if use_this_as_build_directory and do_we_use_a_build_directory? # Most cmake-based projects require a build-directory.
        create_the_build_directory_then_cd_into_it(use_this_as_build_directory) { :be_verbose }
        _ << '.'
      end
      _.squeeze!(' ')
      _.strip!
      # ======================================================================= #
      # === Check for more than one -DCMAKE_INSTALL_PREFIX entry next
      #
      # This should never be the case, so we will do some quality-control
      # and exit if this is the case so.
      # ======================================================================= #
      scanned_result = _.scan(/-DCMAKE_INSTALL_PREFIX=/)
      if scanned_result.size > 1 # More than one.
        # =================================================================== #
        # In this case, something must be wrong. Notify the user about it.
        # That way this can be corrected eventually.
        # =================================================================== #
        opnewarn "There are at the least two definitions of "\
                 "#{royalblue('-DCMAKE_INSTALL_PREFIX=')}"
        opnewarn "in the .yml file at hand."
        opnewarn 'At the least one of these entries must be incorrect '\
                 'and/or superfluous. Consider '
        opnewarn 'correcting this logic-problem before RBT can '\
                 'continue.'
        e
        e 'See:'
        e
        e "  #{sfancy(_)}"
        e
        exit_program
      end
      # ======================================================================= #
      # Finally the cmake-command has been built up. We can now push it
      # towards the general method that handles output.
      # ======================================================================= #
      # reset_error_map_line_to_a_specific_error
      set_full_configure_command(_)
      cmake_object = action(:run_cmake, :do_not_run_yet) { :colourized_and_padded }
      cmake_object.set_full_configure_line(_)
      cmake_object.verbose_run_the_actual_system_command
    else
      # ======================================================================= #
      # The user should be informed that "cmake" could not be found here.
      # ======================================================================= #
      notify_the_user_that_cmake_could_not_be_found
      we_can_not_continue
    end
  end; alias run_the_cmake_steps                                run_cmake_configure_step # === run_the_cmake_steps
       alias run_cmake_command                                  run_cmake_configure_step # === run_cmake_command
       alias run_cmake_via_action                               run_cmake_configure_step # === run_cmake_via_action
       alias run_the_cmake_command                              run_cmake_configure_step # === run_the_cmake_command
       alias run_cmake_then_premake_then_make_then_make_install run_cmake_configure_step # === run_cmake_then_premake_then_make_then_make_install
       alias do_install_via_cmake                               run_cmake_configure_step # === run_cmake_then_premake_then_make_then_make_install

  # ========================================================================= #
  # === return_default_hash_to_be_passed_into_colour_make
  #
  # This method will return a static hash. This can be overruled,
  # though, by providing an argument (another hash) to this
  # method.
  # ========================================================================= #
  def return_default_hash_to_be_passed_into_colour_make(
      optional_input = nil
    )
    hash = {
      use_this_make_command:            use_which_make_command?,
      append_these_options:             parameters_to_make?,
      run_make:                         run_make?,
      run_make_install:                 run_make_install?,
      use_porg:                         use_porg?,
      program_name_and_program_version: program_name_and_program_version?
    }
    if optional_input
      hash.update(optional_input)
    end
    return hash 
  end

  # ========================================================================= #
  # === run_make_install                                   (make install tag)
  #
  # This method will ultimately default to run:
  #    make install
  #
  # Note that some programs do not use "make install" and instead use
  # something such as "ninja install".
  #
  # The method here thus has to remain somewhat flexible, in order
  # to install such programs as well.
  # ========================================================================= #
  def run_make_install
    # ======================================================================= #
    # Do not try to compile via make, if it does not have to be compiled.
    # ======================================================================= #
    return unless has_to_be_compiled?
    return unless can_we_continue?
    # ======================================================================= #
    # Next check for pre-make-install sed instructions:
    # ======================================================================= #
    unless pre_make_install_sed.empty?
      pre_make_install_sed.each {|sed_instruction|
        run_this_sed_instruction(sed_instruction)
      }
    end
    if exit_before_make_install?
      orev 'Exiting now as instructed before reaching '\
           'the "'+orange('make install')+rev+'" stage.'
      exit_program
    end
    # ======================================================================= #
    # === Check whether we may run make install or whether we may not.
    # ======================================================================= #
    if run_make_install?
      # ===================================================================== #
      # Clear the old buffer keeping track of our errors first:
      # ===================================================================== #
      # reset_error_map_line_to_a_specific_error
      use_this_make_command = use_this_make_command?
      if use_this_make_install_command?
        # =================================================================== #
        # In this case we will use that make-install command instead.
        # =================================================================== #
        use_this_make_command = use_this_make_install_command?
      end
      use_this_make_command = use_this_make_command.dup
      # ===================================================================== #
      # Check for porg next:
      # ===================================================================== #
      use_porg = false
      if use_porg? # The user is trying to make use of porg here.
        if porg_is_available?
          use_porg = true
        else
          # =================================================================== #
          # Notify the user that porg is NOT available. We will continue
          # anyway, though.
          # =================================================================== #
          notify_the_user_that_porg_is_not_available
        end
      end
      _ = "#{use_this_make_command} install"
      # ======================================================================= #
      # If the user uses stow then the following clause is entered.
      # ======================================================================= #
      opnn
      if use_stow?
        RBT.colour_make_install {
          return_default_hash_to_be_passed_into_colour_make(
            append: " DESTDIR=/stow/#{compile_which_program?} install",
            porg:                  use_porg,
            use_this_make_command: use_this_make_command,
            pad_the_output:        true,
            run_make:              false
          )
        }
      else # This is the default variant.
        RBT.colour_make_install {
          return_default_hash_to_be_passed_into_colour_make(
            porg:                  use_porg,
            use_this_make_command: use_this_make_command,
            pad_the_output:        true,
            run_make:              false
          )
        }
      end
      # run_through_the_registered_problems_registered_errors_and_behaviour_changes
      # reset_error_map_line_to_a_specific_error
      # if an_error_has_occurred?
      #   # =================================================================== #
      #   # Register that this step led to an error in this case.
      #   # =================================================================== #
      #   @internal_hash[:make_install_stage_was_a_success] = false
      #  end
      # colourized_and_padded_esystem(_)
      # ^^^ this should be used in a padded variant.
    end
  end; alias run_make_install_command                            run_make_install # === run_make_install_command
       alias considering_running_make_install_command_via_system run_make_install # === considering_running_make_install_command_via_system

  # ========================================================================= #
  # === create_the_build_directory
  # ========================================================================= #
  def create_the_build_directory(
      i = DEFAULT_NAME_FOR_THE_BUILD_DIRECTORY
    )
    mkdir(i) unless File.directory?(i)
  end

  # ========================================================================= #
  # === run_the_equivalent_configure_like_stage_for_this_program
  # ========================================================================= #
  def run_the_equivalent_configure_like_stage_for_this_program(
      i = compile_which_program?
    )
    orev "The #{lightgreen('configure-like stage')}"\
         "#{rev} for #{steelblue(i)}#{rev} will now be invoked."
    # ======================================================================= #
    # Next register that we are compiling a program. This should come
    # before we consider building the program statically.
    # ======================================================================= #
    register_that_this_program_is_currently_being_compiled
    # ======================================================================= #
    # We must next determine which configure command is to be used:
    # ======================================================================= #
    configure_command_to_use = configure_command_to_use?
    # ======================================================================= #
    # Here we have to do a decision: either we make use of the
    # use_this_build_system specified value, or we have to infer the
    # build system. Either way we begin this by quering the
    # use_this_build_system value.
    # ======================================================================= #
    use_this_build_system = use_which_build_system? # This will always be a Symbol.
    case use_this_build_system
    # ======================================================================= #
    # === :configure
    # ======================================================================= #
    when :configure
      consider_running_autoconf
      do_run_GNU_configure_then_make_then_make_install
    # ======================================================================= #
    # === :perl
    #
    # Invocation example:
    #
    #   ry archivezip
    #
    # ======================================================================= #
    when :perl
      run_the_perl_steps
    # ======================================================================= #
    # === :cmake
    # ======================================================================= #
    when :cmake
      run_cmake_then_make_then_make_install
    # ======================================================================= #
    # === :meson
    # ======================================================================= #
    when :meson
      run_meson_configure_step
      run_ninja
      run_ninja_install
    # ======================================================================= #
    # === :Python
    # ======================================================================= #
    when :Python, :python # Can be capitalized or not.
      run_python_installation
    # ======================================================================= #
    # === :Makefile
    # ======================================================================= #
    when :Makefile
      run_make
      run_make_install
    # ======================================================================= #
    # === :scons
    # ======================================================================= #
    when :scons
      run_scons
    else
      orev tomato('Not registered build-system type: ')+
           steelblue(use_this_build_system.to_sym)
      orev 'No way to proceed, thus stopping now.'
      return
    end
    do_handle_the_post_installation_steps
  end; alias find_the_corresponding_install_method_and_proceed_from_there run_the_equivalent_configure_like_stage_for_this_program # === find_the_corresponding_install_method_and_proceed_from_there

  # ========================================================================= #
  # === do_compile_this_registered_program                         (main tag)
  # ========================================================================= #
  def do_compile_this_registered_program(i)
    clear_the_original_cookbook_dataset
    initialize_cookbook_dataset_for(i)
    # ======================================================================= #
    # First check if the user wants to ensure a specific licence - this
    # can only come once the dataset has been initialized:
    # ======================================================================= #
    if check_for_this_licence?
      unless licence? == check_for_this_licence?
        opne "#{tomato('The licences do not match. ')}#{rev}Expected licence"
        orev 'was: '+
             steelblue(
               RBT.sanitize_this_licence(check_for_this_licence?)
             )+", specified licence was: #{steelblue(licence?)}"
        return
      end
    end
    # ======================================================================= #
    # === Check whether ccache exists
    #
    # Query for the use and availability of ccache. This should come
    # after any special environment was set.
    # ======================================================================= #
    if use_ccache? and !is_ccache_available?
      notify_the_user_that_ccache_is_not_available
    end
    # ======================================================================= #
    # Next show the infobox:
    # ======================================================================= #
    infobox_for(i)
    local_path = cookbook_dataset?.local_path?
    notify_the_user_as_to_which_program_will_be_compiled_or_installed_next(
      i,
      local_path
    )
    # ======================================================================= #
    # Must check whether this file also exists next:
    # ======================================================================= #
    if File.exist? local_path
      # ===================================================================== #
      # First, check whether we have to remove the old, extracted directory:
      # ===================================================================== #
      to = "#{extract_to_this_directory?}"\
           "#{program_name_and_program_version?}"
      to = rds(to).dup
      if File.directory?(to) and !(to == '/')
        orev 'Removing the old, extracted directory at'
        orev "`#{sdir(to)}#{rev}` next."
        remove_directory(to)
      end
      orev "Now extracting the archive, at `#{sfile(local_path)}#{rev}`."
      orev 'The extracted directory will be located at'
      orev '`'+sfile(to)+"#{rev}`,"
      orev 'if the extraction-step worked properly.'
      # ===================================================================== #
      # Extracting the archive next:
      # ===================================================================== #
      extract_what_to(local_path, File.dirname(to))
      orev lightsalmon('The extraction-step has finished.')
      if File.directory? to
        orev "Entering the directory at #{sdir(to)}#{rev} now."
        cd_to_the_extracted_archive(to)
        run_the_equivalent_configure_like_stage_for_this_program(i)
      else
        orev 'No directory was found at `'+sdir(to)+"#{rev}`."
      end
    else
      opnn; no_file_exists_at(local_path)
    end
  end; alias do_install_this_program do_compile_this_registered_program # === do_install_this_program

  # ========================================================================= #
  # === create_the_build_directory_then_cd_into_it
  #
  # Note that we will only cd into the directory if we actually use a
  # build directory.
  # ========================================================================= #
  def create_the_build_directory_then_cd_into_it(
      i = DEFAULT_NAME_FOR_THE_BUILD_DIRECTORY,
      &block
    )
    if use_build_directory?
      # ===================================================================== #
      # === Handle blocks next
      # ===================================================================== #
      if block_given?
        yielded = yield
        case yielded
        # =================================================================== #
        # === :be_verbose
        # =================================================================== #
        when :be_verbose
          orev "Changing directory into "\
               "#{sdir(absolute_path(i))} "\
               "#{rev}next,"
          orev "thus honouring the #{steelblue('use-build-directory')} "\
               "#{rev}setting."
        end
      end
      create_the_build_directory(i)
      cd i
    end
  end; alias create_build_directory_then_cd_into_it create_the_build_directory_then_cd_into_it # === create_build_directory_then_cd_into_it

  # ========================================================================= #
  # === we_can_not_continue
  # ========================================================================= #
  def we_can_not_continue(
      be_verbose = be_verbose?
    )
    case be_verbose
    when :be_quiet
      be_verbose = false
    end
    if be_verbose
      orev "The RBT scripts can not continue."
    end
    set_can_we_continue(false)
  end; alias can_not_continue we_can_not_continue # === can_not_continue
       alias do_not_continue  we_can_not_continue # === do_not_continue

  # ========================================================================= #
  # === set_can_we_continue
  # ========================================================================= #
  def set_can_we_continue(i = false)
    @internal_hash[:can_we_continue] = i
  end

  # ========================================================================= #
  # === run_meson_configure_step                                  (meson tag)
  #
  # This method essentially does something like:
  #
  #   meson --prefix=/usr --libdir=/usr/lib BUILD_DIRECTORY
  #
  # ========================================================================= #
  def run_meson_configure_step(
      use_this_as_build_directory = use_this_as_build_directory?
    )
    # ======================================================================= #
    # Obtain the configure options for meson next:
    # ======================================================================= #
    if is_meson_installed?
      configure_options = meson_configure_options?
      action(:run_meson_configure_step,
      {
        prefix_to_use:               prefix?,
        configure_options:           configure_options,
        use_this_as_build_directory: use_this_as_build_directory
      }) { :colourized_and_padded }
    else
      orev steelblue('Meson')+rev+' is unavailable. Consider installing it.'
    end
  end; alias run_meson_configure  run_meson_configure_step # === run_meson_configure
       alias run_meson_via_action run_meson_configure_step # === run_meson_via_action

  # ========================================================================= #
  # === set_on_ihash
  #
  # The second argument is the dataset that you wish to use.
  # ========================================================================= #
  def set_on_ihash(
      which_key_to_use = :cookbook_dataset,
      which_dataset_value_to_use
    )
    @internal_hash[which_key_to_use] = which_dataset_value_to_use
  end; alias set_cookbook         set_on_ihash # === set_cookbook
       alias set_cookbook_dataset set_on_ihash # === set_cookbook_dataset

  # ========================================================================= #
  # === initialize_a_new_cookbook_for
  #
  # The first argument to this method should be the program that the user
  # wants to wish to compile or install, as a String, such as "php" or
  # "ruby" or something like that.
  #
  # Since as of January 2018, whenever you need an instance from
  # class RBT::Cookbooks::SanitizeCookbook, this method here
  # should be used.
  #
  # The reason is simple:
  #
  # - This method will do all the required invocations needed
  #   to properly instantiate a new Cookbook instance.
  #
  # ========================================================================= #
  def initialize_a_new_cookbook_for(
      i                 = install_this_program?,
      simple_or_complex = simple_or_complex?
    )
    # ======================================================================= #
    # We need to assign the primary cookbook dataset. This will be handled
    # by class SanitizeCookbook, and will be delegated via RBT.action().
    # ======================================================================= #
    cookbook_dataset = action(:cookbook_dataset, i){{
      make_use_of_aliases_kept_in_the_file_cookbook_aliases: make_use_of_cookbook_aliases?,
      simple_or_complex:                                     simple_or_complex?
    }}
    set_cookbook_dataset(cookbook_dataset) # Use the appropriate setter-method here.
    # ======================================================================= #
    # Sync the make command to be used at once.
    # ======================================================================= #
    if cookbook_dataset_use_this_make_command?
      set_use_this_make_command(
        cookbook_dataset_use_this_make_command?
      )
    end
    return i # Always return the dataset as well here.
  end; alias load_yaml_dataset_for_this_program                      initialize_a_new_cookbook_for # === load_yaml_dataset_for_this_program
       alias update_dataset                                          initialize_a_new_cookbook_for # === update_dataset
       alias ensure_that_the_dataset_exists                          initialize_a_new_cookbook_for # === ensure_that_the_dataset_exists
       alias ensure_that_the_main_dataset_exists                     initialize_a_new_cookbook_for # === ensure_that_the_main_dataset_exists
       alias ensure_that_we_have_a_dataset                           initialize_a_new_cookbook_for # === ensure_that_we_have_a_dataset
       alias ensure_that_the_default_dataset_exists_for_this_program initialize_a_new_cookbook_for # === ensure_that_the_default_dataset_exists_for_this_program
       alias ensure_dataset_for_this_program                         initialize_a_new_cookbook_for # === ensure_dataset_for_this_program
       alias ensure_proper_dataset_for                               initialize_a_new_cookbook_for # === ensure_proper_dataset_for
       alias ensure_that_a_dataset_exists_for_this_program           initialize_a_new_cookbook_for # === ensure_that_a_dataset_exists_for_this_program
       alias ensure_that_the_dataset_exists_for_this_program         initialize_a_new_cookbook_for # === ensure_that_the_dataset_exists_for_this_program
       alias ensure_proper_dataset                                   initialize_a_new_cookbook_for # === ensure_proper_dataset
       alias create_a_new_cookbook_instance                          initialize_a_new_cookbook_for # === create_a_new_cookbook_instance
       alias determine_the_dataset_for_this_program                  initialize_a_new_cookbook_for # === determine_the_dataset_for_this_program
       alias determine_cookbook_dataset_for_this_program             initialize_a_new_cookbook_for # === determine_cookbook_dataset_for_this_program
       alias load_cookbook_dataset_for                               initialize_a_new_cookbook_for # === load_cookbook_dataset_for
       alias prepare_dataset_for                                     initialize_a_new_cookbook_for # === prepare_dataset_for
       alias initialize_cookbook_dataset_for                         initialize_a_new_cookbook_for # === initialize_cookbook_dataset_for
       alias initialize_a_new_cookbook_instance                      initialize_a_new_cookbook_for # === initialize_a_new_cookbook_instance
       alias initialize_dataset_for_this_program                     initialize_a_new_cookbook_for # === initialize_dataset_for_this_program
       alias initialize_dataset_for                                  initialize_a_new_cookbook_for # === initialize_dataset_for
       alias initialize_dataset                                      initialize_a_new_cookbook_for # === initialize_dataset
       alias initialize_for                                          initialize_a_new_cookbook_for # === initialize_for
       alias initialize_cookbook_dataset_for                         initialize_a_new_cookbook_for # === initialize_cookbook_dataset_for
       alias initialize_a_new_cookbook_for                           initialize_a_new_cookbook_for # === initialize_a_new_cookbook_for
       alias initialize_yaml_dataset                                 initialize_a_new_cookbook_for # === initialize_yaml_dataset
       alias initialize_cookbook_dataset                             initialize_a_new_cookbook_for # === initialize_cookbook_dataset
       alias initialize_cookbook_for                                 initialize_a_new_cookbook_for # === initialize_cookbook_for
       alias instantiate_a_new_cookbook_instance                     initialize_a_new_cookbook_for # === instantiate_a_new_cookbook_instance
       alias instantiate_a_new_cookbook_for                          initialize_a_new_cookbook_for # === instantiate_a_new_cookbook_for
       alias instantiate_a_new_cookbook                              initialize_a_new_cookbook_for # === instantiate_a_new_cookbook

  # ========================================================================= #
  # === run_ninja                                                 (ninja tag)
  #
  # This method will simply run "ninja".
  # ========================================================================= #
  def run_ninja
    # reset_error_map_line_to_a_specific_error
    cd use_this_as_build_directory? if do_we_use_a_build_directory?
    orev "Proceeding to run the command "+
         mediumpurple('ninja')+"#{rev} next, from the"
    orev "working directory #{sdir(return_pwd)}#{rev}:"
    _ = 'ninja'
    colourized_and_padded_esystem(_)
    # run_this_system_command_in_a_coloured_and_padded_manner 'ninja'
    # run_through_the_registered_problems_registered_errors_and_behaviour_changes
    # reset_error_map_line_to_a_specific_error
  end; alias do_run_ninja run_ninja # === do_run_ninja

  # ========================================================================= #
  # === return_append_this_to_the_configure_options
  #
  # This method will always at the least return an empty String.
  # ========================================================================= #
  def return_append_this_to_the_configure_options(
      append_this_to_the_configure_options = append_this_to_the_configure_options?
    )
    _ = ''
    if append_this_to_the_configure_options
      _ = _.dup if _.frozen?
      _ << append_this_to_the_configure_options
    end
    return _
  end

  # ========================================================================= #
  # === run_traditional_GNU_configure
  # ========================================================================= #
  def run_traditional_GNU_configure
    _ = './configure --prefix='+prefix?.dup
    configure_options = configure_options?
    if configure_options and !configure_options.empty?
      _ << " #{configure_options}"
    end
    colourized_and_padded_esystem(_)
  end; alias run_the_configure_command run_traditional_GNU_configure # === run_the_configure_command
       alias run_configure_like_stage  run_traditional_GNU_configure # === run_configure_like_stage

  # ========================================================================= #
  # === do_run_GNU_configure_then_make_then_make_install
  # ========================================================================= #
  def do_run_GNU_configure_then_make_then_make_install(
      use_this_configure_command = use_this_build_system?
    )
    run_traditional_GNU_configure
    run_premake_make_then_make_install
  end; alias run_configure_then_make_then_make_install do_run_GNU_configure_then_make_then_make_install # === run_configure_then_make_then_make_install
       alias run_the_gnu_configure_steps               do_run_GNU_configure_then_make_then_make_install # === run_the_gnu_configure_steps

  # ========================================================================= #
  # === consider_running_autoconf_then_run_configure_then_make_then_make_install
  # ========================================================================= #
  def consider_running_autoconf_then_run_configure_then_make_then_make_install
    consider_running_the_pre_configure_steps
    decide_whether_to_invoke_the_autoconf_command
    run_configure_then_make_then_make_install
  end

  # ========================================================================= #
  # === do_handle_the_post_installation_steps
  # ========================================================================= #
  def do_handle_the_post_installation_steps
    if appdir_prefix? and can_we_continue?
      _ = appdir_prefix_points_at?
      orev "Trying to update or create the #{steelblue('Current')}"\
           "#{rev} symlink next."
      action(:ToCurrent, _) # Run ToCurrent here.
      orev 'Trying to symlink various entries from '+steelblue(_)+rev+' next.'
      if force_symlinks_when_appdir_prefix_is_used?
        action(:symlink_from_to_current2, _) # Run SymlinkFromToCurrent here.
      else
        action(:symlink_from_to_current, _) # Run SymlinkFromToCurrent here.
      end
      consider_to_run_ldconfig
    end
  end

  # ========================================================================= #
  # === try_to_batch_compile_this_stack
  #
  # This method can be used to batch-compile several applications.
  #
  # Invocation examples:
  #
  #   rbt --batch-compile=autotools
  #   rbt --batch-compile=vid_suite
  #   rbt --batch-compile=mate
  #   rbt --batch-compile=kde5_porting_aids
  #
  # ========================================================================= #
  def try_to_batch_compile_this_stack(i)
    # ======================================================================= #
    # === Do some sanitizing of the input first.
    # ======================================================================= #
    case i
    # ======================================================================= #
    # === rbt --batch-compile=phonon
    # ======================================================================= #
    when 'phonon',
         /^kde(_|-)?plasma/
      i = 'kde5_plasma'
    # ======================================================================= #
    # === rbt --batch-compile=kde_apps
    # ======================================================================= #
    when /^kde(_|-)?apps/
      i = 'kde5_applications'
    # ======================================================================= #
    # === rbt --batch-compile=protos
    # ======================================================================= #
    when 'protos'
      i = 'xorg_protos'
    end
    case i # case tag
    # ======================================================================= #
    # === rbt --batch-compile=kde5_porting_aids
    # ======================================================================= #
    when /^kde5?(_|-)?porting(_|-)?aids?$/i,
         /^porting(_|-)?aids?$/i
      i = return_kde5_porting_aids
    # ======================================================================= #
    # === rbt --batch-compile=mate
    # ======================================================================= #
    when 'mate',
         /^mate(-|_)?desktop$/
      i = RBT.returned_chained_programs_of_the_mate_desktop
      opne "We will compile all #{sfancy(i.size.to_s)} programs "\
           "of the mate-desktop."
      opne 'These include:'
      e; i.each {|entry|
        e '  - '+sfancy(entry)
      }; e
    else
      hash = return_chained_programs
      i = hash[i]
    end
    if i.is_a? String and i.include? ','
      i = i.split(',').map(&:strip).flatten # Now it should be an Array.
    end
    if i.is_a? Array
      i.map! {|entry| entry.delete('-') }
    end
    return i
  end

  # ========================================================================= #
  # === do_these_logic_steps                                      (logic tag)
  #
  # This is a "meta"-method - it bundles different logic-steps together.
  # ========================================================================= #
  def do_these_logic_steps(*array)
    flattened = array.flatten
    flattened.each {|this_entry|
      case this_entry
      # ======================================================================= #
      # === :consider_applying_a_patch
      # ======================================================================= #
      when :consider_applying_a_patch
        consider_applying_a_patch
      # ======================================================================= #
      # === :consider_creating_the_appdir
      # ======================================================================= #
      when :consider_creating_the_appdir
        consider_creating_the_appdir_skeleton
      else # else assume a method-call as-is.
        send(this_entry.to_sym)
      end
    }
  end; alias do_this_logic_step do_these_logic_steps # === do_this_logic_step

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    determine_the_real_commandline_arguments(
      commandline_arguments?.select {|entry|
        entry.start_with?('--')
      }
    )
    _ = first_non_hyphened_argument?
    if _
      set_compile_this_program(_)
      set_cflags { :be_silent } # ← Initialize the CLFAGS here.
    end
    menu(real_commandline_arguments?)
    # ======================================================================= #
    # === The log directory must exist
    #
    # We must always ensure that the log_directory exists.
    # ======================================================================= #
    ensure_that_the_log_directory_exists
    if _ and !_.empty?
      set_compile_these_programs(compile_which_program?)
      do_install_these_programs(compile_which_program?)
    else
      orev 'Please supply an argument to this action.'
    end
  end

  # ========================================================================= #
  # === set_compile_this_program
  # ========================================================================= #
  def set_compile_this_program(i = nil)
    if i
      # ===================================================================== #
      # === Handle *
      #
      # First we will handle the '*' substring, in any given argument.
      #
      # We will show to the user what programs exist that start with
      # this name.
      #
      # To trigger this method, do:
      #
      #   ry a*
      #   ry e*
      #
      # ===================================================================== #
      if i.respond_to?(:include?) and
         i.include?('*')
        verbose_find_all_keys_starting_with(i)
        exit_program
      end
      # ===================================================================== #
      # We need to sanitize the given input a little bit next:
      # ===================================================================== #
      i = i.to_s.delete_suffix(
            File.extname(i)
          ).delete('-').
          downcase
      if i.end_with?(':')
        i = i.dup if i.frozen?
        i.chop!
      end
      i = i.to_sym
      # ===================================================================== #
      # === Check whether the given input is a registered program
      # ===================================================================== #
      if is_this_program_part_of_the_RBT_project?(i.to_s)
      # ===================================================================== #
      # === else we may have found an alias name
      # ===================================================================== #
      elsif make_use_of_the_cookbook_aliases? and
            cookbook_aliases?.include?(i.to_s)
        new_name = RBT.find_cookbook_alias_for(i)
        orev "An alias was found for the given input "+
             "#{steelblue(i)}#{rev}."
        orev "This alias will next be used, via the derived "\
             "name being #{steelblue(new_name)}#{rev}."
        i = new_name
      # ===================================================================== #
      # === else if we can map an input to a registered program
      # ===================================================================== #
      elsif (new_name = action(:map_this_input_to_that_registered_program, i)) and
            (i != new_name) and
             !new_name.nil?
        orev steelblue(i)+rev+' was not found, but another match was found'
        orev 'called '+steelblue(new_name)+rev+', which is now used.'
        i = new_name
      # ===================================================================== #
      # === else we have found nothing, so report to the user
      # ===================================================================== #
      else
        orev tomato('Unfortunately the program called ')+
             steelblue(i)+
             tomato(' is not')
        orev tomato('registered as part of the RBT project.')
        i = nil
      end
      @internal_hash[:compile_this_program] = i
    end
  end

  # ========================================================================= #
  # === do_install_these_programs
  # ========================================================================= #
  def do_install_these_programs(
      i = compile_which_program?
    )
    [i].flatten.compact.each {|this_program|
      try_to_install_this_program(this_program)
    }
  end; alias batch_compile_the_programs    do_install_these_programs # === batch_compile_the_programs
       alias do_initiate_batch_compilation do_install_these_programs # === do_initiate_batch_compilation

  # ======================================================================= #
  # === try_to_install_this_program
  # ======================================================================= #
  def try_to_install_this_program(
      i = compile_which_program?
    )
    return if i.nil?
    reset_the_cookbook_dataset # Start from a clean state here.
    do_compile_this_registered_program(i)
  end; alias do_install_this_program try_to_install_this_program # === do_install_this_program

  # ========================================================================= #
  # === set_compile_these_programs_then_batch_compile_them
  #
  # This method can be used to start batch-compiling several programs
  # in one go.
  # ========================================================================= #
  def set_compile_these_programs_then_batch_compile_them(
      i, consider_replacing_the_input = :default
    )
    set_compile_these_programs(i, consider_replacing_the_input) 
    batch_compile_these_programs(i)
  end

  # ========================================================================= #
  # === set_compile_these_programs
  #
  # This variant will always be an Array.
  # ========================================================================= #
  def set_compile_these_programs(i)
    @internal_hash[:compile_these_programs] << [i].flatten.compact
    @internal_hash[:compile_these_programs].flatten!
  end

end; end; end
