#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::Aliases
#
# The purpose of this class is fairly simple:
#
#   - Give it a string such as "qt4r" and it will point to "qt4ruby".
#
# Presently, this class only uses the Aliases defined by the file
# "cookbook_aliases.yml".
#
# You can also query and obtain this information.
#
# It may be better to require other files than this class directly, such
# as 'cookbooks/module_methods/is_included.rb', since that file offers
# additional functionality, as well as make available class
# Cookbooks:Aliases anyway.
# =========================================================================== #
# require 'rbt/aliases/aliases.rb'
# RBT.cookbook_aliases
# =========================================================================== #
module RBT

  require 'rbt/lean_prototype/lean_prototype.rb'

  # ========================================================================= #
  # === RBT::COOKBOOK_ALIASES
  #
  # All aliases for the cookbook files are stored here.
  #
  # For the individual yaml files, the constant
  # INDIVIDUAL_COOKBOOKS_DIRECTORY should be
  # used instead.
  # ========================================================================= #
  _ = RBT.file_cookbook_aliases
  if File.exist? _
    COOKBOOK_ALIASES = YAML.load_file(_) # load up the file cookbook_aliases.yml
  else
    COOKBOOK_ALIASES = {} # This will be for most everyone else.
  end

  # ========================================================================= #
  # === RBT.cookbook_aliases?
  # ========================================================================= #
  def self.cookbook_aliases?
    COOKBOOK_ALIASES
  end

  # ========================================================================= #
  # === RBT.cookbook_aliases
  #
  # This will return the registered/available keys, in a sorted manner.
  #
  # This will have about 331 entries (as of August 2018).
  # ========================================================================= #
  def self.cookbook_aliases
    COOKBOOK_ALIASES.keys.sort
  end; self.instance_eval { alias aliases? cookbook_aliases } # === Cookbook.aliases?

  # =========================================================================== #
  # === RBT.return_alias_program_name_for
  #
  # This method will return the (aliased) program name for the
  # given input. If no alias name was found, it will return the
  # input.
  # =========================================================================== #
  def self.return_alias_program_name_for(i)
    RBT::Cookbooks::Aliases.new(i).translate_alias
  end

  # =========================================================================== #
  # === RBT.is_an_alias?
  #
  # This method will return true if the input is an alias, and otherwise
  # it will return false.
  #
  # We will get rid of ':' in the input.
  #
  # To test this method, do:
  #
  #   RBT.is_an_alias? 'htop'
  #   RBT.is_an_alias? 'blkid'
  #
  # =========================================================================== #
  def self.is_an_alias?(i)
    RBT::Cookbooks::Aliases.new(i).is_an_alias?
  end

module Cookbooks # === RBT::Cookbooks

class Aliases < LeanPrototype # === RBT::Cookbooks::Aliases

  # ========================================================================= #
  # === initialize
  #
  # The first argument, `this_program`, will be assigned to the instance
  # variable @this_program, which keeps track of our original input.
  # ========================================================================= #
  def initialize(
      this_program = DEFAULT_PROGRAM_ENTRY,
      run_already  = true
    )
    register_sigint
    reset
    set_search_for_this_program(
      this_program
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === :be_verbose
    # ======================================================================= #
    do_not_be_verbose # By default, we will be fairly quiet.
    # ======================================================================= #
    # === :use_abbreviations_for_the_aliases
    #
    # The next setting determines whether class Cookbooks::Aliases will
    # use abbreviations for its core-defined aliases. The core-defined
    # aliases are those that are written in the .yml file directly;
    # the abbreviations to these aliases, are NOT core aliases.
    #
    # By default, we will NOT use abbreviations - you have to specifically
    # enable this in your code if you want to use abbreviations.
    # ======================================================================= #
    @internal_hash[:use_abbreviations_for_the_aliases] = false
    obtain_dataset
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(namespace?)
  end

  # ========================================================================= #
  # === n_registered_aliases?
  # ========================================================================= #
  def n_registered_aliases?
    @internal_hash[:raw_dataset].keys.size
  end; alias n_registered_programs? n_registered_aliases? # === n_registered_programs?
       alias n_entries?             n_registered_aliases? # === n_entries?

  # ========================================================================= #
  # === search_for_this_program?
  # ========================================================================= #
  def search_for_this_program?
    ihash?[:search_for_this_program]
  end; alias search_for_which_program? search_for_this_program? # === search_for_which_program?
       alias program?                  search_for_this_program? # === program?
       alias program_name?             search_for_this_program? # === program_name?
       alias this_program?             search_for_this_program? # === this_program?
       alias input?                    search_for_this_program? # === input?

  # ========================================================================= #
  # === set_keys
  #
  # This will keep track of how many keys there are in our dataset.
  # ========================================================================= #
  def set_keys(
      i = dataset?.keys.sort
    )
    @internal_hash[:keys] = i
  end

  # ========================================================================= #
  # === is_a_real_alias?
  #
  # This method tells us whether we have a real alias, that is originating
  # from the initial Hash, or whether we have an alias that we deduced.
  # ========================================================================= #
  def is_a_real_alias?(i = input?)
    raw_aliases?.has_key? i
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @internal_hash[:dataset]
  end; alias dataset dataset? # === dataset
       alias data?   dataset? # === data?
       alias data    dataset? # === data

  # ========================================================================= #
  # === report_whether_the_input_is_an_alias
  #
  # This method can be used to report whether the given input is an
  # alias or whether it is not.
  # ========================================================================= #
  def report_whether_the_input_is_an_alias(
      i = program?
    )
    if is_included?(i)
      opne rev+'The program `'+sfancy(i)+rev+'` is an alias, pointing '+
           "at the program `#{simp(points_at?)}."
    else
      opne "#{rev}The program `#{sfancy(i)}#{rev}` is NOT an alias."
    end
  end

  # ========================================================================= #
  # === find
  #
  # find() will output in a verbose manner, regardless of the verbosity
  # level.
  # ========================================================================= #
  def find(i = program?)
    if is_included?(i)
      opne rev+'`'+sfancy(i)+rev+'` matches towards `'+
           sfancy(points_at?(i))+rev+'`.'
    else
      opne "#{rev}`#{sfancy(i)}#{rev}` does not match to anything."
    end
  end; alias show find # === show

  # ========================================================================= #
  # === deep_show
  #
  # This will show all matches. It is thought to be used in conjunction
  # with a query (and obviously, the variable @is_included must be set
  # for this to work.
  # ========================================================================= #
  def deep_show
    if is_included?
      opne "#{rev}Additionally, also showing all aliases starting "\
           "with `#{sfancy(program?)}#{rev}*`."
      all_matches = keys?.select {|key| key =~ /^#{program?}/ }
      e all_matches.sort.join(', ').split(N).map {|entry| '  '+entry }.join(N)
    end
  end; alias show_all_matches deep_show # === show_all_matches

  # ========================================================================= #
  # === points_at?
  #
  # Return the alias-pointer, without any additional checking whatsoever.
  #
  # Note that [] is also available as alias.
  # ========================================================================= #
  def points_at?(i = program_name?)
    dataset?[i]
  end; alias real_alias    points_at? # === real_alias
       alias points_to?    points_at? # === points_at?
       alias expanded_name points_at? # === expanded_name # Just an alias to the above.
       alias replace       points_at? # === replace
       alias value?        points_at? # === value?        # is an alias as well.
       alias []            points_at? # === []

  # ========================================================================= #
  # === show_size
  # ========================================================================= #
  def show_size
    opne "#{rev}We have #{sfancy(n_registered_programs?.to_s)}"\
         " #{rev}unique programs registered."
  end

  # ========================================================================= #
  # === avaiable_programs?
  # ========================================================================= #
  def available_programs?
    raw_aliases?.keys
  end; alias available_programs available_programs? # === available_programs
       alias programs?          available_programs? # === programs?
       alias aliases?           available_programs? # === aliases?

  # ========================================================================= #
  # === enable_aliases
  #
  # This method is required so that keys such as "qt4r" also work,
  # which is an alias of "qt4ruby" pointing towards "qt4qtruby".
  #
  # This will, however had, only be done if the variable at
  # @internal_hash[:use_abbreviations_for_the_aliases] is set to true.
  # ========================================================================= #
  def enable_aliases
    if @internal_hash[:use_abbreviations_for_the_aliases]
      _ = {} # This will become the new Hash.
      # ===================================================================== #
      # Iterate over the old aliases.
      # ===================================================================== #
      dataset?.each_pair {|key, value|
        _[key]   = value
        _[value] = value # Also keep itself as name.
        loop {
          # ================================================================= #
          # Get all but the last two characters repeatedly, so we chop up
          # the keys.
          # ================================================================= #
          key = key[0..-2]
          _[key] = value
          break if key.empty?
        }
      }
      set_dataset(_)
    end
  end

  # ========================================================================= #
  # === keys
  #
  # Assign the keys, then returns all keys. Will be sorted.
  # ========================================================================= #
  def keys?
    @internal_hash[:keys]
  end; alias keys keys? # === keys

  # ========================================================================= #
  # === show_all_available_entries
  # ========================================================================= #
  def show_all_available_entriesx
    pp keys?.sort
  end

  # ========================================================================= #
  # === translate_alias
  #
  # This method will try to "translate" an alias.
  # ========================================================================= #
  def translate_alias(i = program?)
    return dataset?[i] if is_included? # Found the result here.
    return i
  end; alias translate translate_alias # === translate
       alias value?    translate_alias # === value?
       alias []        translate_alias # === []
       alias result?   translate_alias # === result?
       alias result    translate_alias # === result

  # ========================================================================= #
  # === report_n_entries
  # ========================================================================= #
  def report_n_entries(
      i = n_entries?
    )
    e
    opne "#{rev}We have n entries (== core aliases) registered:"
    opne "  #{sfancy(i)}"
    e
  end

  # ========================================================================= #
  # === set_search_for_this_program
  #
  # This setter method will search for the given program at hand.
  # ========================================================================= #
  def set_search_for_this_program(
      i = DEFAULT_PROGRAM_ENTRY
    )
    i = i.first if i.is_a? Array
    i = DEFAULT_PROGRAM_ENTRY if i.nil?
    i = i.to_s.dup # To ensure that it is never nil.
    # ======================================================================= #
    # Remove all '_' characters since as of March 2012.
    # Disabled this again in August 2018 - let's see which variant
    # will be kept in the long run.
    # ======================================================================= #
    # i.delete!('-') if i.include? '-'
    ihash?[:search_for_this_program] = i
  end

  # ========================================================================= #
  # === show_all_abbreviations
  # ========================================================================= #
  def show_all_abbreviations
    abbreviations = dataset?
    pp abbreviations
  end

  # ========================================================================= #
  # === internal_hash?
  # ========================================================================= #
  def internal_hash?
    @internal_hash
  end; alias ihash? internal_hash? # === ihash?

  # ========================================================================= #
  # === is_included?
  # ========================================================================= #
  def is_included?(i = program_name?)
    dataset?.has_key? i
  end; alias is_an_alias? is_included? # === is_an_alias?
       alias includes?    is_included? # === includes?
       alias found?       is_included? # === found?
       alias was_found?   is_included? # === was_found?

  # ========================================================================= #
  # === obtain_raw_dataset
  # ========================================================================= #
  def obtain_raw_dataset
    # ======================================================================= #
    # === :raw_dataset
    #
    # Keep a raw dataset, as reference copy. The raw dataset is the
    # unmodified set of key-value pairs.
    # ======================================================================= #
    @internal_hash[:raw_dataset] = @internal_hash[:dataset].dup
  end

  # ========================================================================= #
  # === raw_dataset?
  # ========================================================================= #
  def raw_dataset?
    @internal_hash[:raw_dataset]
  end; alias raw_data?    raw_dataset? # === raw_data?
       alias raw_aliases? raw_dataset? # === raw_aliases?

  # ========================================================================= #
  # === set_dataset
  #
  # This sets the main dataset. If you ever need the raw dataset, make
  # use of the method called raw_dataset?. Note that this dataset will
  # lateron become expanded via "aliases" to this dataset.
  # ========================================================================= #
  def set_dataset(
      i = RBT.cookbook_aliases?
    )
    @internal_hash[:dataset] = i
  end

  # ========================================================================= #
  # === obtain_raw_dataset
  # ========================================================================= #
  def obtain_dataset
    # ======================================================================= #
    # The main pointer to point at our yaml data. This is a Hash.
    # ======================================================================= #
    set_dataset(
      RBT.cookbook_aliases? # Defined in rbt/aliases/aliases.rb
    )
    obtain_raw_dataset # Also sync at the raw dataset.
  end

  # ========================================================================= #
  # === show_help
  #
  # This method is used to show the available help-options for this
  # class.
  # ========================================================================= #
  def show_help(
      shall_we_exit = false
    )
    shall_we_exit = true if shall_we_exit == :then_exit
    opne 'These options exist:'
    e
    ecomment '  --n_entries # show how many cookbook entries exist'
    e
    exit if shall_we_exit
  end

  # =========================================================================== #
  # === menu
  # =========================================================================== #
  def menu(
      i = search_for_which_program?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ======================================================================= #
      # === ashower --show_all_abbreviations
      # ======================================================================= #
      when /^-?-?show(_|-)?abbreviations$/i, # ashower SHOW_ABBREVIATIONS
           /^-?-?show(_|-)?all(_|-)?abbreviations$/
        show_all_abbreviations
        exit
      # ======================================================================= #
      # === ashower --all
      # ======================================================================= #
      when /^-?-?all$/i # Careful - 0install will match towards this, hence the leading '^'.
        show_all_available_entries
        exit
      # ======================================================================= #
      # === ashower --show_all
      # ======================================================================= #
      when /-?-?show(_|-)?all$/i # ashower SHOW_ALL
        show_all_matches # Show all matches.
      # ======================================================================= #
      # === ashower --n_entries
      # ======================================================================= #
      when /^-?-?n(_|-)?entries$/i
        report_n_entries
        exit
      # ======================================================================= #
      # === ashower --help
      # ======================================================================= #
      when /^-?-?help$/i # <- Otherwise we would match towards "devhelp".
        show_help :then_exit
      end
    end
  end

  # ========================================================================= #
  # === report
  # ========================================================================= #
  def report
    opne 'The available aliases are:'
    opne keys?.join(', ')
    opne 'The available datasets are:'
    pp data?
    opne "The total size of all available aliases-keys is "\
         "#{keys?.size} entries."
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    enable_aliases
    set_keys
    menu
  end

end; end

# =========================================================================== #
# === RBT.find_cookbook_alias_for
#
# This method can be used to find a specific alias for a real, existing
# (and registered) program of the RBT suite.
#
# Usage example:
#
#   RBT.find_cookbook_alias_for 'perlx'
#
# =========================================================================== #
def self.find_cookbook_alias_for(i)
  RBT::Cookbooks::Aliases.new(i).value?
end; self.instance_eval { alias alias?         find_cookbook_alias_for } # === RBT.find_cookbook_alias_for
     self.instance_eval { alias find_alias_for find_cookbook_alias_for } # === RBT.find_alias_for

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  require 'colours/autogenerated/html_colours_methods.rb'
  require 'verbose_truth/autoinclude'
  include Colours
  include Colours::HtmlColoursMethods
  pp RBT.cookbook_aliases?
  e "#{Colours.rev}How many aliases are registered? "\
    "#{Colours::HtmlColoursMethods.steelblue(RBT.cookbook_aliases.size.to_s)}"
  _ = RBT::Cookbooks::Aliases.new(ARGV)
  if ARGV.empty?
    _.find 'glib2' # glib2.
    _.report_n_entries
    e "#{rev}Is ecm an alias? "+ Colours::HtmlColoursMethods.steelblue(_.is_an_alias?('ecm')).to_s
    e
  else
    _.find(ARGV.first)
    _.report_whether_the_input_is_an_alias
    # _.translate_alias # No output here.
    e rev+"We will next feedback information about #{simp(ARGV.first)}#{rev}."
    e rev+'Is it a real alias? '+
      sfancy(VerboseTruth[_.is_a_real_alias?.to_s])
    e rev+'Is it an alias? '+
      sfancy(VerboseTruth[_.is_an_alias?.to_s])
    unless _.points_at?.to_s.empty?
      e rev+'It points towards '+
        sfancy(_.points_at?.to_s)+'.'
    end
  end
end # ashower ruby
    # ashower qt4r
    # ashower ascii; ashower mencoder
    # ashower aclocal
    # ashower xfcem
    # ashower --n_entries       