#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CheckForCorrectDependencies
#
# This class will check whether the entries in the cookbook files do all
# happen to have valid, existing dependencies. If the file is not valid,
# the class will output this information on the commandline. That way
# the problem(s) can be fixed.
#
# Usage example:
#
#   RBT::Cookbooks::CheckForCorrectDependencies.new(ARGV)
#
# =========================================================================== #
# require 'rbt/checks_and_validations/check_for_correct_dependencies.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class CheckForCorrectDependencies < RBT::Base # === RBT::Cookbooks::CheckForCorrectDependencies

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @raw_cookbook
    # ======================================================================= #
    @raw_cookbook = action(:raw_cookbook)
    # ======================================================================= #
    # === @n_erroneous_entries_were_found
    # ======================================================================= #
    @n_erroneous_entries_were_found = 0
  end

  # ========================================================================= #
  # === format_program
  # ========================================================================= #
  def format_program(i)
    _ = "#{i}:"
    _ = _.ljust(22)
    splitted = _.split(':')
    splitted[0] = sfancy(splitted.first)
    _ = splitted.join(':')
    _
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e
    e 'Currently this class has no special commandline flags.'
    e
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === correct_deps? --help
      # ===================================================================== #
      when /^help$/
        show_help
      else
        opne "#{rev}Unregistered commandline argument: #{sfancy(i)}#{rev}"
      end
    end
  end

  # ========================================================================= #
  # === check_for_every_entry
  #
  # This method will check for every available (registered) program.
  # ========================================================================= #
  def check_for_every_entry(
      use_this_as_the_key_for_required_deps_on =
        'required_deps_on'.to_sym
    )
    opne "#{rev}Now checking every program. This may take a while."
    available_programs?.each {|program|
      program = File.basename(program).sub(/\.yml$/,'')
      # ===================================================================== #
      # The next line is very verbose, so it may be best to enable it only
      # for debugging purposes really.
      # ===================================================================== #
      # opne 'Working on the program `'+sfancy(program)+'` next.'
      # ===================================================================== #
      # We have two ways for the next line - we will either instantiate a
      # new Cookbooks object - or we will make use of an already existing,
      # expanded .yml file that holds the dataset for the program at hand.
      # ===================================================================== #
      if expanded_directory_exists? and
         File.exist?(
           path_to_this_expanded_cookbooks_dataset(program)
         )
        dataset = YAML.load_file(
          path_to_this_expanded_cookbooks_dataset(program)
        )
        deps = dataset[use_this_as_the_key_for_required_deps_on]
      else
        @raw_cookbook.reset_the_main_hash
        dataset = @raw_cookbook.find(program)
        deps    = @raw_cookbook.deps?
      end
      result = nil
      if deps
        result = deps.all? {|entry|
          RBT.really_does_include?(entry)
        }
      end
      if result # Ok, in this case all entries were found.
      else # else it was not found, so work on that.
        if deps
          @n_erroneous_entries_were_found += 1
          deps.each {|entry|
            if entry.nil?
              opne rev+'Nil value found for `'+steelblue(program)+'`. '\
                   'This is probably an empty entry.'
            end
            if entry
              if entry.include?(' ')
                entry = entry.split(' ').first.strip
              end
              unless RBT.really_does_include? entry
                e "#{rev}#{format_program(program)} #{rev}No "\
                  "entry for `#{simp(entry)}#{rev}` was found."
              end
            end
          }
        end
      end
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    unless commandline_arguments?.empty?
      menu
    else # else check for every entry
      check_for_every_entry
    end
  end

  # ========================================================================= #
  # === RBT::Cookbooks::CheckForCorrectDependencies[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CheckForCorrectDependencies.new(ARGV)
end # correctdependencies