#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::ValidateArchiveSize
#
# This class will simply check the archive size, and report all entries
# with an archive size of 0. An archive size of 0 is not a good result;
# it meant that something did not work, somewhere.
#
# Usage example:
#
#   RBT::Cookbooks::ValidateArchiveSize.new(ARGV)
#
# =========================================================================== #
# require 'rbt/checks_and_validations/validate_archive_size.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class ValidateArchiveSize < RBT::Base # === RBT::Cookbooks::ValidateArchiveSize

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @n_problematic_archives_were_found
    # ======================================================================= #
    @n_problematic_archives_were_found = 0
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === validate_blfs_entries --help
      # ===================================================================== #
      when /^help$/
        show_help
      else
        opne "#{rev}Unregistered commandline argument: #{sfancy(i)}"
      end
    end
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help
    e 'Currently no help options are documented.'
  end

  # ========================================================================= #
  # === notify_the_user_if_any_problematic_archive_was_discovered
  # ========================================================================= #
  def notify_the_user_if_any_problematic_archive_was_discovered
    if @n_problematic_archives_were_found > 0
      e
      e rev+'A total of '+steelblue(@n_problematic_archives_were_found.to_s)+
        rev+' programs was found.'
      e
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu(commandline_arguments?)
    target = "#{directory_expanded_cookbooks?}*.yml"
    opnn { :no_colon }; e; e
    Dir[target].each {|this_yaml_file|
      dataset = File.read(this_yaml_file)
      archive_size_is = dataset.scan(
        /archive_size: (\d{1,15})/
      ).flatten.first.to_i
      if archive_size_is == 0
        @n_problematic_archives_were_found += 1
        e rev+'The archive at '+sfile(this_yaml_file)+
          rev+' is problematic: size of 0.'
      end
    }
    notify_the_user_if_any_problematic_archive_was_discovered
  end

  # ========================================================================= #
  # === RBT::Cookbooks::ValidateArchiveSize[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::ValidateArchiveSize.new(ARGV)
end # ValidateArchiveSize