#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cmake
#
# The task of this class is simple: Contain a CMake string. Additional goals
# are to be able to call this from the commandline (and allow the user to
# manipulate things that way).
#
# The RBT::Cmake class was created after noticing that class RConfigureString
# does such a great job of simplifying handling configure line options, so 
# it made sense to use the same for all cmake-based projects.
#
# Ironically enough, RConfigureString was later removed but class RBT::Cmake
# remained in use - and is still in use, today. (It had another name back
# then, though, called RBT::CmakeString).
#
# Some modifications were needed, and a few of these modifications may be
# explained next.
#
# The class RBT:Cmake has two "modus operandi". The default way is to
# run "standalone", that is - simply invoke it and it will do its job.
#
# But a second way is to feedback information into class RBT::Action::Installer,
# so that class RBT::Action::Installer can handle and process all the information
# used. The proper way to assign a parent object is to pass a block.
# =========================================================================== #
# === Specification constraints for this class:
#
#   - The '.' or '..' must come last in the @cmake_string
#
# Please note that we can omit both and just pass the absolute directory
# instead. Since as of August 2011, the latter is the preferred way.
# =========================================================================== #
# require 'rbt/cmake/cmake.rb'
# RBT.cmake
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class Cmake < Base # === RBT::Cmake

  require 'rbt/misc/colourize_parser.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      use_this_prefix = :guess_prefix,
      run_already     = true,
      &block
    )
    reset
    case use_this_prefix
    when :do_not_run_yet
      run_already = false
      use_this_prefix = :guess_prefix
    end
    case run_already
    # ======================================================================= #
    # === :do_not_run_yet
    # ======================================================================= #
    when :do_not_run_yet
      run_already = false
    end
    set_use_this_prefix(use_this_prefix)
    set_base_directory
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      # ===================================================================== #
      # Use it as at the parent object.
      # ===================================================================== #
      if yielded.is_a?(Symbol)
        case yielded
        # =================================================================== #
        # ===  :colourized_and_padded
        # =================================================================== #
        when :colourized_and_padded
          @use_extra_padding_for_the_system_command = true
        else
          e tomato("Not found the following symbol: :#{yielded}")
        end
      elsif yielded.is_a?(Hash)
        set_parent_object(yielded[:self]) if yielded.has_key? :self
        set_use_opn(yielded[:use_opn]) if yielded.has_key? :use_opn
      else
        set_parent_object(yielded)
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @current_directory
    # ======================================================================= #
    @current_directory = return_pwd
    # ======================================================================= #
    # === @cmake_string
    # ======================================================================= #
    @cmake_string      = 'cmake'.dup
    # ======================================================================= #
    # === @may_we_exit
    # ======================================================================= #
    @may_we_exit       = false
    @parent_object     = nil
    @has_been_run      = false
    # ======================================================================= #
    # === @use_opn
    # ======================================================================= #
    @use_opn           = true
    # ======================================================================= #
    # === @use_extra_padding_for_the_system_command
    # ======================================================================= #
    @use_extra_padding_for_the_system_command = false
    set_base_directory(temp_directory?)
  end

  # ========================================================================= #
  # === set_parent_object
  #
  # The parent object, presently, may only be of class RBT::Action::Installer
  # or it is simply nil.
  # ========================================================================= #
  def set_parent_object(i = nil)
    @parent_object = i
  end

  # ========================================================================= #
  # === set_use_this_prefix
  # ========================================================================= #
  def set_use_this_prefix(i)
    case i
    # ======================================================================= #
    # === :guess
    # ======================================================================= #
    when :guess,
         :guess_prefix
      i = guess_prefix
    end
    _ = " -DCMAKE_INSTALL_PREFIX=#{i}"
    @use_this_prefix = _
  end; alias set_prefix set_use_this_prefix # === set_prefix

  # ========================================================================= #
  # === use_this_prefix?
  # ========================================================================= #
  def use_this_prefix?
    @use_this_prefix
  end

  # ========================================================================= #
  # === guess_prefix
  #
  # The argument `of_this_dir` passed to this method should include a '-'.
  #
  # guess_prefix will simply return the new prefix.
  # ========================================================================= #
  def guess_prefix(
      of_this_dir = return_pwd
    )
    basename = File.basename(of_this_dir)
    basename = of_this_dir.split('-') if of_this_dir.include? '-'
    _ = programs_dir?.dup
    if of_this_dir.include? '-'
      _ << File.basename(basename.first).capitalize+'/'+basename[1]
    else
      _ << basename
    end
    return _
  end; alias prefix?       guess_prefix # === prefix?
       alias guess_prefix? guess_prefix # === guess_prefix?

  # ========================================================================= #
  # === check_if_cmakelists_file_exists?
  #
  # This method checks if a CMakeLists.txt file exists.
  # ========================================================================= #
  def check_if_cmakelists_file_exists?(where = Dir.pwd)
    if File.exist? where+'/CMakeLists.txt'
      true
    else
      false
    end
  end

  # ========================================================================= #
  # === create_cmake_file_add_subdirectory
  #
  # From this point on we list only methods which are to be used to create 
  # a cmake file. These should one day be used to automatically create a 
  # cmake project.
  # ========================================================================= #  
  def create_cmake_file_add_subdirectory(name)
  end

  # ========================================================================= #
  # === set_may_we_exit
  # ========================================================================= #
  def set_may_we_exit
    @may_we_exit = true # We may exit.
  end

  # ========================================================================= #
  # === append
  #
  # This method will simply append data onto the @cmake_string.
  #
  # NO checking involved, so call it only when you are sure that you want
  # to add this.
  #
  # Use this method and ONLY this method when you wish to extend
  # the @cmake_string.
  # ========================================================================= #
  def append(i)
    @cmake_string << " #{i}"
    @cmake_string.strip! # Clean up too.
    @cmake_string.squeeze!(' ')
  end

  # ========================================================================= #
  # === prepend
  #
  # This will prepend to the cmake string. Best way to use this is via
  # the prefix variable.
  # ========================================================================= #
  def prepend(i)
    # ======================================================================= #
    # We must find the position of the 'cmake' string.
    # ======================================================================= #
    if @cmake_string.include? 'cmake'
      start_position = @cmake_string.index('cmake')+('cmake'.size)
      @cmake_string[start_position,0] = ' '+i
    else 
      @cmake_string.prepend(" #{i}")
    end
    @cmake_string.strip! # Clean up too.
    @cmake_string.squeeze!(' ')
  end

  # ========================================================================= #
  # === set_base_directory
  #
  # The method set_base_directory sets the variable @base_directory to use.
  # By default, it uses ' .' as its target.
  # ========================================================================= #
  def set_base_directory(
      i = ' .'
    )
    @base_directory = i
  end

  # ========================================================================= #
  # === we_use_a_build_directory
  # ========================================================================= #
  def we_use_a_build_directory
    set_base_directory(' ..')
  end

  # ========================================================================= #
  # === append_base_directory
  # ========================================================================= #
  def append_base_directory
    append @base_directory
  end; alias add_prefix append_base_directory # == add_prefix

  # ========================================================================= #
  # === check_if_cmake_is_available
  #
  # We must find out whether cmake is available or not.
  #
  # If it is not available, we must exit, but only if the argument
  # provided allows us to do this.
  # ========================================================================= #
  def check_if_cmake_is_available(
      may_we_exit = @may_we_exit
    )
    case may_we_exit
    when :do_not_exit
      may_we_exit = false
    end
    _ = `cmake 2>&1`
    if _.include? 'not found'
      opne swarn('Cmake is not available. Please install `cmake` first.')
      if may_we_exit
        opnwarn 'Exiting now.'
        exit
      end
      return false
    else
      return true
    end
  end; alias cmake_is_available? check_if_cmake_is_available # === cmake_is_available?

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    super(namespace?) if use_opn?
  end

  # ========================================================================= #
  # === run_full
  #
  # This method will also run "make" and "make install".
  #
  # Afterwards, it will invoke CreateProgram, and then AdvancedSymlink.
  #
  # You probably won't need this method if you use this class here as
  # part of RBT, but in the event that you also wish to call this
  # class standalone, this code should suffice.
  #
  # Since Aug 2011 we use the variable @base_directory.
  # ========================================================================= #
  def run_full(
      also_run_make_and_make_install = false,
      full_prefix                    = prefix?
    )
    append_base_directory
    run
    if also_run_make_and_make_install
      run_make_command
      run_make_install_command
      # ===================================================================== #
      # === RBT::CreateProgram
      #
      # Next, delegate towards RBT::CreateProgram.
      # ===================================================================== #
      create_program = action(:app_dir_skeleton, full_prefix)
      create_program.create_the_directories
      # ===================================================================== #
      # Since Aug 2011 we also run AdvancedSymlink. @default_prefix
      # must be set when we arrive at this point here.
      # ===================================================================== #
      @advanced_symlink = RBT::AdvancedSymlink.new(full_prefix)
    end
  end

  # ========================================================================= #
  # === has_been_run?
  # ========================================================================= #
  def has_been_run?
    @has_been_run
  end

  # ========================================================================= #
  # === run_this_system_command_via_io_popen
  # ========================================================================= #
  def run_this_system_command_via_io_popen(i)
    io_object = IO.popen(i, {:err => [:child, :out]}).each { |line|
      if @parent_object
        # =================================================================== #
        # In this case, delegate to the parent object.
        # =================================================================== #
        @parent_object.parse_this_line_obtained_via_io_popen(line)
      else
        # =================================================================== #
        # Else, handle everything here in this class.
        # =================================================================== #
        if RBT.const_defined? :ColourizeParser
          line = ColourizeParser.parse_this_line(line)
        end
        e line
      end
    }
    io_object.close
  end

  # ========================================================================= #
  # === cmake_string?
  # ========================================================================= #
  def cmake_string? # contains the cmake string.
    @cmake_string
  end; alias cmake_string                           cmake_string? # === cmake_string
       alias return_configure_command_that_was_used cmake_string? # === return_configure_command_that_was_used
       alias string                                 cmake_string? # === string

  # ========================================================================= #
  # === run_make_command
  # ========================================================================= #
  def run_make_command
    run_this_system_command_via_io_popen 'make'+ERROR_LINE
  end

  # ========================================================================= #
  # === run_make_install_command
  # ========================================================================= #
  def run_make_install_command
    run_this_system_command_via_io_popen 'make install'+ERROR_LINE
  end

  # ========================================================================= #
  # === notify_the_user_which_command_we_will_run
  #
  # This useful method will notify the user which specific command
  # will be run.
  # ========================================================================= #
  def notify_the_user_which_command_we_will_run
    orev "Running the following command "\
         "from #{sdir(@current_directory)}#{rev}:"
    e if @use_extra_padding_for_the_system_command
    e cornflowerblue("  #{@cmake_string}")
    e if @use_extra_padding_for_the_system_command
    run_this_system_command_via_io_popen(@cmake_string)
    @has_been_run = true # Keep track of the fact that we already run this once.
  end; alias verbose_run_the_actual_system_command notify_the_user_which_command_we_will_run # === verbose_run_the_actual_system_command

  # ========================================================================= #
  # === set_full_configure_line
  #
  # This method can be specifically used from the outside.
  # ========================================================================= #
  def set_full_configure_line(i)
    @cmake_string = i
  end

  # ========================================================================= #
  # === run
  #
  # If you need some examples for a proper cmake string, have a look:
  #
  #   cmake . -DCMAKE_INSTALL_PREFIX=$MY_TEMP/install_test
  #   cmake -DCMAKE_INSTALL_PREFIX=/usr ..
  #
  # ========================================================================= #
  def run
    check_if_cmake_is_available
    prepend(use_this_prefix?)
    notify_the_user_which_command_we_will_run
  end

end

# =========================================================================== #
# === RBT.cmake
# =========================================================================== #
def self.cmake(i = ARGV, &block)
  RBT::Cmake.new(i, &block)
end

end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::Cmake.new
  _.check_if_cmake_is_available
  _.guess_prefix(Dir.pwd)
  _.set_prefix
  _.set_may_we_exit
  _.run_full(true)
end # rcmake
    # rcmake -DOPENBABEL2_LIBRARIES=$SLL/ -DOPENBABEL2_INCLUDE_DIR=$SLH
