#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Commandline
# =========================================================================== #
# require 'rbt/commandline/commandline.rb'
# RBT::Commandline.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

  # ========================================================================= #
  # === @commandline_options
  #
  # This must be defined on the toplevel.
  # ========================================================================= #
  @commandline_options = []

  # ========================================================================= #
  # === RBT.commandline_options?
  # ========================================================================= #
  def self.commandline_options?
    @commandline_options
  end; self.instance_eval { alias commandline_arguments? commandline_options? } # === RBT.commandline_arguments?

  # ========================================================================= #
  # === RBT.set_commandline_options
  # ========================================================================= #
  def self.set_commandline_options(i)
    @commandline_options = [i].flatten.compact
  end

class Commandline < Base

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARGV,
      run_already = true
    )
    RBT.set_commandline_options(i) # ← Keep a reference to the commandline-options.
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @let_rbt_compile_handle_the_input
    # ======================================================================= #
    @let_rbt_compile_handle_the_input = false
  end

  # ========================================================================= #
  # === require_class_compile
  # ========================================================================= #
  def require_class_compile
    require 'rbt/requires/require_the_installer_class.rb'
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    ::RBT.commandline_arguments?
  end

  # ========================================================================= #
  # === menu_for_the_commandline_arguments_as_string
  #
  # This menu is specifically used to handle the full commandline argument.
  # ========================================================================= #
  def menu_for_the_commandline_arguments_as_string(
      i = commandline_arguments?
    )
    require 'rbt/requires/require_actions.rb'
    @let_rbt_compile_handle_the_input = false
    _ = i.join(' ').strip
    case _ # case tag
    # ======================================================================= #
    # === rbt --download htop
    # ======================================================================= #
    when /^-?-?download (.+)$/
      require_class_installer
      action(:Installer, $1.to_s.dup) { :do_not_run_yet }.download
    # ======================================================================= #
    # === rbt --extract htop
    # ======================================================================= #
    when /^-?-?extract(=| )(.+)$/
      require 'rbt/toplevel_methods/extract.rb'
      require 'rbt/requires/require_the_cookbook_class.rb'
      target_program = $2.to_s.dup
      if File.exist? target_program
        RBT.extract(target_program)
      # ===================================================================== #
      # === Check for a locally existing file next
      # ===================================================================== #
      elsif File.exist? File.basename(target_program)
        RBT.extract(
          File.basename(target_program)
        )
      else
        dataset = action(:SanitizeCookbook, target_program) { :fast }
        target_program = dataset.program_path?
        if File.exist? target_program
          RBT.extract(target_program)
        else
          opne "No file could be found at #{steelblue(target_program)}."
          opne 'Perhaps consider downloading the program if it is'
          opne 'registered in the RBT project.'
        end
      end
    else
      @let_rbt_compile_handle_the_input = true
    end
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === rbt --colour-make-install
      # ===================================================================== #
      when /^-?-?colour(_|-)?make(_|-)?install$/i
        require 'rbt/make/colour_make_install.rb'
        action(:ColourMakeInstall) { :run }
      # ===================================================================== #
      # === rbt programs-dir?
      # ===================================================================== #
      when /^programs(_|-)?dir\??$/i,
           /^programs(_|-)?directory\??$/i
        e steelblue(RBT.programs_directory?)
        exit
      # ===================================================================== #
      # === rbt --source-dir?
      # ===================================================================== #
      when *REGEXES_TO_SHOW_THE_SOURCE_DIRECTORY
        e sdir(source_directory?)
        remove_this_commandline_argument(i, commandline_arguments?)
        exit
      # ===================================================================== #
      # === rbt --colour-make
      #
      # This entry point allows us to use colour-make.
      # ===================================================================== #
      when /^-?-?colour(_|-)?make$/i
        require 'rbt/make/make.rb'
        RBT::Make.new { :run }
      # ===================================================================== #
      # === rbt --version
      #
      # Simply display the current version of RBT. Since as of August 2019,
      # this will also display when we last updated the RBT project.
      # ===================================================================== #
      when *ARRAY_HELP_VERSION_OPTIONS
        require 'rbt/version/version.rb'
        RBT.report_rbt_version(:also_show_when_the_project_was_last_updated)
        exit
      # ===================================================================== #
      # === rbt is_the_gobolinux_configuration_enabled?
      # ===================================================================== #
      when /^-?-?is(_|-)?the(_|-)?gobolinux(_|-)?configuration(_|-)?enabled\??$/i
        e 'Do we use the GoboLinux configuration: '+
          steelblue(
            RBT.configuration?.enable_gobolinux_config.to_s
          )
      # ===================================================================== #
      # === Feedback all available aliases
      #
      # Invocation example:
      #
      #   rbt --available_aliases?
      #
      # ===================================================================== #
      when /^-?-?available(_|-)?aliases\??$/i
        pp RBT.hash_aliases_to_the_available_programs?
        remove_this_commandline_argument(i, commandline_arguments?)
      # ===================================================================== #
      # === rbt --be_verbose
      # ===================================================================== #
      when /^-?-?be(_|-)?verbose$/
        pp RBT.configuration?.be_verbose
        remove_this_commandline_argument(i, commandline_arguments?)
      else
        # =================================================================== #
        # === Handle URLs given to rbt
        #
        # Invocation example:
        #
        #   rbt http://ftp.gnu.org/pub/gnu/kawa/kawa-3.1.tar.gz
        #
        # =================================================================== #
        if i.start_with?('http')
          create_new_cookbook_for(i)
        else
          # ================================================================= #
          # The following instance variable means that we will delegate
          # all of this into the class RBT::Compile.
          # ================================================================= #
          @let_rbt_compile_handle_the_input = true
        end
      end
    end
  end

  # ========================================================================= #
  # === create_new_cookbook_for
  #
  # This method can create a new cookbook file - but only if the program
  # has not yet been registered as part of RBT.
  # ========================================================================= #
  def create_new_cookbook_for(i)
    require 'rbt/cookbooks/create_cookbook_yaml_file/create_cookbook_yaml_file.rb'
    _ = RBT.remove_archive_from_the_end(File.basename(i))
    program_name = ProgramInformation.return_program_name(_)
    if RBT.is_a_registered_program? program_name
      e "The program #{program_name} is registered."
    else # else register it.
      e rev+'The program '+sfancy(program_name)+
        rev+' is not registered.'
      e rev+'Thus, delegating towards class '+
        steelblue('RBT::Cookbooks::CreateCookbookYamlFile')+
        '.'
      mkdir(src_dir?+program_name)
      RBT::Cookbooks::CreateCookbookYamlFile.new(i) { :interactive_mode }
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    # ======================================================================= #
    # We will first pass all commandline arguments into menu() here,
    # before we typically then delegate towards class
    # RBT::Action::Installer.
    # ======================================================================= #
    menu_for_the_commandline_arguments_as_string
    if @let_rbt_compile_handle_the_input
      menu(commandline_arguments?)
    end
    # ======================================================================= #
    # And check it again.
    # ======================================================================= #
    if @let_rbt_compile_handle_the_input
      require_class_compile
      do_compile_these_programs(commandline_arguments?)
    end
  end

  # ========================================================================= #
  # === do_compile_these_programs
  #
  # This method can be used to compile
  # ========================================================================= #
  def do_compile_these_programs(
      i = commandline_arguments?
    )
    require 'rbt/requires/require_the_installer_class.rb'
    i = [i].flatten
    action(:installer, i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Commandline.new(ARGV)
end # rbt $USERS/x/src/yakuake/yakuake-3.0.5.tar.xz