#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::CreateCookbookYamlFile
#
# This class, written from scratch in May 2019, is able to generate a new
# cookbook file (.yml format).
#
# It will make use of a template file and then fill in relevant information
# into that template. The primary reason as to why a template file is used
# is because it allows us to easily re-arrange it at any later moment in
# time - we can just modify the template, without having to modify the
# ruby code really. Even regular users should be able to do so (There was
# an older class which had hardcoded entries in the .rb file directly,
# so this was also a change away from that old approach).
#
# You can pass an URL to this class here and the class will figure out
# the name of the yaml file in question, based on that given URL. The
# URL should, in this case, point towards the remote location of a
# tarball.
#
# Take note that this class also allows for interactive use - see
# "--help" for more information about it. The interactive component
# can be started via "--interactive".
#
# Usage example:
#
#   RBT::Cookbooks::CreateCookbookYamlFile.new(ARGV)
#
# =========================================================================== #
# require 'rbt/cookbooks/create_cookbook_yaml_file/create_cookbook_yaml_file.rb'
# RBT::Cookbooks::CreateCookbookYamlFile.new(ARGV)
# RBT.create_cookbook_yaml_file
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class CreateCookbookYamlFile < RBT::Base # === RBT::Cookbooks::CreateCookbookYamlFile

  # ========================================================================= #
  # === YAML_EXTENSION
  # ========================================================================= #
  YAML_EXTENSION = '.yml'

  # ========================================================================= #
  # === USE_THIS_EDITOR
  # ========================================================================= #
  USE_THIS_EDITOR = RBT.editor?

  # ========================================================================= #
  # === OPEN_IN_EDITOR_AFTER_CREATION
  # ========================================================================= #
  OPEN_IN_EDITOR_AFTER_CREATION = false

  # ========================================================================= #
  # === initialize
  #
  # The first argument to this class can be the full URL to the target
  # program at hand.
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    # ======================================================================= #
    # === Handle blocks given to the class
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :interactive_mode
      # ===================================================================== #
      when :interactive_mode
        append_to_the_commandline_arguments('--interactive'.dup)
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @editor
    # ======================================================================= #
    @editor = USE_THIS_EDITOR
    # ======================================================================= #
    # === @template_file_to_use
    # ======================================================================= #
    @template_file_to_use =
      RBT.file_base_template_for_cookbook_yaml_files
    # ======================================================================= #
    # === @dataset
    # ======================================================================= #    
    @dataset = []
    # ======================================================================= #
    # === @hash_information_about_the_program
    # ======================================================================= #
    @hash_information_about_the_program = {}
    # ======================================================================= #
    # === @open_in_editor_after_creation
    # ======================================================================= #
    @open_in_editor_after_creation = OPEN_IN_EDITOR_AFTER_CREATION
    if is_on_roebe?
      @open_in_editor_after_creation = true
    end
    # ======================================================================= #
    # === @file
    # ======================================================================= #
    @file = nil
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ======================================================================= #
      # === --interactive
      #
      # This entry point enters the interactive component of this class.
      # ======================================================================= #
      when /^-?-?interactive$/i,
           /^-?-?enter(_|-)?interactive(_|-)?part$/
        enter_interactive_part
      # ======================================================================= #
      # === --help
      # ======================================================================= #
      when /^-?-?help$/i,'h','-h'
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === set_description
  # ========================================================================= #
  def set_description(
      i = 'Default description of the program.'
    )
    i = i.to_s.dup
    @hash_information_about_the_program[:description] = i
  end

  # ========================================================================= #
  # === enter_interactive_part
  #
  # This method allows the user to specifically designate some
  # parts of the recipe at hand, interactively.
  #
  # Invocation example:
  #
  #   ccook --interactive
  #
  # ========================================================================= #
  def enter_interactive_part(
      use_this_url = first_argument?
    )
    @interactive_mode = true
    e 'A few questions may be asked next, about the values that should be'
    e 'added to the yaml file at hand.'
    e
    e 'Many of these can be skipped.'
    e
    if use_this_url
    else
      e slateblue('Remote URL?')
      set_url1($stdin.gets.chomp)
    end
    e
    e slateblue('Description of the program at hand? [Or just hit enter ')
    e slateblue('for no description at all]')
    set_description($stdin.gets.chomp)
    e
    e 'Done!'
  end

  # ========================================================================= #
  # === name_of_the_program?
  # ========================================================================= #
  def name_of_the_program?
    File.basename(
      first_argument?
    )
  end; alias program_name? name_of_the_program? # === program_name?
       alias name?         name_of_the_program? # === name?

  # ========================================================================= #
  # === set_program_name
  #
  # This sets the short name of the program in question.
  # ========================================================================= #
  def set_program_name(i = '')
    # ======================================================================= #
    # First, do some sanitization.
    # ======================================================================= #
    if i.include? '.' # Assume that a file extension was given. Chop it away thus.
      i = i.to_s.dup
      i.sub!(/#{File.extname(i)}/,'')
    end
    i.delete!('_') if i.include? '_'
    i.downcase!
    @hash_information_about_the_program[:program_name] = i # This variable can be inferred from the url.
  end; alias set_short_name set_program_name # === set_short_name

  # ========================================================================= #
  # === automatically_determine_the_program_name_based_on_the_url1_entry
  # ========================================================================= #
  def automatically_determine_the_program_name_based_on_the_url1_entry(
      i = @hash_information_about_the_program[:url1]
    )
    program_information = ProgramInformation.new(File.basename(i))
    @hash_information_about_the_program[:program_name] = program_information.program_name?
  end

  # ========================================================================= #
  # === short_program_name?
  # ========================================================================= #
  def short_program_name?
    _ = program_name?
    if @hash_information_about_the_program.has_key? :program_name
      _ = @hash_information_about_the_program[:program_name]
    end
    _
  end

  # ========================================================================= #
  # === sanitize_dataset                                       (sanitize tag)
  #
  # This method will, in particular, replace some of the generic MACROS
  # in the file with corresponding "sane" values.
  # ========================================================================= #
  def sanitize_dataset
    @dataset.map! {|line|
      case line
      # ===================================================================== #
      # === description:
      # ===================================================================== #
      when /description: /
        if @hash_information_about_the_program.has_key? :description
          line = ' description: |'+N+
                 '  '+description?
        end
      # ===================================================================== #
      # === NAME_OF_THE_PROGRAM
      # ===================================================================== #
      when /NAME_OF_THE_PROGRAM/
        line = program_name?.dup
        if @hash_information_about_the_program.has_key? :url1
          unless @hash_information_about_the_program.has_key? :program_name
            automatically_determine_the_program_name_based_on_the_url1_entry
          end
          line = @hash_information_about_the_program[:program_name].dup
        end
        line << ':' unless line.end_with? ':'
      # ===================================================================== #
      # === homepage:
      # ===================================================================== #
      when / homepage:/
        line = ' homepage: '+homepage?
      # ===================================================================== #
      # === required_deps_on:
      # ===================================================================== #
      when / required_deps_on:/
        line = " required_deps_on:#{add_hint_for_dependencies}"
      # ===================================================================== #
      # === last_update
      # ===================================================================== #
      when /last_update: DD MM YYYY/
        line = " last_update: #{dd_mm_yyyy}"
      # ===================================================================== #
      # === url1
      # ===================================================================== #
      when /url1:/
        if @hash_information_about_the_program.has_key? :url1
          line = ' url1: '+@hash_information_about_the_program[:url1]
        end
      end
      line
    }
  end; alias update sanitize_dataset # === update

  # ========================================================================= #
  # === store_dataset_into_file                                  (store tag)
  #
  # This method will store the dataset describing out .yml file into a
  # (local) .yml file finally.
  # ========================================================================= #
  def store_dataset_into_file
    what = @dataset.join(N)
    into = short_program_name?+'.yml'
    if File.exist? into
      if File.directory? into
        opne sdir(into)+swarn(' seems to be a directory. Will not overwrite it.')
      else
        opnwarn 'But there already is a file at '+sfile(into)+swarn('.')
        opnwarn 'Shall we really overwrite this file?'+
                 steelblue(' [y/n]')
        case $stdin.gets.chomp
        when 'y','yes','j','ja'
          opnwarn 'Overwriting `'+sfile(into)+swarn('` now.')
          write_what_into(what, into)
        else
          opnwarn 'Ok - not overwriting.'
        end
      end
    else
      opne "Saving into the file `#{sfile(into)}#{rev}` next."
      write_what_into(what, into)
    end
  end; alias store                                 store_dataset_into_file # === store
       alias save                                  store_dataset_into_file # === save
       alias consider_creating_a_new_cookbook_file store_dataset_into_file # === consider_creating_a_new_cookbook_file

  # ========================================================================= #
  # === verbose_make_use_of_the_template_file_and_then_sanitize_the_dataset
  # ========================================================================= #
  def verbose_make_use_of_the_template_file_and_then_sanitize_the_dataset
    _ = @template_file_to_use
    if File.exist? _
      opne "Making use of a template file (#{sfile(File.basename(_))}#{rev})."
      @dataset = File.readlines(_).map(&:chomp)
      sanitize_dataset
    end
  end

  # ========================================================================= #
  # === set_url1
  # ========================================================================= #
  def set_url1(
      i = ''
    )
    @hash_information_about_the_program[:url1] = i
    unless i.empty?
      i = return_program_information(
            remove_archive_stuff_at_the_end(
              File.basename(i)
            )
          )
      set_filename(i)
      set_short_name(i)
    end
  end; alias set_url set_url1 # === set_url

  # ========================================================================= #
  # === do_not_open_in_editor
  # ========================================================================= #
  def do_not_open_in_editor
    @open_in_editor_after_creation = false
  end

  # ========================================================================= #
  # === do_open_in_editor
  # ========================================================================= #
  def do_open_in_editor
    @open_in_editor_after_creation = false
  end

  # ========================================================================= #
  # === description?
  # ========================================================================= #
  def description?
    @hash_information_about_the_program[:description]
  end

  # ========================================================================= #
  # === file?
  # ========================================================================= #
  def file?
    @file
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e
    e 'How to create a new cookbook:' # newcookbook --help
    e
    e 'Simply provide the name of the .yml file, such as:'
    e
    e '   newcookbook php.yml'
    e
    e 'Alternatively, provide a remote URL which we will use to '\
      'generate such a file.'
    e
    e '  --interactive # this will ask you several questions before'
    e '                # the new .yml file will be created'
    e
  end

  # ========================================================================= #
  # === add_hint_for_dependencies
  # ========================================================================= #
  def add_hint_for_dependencies
    if is_on_roebe?
      ''
    else # Add this line to help newbies.
      "# The dependencies go in here.#{N}"
    end
  end

  # ========================================================================= #
  # === set_filename
  #
  # This method will store the filename, i.e. where to store the file.
  # ========================================================================= #
  def set_filename(i)
    i = i.to_s.dup
    i = return_program_information(i)
    if i
      i << YAML_EXTENSION unless i.end_with? '.yml'
      # ===================================================================== #
      # Since as of March 2014 we will also downcase the name.
      # ===================================================================== #
      i.downcase!
    end
    set_file(i)
  end; alias set_real_name set_filename # === set_real_name

  # ========================================================================= #
  # === set_file
  #
  # This method is not allowed to do any modifications - it will simply
  # assign towards the @file variable.
  # ========================================================================= #
  def set_file(i)
    @file = i
  end

  # ========================================================================= #
  # === open_in_editor?
  # ========================================================================= #
  def open_in_editor?
    @open_in_editor_after_creation
  end

  # ========================================================================= #
  # === homepage?
  # ========================================================================= #
  def homepage?
    require 'uri'
    _ = ''.dup
    if @hash_information_about_the_program.has_key? :url1
      use_this_url = @hash_information_about_the_program[:url1]
      if is_on_roebe?
        opne 'Using information from this url: '
        e
        e "  #{steelblue(use_this_url)}"
        e
      end
      uri = URI(use_this_url)
      if uri
        _ = "#{uri.scheme}://#{uri.host}"
      end
    end
    _
  end

  # ========================================================================= #
  # === sanitize_first_argument
  # ========================================================================= #
  def sanitize_first_argument
    _ = first_argument?
    if _.include? '-' and (_ =~ /\d+/)
      # ===================================================================== #
      # In this case assume a remote URL:
      # ===================================================================== #
      program_information = ProgramInformation.new(File.basename(_))
      @hash_information_about_the_program[:program_name]    = program_information.program_name?
      @hash_information_about_the_program[:program_version] = program_information.program_version?
      set_url1 _
    end
  end

  # ========================================================================= #
  # === consider_opening_in_editor
  #
  # We may use the newly created file in our main editor.
  # ========================================================================= #
  def consider_opening_in_editor(
      this_file = file?
    )
    if this_file.nil? and !commandline_arguments?.empty?
      this_file = commandline_arguments?.first
      unless File.exist? this_file
        this_file = "#{this_file}.yml"
      end
    end
    if open_in_editor?
      _ = "#{@editor} #{this_file}"
      opne 'We will next open this file in our editor, as instructed due'
      opne 'to the ivar '+sfancy('@open_in_editor_after_creation')
      opne "#{rev}having been set to true."
      system(_)
    end
  end; alias edit_this_file consider_opening_in_editor # === edit_this_file

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    sanitize_first_argument
    menu # Should come before 
    verbose_make_use_of_the_template_file_and_then_sanitize_the_dataset
    if @dataset.empty?
      opne 'Empty dataset - can not store anything.'
    else
      store_dataset_into_file
      consider_opening_in_editor
    end
  end

  # ========================================================================= #
  # === RBT::Cookbooks::CreateCookbookYamlFile[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.create_cookbook_yaml_file
# =========================================================================== #
def self.create_cookbook_yaml_file(i = 'php')
  RBT::Cookbooks::CreateCookbookYamlFile.new(i)
end; self.instance_eval { alias create_new_cookbook create_cookbook_yaml_file } # === RBT.create_new_cookbook

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::CreateCookbookYamlFile.new(ARGV)
end # ccook php
    # ccook https://download.gimp.org/pub/gegl/0.4/gegl-0.4.16.tar.bz2
    # ccook https://ftp.gnome.org/pub/GNOME/sources/gnome-online-miners/3.9/gnome-online-miners-3.9.90.tar.xz