#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::DownloadAllSourceArchives
#
# This class can be used to download all source archives into the
# corresponding "archive directory".
#
# On my system the archive directory defaults to the path /home/x/src/
# since as of April 2021, but other users of the RBT project may
# wish to designate another "archive directory" to be used, from
# the commandline.
#
# Into this so called "archive directory", source archives will be
# downloaded, but only if they do not yet exist in that directory.
#
# This works in batch mode, that is - everything will be downloaded
# one after the other, in series. It thus constitutes a "bootstrap"
# variant - people can get up-and-running by downloading all 
# remote source archives.
#
# Note that this class here also allows you to restrict the candidates
# that will be downloaded. Presently, the only way that this
# is via the registered "tags".
#
# So, for example - if you pass in the argument "--tags=plasma",
# then this class here will only download entries that have that
# tag "plasma" registered.
#
# That way you can selectively download only a specific subset of
# the cookbook dataset at hand.
#
# Usage example:
#
#   RBT::Cookbooks::DownloadAllSourceArchives.new(ARGV)
#
# =========================================================================== #
# require 'rbt/cookbooks/download_all_source_archives/download_all_source_archives.rb'
# RBT::Cookbooks::DownloadAllSourceArchives.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class DownloadAllSourceArchives < Base # === RBT::Cookbooks::DownloadAllSourceArchives

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @tag
    # ======================================================================= #
    @tag = nil
    # ======================================================================= #
    # === @archive_directory
    # ======================================================================= #
    @archive_directory = source_directory?
    # ======================================================================= #
    # === @available_programs
    # ======================================================================= #
    @available_programs = available_programs? # <- Must come before the menu() call.
  end

  # ========================================================================= #
  # === set_use_these_cookbooks
  #
  # The input to this method should be an Array.
  # ========================================================================= #
  def set_use_these_cookbooks(i)
    if i.is_a? Array
      i.flatten!
      i.sort!
    end
    @available_programs = i
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @available_programs
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === download_all_source_archives --tags=plasma
      #
      # This is the entry point to select a particular tag.
      #
      # Example:
      #
      #   download_all_source_archives --tags=ruby
      #
      # ===================================================================== #
      when /^-?-?tags=(.+)$/,
           /^-?-?tag=(.+)$/
        register_this_tag($1.to_s.dup)
        set_use_these_cookbooks(select_only_these_tags(@tag))
      # ===================================================================== #
      # === download_all_source_archives --help
      # ===================================================================== #
      when /help/
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === register_this_tag
  # ========================================================================= #
  def register_this_tag(i)
    @tag = i
  end

  # ========================================================================= #
  # === report_the_archive_directory_in_use
  # ========================================================================= #
  def report_the_archive_directory_in_use
    if @tag
      opne "#{rev}All registered programs with the tag #{sfancy(@tag)} "\
           "#{rev}will be downloaded into the directory:"
    else
      opne 'All registered programs will be downloaded into the directory:'
    end
    e
    e sfile("  #{@archive_directory}")
    e
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    opne 'This class allows you to download source archives.'
    opne 'You can also selectively download only certain entries, in'
    opne 'particular those who have registered tags.'
    e
    opne 'For example, if you wish to download all KDE-plasma related entries'
    opne 'then you can issue the following command:'
    e
    opnfancy '  download_all_source_archives --tags=plasma'
    e
  end

  # ========================================================================= #
  # === consider_downloading_this_program
  # ========================================================================= #
  def consider_downloading_this_program(i)
    cd_to_the_archive_directory # Always enter the archive directory. 
    # ======================================================================= #
    # Next, check whether the program exists.
    # ======================================================================= #
    _ = action(:SanitizeCookbook, i) { :fast }
    program_path = _.program_path?
    remote_url   = _.remote_url?
    unless File.exist? program_path
      this_directory = "#{@archive_directory}#{i}/"
      unless File.directory? this_directory
        opne "Creating the directory `#{sdir(this_directory)}` next."
        mkdir this_directory
      end
      cd this_directory
      unless File.exist? program_path
        download(remote_url)
        opne "#{rev}Next downloading `#{simp(remote_url)}#{rev}`."
      end
    end
  end

  # ========================================================================= #
  # === download
  #
  # The download-functionality requires wget right now.
  # ========================================================================= #
  def download(i)
    esystem "wget #{i}"
  end

  # ========================================================================= #
  # === cd_to_the_archive_directory
  # ========================================================================= #
  def cd_to_the_archive_directory
    cd @archive_directory, :be_quiet
  end

  # ========================================================================= #
  # === select_only_these_tags
  #
  # To invoke this method, try:
  #
  #   download_all_source_archives --tags=plasma
  #
  # ========================================================================= #
  def select_only_these_tags(i)
    # ======================================================================= #
    # Ok, the user provided a tag, so we have to selectively filter for
    # programs that include only that tag. For now, we only use the
    # expanded cookbook dataset for this. Of course we must check whether
    # this directory exists or whether it does not.
    # ======================================================================= #
    if File.directory? directory_expanded_cookbooks?
      entries = Dir[directory_expanded_cookbooks?+'*.yml']
      results = entries.map {|entry|
        YAML.load_file(entry)
      }.select {|entry| entry['tags'].include? i }
      results.map! {|entry| entry['real_short_name'] }
      return results
    else
      opne "No directory called #{sdir(directory_expanded_cookbooks?)} "\
           "#{rev}exists."
      opne 'This class currently requires this directory to work.'
      opne 'If you wish to auto-generate the directory and the files or that'
      opne 'directory, then issue this command:'
      e
      efancy '  cookbooks --expand'
      e
      exit
    end
  end

  # ========================================================================= #
  # === process_the_programs_one_after_the_other
  # ========================================================================= #
  def process_the_programs_one_after_the_other(
      i = @available_programs
    )
    i.each {|this_program|
      consider_downloading_this_program(this_program)
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    report_the_archive_directory_in_use
    process_the_programs_one_after_the_other
  end

  # ========================================================================= #
  # === RBT::Cookbooks::DownloadAllSourceArchives[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::DownloadAllSourceArchives.new(ARGV)
end # downloadallsourcearchives