#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::FeedbackLicenses
#
# This will feedback all licenses. Optionally you can search for only
# a specific version.
# =========================================================================== #
# require 'rbt/cookbooks/utility_scripts/feedback_licenses.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class FeedbackLicenses < Base # === RBT::Cookbooks::FeedbackLicenses

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ======================================================================= #
  # === PADDING_TO_USE
  # ======================================================================= #
  PADDING_TO_USE = '25'

  # ======================================================================= #
  # === TOKEN_LEFT_THEN_PADDING
  # ======================================================================= #
  TOKEN_LEFT_THEN_PADDING = '| ' # The leading padding to use.

  # ======================================================================= #
  # === DEFAULT_FILTER_TO_USE
  # ======================================================================= #
  DEFAULT_FILTER_TO_USE = '*' # By default, we show ALL known licenses.

  # ======================================================================= #
  # === initialize
  #
  # Usage example:
  #
  #   FeedbackLicenses.new(filter_for_licenses = DEFAULT_FILTER_TO_USE, run_already = true)
  #
  # If the filter is * then we will filter for ALL licenses.
  # ======================================================================= #
  def initialize(
      filter_for_licenses = DEFAULT_FILTER_TO_USE,
      run_already         = true
    )
    reset
    set_filter_for_licenses(
      filter_for_licenses
    )
    run if run_already
  end

  # ======================================================================= #
  # === reset
  # ======================================================================= #
  def reset
    super()
    infer_the_namespace
    # ===================================================================== #
    # === @cookbook
    # ===================================================================== #
    @cookbook = action(:SanitizeCookbook) { :do_not_run_yet }
    # ===================================================================== #
    # === @filter
    # ===================================================================== #
    @filter   = ''.dup # Set the filter to use.
  end

  # ======================================================================= #
  # === set_filter_for_licenses
  # ======================================================================= #
  def set_filter_for_licenses(i = DEFAULT_FILTER_TO_USE)
    i = i.first if i.is_a? Array
    i = DEFAULT_FILTER_TO_USE if i.nil?
    i = DEFAULT_FILTER_TO_USE if i.empty?
    i = i.to_s.dup
    i = i.downcase
    @filter = i
  end; alias filter_for set_filter_for_licenses # === filter_for

  # ======================================================================= #
  # === set_which_program
  # ======================================================================= #
  def set_which_program(i) # Set which program we are seeking.
    @which_program = i
  end

  # ======================================================================= #
  # === output_header
  # ======================================================================= #
  def output_header
    # ===================================================================== #
    # This is the header content.
    # ===================================================================== #
    e TOKEN_LEFT_THEN_PADDING+('%-25s' % 'Name  ')+
      '  | License                                         |'
  end

  # ======================================================================= #
  # === filter?
  # ======================================================================= #
  def filter?
    @filter
  end

  # ======================================================================= #
  # === feedback_licenses
  # ======================================================================= #
  def feedback_licenses
    cliner {
      _ = 'known licenses' # Placeholder for now.
      _ = filter? unless filter? == DEFAULT_FILTER_TO_USE
      opne 'Feedback licenses matching search-criteria for'
      opne '`'+sfancy(_.to_s)+rev+'` in alphabetical order next:'
    }
    output_header
    cliner {
      programs = available_programs?
      programs.each { |program|
        @cookbook = action(:SanitizeCookbook, program) { :do_not_run_yet }
        @cookbook.metafind(program)
        license = return_license
        if license.is_a? Array
          license = license.flatten.compact.join(', ') # We want to see it on the same line here.
        end
        next if license == 'unknown'
        program = File.basename(program).delete_suffix('.yml')
        padding_left = '%-25s' % program
        padding_left = steelblue(padding_left).dup
        padding_left.prepend(TOKEN_LEFT_THEN_PADDING)
        padding_right = '%-10s' % license
        full_license_text = "#{padding_left}"\
                            "#{rev}  | #{royalblue(padding_right)}#{rev}"
        if @filter == DEFAULT_FILTER_TO_USE
          e full_license_text
        else # else we assume user wants filtering done
          unless license.to_s.downcase.scan(/#{filter?}/).empty?
            e full_license_text 
          end
        end
      }
    }
  end

  # ======================================================================= #
  # === return_license
  # ======================================================================= #
  def return_license
    @cookbook.license?.to_s
  end

  # ======================================================================= #
  # === run                                                       (run tag)
  # ======================================================================= #
  def run
    feedback_licenses
  end

  # ======================================================================= #
  # === RBT::Cookbooks::FeedbackLicenses[]
  # ======================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.feedback_licenses
# =========================================================================== #
def self.feedback_licenses(i = ARGV)
  RBT::Cookbooks::FeedbackLicenses.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::FeedbackLicenses.new(ARGV)
end # flic gpl # Feedback all GPL-licenses.