#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT:::Cookbooks::FindHeaders
#
# Find all headers of a given single program.
#
# Pass the name of the program as argument to this class.
#
# Usage examples:
#
#   RBT::Cookbooks::FindHeaders.new :ruby
#   RBT::Cookbooks::FindHeaders.new 'php'
#
# =========================================================================== #
# require 'rbt/cookbooks/find_headers.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class FindHeaders < Base # === RBT::Cookbooks::FindHeaders

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup if i
    @input = i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === obtain_headers
  # ========================================================================= #
  def obtain_headers(i = input?)
    headers = action(:SanitizeCookbook, i) { :fast }.headers?
    return headers
  end

  # ========================================================================= #
  # === report_that_we_have_found_these_headers
  #
  # We do assume here that the input argument to this method
  # is a String which will contain ',' characters.
  # ========================================================================= #
  def report_that_we_have_found_these_headers(headers)
    if headers.include? ','
      splitted = headers.split(',')
    else
      splitted = (headers+',').split(',')
    end
    opne "The following #{slateblue(splitted.size.to_s)}#{rev} header files "\
         "were found for the program `#{sfancy(input?)}#{rev}`."
    splitted.each {|the_header|
      e "  - #{the_header}"
    }
  end

  # ========================================================================= #
  # === do_find_headers
  # ========================================================================= #
  def do_find_headers
    headers = obtain_headers
    if headers.is_a? Array # Convert the Array into a String here.
      headers = headers.join(',').strip
    end
    headers.delete!(' ') if headers
    if headers.to_s.empty?
      opne "#{rev}No headers were found for #{steelblue(@input)}#{rev}."
    else
      report_that_we_have_found_these_headers(headers)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_find_headers
  end

end; end

# =========================================================================== #
# === RBT.find_headers
#
# This method exists mostly because it is more convenient to type it.
#
# Invocation example:
#
#   RBT.find_headers('libxcb')
#
# =========================================================================== #
def self.find_headers(i = ARGV)
  RBT::Cookbooks::FindHeaders.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::FindHeaders.new(ARGV)
end # headers? libxcb