#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::Homepage
#
# This class can show the homepage of a given program.
#
# It can also report all URLs that do not have a homepage
# entry, such as via:
#
#   homepage --missing-url?
#
# Usage example:
#
#   RBT::Cookbooks::Homepage.new(ARGV)
#
# =========================================================================== #
# require 'rbt/cookbooks/homepage/homepage.rb'
# RBT.return_homepage_of?
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class Homepage < Base # === RBT::Cookbooks::Homepage

  # ========================================================================= #
  # === MODIFY_THE_XORG_BUFFER_IF_A_HOMEPAGE_ENTRY_EXISTS
  #
  # If the following constant is set to true then we will also try
  # to modify the xorg buffer.
  # ========================================================================= #
  MODIFY_THE_XORG_BUFFER_IF_A_HOMEPAGE_ENTRY_EXISTS = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      this_program              = ARGV.first,
      optional_use_this_dataset = nil,
      run_already               = true
    )
    reset
    set_this_program(
      this_program
    )
    determine_the_dataset_to_use(
      optional_use_this_dataset
    )
    # ======================================================================= #
    # === Handle blocks given to this class
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :be_quiet
      # ===================================================================== #
      when :be_quiet
        be_quiet
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @modify_the_xorg_buffer_if_a_homepage_entry_exists
    # ======================================================================= #
    @modify_the_xorg_buffer_if_a_homepage_entry_exists =
      MODIFY_THE_XORG_BUFFER_IF_A_HOMEPAGE_ENTRY_EXISTS
  end

  # ========================================================================= #
  # === modify_the_xorg_buffer_if_a_homepage_entry_exists
  # ========================================================================= #
  def modify_the_xorg_buffer_if_a_homepage_entry_exists?
    @modify_the_xorg_buffer_if_a_homepage_entry_exists
  end; alias modify_the_xorg_buffer? modify_the_xorg_buffer_if_a_homepage_entry_exists? # === modify_the_xorg_buffer?

  # ========================================================================= #
  # === make_sure_that_the_cookbook_class_is_available
  # ========================================================================= #
  def make_sure_that_the_cookbook_class_is_available
    require 'rbt/requires/require_the_cookbook_class.rb'
  end

  # ========================================================================= #
  # === set_this_program
  # ========================================================================= #
  def set_this_program(i = '')
    i = i.first if i.is_a? Array
    i = i.to_s.dup
    menu(i)
    @this_program = i
  end

  # ========================================================================= #
  # === which_program?
  # ========================================================================= #
  def which_program?
    @this_program
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === homepage --help
      # ===================================================================== #
      when /^-?-?help$/
        show_help
        exit
      # ===================================================================== #
      # === homepage --missing-url?
      # ===================================================================== #
      when /^-?-?missing(-|_)?url\??$/,
           /^-?-?missing$/
        show_all_programs_that_miss_a_homepage_entry
        exit
      end
    end
  end

  # ========================================================================= #
  # === show_all_programs_that_miss_a_homepage_entry
  #
  # This method will show all programs that miss a homepage entry.
  #
  # Invocation example:
  #
  #   homepage --missing-url?
  #
  # ========================================================================= #
  def show_all_programs_that_miss_a_homepage_entry(
      available_programs = available_programs?
    )
    make_sure_that_the_cookbook_class_is_available
    cliner
    opne "Now working through #{steelblue(available_programs.size)} "\
         "different programs."
    opne "This may take a while, so please be patient."
    cliner
    available_programs.each {|this_program|
      dataset = action(:SanitizeCookbook, this_program) { :be_fast }
      homepage_entry = dataset.homepage?.to_s
      if homepage_entry.empty?
        _ = ("`#{this_program}`").ljust(15)
        opne "#{rev}The program #{sfancy(_)} #{rev}has no "\
             "#{sfancy('homepage: ')} #{rev}entry."
      end
    }
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  #
  # Invoke this entry point via:
  #
  #   homepage --help
  #
  # ========================================================================= #
  def show_help
    opne 'Currently these options are available for `'+
          sfancy("class #{namespace?}")+'`:'
    e
    e "  --missing-url?      # show which cookbook entries lack "\
      "a proper #{sfancy('homepage: entry')}"
    e
  end

  # ========================================================================= #
  # === this_program?
  # ========================================================================= #
  def this_program?
    @this_program
  end

  # ========================================================================= #
  # === dataset?
  # ========================================================================= #
  def dataset?
    @dataset
  end

  # ========================================================================= #
  # === return_homepage
  # ========================================================================= #
  def return_homepage
    dataset?.homepage?
  end; alias homepage? return_homepage # === homepage?
       alias result?   return_homepage # === result?

  # ========================================================================= #
  # === show_homepage_of_this_program
  # ========================================================================= #
  def show_homepage_of_this_program(
      this_program = which_program?
    )
    if this_program.include? '.yml'
      this_program.delete_suffix!('.yml')
    end
    _ = return_homepage
    if _
      this_program = File.basename(this_program).sub(/\.yml$/,'')
      if _.empty?
        if be_verbose?
          opne "#{rev}No homepage has been registered for the program "\
               "`#{sfancy(this_program)}#{rev}`."
        end 
      else
        # =================================================================== #
        # Else we have found a homepage entry and will display
        # our findings to the end-user.
        #
        # To test this subsection, try:
        #
        #   homepage geany
        #
        # =================================================================== #
        if be_verbose?
          opne "#{rev}The homepage of the program `#{sfancy(this_program)}"\
               "#{rev}` is at:"
          e
          e "  #{orange(_)}"
          e
          if modify_the_xorg_buffer?
            set_xorg_buffer(_)
          end
        end
      end
    else
      if be_verbose?
        opne "No homepage entry is registered in `"+
             "#{sfancy(this_program)}#{rev}` "\
             "(The .yml file should be at "
        opne simp("  #{individual_cookbooks_dir?}#{this_program}.yml")+
              ')'
      end
    end
  end; alias report_the_homepage show_homepage_of_this_program # === report_the_homepage

  # ========================================================================= #
  # === consider_opening_the_url_to_the_homepage_in_the_browser
  # ========================================================================= #
  def consider_opening_the_url_to_the_homepage_in_the_browser
    if is_on_roebe? and be_verbose?
      open_in_browser(return_homepage)
    end
  end

  # ========================================================================= #
  # === determine
  # ========================================================================= #
  def determine_the_dataset_to_use(i = nil)
    if i.nil?
      @dataset = action(:SanitizeCookbook, which_program?) {
        :fast_and_be_verbose
      }
    else
      @dataset = i
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    report_the_homepage
    consider_opening_the_url_to_the_homepage_in_the_browser
  end

  # ========================================================================= #
  # === RBT::Cookbooks::Homepage[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

# =========================================================================== #
# === RBT.return_homepage_of?
#
# This variant will silently return the homepage of a registered program,
# as a String.
#
# If no homepage is registered in the .yml file then an empty String will
# be returned instead.
#
# Usage example:
#
#   x = RBT.return_homepage_of? :htop # => "https://hisham.hm/htop/"
#
# =========================================================================== #
def self.return_homepage_of?(i = ARGV)
  _ = RBT::Cookbooks::Homepage.new(i) { :be_quiet }
  return _.homepage?
end; self.instance_eval { alias homepage_of return_homepage_of? } # === RBT.homepage_of

# =========================================================================== #
# === RBT.homepage
# =========================================================================== #
def self.homepage(i = ARGV)
  RBT::Cookbooks::Homepage.new(i)
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::Homepage.new(ARGV)
end # homepage bluefish
    # homepage libx11