#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::MultiUrlDisplayer
#
# This class can show all URLs belonging to a particular component.
#
# Usage example:
#
#   RBT::Cookbooks::MultiUrlDisplayer.new(:audio_suite)
#
# =========================================================================== #
# require 'rbt/cookbooks/multi_url_displayer.rb'
# RBT::Cookbooks::MultiUrlDisplayer.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class MultiUrlDisplayer < Base

  require 'rbt/toplevel_methods/url.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_arguments = ARGV,
      run_already        = true
    )
    reset
    set_commandline_arguments(optional_arguments)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @show_only_the_URLs
    # ======================================================================= #
    @show_only_the_URLs = false
  end

  # ========================================================================= #
  # === try_to_report_the_URLs_of_this_component
  #
  # This is the main method that will report the actual URL of a
  # particular component.
  # ========================================================================= #
  def try_to_report_the_URLs_of_this_component(this_component)
    this_component = this_component.to_s
    if @dataset.has_key? this_component
      array = @dataset[this_component]
      # ===================================================================== #
      # First notify the user as to how many entries were found.
      # ===================================================================== #
      e
      opne "#{tomato(array.size)} entries were found."
      e
      array.each {|this_entry|
        remote_URL = RBT.remote_url_for?(this_entry.to_sym)
        if remote_URL.is_a? Array
          remote_URL = remote_URL.first
        end
        if @show_only_the_URLs
          left_part = ''
        else # this here is the default entry point.
          left_part = (
            steelblue("  #{this_entry}")+
            lightgreen(': ')
          ).ljust(82)
        end
        e left_part+
          royalblue(remote_URL)
      }
      e
    else
      unless this_component.start_with? '--' # Exclude commandline argument flags here.
        opne 'No key called '+steelblue(this_component)+' was found.'
      end
    end
  end

  # ========================================================================= #
  # === sanitize_this_component
  #
  # This can modify some abbreviations, on an ad-hoc basis. May have
  # to be moved to another file one day - let's see (written this in
  # May 2020).
  # ========================================================================= #
  def sanitize_this_component(i)
    case i.to_s
    # ======================================================================= #
    # === plasma5
    # ======================================================================= #
    when /^plasma5$/
      i = 'kde5_plasma'
    end
    return i # Don't forget to return our finding here.
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(
      i = return_commandline_arguments_with_hyphens
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === multiurldisplayer kde5_plasma --show-only-URLs
      # ===================================================================== #
      when /^-?-?show(-|_)?only(-|_)?URLs$/i, # murl --show-only-urls
           /^-?-?only(-|_)?URLs$/i,
           /^-?-?raw$/i
        @show_only_the_URLs = true
      end
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    menu
    _ = RBT.file_cookbook_chained_programs
    if File.exist? _
      work_on_these_programs = commandline_arguments?
      opne "#{rev}Loading the dataset from the file #{sfile(_)} #{rev}next."
      @dataset = YAML.load_file(_)
      if work_on_these_programs.empty?
        opne 'Please supply the names of programs to this class.'
      else
        work_on_these_programs.each {|this_component|
          this_component = sanitize_this_component(this_component)
          try_to_report_the_URLs_of_this_component(this_component)
        }
      end
    else
      no_file_exists_at(_)
    end
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::MultiUrlDisplayer.new(ARGV)
end # multiurldisplayer audio_suite
    # multiurldisplayer kde5_plasma