#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::RequiredDependenciesOn
#
# This class originated, as conceptual idea, from an older class called
# DependencyCollector. We wish to be able to determine the dependencies
# for any given program.
#
# Usage example:
#
#   RBT::RequiredDependenciesOn.new(ARGV)
#
# =========================================================================== #
# require 'rbt/cookbooks/required_dependencies_on/required_dependencies_on.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class RequiredDependenciesOn < Base # === RBT::RequiredDependenciesOn

  require 'rbt/actions/individual_actions/cookbooks/raw_cookbook/raw_cookbook.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    set_be_verbose
    infer_the_namespace
  end

  alias program_name? first_argument? # === program_name?

  # ========================================================================= #
  # === return_dependencies_of_this_program
  #
  # The argument to this method should be the name of the program that
  # we are trying to find.
  # ========================================================================= #
  def return_dependencies_of_this_program(
      i = program_name?
    )
    # ======================================================================= #
    # The input may include strings such as '>=', such as in 'mpc >= 0.8.1'.
    # In this case we will split the input.
    # ======================================================================= #
    if i.include? '='
      i = i.split('=').first.strip
    end
    i.delete!('><')
    i.strip!
    action(:raw_cookbook, i).required_dependencies_on?
  end

  # ========================================================================= #
  # === dependencies?
  # ========================================================================= #
  def dependencies?
    @cookbook.required_dependencies_on?
  end

  # ========================================================================= #
  # === report                                                  (report tag)
  # ========================================================================= #
  def report(
      the_dependencies = dependencies?,
      be_verbose       = be_verbose?
    )
    program_name = program_name?
    if be_verbose
      opne "The following dependencies are known (and "\
           "have been registered) for #{royalblue(program_name)}:"
      e
      cliner
      e royalblue(program_name)
    end
    purple_arrow = mediumpurple('→')
    the_dependencies.each {|this_dependency|
      e "  #{purple_arrow} #{slateblue(this_dependency)}"
      dependencies_for_that_program = return_dependencies_of_this_program(this_dependency)
      if dependencies_for_that_program
        e "    #{slateblue(this_dependency)} #{rev}in turn depends on:"
        dependencies_for_that_program.each {|entry|
          e "      #{tomato(entry.to_s)}#{rev}"
        }
      end
    }
    if be_verbose
      e
      cliner
      e
    end
  end; alias report_dependencies report # === report_dependencies

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    program_name = program_name?
    if RBT.has_this_program_registered?(program_name)
      report(RBT.raw_cookbook(program_name).required_dependencies_on?)
    else
      opne "#{rev}The program #{sfancy(program_name)} #{rev}is "\
           "not registered as part of the RBT project."
    end
  end

  # ========================================================================= #
  # === RBT::RequiredDependenciesOn[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::RequiredDependenciesOn.new(ARGV)
end # requireddependencieson gcc