#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::ScanSourceArchive
#
# This class will scan through our archives and report if we have
# packages which are not included.
# =========================================================================== #
# require 'rbt/cookbooks/scan_source_archive/scan_source_archive.rb'
# RBT.scan_source_archive
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class ScanSourceArchive < Base # === RBT::Cookbooks::ScanSourceArchive

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === ARRAY_SKIP_THESE_DIRECTORIES
  #
  # Next comes a listing of Sub-Directories to skip (forbidden dirs).
  # Careful with what you enter here.
  # ========================================================================= #
  ARRAY_SKIP_THESE_DIRECTORIES = %w(
    /windows/
    java
    rpg
    firefox
    extensions
    opera
    firmware
    ruby
    patches
    gobolinux
    fonts
    hardware
    rarlinux
    libreoffice
    nvidia
    jdk
    bios
    flash
    jquery
    ntfs3g
    pdf
    polkitqt
    wlan
    youtubedl
  )

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    register_sigint
    super()
    reset
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @cookbook_dataset
    # ======================================================================= #
    @cookbook_dataset = nil
    # ======================================================================= #
    # === @array_registered_files
    # ======================================================================= #
    @array_registered_files = [] # Keep them registered.
  end

  # ========================================================================= #
  # === set_target_directory
  # ========================================================================= #
  def set_target_directory(i)
    @target_directory = i
  end

  # ========================================================================= #
  # === target_directory?
  # ========================================================================= #
  def target_directory?
    @target_directory
  end; alias target_directory target_directory? # === target_directory

  # ========================================================================= #
  # === register_file
  #
  # Simply append to @array_registered_file.
  #
  # The second argument shall specify the target location. 
  # ========================================================================= #
  def register_file(f, real_location)
    @array_registered_files << [
      f, real_location
    ] #unless @array_registered_files
  end

  # ========================================================================= #
  # === scan
  #
  # This is the actual scanning action.
  # ========================================================================= #
  def scan(
      target = source_directory?
    )
    set_target_directory(target)
    opne "Scanning through `#{sdir(target_directory?)}"\
         "` and all of its subdirectories next."
    opne "This may take a while - please be patient.#{N}"
    all_files = Dir["#{target_directory?}**/*"].sort
    # ======================================================================= #
    # Filter away directories.
    # ======================================================================= #
    all_files.reject! {|entry| File.directory? entry }
    # ======================================================================= #
    # Report to the user how many entries were foudn.
    # ======================================================================= #
    opne "#{royalblue(all_files.size.to_s)} entries (== files) were found."
    all_files.each { |file|
      original_file = file.to_s.dup # Keep a reference here.
      next if !file.include? '-'
      next if forbidden?(file)
      file = remove_archive_at_the_end(
        File.basename(file).downcase
      )
      # ===================================================================== #
      # Obtain only the assumed "program name".
      # ===================================================================== #
      name_of_the_program = ProgramInformation::ProgramInformation.new(file).return_name
      name_of_the_program = name_of_the_program.to_s.
                            tr('-','').downcase.
                            tr('_','')
      if name_of_the_program.empty?
        opne "Something went wrong for #{sfile(file)}#{rev}, "\
             "#{sfancy(original_file)} #{rev}(#{simp(name_of_the_program)}#{rev})."
        opne 'Please correct this mistake.'
      elsif file.end_with? '.gem' # Ignore ruby .gem files.
        name_of_the_program = nil
      end
      begin
        # =================================================================== #
        # Register the file here.
        # =================================================================== #
        register_file(
          name_of_the_program, original_file
        ) if name_of_the_program
      rescue Exception => error
        opne 'An error happened (error type: '+sfancy(error.class.to_s)+
             ') at line '+swarn(__LINE__.to_s)
        opne 'The original_file entry was: '+simp(original_file.to_s)
        pp error
      end
    }
    process_registered_programs
  end

  # ========================================================================= #
  # === forbidden?
  #
  # Skip directories listed in our array.
  # ========================================================================= #
  def forbidden?(i)
    ARRAY_SKIP_THESE_DIRECTORIES.any? { |entry|
      return true if i.include? entry
    }
    return false
  end

  # ========================================================================= #
  # === files?
  # ========================================================================= #
  def files?
    @array_registered_files
  end

  # ========================================================================= #
  # === array_registered_files?
  # ========================================================================= #
  def array_registered_files?
    @array_registered_files
  end; alias array_registered_files array_registered_files? # === array_registered_files

  # ========================================================================= #
  # === query_database?
  # ========================================================================= #
  def query_database?
    @query_database
  end; alias query_database query_database? # === query_database

  # ========================================================================= #
  # === process_registered_programs
  # ========================================================================= #
  def process_registered_programs(
      i = files?
    )
    available_programs = available_programs?
    i.each { |name_of_the_program, real_location|
      if available_programs.include? name_of_the_program
        # opne sfancy(name_of_program)+' is included.' Then be silent.
      else # Else it is not included.
        opne swarn('`')+
             sfancy(name_of_the_program.to_s)+
             swarn('` is not included.')
        opne "#{rev}It can be found at `#{sfile(real_location)}#{rev}`."
      end
    }
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    scan
  end

end; end

# =========================================================================== #
# === RBT.scan_source_archive
# =========================================================================== #
def self.scan_source_archive
  RBT::Cookbooks::ScanSourceArchive.new
end

end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::ScanSourceArchive.new
  # pp _.array_registered_files
end # sarchive