#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::ShowDependenciesOn
#
# This class will list all programs that depend on "X" e.g. all that
# depend on something such as "ruby" or "webkitgtk" or "php".
#
# It is part of the Cookbooks submodule of the RBT project.
#
# Usage Examples:
#
#   sdo gtk2
#   sdo php
#   sdo x11
#   sdo qt
#
# =========================================================================== #
# require 'rbt/cookbooks/show_dependencies_on.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Cookbooks

class ShowDependenciesOn < RBT::Base # === RBT::Cookbooks::ShowDependenciesOn

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @array_programs
    # ======================================================================= #
    @array_programs = []
  end

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i)
    i = i.join(' ') if i.is_a? Array
    @input = i
  end

  # ========================================================================= #
  # === add
  # ========================================================================= #
  def add(i)
    if i.include? '.'
      i = File.basename(i).gsub(File.extname(i), '')
    end
    @array_programs << i
  end

  # ========================================================================= #
  # === input?
  # ========================================================================= #
  def input?
    @input
  end

  # ========================================================================= #
  # === report_which_programs_depend_on_the_given_input          (report tag)
  # ========================================================================= #
  def report_which_programs_depend_on_the_given_input(
      i = @array_programs
    )
    opne 'These '+sfancy(i.size.to_s)+' programs '\
         'depend on '+simp(input?.to_s)+':'
    e
    i.sort.each_with_index {|entry, index| # Sort it alphabetically here.
      index += 1
      index = index.to_s.rjust(i.size.to_s.size)
      index = slateblue(index.to_s)
      index = ('('+index.to_s+')').rjust(4)
      index.sub!(/\(/, royalblue('('))
      index.sub!(/\)/, royalblue(')'))
      entry = teal(entry)
      e "  #{index} #{entry}"
    }
    e
  end; alias report report_which_programs_depend_on_the_given_input # === report

  # ========================================================================= #
  # === notify_the_user_that_this_might_take_a_while
  # ========================================================================= #
  def notify_the_user_that_this_might_take_a_while
    opne 'Scanning through the available programs next to find out'
    opne 'which programs depend on '+sfancy(input?)+rev+'. This may '\
         'take a while,'
    opne 'please be patient.'
  end

  # ========================================================================= #
  # === fetch_all_programs_that_depend_on_the_given_input
  # ========================================================================= #
  def fetch_all_programs_that_depend_on_the_given_input
    all_cookbook_files = cookbook_files?(:show_full_path)
    all_cookbook_files.each {|file|
      file = File.basename(file)
      dataset = action(:SanitizeCookbook, file) { :fast }
      data = dataset.dependencies_on?
      unless data.nil?
        add(file) if data.include? input? # Add it if the input is part of the dependencies. 
      end
    }
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    notify_the_user_that_this_might_take_a_while
    fetch_all_programs_that_depend_on_the_given_input
    report_which_programs_depend_on_the_given_input
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Cookbooks::ShowDependenciesOn.new(ARGV)
end # sdo php
    # sdo ruby