#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::CreatePkgconfigFile
#
# This class can be used to generate a .pc file.
#
# Usage examples:
#
#   RBT::CreatePkgconfigFile.new
#
#   require 'rbt/generate_files/create_pkgconfig_file.rb'
#   pc_file = RBT::CreatePkgconfigFile.new(
#     pkgconfig_filename: '/usr/lib/pkgconfig/mad.pc',
#     prefix: '/usr',
#     version: program_version?,
#     description: short_desc?
#   )
#
# =========================================================================== #
# require 'rbt/generate_files/create_pkgconfig_file.rb'
# RBT::CreatePkgconfigFile.new(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class CreatePkgconfigFile < Base # === RBT::CreatePkgconfigFile

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      run_already = true
    )
    reset
    set_input_hash(i)
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @store_this_string
    # ======================================================================= #
    @store_this_string  = ''.dup
    # ======================================================================= #
    # === @prefix
    # ======================================================================= #
    @prefix             = '/usr' # This is the default prefix.
    # ======================================================================= #
    # === @program_version
    # ======================================================================= #
    @program_version    = '1.0.0'
    # ======================================================================= #
    # === @pkgconfig_filename
    # ======================================================================= #
    @pkgconfig_filename = 'mad'
    # ======================================================================= #
    # === @description
    # ======================================================================= #
    @description        = ''
    # ======================================================================= #
    # === @extra_libs
    # ======================================================================= #
    @extra_libs         = ''.dup
  end

  # ========================================================================= #
  # === store_this_string?
  # ========================================================================= #
  def store_this_string?
    @store_this_string
  end; alias main_string? store_this_string? # === main_string?

  # ========================================================================= #
  # === description?
  # ========================================================================= #
  def description?
    @description
  end

  # ========================================================================= #
  # === prefix?
  # ========================================================================= #
  def prefix?
    @prefix
  end

  # ========================================================================= #
  # === set_input_hash
  # ========================================================================= #
  def set_input_hash(
      i = {}
    )
    i = i.first if i.is_a? Array
    if i.is_a?(String) and i.start_with?(':')
      cookbook_dataset = action(:sanitize_cookbook, i.delete(':'))
      # ===================================================================== #
      # Build up the most important Hash next:
      # ===================================================================== #
      i = {
        prefix:             cookbook_dataset.prefix_to_use?,
        pkgconfig_filename: cookbook_dataset.prefix_to_use?+
                            '/lib/pkgconfig/'+cookbook_dataset.real_short_name?.to_s+'.pc',
        version:            cookbook_dataset.program_version?,
        description:        cookbook_dataset.short_description?
      }
    end
    @input_hash = i
    sanitize_the_input_hash
  end

  # ========================================================================= #
  # === sanitize_the_input_hash
  #
  # The main idea for this method is to turn symbol input such as :htop,
  # or pseudo-symbols such as ':htop', into the real dataset.
  # ========================================================================= #
  def sanitize_the_input_hash(
      i = @input_hash
    )
    if i.is_a?(Symbol) or (i.is_a?(String) and i.start_with?(':'))
      require 'rbt/requires/require_the_installer_class.rb'
      @input_hash = action(:installer, i, :do_not_run_yet)
      # ======================================================================= #
      # We will also generate another filename when RBT::Action::Installer
      # is used.
      # ======================================================================= #
      set_pkgconfig_filename(i)
    end
  end

  # ========================================================================= #
  # === input_hash?
  # ========================================================================= #
  def input_hash?
    @input_hash
  end

  # ========================================================================= #
  # === pkgconfig_filename?
  # ========================================================================= #
  def pkgconfig_filename?
    @pkgconfig_filename
  end

  # ========================================================================= #
  # === program_name?
  #
  # This method is guaranteed to return the name without the .pc name.
  # ========================================================================= #
  def program_name?
    File.basename(pkgconfig_filename?).
    delete_suffix('.pc').dup
  end

  # ========================================================================= #
  # === add
  # ========================================================================= #
  def add(i = '')
    @store_this_string << i+N
  end

  # ========================================================================= #
  # === set_pkgconfig_filename
  # ========================================================================= #
  def set_pkgconfig_filename(i)
    i = i.to_s.dup
    i = rds(i).dup
    if i.start_with? ':'
      i[0,1] = ''
    end
    @pkgconfig_filename = i
  end

  # ========================================================================= #
  # === parse_input_hash
  #
  # This method can be used to decompose the given input at hand. It is
  # the first method that is called from within run() itself.
  # ========================================================================= #
  def parse_input_hash
    _ = input_hash?
    # ======================================================================= #
    # === :pkgconfig_filename
    # ======================================================================= #
    if _.has_key? :pkgconfig_filename
      set_pkgconfig_filename(_.delete(:pkgconfig_filename))
    end
    # ======================================================================= #
    # === :prefix
    # ======================================================================= #
    if _.has_key? :prefix
      @prefix = _.delete(:prefix)
    end
    # ======================================================================= #
    # === :program_version
    # ======================================================================= #
    if _.has_key? :program_version
      set_program_version _.delete(:program_version)
    # ======================================================================= #
    # === :version
    # ======================================================================= #
    elsif _.has_key? :version
      set_program_version _.delete(:version)
    end
    # ======================================================================= #
    # === :description
    # ======================================================================= #
    if _.has_key? :description
      @description = _.delete(:description)
    end
  end

  # ========================================================================= #
  # === short_description?
  # ========================================================================= #
  def short_description?
    if @input_hash.is_a? Hash
      @description
    elsif @input_hash.is_a? ::RBT::Action::Installer
      @input_hash.short_description?
    end
  end

  # ========================================================================= #
  # === set_program_version
  # ========================================================================= #
  def set_program_version(i)
    @program_version = i
  end

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    @program_version
  end

  # ========================================================================= #
  # === word_wrapped_description?                                  (wrap tag)
  #
  # Wrap the description a bit.
  # ========================================================================= #
  def word_wrapped_description?
    _ = short_description?.tr(N, ' ').squeeze(' ')
    _ = word_wrap(_, 68) if _.size > 66
    return _
  end

  # ========================================================================= #
  # === store_pc_file
  #
  # We must ensure that the pkgconfig/ directory also exists.
  # ========================================================================= #
  def store_pc_file
    what = main_string?
    into = File.basename(pkgconfig_filename?.dup)
    into << '.pc' unless into.end_with? '.pc'
    ensure_that_the_pkgconfig_directory_exists_for(into)
    opne "Now storing into the file `#{sfile(into)}`."
    opne 'This .pc file could be moved onto'
    opne sdir(pkgconfig_filename?)
    store_what_into(what, into)
  end

  # ========================================================================= #
  # === raw_name_of_the_program?
  # ========================================================================= #
  def raw_name_of_the_program?
    return File.basename(pkgconfig_filename?.dup).delete_suffix('.pc')
  end

  # ========================================================================= #
  # === ensure_that_the_pkgconfig_directory_exists_for
  # ========================================================================= #
  def ensure_that_the_pkgconfig_directory_exists_for(i)
    i = File.dirname(i)
    mkdir(i, :be_quiet) unless File.directory? i
  end

  # ========================================================================= #
  # === build_up_the_main_string                      (main string, main tag)
  #
  # This method depends on information made available prior to calling
  # it, e. g. the method description? or prefix? and so forth.
  # ========================================================================= #
  def build_up_the_main_string
    add 'prefix='+prefix?.to_s # Need .to_s here since we can also pass class Prefix.
    add 'exec_prefix=${prefix}'
    add 'libdir=${exec_prefix}/lib'
    add 'includedir=${prefix}/include'
    add
    add 'Name: '+program_name?.dup
    add "Description: #{word_wrapped_description?}"
    add 'Requires:'
    add 'Version: '+program_version?
    add (
          'Libs: -L${libdir} -l'+program_name?+' '+extra_libs?.to_s
        ).rstrip
    add 'Cflags: -I${includedir}'
  end

  # ========================================================================= #
  # === extra_libs?
  # ========================================================================= #
  def extra_libs?
    @extra_libs
  end

  # ========================================================================= #
  # === handle_special_programs
  # ========================================================================= #
  def handle_special_programs
    case raw_name_of_the_program?
    when 'lua'
      @extra_libs << '-lm -ldl'
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    parse_input_hash # Must parse the Hash prior to String build-up.
    handle_special_programs
    build_up_the_main_string
    store_pc_file
  end

  # ========================================================================= #
  # === RBT::CreatePkgconfigFile[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  if ARGV.empty? # For testing purposes.
   prefix_to_use = '/usr/'
   RBT::CreatePkgconfigFile.new(
     prefix: prefix_to_use,
     pkgconfig_filename: prefix_to_use+'/lib/pkgconfig/mad.pc',
     version: '1.0.3',
     description: 'bla bla bla'
   )
  else
    RBT::CreatePkgconfigFile.new(ARGV)
  end
end # createpkgconfig
    # createpkgconfig :binutils
    # createpkgconfig :lua