#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GenerateFlatpakManifestFile
#
# This class can be used to generate a "manifest" file describing
# a flatpak.
#
# Usage example:
#
#   RBT::GenerateFlatpakManifestFile.new(ARGV)
#
# =========================================================================== #
# Useful resources for flatpaks can be found here:
#
#   https://fedoramagazine.org/getting-started-flatpak/
#
# =========================================================================== #
# require 'rbt/generate_files/generate_flatpak_manifest_file.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class GenerateFlatpakManifestFile < Base # === RBT::GenerateFlatpakManifestFile

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === do_work_on_this_program
  # ========================================================================= #
  def do_work_on_this_program(i)
    require 'json'
    dataset = action(:SanitizeCookbook, i)
    # ======================================================================= #
    # Now we can collect all information that will be necessary for
    # the "manifest" file.
    # ======================================================================= #
    program_name = dataset.short_name?
    hash = {}
    hash['app-id'] = 'org.gnome.'+program_name.capitalize
    hash['runtime-id'] = 'org.gnome.Platform'
    hash['runtime-version'] = dataset.program_version?
    hash['sdk'] = 'org.gnome.Sdk'
    hash['command'] = dataset.short_name?
    hash['finish-args'] = [
      '--filesystem=xdg-documents',
      '--socket=x11',
      # Wayland access:
      '--socket=wayland',
      '--share=network'
    ]
    # ======================================================================= #
    # === Modules
    #
    # The module list specifies each of the modules that are to be built
    # as part of the build process.
    # ======================================================================= #
    hash['modules'] = {
      'name': dataset.short_name?,
      'sources': [
        'type':   'archive',
        'url':     dataset.url1?,
        'sha256': `sha256sum #{dataset.program_path?}`.strip
      ]
    }
    # what = hash.to_json
    what = YAML.dump(hash)
    into = 'manifest'
    opne 'Storing into `'+sfile(into)+'`.'
    write_what_into(what, into)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_work_on_this_program(first_argument?)
  end

  # ========================================================================= #
  # === RBT::GenerateFlatpakManifestFile[]
  # ========================================================================= #
  def self.[](i = '')
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::GenerateFlatpakManifestFile.new(ARGV)
end # generateflatpakmanifestfile gnomedictionary