#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GenerateMachomebrewFormula
#
# This class should ideally be able to generate valid homebrew-formula.
#
# Usage examples:
#
#   RBT::GenerateMachomebrewFormula.new(ARGV)
#
# ============================================================================ #
# require 'rbt/generate_files/generate_machomebrew_formula.rb'
# ============================================================================ #
require 'rbt/base/base.rb'

module RBT

class GenerateMachomebrewFormula < Base # === RBT::GenerateMachomebrewFormula 

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === DEFAULT_PROGRAM
  # ========================================================================= #
  DEFAULT_PROGRAM = 'htop'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @program
    # ======================================================================= #
    @program = nil
    # ======================================================================= #
    # === @_
    # ======================================================================= #
    @_ = ''.dup # This is the string we use to generate a machomebrew formula.
    setup_dataset
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(
      i
    )
    i = [i].flatten.compact
    if i.any? {|entry| entry.start_with?('--') }
      selection = i.select {|entry| entry.start_with?('--') }
      menu(selection)
      i.reject! {|entry| entry.start_with?('--') }
    end
    @internal_hash[:commandline_arguments] = i
    determine_the_program
  end

  # ========================================================================= #
  # === determine_the_program
  # ========================================================================= #
  def determine_the_program
    set_program(first_argument?)
  end

  # ========================================================================= #
  # === set_program
  # ========================================================================= #
  def set_program(
      i = DEFAULT_PROGRAM
    )
    i = DEFAULT_PROGRAM if i.nil?
    @program = i.to_s.dup
  end

  # ========================================================================= #
  # === program?
  # ========================================================================= #
  def program?
    @program
  end; alias program_name? program? # === program_name?

  # ========================================================================= #
  # === setup_dataset
  #
  # @program must exist before calling this method.
  # ========================================================================= #
  def setup_dataset(i = program?)
    @dataset = action(:SanitizeCookbook, i)
  end

  # ========================================================================= #
  # === find_remote_homepage
  #
  # This method finds the last URL.
  # ========================================================================= #
  def find_remote_homepage
    @dataset.remote_homepage?.to_s
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e
    e 'Currently this class has no documented help options.'
    e
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i = nil)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === homebrew --help
      # ===================================================================== #
      when /help$/i
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === return_short_description
  # ========================================================================= #
  def return_short_description
    @dataset.send(__method__)
  end

  # ========================================================================= #
  # === program_path?
  # ========================================================================= #
  def program_path?
    @dataset.send(__method__)
  end

  # ========================================================================= #
  # === shasum?
  # ========================================================================= #
  def shasum?(
      i = program_path?
    )
    return RBT.shasum(i)
  end

  # ========================================================================= #
  # === prefix?
  # ========================================================================= #
  def prefix?
    @dataset.send(__method__)
  end

  # ========================================================================= #
  # === extra_information?
  # ========================================================================= #
  def extra_information?
    @dataset.send(__method__)
  end

  # ========================================================================= #
  # === generate_main_string
  # ========================================================================= #
  def generate_main_string
    short_name        = @dataset.short_name?.to_s.capitalize
    if short_name.empty?
      short_name = program_name?.to_s.dup.capitalize
    end
    link_to_remote_package = @dataset.url1?
    dependencies      = @dataset.required_deps_on?
    configure_options = @dataset.configure_options_or_meson_options?
    program_homepage  = find_remote_homepage
    @_ << "require 'formula'

class #{short_name.to_s} < Formula

  desc '"+return_short_description.to_s.chomp('.')+"'
  homepage '"+program_homepage+"'
  url '"+link_to_remote_package+"'
  sha256 '#{shasum?}'
  revision 1

"
  # Next, add the dependencies:
  dependencies.each {|dep|
    @_ << "  depends_on '"+dep+"'"+N
  }

  @_ << "

  skip_clean :all

  def options
    [['--disable-etcdir', 'Disable the reading of Zsh rc files in /etc']]
  end

  def install
    args = %w(
"

    # ======================================================================= #
    # Next, add the dependencies onto args:
    # ======================================================================= #
    if configure_options.is_a? String
      @_ << "#{configure_options}\n"
    elsif configure_options.is_a? Array
      configure_options.each {|option|
        @_ << "#{option}#{N}"
      } unless configure_options.empty?
    end
    @_ << ")\n"
    # ======================================================================= #
    # And add more to the main String:
    # ======================================================================= #
    @_ << "

    system './configure', \"--prefix=#{prefix?}\", *args
    system 'make install'
  end

  def caveats; <<-EOS
#{extra_information?}

EOS

  end
end
"
  end

  # ========================================================================= #
  # === save_string
  # ========================================================================= #
  def save_string
    _ = "#{rbt_log_directory?}homebrew_formulas/#{program?}.rb"
    unless File.directory? File.dirname(_)
      mkdir(File.dirname(_))
    end
    orev "Saving into the file `#{sfile(_)}#{rev}`."
    save_what_to(@_, _)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    setup_dataset # Set it up anew again.
    generate_main_string
    save_string
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::GenerateMachomebrewFormula.new(ARGV)
end # gmf htop
