#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GenerateMakefile
#
# This class attempts to generate a valid Makefile script. Actually it can
# not do so, but it wants to be able to generate valid Makefiles one day.
# =========================================================================== #
#
#   $@ is the name of the file to be made.
#   $? is the names of the changed dependents.
#   $< the name of the related file that caused the action.
#   $* the prefix shared by target and dependent files.
#
# =========================================================================== #
# Note for myself:
#   We do NOT need to include Distribution-specific things here.
# =========================================================================== #
#
# A GNU Makefile describes the relationships among files in your program 
# and provides commands for updating each file. This ruby script shall 
# generate a Makefile for you automagically.
#
# Additionally, we can use this class to modify an existing Makefile.
#
# For instance, some Makefiles generate something like:
#
#   install -d -m 755 -g $(IGROUP) -o $(IUSER) $(DESTDIR)$(prefix)/include/sound;
# 
# We will automatically replace this with the real IGROUP and IUSER name.
#
# =========================================================================== #
# require 'rbt/generate_files/generate_makefile.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class GenerateMakefile < Base # === RBT::GenerateMakefile

  attr_writer :base_dir

  # ========================================================================= #
  # === WHICH_SHELL_TO_USE
  # ========================================================================= #
  WHICH_SHELL_TO_USE = '/bin/sh'

  # ========================================================================= #
  # === NAME_OF_MAKEFILE
  # ========================================================================= #
  NAME_OF_MAKEFILE = 'Makefile'

  # ========================================================================= #
  # === NAME_OF_THE_FILE_HERE
  # ========================================================================= #
  NAME_OF_THE_FILE_HERE = 'generate_makefile.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_makefile = nil,
      run_already       = true
    )
    reset
    set_input(optional_makefile)
    run if run_already
  end

  # ========================================================================= #
  # === input
  # ========================================================================= #
  def input(i)
    i = i.join(' ').strip if i.is_a? Array
    @input = i
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @_
    # ======================================================================= #
    @_ = ''.dup # string that will be generated
    # ======================================================================= #
    # === @base_dir
    # ======================================================================= #
    @base_dir = return_pwd.dup # Trailing / is easier.
    # ======================================================================= #
    # === @prefix
    # ======================================================================= #
    @prefix = '/usr/' # prefix to use
    # ======================================================================= #
    # === @may_we_overwrite_existing_makefiles
    # ======================================================================= #
    @may_we_overwrite_existing_makefiles = false
    # ======================================================================= #
    # === @array_available_C_files
    # ======================================================================= #
    @array_available_C_files = Dir['*.c']
  end

  # ========================================================================= #
  # === array_available_C_files?
  # ========================================================================= #
  def array_available_C_files?
    @array_available_C_files
  end; alias all_c_files? array_available_C_files? # === all_c_files?

  # ========================================================================= #
  # === add_header
  # ========================================================================= #
  def add_header
    add_comment
    @_ << "# Autogenerated Makefile from "\
          "$RBT/generate_files/#{NAME_OF_THE_FILE_HERE}#{N}"
    add_comment
    add_newline
  end

  # ========================================================================= #
  # === add_newline
  # ========================================================================= #
  def add_newline
    @_ << N
  end
  
  # ========================================================================= #
  # === add_tabs
  #
  # This will replace the first 4 spaces with a tab
  # ========================================================================= #
  def add_tabs
    @_.gsub!(/^    /, TABULATOR)
  end
  
  # ========================================================================= #
  # === sanitize
  #
  # You can use this method to sanitize an existing makefile as well.
  # ========================================================================= #
  def sanitize
    @_.gsub!(/chown root/,'chown 0')
    @_.gsub!(/\/usr\/bin\//,'')
  end

  # ========================================================================= #
  # === create_rule
  #
  # Use this to create a specific rule.
  # ========================================================================= #
  def create_rule(
      name_of_rule,
      action,
      extended_instruction_set = ''
    )
    extended_instruction_set = ' '+extended_instruction_set unless extended_instruction_set.empty?
    @_ << N # right now, we always add a newline
    @_ << name_of_rule+':'+extended_instruction_set+N
    @_ << TABULATOR+action+N # rm *.o temp
  end

  # ========================================================================= #
  # === create_makefile_variable
  #
  # Creates a makefile variable for you to use.
  # ========================================================================= #
  def create_makefile_variable(this_variable)
  end

  # ========================================================================= #
  # === data?
  # ========================================================================= #
  def data?
    @_
  end

  # ========================================================================= #
  # === add_comment
  #
  # Add a comment to the Makefile.
  # ========================================================================= #
  def add_comment
    @_ << '# =========================================================================== #'+N
  end

  # ========================================================================= #
  # === fill_up_string
  # ========================================================================= #
  def fill_up_string
    @_ << 
'# =========================================================================== #
# MACRO LISTING
# =========================================================================== #
SHELL='+WHICH_SHELL_TO_USE+'
    
# =========================================================================== #
# === CFLAGS TO USE
# =========================================================================== #
CFLAGS = # -O -systype bsd43

# =========================================================================== #
# === COMPILER TO USE
#
# This variable can be used to keep track as to which compiler has to be
# used.
# =========================================================================== #
CC=gcc
    
CCOPTS=-Wall
    
# =========================================================================== #
# === TARGET
# =========================================================================== #
# foobar.dvi: $(SRCS)
# 	$(DITROFF) $(MACROS) $(SRCS) >paper.dvi
       
dummy: '+all_c_files?.join(' ')+'
	$(CC) $(CCOPTS) -o dummy '+all_c_files?.join(' ')+'
  	
prefix		= '+prefix?+'
exec_prefix	= ${prefix}


'
  end

  # ========================================================================= #
  # === prefix?
  # ========================================================================= #
  def prefix?
    @prefix
  end

  # ========================================================================= #
  # === report_what_we_will_do
  # ========================================================================= #
  def report_what_we_will_do
    _ = @base_dir+NAME_OF_MAKEFILE
    opne "#{rev}Generating Makefile in `#{sfile(_)}`."
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  # ========================================================================= #
  def show_help
    e 'These are the available options:'
  end

  # ========================================================================= #
  # === menu
  # ========================================================================= #
  def menu(
      i = nil
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ======================================================================= #
      # === generate_makefile --help
      # ======================================================================= #
      when /-?-?help$/i
        show_help
        exit
      end
    end
  end; alias check_against_input menu # === check_against_input

  # ========================================================================= #
  # === consider_generating_a_new_makefile
  # ========================================================================= #
  def consider_generating_a_new_makefile
    target = 'Makefile'
    if File.exist?(target) and !@may_we_overwrite_existing_makefiles
      opne 'A file '+sfile(target)+rev+' already exists. We may '\
           'not generate the same-named file.'
    end
    save_to(target)
  end

  # ========================================================================= #
  # === could_we_find_a_C_file?
  # ========================================================================= #
  def could_we_find_a_C_file?
    !@array_available_C_files.empty?
  end

  # ========================================================================= #
  # === save_to
  #
  # Where to save the Makefile.
  # ========================================================================= #
  def save_to(
      where_to    = 'Makefile',
      mode_to_use = 'w+'
    )
    if File.exist? where_to
      name = 'old_Makefile'
      opne 'A Makefile already exists in this directory. We will rename this'
      opne 'Makefile to '+sfile(name)+' before generating a new Makefile.'
      opnn; mv('Makefile', name)
    end
    save_what_into(data?, where_to)
    opne 'A new '+steelblue('Makefile')+rev+' was generated!'
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  #
  # Bundle together the various actions.
  # ========================================================================= #
  def run
    check_against_input
    report_what_we_will_do
    add_header
    fill_up_string
    if could_we_find_a_C_file?
    else
      create_rule 'joe_goes_crazy', 'echo "ok joe is crazy..."'
      create_rule 'all','g++ main.cpp hello.cpp factorial.cpp -o hello'
      create_rule 'hello','g++ main.o factorial.o hello.o -o hello','main.o factorial.o hello.o'
    end
    sanitize
    add_tabs # should be called right before save_to is invoked
    consider_generating_a_new_makefile
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::GenerateMakefile.new(ARGV)
end # gmake