#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GUI::UniversalWidgets::Notebook
#
# Usage example:
#
#   RBT::GUI::UniversalWidgets::Notebook.new(ARGV)
#
# =========================================================================== #
# require 'rbt/gui/universal_widgets/installed_programs/installed_programs.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module GUI

module UniversalWidgets

class Notebook < ::RBT::Base # === RBT::GUI::UniversalWidgets::Notebook

  require 'universal_widgets/base_module/base_module.rb'
  include ::UniversalWidgets::BaseModule

  require 'rbt/requires/require_the_installer_class.rb'
  require 'rbt/toplevel_methods/infer_build_system.rb'
  require 'rbt/gui/universal_widgets/compile_program/compile_program.rb'

  # ========================================================================= #
  # === HASH_DESIGNATED_KEY_COMBINATIONS
  # ========================================================================= #
  HASH_DESIGNATED_KEY_COMBINATIONS = {
    'alt+1': 'focus_on_this_tab(0)',
    'alt+2': 'focus_on_this_tab(1)',
    'alt+3': 'focus_on_this_tab(2)',
    'alt+4': 'focus_on_this_tab(3)',
    'alt+5': 'focus_on_this_tab(4)'
  }

  # ========================================================================= #
  # === USE_THESE_CSS_RULES
  # ========================================================================= #
  USE_THESE_CSS_RULES = '
  '

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Notebook'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = '82% or minimum 1280px'

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = '40% or minimum 800px'

  # ========================================================================= #
  # === USE_THIS_FONT
  # ========================================================================= #
  USE_THIS_FONT = :dejavu_condensed_19

  # ========================================================================= #
  # === LARGER_FONT
  # ========================================================================= #
  LARGER_FONT   = :dejavu_condensed_22

  # ========================================================================= #
  # === SMALLER_FONT
  # ========================================================================= #
  SMALLER_FONT  = :dejavu_condensed_18

  # ========================================================================= #
  # === USE_THIS_MONOSPACED_FONT
  # ========================================================================= #
  USE_THIS_MONOSPACED_FONT = :hack_16

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments       = ARGV,
      run_already                 = true
    )
    super(:vertical) if use_gtk3?
    determine_the_GUI_to_be_used(commandline_arguments) # This must come first, even before reset().
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    on_delete_event_quit_the_application
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super() if respond_to?(:super)
    reset_the_shared_module # This can come early.
    reset_the_base_module
    reset_the_internal_variables
    infer_the_namespace
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, namespace?]
    # ======================================================================= #
    # === Set the title, width, height and the font in use.
    # ======================================================================= #
    title_width_height_font(TITLE, WIDTH, HEIGHT, USE_THIS_FONT)
    if use_gtk3?
      handle_CSS_rules
    end
    infer_the_size_automatically
  end

  # ========================================================================= #
  # === favicon?
  # ========================================================================= #
  def favicon?
    :tabble
  end

  # ========================================================================= #
  # === smaller_font?
  # ========================================================================= #
  def smaller_font?
    SMALLER_FONT
  end

  # ========================================================================= #
  # === handle_CSS_rules                                            (CSS tag)
  # ========================================================================= #
  def handle_CSS_rules
    use_gtk_paradise_project_css_file
    append_project_css_file
    more_CSS_then_apply_it(USE_THESE_CSS_RULES)
  end

  # ========================================================================= #
  # === reset_the_shared_module
  #
  # This method can be used for ruby-gtk3 and ruby-libui, among other
  # toolkits.
  # ========================================================================= #
  def reset_the_shared_module
    # ======================================================================= #
    # === @array_available_programs
    # ======================================================================= #
    _ = RBT.file_available_programs
    if File.exist? _
      @array_available_programs = YAML.load_file(_)
    end
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    0
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    0
  end

  require 'rbt/toplevel_methods/report_total_size_of_all_archives.rb'
  # ========================================================================= #
  # === do_update_stuff
  # ========================================================================= #
  def do_update_stuff
    total_size = RBT.return_total_size_of_all_archives
    _n_kilobytes = total_size  / 1024.0
    _n_megabytes = _n_kilobytes / 1024.0
    n_gigabytes  = _n_megabytes / 1024.0
    
    @label_total_size_of_all_archives.set_text(
      'All local archives amount to a total size of <b>'+
      n_gigabytes.round(3).to_s+
      ' GB</b>.'
    )
    @label_total_size_of_all_archives.do_markify
    @label_total_size_of_all_archives.left_align
  end

  # ========================================================================= #
  # === focus_on_this_tab
  # ========================================================================= #
  def focus_on_this_tab(i = 2)
    @notebook.focus_on_this_tab(i.to_i)
  end

  # ========================================================================= #
  # === return_widget_containing_package_related_components
  #
  # This is the widget for dealing with packages (creating or
  # installing packages).
  # ========================================================================= #
  def return_widget_containing_package_related_components
    vbox = create_vbox
    vbox.set_border_width(2)
    vbox.add(create_radio_button)
    vbox.add(create_radio_button)
    vbox.add(create_text('This is incomplete.'))
    return vbox
  end

  require 'rbt/toplevel_methods/available_programs.rb'
  # ========================================================================= #
  # === return_the_combo_box_containing_all_available_programs
  # ========================================================================= #
  def return_the_combo_box_containing_all_available_programs
    array = @array_available_programs
    _ = create_combo_box_text(array[0 ... 751]) { :first_element_is_active }
    _.use_this_font = :hack_15
    _.on_changed {
      set_compile_this_program(_.active_text?.to_s)
    }
    _.bblack1
    return _
  end

  # ========================================================================= #
  # === set_compile_this_program
  # ========================================================================= #
  def set_compile_this_program(i)
    @entry_compile_this_program.set_text(i.to_s)
  end

  # ========================================================================= #
  # === do_compile_this_program
  # ========================================================================= #
  def do_compile_this_program(
      i = @entry_compile_this_program.text?
    )
    if i and !i.empty?
      _ = ::RBT::Action::Installer.new(i) { :do_not_run_yet }
      _.set_prefix(
        @entry_prefix_to_use.text?
      )
      @entry_program_path.set_text(
        _.returned_the_assumed_program_path.to_s
      )
      _.determine_which_programs_to_compile
      _.do_extract_the_archive
      _.initialize_a_new_cookbook_instance
      _.determine_the_program_path
      Dir.chdir(_.base_dir?) if File.directory?(_.base_dir?)
      text = RBT.infer_build_system.to_s
      @entry_inferred_build_type.set_text(text)
      # Thread.new { # We let the compilation run in a thread.
      _.do_initiate_batch_compilation
      consider_updating_when_this_program_was_compiled(i)
    end
  end; alias do_compile_the_program do_compile_this_program # === do_compile_the_program

  # ========================================================================= #
  # === consider_updating_when_this_program_was_compiled
  # ========================================================================= #
  def consider_updating_when_this_program_was_compiled(i)
    if RBT.is_this_program_registered?(i)
      @text_status.set_text(
        'Finished compiling <b>'+i.to_s+
        '</b> at '+Time.now.to_s+'.'
      )
      @text_status.do_markify
    end
  end

  require 'rbt/toplevel_methods/available_programs.rb'
  # ========================================================================= #
  # === return_widget_containing_statistical_information
  # ========================================================================= #
  def return_widget_containing_statistical_information
    vbox = create_vbox
    vbox.set_border_width(2)
    @label_total_size_of_all_archives = label
    @label_total_size_of_all_archives.make_selectable

    label_n_programs_are_registered = markified_text(
      "There are a total of <b>#{RBT.available_programs?.size}</b> "\
      "programs available in the RBT project."
    )
    label_n_programs_are_registered.left_align
    label_n_programs_are_registered.make_selectable
    vbox.minimal(label_n_programs_are_registered,   2)
    vbox.minimal(@label_total_size_of_all_archives, 2)
    return vbox
  end

  # ========================================================================= #
  # === return_widget_containing_compile_related_components
  # ========================================================================= #
  def return_widget_containing_compile_related_components
    vbox = create_vbox
    vbox.set_border_width(2)

    hbox1 = create_hbox
    hbox1 << text('Compile this program')
    @entry_compile_this_program = entry('php') # Or: gtk_entry_completion(@array_available_programs)
    @entry_compile_this_program.on_click_select_everything
    @entry_compile_this_program.bblack1
    @entry_compile_this_program.on_enter {
      do_compile_this_program(@entry_compile_this_program.text?)
    }
    hbox1.maximal(@entry_compile_this_program, 1)
    hbox1.maximal(return_the_combo_box_containing_all_available_programs, 1)
    vbox.minimal(hbox1)

    hbox2 = createhbox
    hbox2 << text('Program path:')
    @entry_program_path = createentry
    @entry_program_path.bblack1
    @entry_program_path.disallow_changes
    hbox2.maximal(@entry_program_path, 1)
    vbox.minimal(hbox2, 1)

    hbox3 = createhbox
    hbox3 << text('Prefix to use:')
    @entry_prefix_to_use = createentry('/usr/')
    @entry_prefix_to_use.bblack1
    hbox3.maximal(@entry_prefix_to_use, 1)
    vbox.minimal(hbox3, 1)

    hbox4 = hbox
    hbox4 << text('Inferred build type:')
    @entry_inferred_build_type = create_entry
    @entry_inferred_build_type.bblack1
    hbox4.maximal(@entry_inferred_build_type, 1)
    vbox.minimal(hbox4, 1)

    @compile_button = button('Compile the program')
    @compile_button.bblack1
    @compile_button.hint = 'Click this button to '\
      'compile the program.'
    @compile_button.on_clicked {
      do_compile_the_program
    }
    vbox.minimal(@compile_button, 1)
    @text_status = create_text
    vbox.minimal(@text_status, 1)
    return vbox
  end

  # ========================================================================= #
  # === create_the_skeleton                        (create tag, skeleton tag)
  # ========================================================================= #
  def create_the_skeleton
    # ======================================================================= #
    # === @notebook
    # ======================================================================= #
    @notebook = create_notebook
    @notebook.allow_scrolling
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_super
  end

  # ========================================================================= #
  # === connect_the_skeleton                                    (connect tag)
  # ========================================================================= #
  def connect_the_skeleton
    abort_on_exception

    outer_vbox = create_vbox # A new vbox.

    @notebook = create_notebook(:useful_defaults)
    @notebook.append_this_widget(
      return_widget_containing_compile_related_components,
      'Compile a program'
    )
    @notebook.append_this_widget(
      RBT::GUI::Gtk::CompileProgram.new,
      'Compile a program (advanced widget)'
    )
    @notebook.append_this_widget(
      return_widget_containing_package_related_components,
      'Create or install packages'
    )
    @notebook.append_this_widget(
      return_widget_containing_recipe_related_components,
      'Create a new recipe'
    )
    # ======================================================================= #
    # The VTE does not work very well. We may have to
    # find a solution to this; perhaps just a custom
    # non-vte shell or something.
    #
    #   @notebook.append_this_widget(
    #     return_widget_containing_vte,
    #     'VTE'
    #   )
    # ======================================================================= #
    @notebook.append_this_widget(
      return_widget_containing_statistical_information,
      'Statistics'
    )
    outer_vbox.add(@notebook)

    window = create_window_or_runner
    window << outer_vbox

    properly_prepare_this_window(window,
      {
        title:       title?,
        font:        font?,
        width:       width?,
        height:      height?,
        padding:     padding?,
        border_size: border_size?
      }
    )
    window.show_all
    window.top_left
    do_all_startup_related_actions
    run_main
  end

  # ========================================================================= #
  # === return_widget_containing_recipe_related_components
  # ========================================================================= #
  def return_widget_containing_recipe_related_components
    require 'rbt/gui/gtk3/recipe_maker/recipe_maker.rb'
    vbox = create_vbox
    vbox.set_border_width(2)
    vbox.maximal(RBT::GUI::Gtk::RecipeMaker.new, 1)
    return vbox
  end

  # ========================================================================= #
  # === return_widget_containing_vte
  # ========================================================================= #
  def return_widget_containing_vte
    require 'gtk_paradise/widgets/gtk3/flexible_vte/flexible_vte.rb'
    return ::Gtk::FlexibleVte.new {{ n_tabs: 3 }}
  end

  # ========================================================================= #
  # === do_all_startup_related_actions
  # ========================================================================= #
  def do_all_startup_related_actions
    Thread.new {
      sleep 0.3
      do_update_stuff
    }
  end

  # ========================================================================= #
  # === RBT::GUI::Gtk::Notebook.run
  # ========================================================================= #
  def self.run(
      i = ARGV
    )
    require 'gtk_paradise/run'
    _ = ::RBT::GUI::Gtk::Notebook.new(i)
    r = ::Gtk.run { :add_a_new_accel_group }
    r << _
    _.set_parent_widget(r)
    _.enable_these_key_combinations(HASH_DESIGNATED_KEY_COMBINATIONS)
    r.automatic_size_then_automatic_title
    r.enable_quick_exit
    r.set_background :white
    r.is_resizable # <- This line was added due to a possible prior .maximize action.
    r.top_left_then_run
  end; self.instance_eval { alias start_gui_application run } # === RBT::GUI::Gtk::Notebook.start_gui_application

  # ========================================================================= #
  # === RBT::GUI::UniversalWidgets::Notebook[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::GUI::UniversalWidgets::Notebook.new(ARGV)
end # gui_rbt