#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::GUI::UniversalWidgets::Statistics
#
# This widget will show the compile-time statistics, and ideally
# also other statistics. That way the user can investigate the
# computer based on a set of gtk-widgets - at the least this is
# the design goal in the long run.
#
# Usage example:
#
#   RBT::GUI::UniversalWidgets::Statistics.new(ARGV)
#
# =========================================================================== #
# require 'rbt/gui/universal_widgets/statistics/statistics.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module GUI

module UniversalWidgets

class Statistics < ::RBT::Base # === RBT::GUI::UniversalWidgets::Statistics

  require 'universal_widgets/base_module/base_module.rb'
  include ::UniversalWidgets::BaseModule

  require 'rbt/requires/require_the_installer_class.rb'
  require 'rbt/shell/shell.rb'

  # ========================================================================= #
  # === USE_THESE_CSS_RULES
  # ========================================================================= #
  USE_THESE_CSS_RULES = '
treeview.view header button {
  font-size: 1.2em;
  color: steelblue;
  background: lightblue;
  font-weight: bold;
  text-shadow: none;
  box-shadow:  none;
  border: 1px solid black;
}'

  # ========================================================================= #
  # === EXPLANATION_ABOUT_THE_PURPOSE_OF_THIS_WIDGET
  # ========================================================================= #
  EXPLANATION_ABOUT_THE_PURPOSE_OF_THIS_WIDGET = 
    "This widget shows some information as to how long it took\n"\
    "to compile different programs on the given computer system,\n"\
    "in seconds and in minutes."

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Statistics about the RBT project' # Or: 'RBT::GUI::UniversalWidgets::Statistics'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = '82% or minimum 1280px'

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = '40% or minimum 300px'

  # ========================================================================= #
  # === USE_THIS_FONT
  # ========================================================================= #
  USE_THIS_FONT = :dejavu_condensed_20

  # ========================================================================= #
  # === LARGER_FONT
  # ========================================================================= #
  LARGER_FONT   = :dejavu_condensed_22

  # ========================================================================= #
  # === SMALLER_FONT
  # ========================================================================= #
  SMALLER_FONT  = :dejavu_condensed_18

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    super(:vertical) if use_gtk3?
    determine_the_GUI_to_be_used(commandline_arguments) # This must come first, even before reset().
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    on_delete_event_quit_the_application
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super() if respond_to?(:super)
    reset_the_shared_module # This can come early.
    reset_the_base_module
    reset_the_internal_variables
    infer_the_namespace
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, namespace?]
    # ======================================================================= #
    # === Set the title, width, height and the font in use.
    # ======================================================================= #
    title_width_height_font(TITLE, WIDTH, HEIGHT, USE_THIS_FONT)
    if use_gtk3?
      handle_CSS_rules
    end
    infer_the_size_automatically
    # ======================================================================= #
    # === @compile_time_statistics
    #
    # The next instance-variable is a Hash. We will tap into class Shell
    # for this information.
    # ======================================================================= #
    @compile_time_statistics =
      ::RBT::Installer::Shell.new(:dont_run_yet).return_compile_time
    if @compile_time_statistics.nil?
      e 'No statistics were found. Exiting now.'
      exit
    end
  end

  # ========================================================================= #
  # === favicon?
  # ========================================================================= #
  def favicon?
    :tabble
  end

  # ========================================================================= #
  # === smaller_font?
  # ========================================================================= #
  def smaller_font?
    SMALLER_FONT
  end

  # ========================================================================= #
  # === create_the_scrolled_window
  # ========================================================================= #
  def create_the_scrolled_window
    @scrolled_window = create_scrolled_window { :automatic }
  end

  # ========================================================================= #
  # === make_the_treeview_sortable
  # ========================================================================= #
  def make_the_treeview_sortable
    columns = @treeview.columns?
    
    columns[0].sortable_based_on(0)
    columns[0].align_to_the_center
    columns[1].sortable_based_on(1)
    columns[1].align_to_the_center
    columns[2].sortable_based_on(2)
    columns[2].align_to_the_center
    columns[3].sortable_based_on(3)
    columns[3].align_to_the_center

    # ======================================================================= #
    # Use a custom sort-function next.
    # ======================================================================= #
    list_store?.set_sort_func(2) { |_model, iter1, iter2|
      iter2[2].to_f <=> iter1[2].to_f
    }
    # ======================================================================= #
    # Sort the minutes as well.
    # ======================================================================= #
    list_store?.set_sort_func(3) { |_model, iter1, iter2|
      iter2[3].to_f <=> iter1[3].to_f
    }
  end

  # ========================================================================= #
  # === do_prepare_the_list_store
  # ========================================================================= #
  def do_prepare_the_list_store(
      i = @list_store
    )
    # ======================================================================= #
    # Define the dataset that is to be used for this widget.
    # ======================================================================= #
    use_this_dataset = return_time_sorted_dataset
    index = 0
    use_this_dataset.each {|name_of_the_program, compile_time| index += 1
      iter = i.append
      iter.set_value(0, index.to_s.rjust(2, ' '))
      iter.set_value(1, name_of_the_program)
      _ = ('%.2f' % compile_time.round(2)).rjust(10,' ')
      iter.set_value(2, _)
      # ===================================================================== #
      # Next, set the time in n minutes
      # ===================================================================== #
      new_value = (_.strip.to_f / 60.0).round(2).to_s
      new_value = '%.2f' % new_value.to_f
      iter.set_value(
        3,
        new_value
      )
    }
  end; alias populate do_prepare_the_list_store # === populate

  # ========================================================================= #
  # === handle_CSS_rules                                            (CSS tag)
  # ========================================================================= #
  def handle_CSS_rules
    use_gtk_paradise_project_css_file
    append_project_css_file
    more_CSS_then_apply_it(USE_THESE_CSS_RULES)
  end

  # ========================================================================= #
  # === reset_the_shared_module
  #
  # This method can be used for ruby-gtk3 and ruby-libui, among other
  # toolkits.
  # ========================================================================= #
  def reset_the_shared_module
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    0
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    2
  end

  # ========================================================================= #
  # === do_all_startup_related_actions
  # ========================================================================= #
  def do_all_startup_related_actions
  end

  # ========================================================================= #
  # === hash?
  # ========================================================================= #
  def hash?
    @compile_time_statistics
  end

  # ========================================================================= #
  # === create_the_liststore
  # ========================================================================= #
  def create_the_liststore
    @list_store = ::Gtk::ListStore.new(#gtk_list_store(
      String,
      String,
      String,
      String
    )
  end

  # ========================================================================= #
  # === list_store?
  # ========================================================================= #
  def list_store?
    @list_store
  end

  # ========================================================================= #
  # === treeview?
  # ========================================================================= #
  def treeview?
    @treeview
  end

  # ========================================================================= #
  # === create_the_treeview
  # ========================================================================= #
  def create_the_treeview
    # ======================================================================= #
    # === @treeview
    # ======================================================================= #
    @treeview = tree_view
    @treeview.the_headers_are_clickable
    @treeview.enable_search # Enable searching.
    @treeview.rubber_banding
    @treeview.enable_drag_and_drop
    @treeview.all_are_sortable
    @treeview.allow_resizing
    @treeview.set_reorderable(true)
    @treeview.set_size_request(500, 800)
  end

  # ========================================================================= #
  # === create_the_buttons                                      (buttons tag)
  # ========================================================================= #
  def create_the_buttons
    # ======================================================================= #
    # === @button_update_the_tree_view
    # ======================================================================= #
    @button_update_the_tree_view = bold_button(
      '_Update the above listing'
    ) { :use_mnemonics }
    @button_update_the_tree_view.hint = 'Click '\
      'on this button to update the tree-view data above.'
    @button_update_the_tree_view.remove_background
    @button_update_the_tree_view.on_hover(:lightgreen)
    @button_update_the_tree_view.bblack1
    @button_update_the_tree_view.on_clicked {
      do_prepare_the_list_store
    }
  end

  # ========================================================================= #
  # === return_time_sorted_dataset
  # ========================================================================= #
  def return_time_sorted_dataset
    _ = hash?
    _ = _.sort_by {|name, time|
      time
    }.reverse
    _
  end

  # ========================================================================= #
  # === style_the_treeview_properly
  # ========================================================================= #
  def style_the_treeview_properly
    # ======================================================================= #
    # Create a new GtkCellRendererText, add it to the tree
    # view column and append the column to the tree view.
    # ======================================================================= #
    renderer = cell_renderer_text
    renderer.foreground = '#0c087d' # This is a darkblue colour.
    renderer_centered = centered_cell_renderer_text
    renderer_centered.xalign = 0.5 # Center it here.
    very_left_column = tree_view_column(
      '#', renderer_centered, text: 0
    )
    very_left_column.center
    @treeview.append_column(very_left_column)
    # ======================================================================= #
    # === Name of the program
    # ======================================================================= #
    left_column = tree_view_column(
      'Name of the program', renderer, text: 1
    )
    left_column.set_alignment(0.5)
    @treeview.append_column(left_column)
    renderer = cell_renderer_text
    renderer.foreground = '#037317' # This is a darkgreen colour.
    # ======================================================================= #
    # === right_column
    # ======================================================================= #
    right_column = tree_view_column(
      'Time (in seconds)', renderer_centered, text: 2
    )
    right_column.alignment = 0.5 
    @treeview.append_column(right_column)
    # ======================================================================= #
    # === righter_column
    # ======================================================================= #
    righter_column = tree_view_column(
      'Time (in minutes)', renderer_centered, text: 3
    )
    @treeview.append_column(righter_column)

    do_prepare_the_list_store(@list_store)
    @treeview.model = @list_store
    @treeview.on_button_press_event { |widget, event|
      if double_click?(event) # Act on double-click events.
        selected = @treeview.selection.selected[1]
        e ::Colours.steelblue(selected)
      end
    }
  end

  # ========================================================================= #
  # === on_right_mouse_button_deselect_the_treeview_selection
  # ========================================================================= #
  def on_right_mouse_button_deselect_the_treeview_selection
    treeview?.on_button_press_event { |widget, event|
      case event.button
      when 3 # Right-mouse button was clicked here.
        treeview?.deselect
      end
    }
  end

  # ========================================================================= #
  # === create_the_skeleton                        (create tag, skeleton tag)
  # ========================================================================= #
  def create_the_skeleton
    create_the_liststore
    create_the_buttons
    create_the_scrolled_window
    create_the_treeview
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    run_super
  end

  # ========================================================================= #
  # === connect_the_skeleton                                    (connect tag)
  # ========================================================================= #
  def connect_the_skeleton
    abort_on_exception

    outer_vbox = create_vbox # A new vbox.

    style_the_treeview_properly
    make_the_treeview_sortable
    on_right_mouse_button_deselect_the_treeview_selection
    container_for_the_treeview = vbox
    container_for_the_treeview.maximal(@treeview, 10)
    @scrolled_window.add(container_for_the_treeview)
    @scrolled_window.set_size_request(820, 820)
    info_text = bold_text(EXPLANATION_ABOUT_THE_PURPOSE_OF_THIS_WIDGET)

    outer_vbox.maximal(info_text, 20)
    outer_vbox.maximal(@scrolled_window)

    small_container = create_vbox
    small_container.minimal(@button_update_the_tree_view, 2)
    small_container.hcenter
    # @button_update_the_tree_view.hcenter
    # @button_update_the_tree_view.align_to_center
    #_.minimal(small_container)
    outer_vbox.minimal(small_container, 12)

    window = create_window_or_runner
    window << outer_vbox

    properly_prepare_this_window(window,
      {
        title:       title?,
        font:        font?,
        width:       width?,
        height:      height?,
        padding:     padding?,
        border_size: border_size?
      }
    )
    window.show_all
    window.top_left
    do_all_startup_related_actions
    run_main
  end

  # ========================================================================= #
  # === RBT::GUI::Gtk::Statistics.run
  # ========================================================================= #
  def self.run(
      i = ARGV
    )
    require 'gtk_paradise/run'
    _ = ::RBT::GUI::Gtk::Statistics.new(i)
    r = ::Gtk.run
    _.set_parent_widget(r) # Must come before we enable the key-combinations.
    r << _
    r.automatic_size_then_automatic_title
    r.enable_quick_exit
    r.set_background :white
    r.top_left_then_run
  end; self.instance_eval { alias start_gui_application run } # === RBT::GUI::Gtk::Statistics.start_gui_application

  # ========================================================================= #
  # === RBT::GUI::UniversalWidgets::Statistics[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::GUI::UniversalWidgets::Statistics.new(ARGV)
end # gui_microsoft_controller