#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Linux::ParseBlfsWebpage
#
# This class can parse the content of a BLFS webpage and extract the
# instructions. These instructions may then be used by other classes
# for doing a LFS/BLFS conforming installation.
#
# This class must be able to work on both a local dataset and a remote
# webpage. It currently requires nokogiri to be installed on the
# target computer.
#
# It will carry all matching results.
#
# Usage example:
#
#   RBT::Linux::ParseBlfsWebpage.new(ARGV)
#
# =========================================================================== #
# require 'rbt/linux/linux_from_scratch/parse_blfs_webpage.rb'
# _ = RBT::Linux::ParseBlfsWebpage.new(ARGV).shell_commands?
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Linux

class ParseBlfsWebpage < Base # === RBT::Linux::ParseBlfsWebpage

  begin
    require 'nokogiri'
  rescue LoadError; end

  # ========================================================================= #
  # === DEFAULT_FILE
  # ========================================================================= #
  DEFAULT_FILE = RBT.log_dir?+'blfs_webpages/blfs_page_for_krb5.html'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    if block_given?
      case yield
      when :do_report
        @shall_we_report = true
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @dataset_after_having_applied_the_regex
    # ======================================================================= #
    @dataset_after_having_applied_the_regex = []
    # ======================================================================= #
    # === @shall_we_report
    # ======================================================================= #
    @shall_we_report = false
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = :default)
    if i.is_a? Array and i.empty?
      i = :default
    end
    case i
    when :default
      i = [DEFAULT_FILE]
    end
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === apply_nokogiri
  # ========================================================================= #
  def apply_nokogiri
    _ = Nokogiri::HTML(@dataset)
    _.xpath('//kbd[starts-with(@class, "command")]').each { |element|
      @dataset_after_having_applied_the_regex << element.children[0].text
    }
  end

  # ========================================================================= #
  # === shell_instructions?
  # ========================================================================= #
  def shell_instructions?
    @dataset_after_having_applied_the_regex
  end; alias commands?       shell_instructions? # === commands?
       alias shell_commands? shell_instructions? # === shell_commands?

  # ========================================================================= #
  # === report                                                   (report tag)
  # ========================================================================= #
  def report(
      i = @dataset_after_having_applied_the_regex
    )
    if @shall_we_report
      if i.is_a? Array
        i.each {|entry| e entry }
      else
        e i
      end
    end
  end

  # ========================================================================= #
  # === try_to_simplify_the_dataset
  # ========================================================================= #
  def try_to_simplify_the_dataset
    # ======================================================================= #
    # Simplify the dataset next:
    # ======================================================================= #
    if @dataset.include? '<div class="commands"'
      # ===================================================================== #
      # This is to exclude the explanation for the commands:
      # ===================================================================== #
      @dataset = @dataset[
        0 .. @dataset.index('<div class="commands"')
      ]
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    first = first_argument?
    if first.nil?
      opne 'Please provide a page here.'
    end
    # ======================================================================= #
    # First: handle locally existing files:
    # ======================================================================= #
    if first and File.exist?(first)
      @dataset = File.read(first, encoding: 'iso-8859-1')
    elsif first and first.is_a?(String)
      @dataset = first
    else
      opne 'No file could be found. Please provide valid input.'
    end
    if @dataset
      try_to_simplify_the_dataset
      apply_nokogiri
    end
    report if @shall_we_report
  end

  # ========================================================================= #
  # === RBT::Linux::ParseBlfsWebpage[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::Linux::ParseBlfsWebpage.new(ARGV) { :do_report }.report
end # parseblfswebpage