#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Linux::Slackware::GenerateSlackDescFile
#
# The class RBT::Linux::Slackware::GenerateSlackDescFile will generate
# a slack-desc file.
#
# A description of this can be seen here:
#
#   http://www.slackwiki.com/Slack-desc
#   https://slackbuilds.org/templates/slack-desc
#
# Interestingly enough, the slack-desc file must have exactly 11 lines.
#
# The maximum length per line shall not exceed 71 characters.
#
# Lines without any other text should *not* have a space after the ':'.
# =========================================================================== #
# require 'rbt/linux/slackware/generate_slack_desc_file.rb'
# RBT::Linux::SlackwareGenerateSlackDescFile
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Linux

module Slackware

class GenerateSlackDescFile < Base # === RBT::Linux::Slackware::GenerateSlackDescFile

  require 'rbt/requires/require_the_cookbook_class.rb'

  # ========================================================================= #
  # === USE_THIS_FILE_SUFFIX
  #
  # A proper slack-desc file ends with a .slack-desc suffix.
  # ========================================================================= #
  USE_THIS_FILE_SUFFIX = '.slack-desc'

  # ========================================================================= #
  # === CHARACTERS_PER_LINE_MAXIMUM
  # ========================================================================= #
  CHARACTERS_PER_LINE_MAXIMUM = 68

  # ========================================================================= #
  # === HANDY_RULER
  # ========================================================================= #
  HANDY_RULER = 
    '       |-----handy-ruler------------------------------------------------------|'

  # ========================================================================= #
  # === initialize
  #
  # The first argument should be the name of the file in question.
  # ========================================================================= #
  def initialize(
      this_program = ARGV,
      run_already  = true,
      &block
    )
    reset
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :use_short_name
      # ===================================================================== #
      when :use_short_name
        @use_long_name_or_short_name = :short_name
      # ===================================================================== #
      # === :also_show_the_content_of_the_generated_file
      # ===================================================================== #
      when :also_show_the_content_of_the_generated_file
        @show_the_content_of_the_generated_file = true
      # ===================================================================== #
      # === :also_show_the_content_of_the_generated_file_if_on_roebe
      # ===================================================================== #
      when :also_show_the_content_of_the_generated_file_if_on_roebe
        if is_on_roebe?
          @show_the_content_of_the_generated_file = true
        end
      end
    end
    set_this_program(this_program)
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @use_long_name_or_short_name
    # ======================================================================= #
    @use_long_name_or_short_name = :long_name
    # ======================================================================= #
    # === @_
    #
    # Should be an Array.
    # ======================================================================= #
    @_ = []
    # ======================================================================= #
    # === @show_the_content_of_the_generated_file
    #
    # If the following instance variable is set to true then the class
    # will also show the content of the generated file (if the file
    # exists, too).
    # ======================================================================= #
    @show_the_content_of_the_generated_file = false
  end

  # ========================================================================= #
  # === set_this_program
  # ========================================================================= #
  def set_this_program(i)
    i = i.first if i.is_a? Array
    @this_program = i
  end

  # ========================================================================= #
  # === this_program?
  # ========================================================================= #
  def this_program?
    @this_program
  end; alias name?            this_program? # === name?
       alias program?         this_program? # === program?
       alias name_of_program? this_program? # === name_of_program?
       alias program_name?    this_program? # === program_name?

  # ========================================================================= #
  # === length_of_program_name?
  # ========================================================================= #
  def length_of_program_name?
    @this_program.size
  end

  # ========================================================================= #
  # === description?
  # ========================================================================= #
  def description?
    return @dataset.description?.to_s.dup.
           tr("\n",' ') # Replace newlines.
  end

  # ========================================================================= #
  # === short_description?
  # ========================================================================= #    
  def short_description?
    @dataset.send(__method__)
  end

  # ========================================================================= #
  # === homepage?
  # ========================================================================= #
  def homepage?
    @dataset.send(__method__)
  end

  # ========================================================================= #
  # === chunk
  #
  # Split at 80 characters.
  # ========================================================================= #
  def chunk(
      string, size = CHARACTERS_PER_LINE_MAXIMUM
    )
    size -= length_of_program_name?
    string.gsub(/(.{1,#{size}})(\s+|$)/, "\\1\n")
  end

  # ========================================================================= #
  # === generate_main_string
  #
  # This will generate the main String, into the variable @_.
  # ========================================================================= #
  def generate_main_string
    result = ''.dup
    _ = name_of_program?
    _ = _.dup
    if short_description?
      _ << " (#{short_description?})" unless short_description?.empty?
    else
      e swarn('No short_description entry could be found for:')
      e steelblue("  #{_}")
      if description? and !description?.empty?
        opne 'Substituting via the :description entry for this instead.'
        _ << " (#{description?})"
      end
    end
    _ << N
    result << _
    result << N
    result << chunk(description?)
    if result.is_a? String
      result = result.split(N)
    end
    @_ << result
    @_.flatten!
    # ======================================================================= #
    # Next, fill in the missing slots:
    # ======================================================================= #
    (11 - @_.size).times {
      @_ << ''
    }
    # ======================================================================= #
    # And then, prepend the program name to it.
    # ======================================================================= #
    @_.map! {|line|
      line = line.dup if line.frozen?
      if line.empty?
        line.prepend("#{program_name?}:")
      else
        line.prepend("#{program_name?}: ")
      end
      line
    }
    # ======================================================================= #
    # Next, we could add a handy ruler - but I personally find this
    # useless, so the following line is commented away.
    # ======================================================================= #
    # @_[0,0] = return_handy_ruler+N
    append_the_homepage_if_it_exists
  end

  # ========================================================================= #
  # === append_the_homepage_if_it_exists
  # ========================================================================= #
  def append_the_homepage_if_it_exists
    homepage = homepage?
    if homepage and !homepage.empty?
      @_[-1] << " Homepage: #{homepage}"
    end
  end

  # ========================================================================= #
  # === return_name_of_slack_desc_file
  # ========================================================================= #
  def return_name_of_slack_desc_file
    program_name?+USE_THIS_FILE_SUFFIX
  end

  # ========================================================================= #
  # === return_short_name
  # ========================================================================= #
  def return_short_name
    USE_THIS_FILE_SUFFIX.delete('.')
  end; alias return_short_name_of_slack_desc_file return_short_name # === return_short_name_of_slack_desc_file

  # ========================================================================= #
  # === path?
  # ========================================================================= #
  def path?
    path = return_pwd.dup
    case @use_long_name_or_short_name # case tag
    # ======================================================================= #
    # === :long_name
    # ======================================================================= #
    when :long_name,
         :default # This is the default.
      path << return_name_of_slack_desc_file
    # ======================================================================= #
    # === :short_name
    # ======================================================================= #
    when :short_name
      path << return_short_name_of_slack_desc_file
    end
    path
  end

  # ========================================================================= #
  # === create_slack_desk_file
  # ========================================================================= #
  def create_slack_desk_file(
      into = path?
    )
    opnn { :no_trailing }
    e
    e
    e "#{rev}Now creating a #{springgreen('slack-desc file')}#{rev} for "+
      "#{File.basename(into).delete_suffix('.slack-desc')}"
    e "#{rev}(#{sfile(into)}#{rev})."
    e
    what = @_.join(N)+N # Grab the content and adjust it a little.
    write_what_into(what, into)
  end

  # ========================================================================= #
  # === consider_showing_the_content_of_the_generated_file
  # ========================================================================= #
  def consider_showing_the_content_of_the_generated_file
    if @show_the_content_of_the_generated_file
      _ = path?
      if File.exist?(_)
        e
        e "Showing the content of the file #{sfile(_)}:"
        e
        e File.read(_)
        e
      end
    end
  end

  # ========================================================================= #
  # === obtain_main_dataset
  # ========================================================================= #
  def obtain_main_dataset(i = program?)
    @dataset = action(:SanitizeCookbook, i) { :fast }
  end

  # ========================================================================= #
  # === file?
  # ========================================================================= #
  def file?
    @dataset.file?
  end

  # ========================================================================= #
  # === return_handy_ruler
  # ========================================================================= #
  def return_handy_ruler
    HANDY_RULER
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    obtain_main_dataset
    generate_main_string
    create_slack_desk_file
    consider_showing_the_content_of_the_generated_file
  end

  # ========================================================================= #
  # === RBT::Linux::Slackware::GenerateSlackDescFile[]
  # ========================================================================= #
  def self.[](i = ARGV, &block)
    new(i, &block)
  end

end; end; end

# =========================================================================== #
# === RBT.generate_slackware_description_file
# =========================================================================== #
def self.generate_slackware_description_file(
    i, &block
  )
  object = RBT::Linux::Slackware::GenerateSlackDescFile[i, &block]
  result = object.path?
  return result
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.generate_slackware_description_file(ARGV) {
    :also_show_the_content_of_the_generated_file_if_on_roebe
  }
end # slackdescfor htop
    # slackdescfor nano
    # slackdescfor ruby