#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Linux::Slackware::InstallThisSlackwarePackage
#
# This class can be used to quickly install a (remote or local) slackware
# package.
#
# The two main functions covered by this class are:
#
#    a) download the package if it does not exist locally, and
#    b) run "installpkg" on the package.
#
# =========================================================================== #
# require 'rbt/linux/slackware/install_this_slackware_package.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module Linux

module Slackware

class InstallThisSlackwarePackage < Base # === RBT::Linux::Slackware::InstallThisSlackwarePackage

  # ========================================================================= #
  # === INSTALL_PKG_COMMAND_TO_USE
  # ========================================================================= #
  INSTALL_PKG_COMMAND_TO_USE = 'installpkg'

  # ========================================================================= #
  # === initialize
  #
  # The first argument is the name of the program.
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true,
      &block
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    # ======================================================================= #
    # === Hanld blocks given to the class next
    # ======================================================================= #
    if block_given?
      yielded = yielded
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @install_as_appdir
    # ======================================================================= #
    @install_as_appdir = false
  end

  # ========================================================================= #
  # === download_this_package
  # ========================================================================= #
  def download_this_package(i)
    esystem "wget #{i}"
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = commandline_arguments?
    )
    # ======================================================================= #
    # Presently we will only handle arguments that start with a leading
    # '--' in this method.
    # ======================================================================= #
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      if i.start_with? '--'
        case i # case tag
        # =================================================================== #
        # === islackware binutils-2.26-x86_64-3.txz --appdir
        # =================================================================== #
        when /^-?-?appdir$/i,
             /^-?-?app$/i
          @install_as_appdir = true
        end
      end
    end
  end

  # ========================================================================= #
  # === determine_the_shorter_name
  # ========================================================================= #
  def determine_the_shorter_name(i)
    shorter_name = i.sub(/-x86_64-\d{0,1}/,'') 
    if shorter_name.include? '-x86'
      shorter_name.sub!(/-x86-?\d{0,1}/,'')
    end
    @shorter_name = i
  end

  # ========================================================================= #
  # === install_this_package_as_appdir
  # ========================================================================= #
  def install_this_package_as_appdir(i)
    i = remove_archive_at_the_end(i)
    determine_the_shorter_name(i)
    version = ProgramInformation.return_version(
      @shorter_name
    )
    program_name = ProgramInformation.return_program_name(
      @shorter_name
    )
    target = 'temp_for_'+program_name.tr('-','_')+'-'+
             @shorter_name.tr('-','_').
             squeeze('_')
    # ======================================================================= #
    # Next, add-hoc fix the name:
    # ======================================================================= #
    if program_name.include? 'kernelheaders'
      program_name.sub!(/kernelheaders/,'linux')
    end
    mkdir target
    cd target
    cp "../#{i}.txz", '.'
    extract "#{i}.txz"
    cd 'usr'
    new_target = "#{program_name}-#{version}"
    rcp new_target
    new_target = programs_dir?+
                 program_name.capitalize+'/'+
                 version+'/'
    opne "#{rev}Now copying into #{sdir(new_target)}#{rev}:"
    cpr('.', new_target)
    cd new_target
    opne "#{rev}Next symlinking from that directory."
    rnsymc(new_target)
    opne mediumseagreen('All done!')
  end

  # ========================================================================= #
  # === extract
  # ========================================================================= #
  def extract(this)
    require 'extracter'
    Extracter.what_to(this)
  end

  # ========================================================================= #
  # === rnsymc
  # ========================================================================= #
  def rnsymc(
      i = return_pwd
    )
    action(:SymlinkFromToCurrent, i)
  end

  # ========================================================================= #
  # === rcp
  # ========================================================================= #
  def rcp(i)
	  action(:create_appdir_skeleton, i)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    menu
    commandline_arguments?.each {|this_program|
      _ = this_program
      # ===================================================================== #
      # Consider downloading it if a remote URL was given:
      # ===================================================================== #
      if _.start_with?('http') and !File.exist?(_)
        download_this_package(_)
      end
      basename = File.basename(_)
      if !File.exist? basename
        opnn; no_file_exists_at(basename)
      else
        opne "#{rev}Trying to install"
        opne "#{steelblue(basename)} #{rev}next."
        if install_as_appdir?
          install_this_package_as_appdir(basename)
        elsif File.exist? basename
          esystem "#{INSTALL_PKG_COMMAND_TO_USE} #{basename}"
        end
      end
    }
  end

  # ========================================================================= #
  # === install_as_appdir?
  # ========================================================================= #
  def install_as_appdir?
    @install_as_appdir
  end

  # ========================================================================= #
  # === RBT::Linux::Slackware::InstallThisSlackwarePackage[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end

# =========================================================================== #
# === RBT::InstallThisSlackwarePackage
# =========================================================================== #
InstallThisSlackwarePackage = RBT::Linux::Slackware::InstallThisSlackwarePackage

end

if __FILE__ == $PROGRAM_NAME
  RBT::Linux::Slackware::InstallThisSlackwarePackage.new(ARGV)
end # islackware http://mirrors.slackware.com/slackware/slackware64-14.2/slackware64/d/gcc-5.3.0-x86_64-3.txz
    # islackware binutils-2.26-x86_64-3.txz --appdir