#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ClassesForIndividualPrograms::Gcc::ChangeLib64ToLib
#
# This class will locate "/lib64" entries in a file and replace them
# with "/lib" entries.
#
# Usage example:
#
#   RBT::ClassesForIndividualPrograms::Gcc::ChangeLib64ToLib.new
#
# =========================================================================== #
# require 'rbt/misc/classes_for_individual_programs/change_lib64_to_lib.rb'
# RBT::ClassesForIndividualPrograms::Gcc::ChangeLib64ToLib.new
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

module ClassesForIndividualPrograms

class Gcc

class ChangeLib64ToLib < Base # === RBT::ClassesForIndividualPrograms::Gcc::ChangeLib64ToLib

  # ========================================================================= #
  # === SEEK_THIS_SUBSTRING
  # ========================================================================= #
  SEEK_THIS_SUBSTRING = 'm64=../lib64'

  # ========================================================================= #
  # === DEFAULT_FILE
  #
  # This constant specifies the default that is to be used. After all the
  # whole class here was written in order to modify that particular file
  # anyway.
  # ========================================================================= #
  DEFAULT_FILE = 'gcc/config/i386/t-linux64'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @dataset
    # ======================================================================= #
    @dataset = nil # <- Initial state.
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = ARGV)
    case i
    when :default
      i = DEFAULT_FILE
    end
    i = [i].flatten.compact
    if i.empty? and is_on_roebe?
      # ===================================================================== #
      # Use a default, hardcoded file path in this event.
      # ===================================================================== #
      i << DEFAULT_FILE
    end
    internal_hash_set_commandline_arguments(i)
    sanitize_commandline_arguments
  end

  # ===================================================================== #
  # === sanitize_commandline_arguments
  # ===================================================================== #
  def sanitize_commandline_arguments
    commandline_arguments?.map! {|entry|
      case entry
      when 'default'
        entry = DEFAULT_FILE
      end unless File.exist?('default')
      entry
    }
  end

  # ========================================================================= #
  # === set_modify_this_file
  # ========================================================================= #
  def set_modify_this_file(i)
    @modify_this_file = i
  end

  # ========================================================================= #
  # === try_to_read_in_the_main_dataset
  # ========================================================================= #
  def try_to_read_in_the_main_dataset
    _ = @modify_this_file
    if File.exist? _
      @dataset = default_readlines(_).join
    else
      opnn; no_file_exists_at(_)
    end
  end

  # ========================================================================= #
  # === store_file
  # ========================================================================= #
  def store_file(
      what = @dataset, into = @modify_this_file
    )
    into = File.absolute_path(into)
    opne "Saving into the file:"
    e
    e "  #{sfile(into)}"
    e
    write_what_into(what, into)
  end

  # ========================================================================= #
  # === return_the_default_regex
  # ========================================================================= #
  def return_the_default_regex
    Regexp.new(SEEK_THIS_SUBSTRING)
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    query_whether_an_argument_has_been_given
    if first_argument?
      set_modify_this_file(first_argument?)
      try_to_read_in_the_main_dataset
      if @dataset
        # =================================================================== #
        # The replacement will only work on 64-bit systems.
        # =================================================================== #
        if is_a_64bit_system?
          if @dataset.include? SEEK_THIS_SUBSTRING
            use_this_regex = return_the_default_regex
            opne 'The regex to be applied is: '+steelblue(use_this_regex)
            @dataset.gsub!(use_this_regex, 'm64=../lib')
            store_file(@dataset)
          else
            opne "The file does not contain the "\
                 "substring #{steelblue(SEEK_THIS_SUBSTRING)}."
          end
        else
          opne 'Applying the regex replacing m64=../lib only works'
          opne 'for 64-bit systems.'
        end
      end
    end
  end

  # ========================================================================= #
  # === query_whether_an_argument_has_been_given
  # ========================================================================= #
  def query_whether_an_argument_has_been_given
    _ = first_argument?
    if _.nil? or _.nil?
      opne 'Please provide a valid file path to this class.'
    end
  end

  # ========================================================================= #
  # === RBT::ClassesForIndividualPrograms::Gcc:ChangeLib64ToLib[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  RBT::ClassesForIndividualPrograms::Gcc::ChangeLib64ToLib.new(ARGV)
end # changelib64tolib gcc/config/i386/t-linux64