#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ReverseDependencies
#
# This class can show the reverse dependencies of given programs.
#
# It currently absolutely depends on the "expanded cookbook dataset".
#
# By default the class is quite verbose; if you only need an Array
# showing the dependencies then you should consider using the
# class-level method instead (it is defined near the bottom of
# this file).
#
# Usage example:
#
#   RBT::ReverseDependencies.new(ARGV)
#
# =========================================================================== #
# require 'rbt/misc/reverse_dependencies.rb'
# RBT::ReverseDependencies.dependencies?
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ReverseDependencies < Base # === RBT::ReverseDependencies

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      work_on_these_programs = [
        'atk',
        'glib',
        'libjpeg',
        'pango',
        'tiff',
        'libxft',
        'cairo >= 1.6'
      ],
      run_already           = true
    )
    reset
    set_work_on_these_programs(
      work_on_these_programs
    )
    # ======================================================================= #
    # === Handle Blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      when :be_quiet
        @be_verbose = false
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === :be_verbose
    #
    # This should be true by default.
    # ======================================================================= #
    set_be_verbose
    # ======================================================================= #
    # === @array_unique_dependencies
    # ======================================================================= #
    @array_unique_dependencies = []
    # ======================================================================= #
    # === @hash
    # ======================================================================= #
    @hash = {}
    all_programs.each {|this_program|
      this_program.strip!
      if this_program.include? ' '
        this_program = this_program.split(' ').first
      end
      dataset = load_dataset_from_this_expanded_cookbook(this_program)
      @hash.update(
        { this_program.to_sym => dataset }
      )
    }
  end

  # ========================================================================= #
  # === set_work_on_these_programs
  # ========================================================================= #
  def set_work_on_these_programs(i)
    i = [i].flatten.compact.map {|entry|
      if entry.include? ' '
        entry = entry.split(' ').first
      end
      entry
    }
    @work_on_these_programs = i
  end

  # ========================================================================= #
  # === add                                                         (add tag)
  # ========================================================================= #
  def add(i)
    i = [i].flatten.compact.uniq
    i.each {|entry|
      if entry.include? ' '
        entry = entry.split(' ').first
      end
      unless @array_unique_dependencies.include? entry
        @array_unique_dependencies << entry
      end
    }
  end

  # ========================================================================= #
  # === return_all_dependencies_of_this_program
  #
  # This method can be used to return all dependencies of the given
  # program at hand.
  # ========================================================================= #
  def return_all_dependencies_of_this_program(this_program)
    if this_program.include? ' '
      this_program = this_program.split(' ').first
    end
    this_program = this_program.to_sym
    if @hash.include? this_program 
      @hash[this_program]['required_deps_on']
    else
      e "\n"+
        tomato('The program ')+steelblue(this_program)+
        tomato(' is not included.')
    end
  end

  # ========================================================================= #
  # === e
  # ========================================================================= #
  def e(i = '')
    puts i if be_verbose?
  end

  # ========================================================================= #
  # === ee
  # ========================================================================= #
  def ee(i = '')
    print i if be_verbose?
  end

  # ========================================================================= #
  # === do_work_on_the_programs
  # ========================================================================= #
  def do_work_on_the_programs
    @work_on_these_programs.each {|this_program|
      this_program = this_program.split(' ').first if this_program.include? ' '
      entries = return_all_dependencies_of_this_program(this_program)
      if entries.size == 0
        e "#{rev}The program #{cadetblue(this_program)} has no "\
          "listed dependent programs."
      else
        e "#{rev}The program #{cadetblue(this_program)} depends "\
          "on these #{entries.size} programs:"
        ee '    '
        add(entries) # ← Add these dependencies to the unique deps.
        ee steelblue(entries.join(', ').strip)
      end
      e unless entries.empty?
      entries.each {|inner_this_program|
        _ = return_all_dependencies_of_this_program(inner_this_program)
        # =================================================================== #
        # Indent towards the right side once.
        # =================================================================== #
        if _.empty?
          ee "    No #{tomato('secondary dependencies')} "\
             "were found for #{mediumpurple(inner_this_program)}.\n"
        else
          ee "    And the #{tomato('secondary dependencies')} "\
             "of #{mediumpurple(inner_this_program)} are: "\
             "#{steelblue(_.join(', ').strip)}\n"
          add(_)
        end
      }
      e
    }
    report_the_unique_dependencies if @be_verbose
  end

  # ========================================================================= #
  # === report_the_unique_dependencies
  # ========================================================================= #
  def report_the_unique_dependencies
    e 'The unique dependencies are:'
    e
    pp @array_unique_dependencies
    e
  end

  # ========================================================================= #
  # === unique_dependencies?
  # ========================================================================= #
  def unique_dependencies?
    @array_unique_dependencies.sort
  end; alias results? unique_dependencies? # === results?

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_work_on_the_programs
  end

  # ========================================================================= #
  # === RBT::ReverseDependencies[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

  # ========================================================================= #
  # === RBT::ReverseDependencies.dependencies?
  # ========================================================================= #
  def self.dependencies?(i)
    RBT::ReverseDependencies.new(i) { :be_quiet }.results?
  end

end; end

if __FILE__ == $PROGRAM_NAME
  _results = RBT::ReverseDependencies.new.results? # (ARGV)
  pp _results
end # reversedependencies