#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Prefix
#
# This class simply stores the default prefix.
#
# It will be used for EVERY occurance of a prefix in the RBT scripts.
#
# It will only include the target at where it will point to.
#
# Basic Usage example to query the prefix to use goes like this:
#
#   @prefix = RBT::Prefix.new
#
# =========================================================================== #
# To require this class in your project, do:
#
#   require 'rbt/prefix/prefix.rb'
#   prefix = RBT::Prefix.new
#   RBT.prefix_object
#
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class Prefix < Base # === RBT::Prefix

  # ========================================================================= #
  # === USR_PREFIX
  # ========================================================================= #
  USR_PREFIX = '/usr/' # This is the default prefix.
    DEFAULT_PREFIX = USR_PREFIX # === DEFAULT_PREFIX

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      optional_use_this_program_name = nil,
      run_already                    = true
    )
    reset # Must always come first.
    if optional_use_this_program_name
      set_original_input(optional_use_this_program_name)
      set_program(optional_use_this_program_name)
    end
    if block_given?
      yielded = yield
      case yielded
      # === :ntrad
      when :ntrad
        set_prefix(:ntrad)
      else
        run if run_already
      end
    else
      run if run_already
    end
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === :prefix
    # ======================================================================= #
    set_prefix(nil) # nil initially. This is our prefix variable.
    set_program
    set_be_quiet
    set_program_version
    # ======================================================================= #
    # === @do_we_want_to_use_an_appdir_prefix
    #
    # We want to be able to store here whether an AppDir prefix is to
    # be used.
    # ======================================================================= #
    @do_we_want_to_use_an_appdir_prefix = false
  end

  # ========================================================================= #
  # === do_we_want_to_use_an_appdir_prefix?
  # ========================================================================= #
  def do_we_want_to_use_an_appdir_prefix?
    @do_we_want_to_use_an_appdir_prefix
  end

  # ========================================================================= #
  # === we_will_use_an_appdir_prefix
  # ========================================================================= #
  def we_will_use_an_appdir_prefix
    @do_we_want_to_use_an_appdir_prefix = true
  end; alias do_use_appdir_prefix we_will_use_an_appdir_prefix # === do_use_appdir_prefix

  # ========================================================================= #
  # === sub!
  # ========================================================================= #
  def sub!(old, new)
    @internal_hash[:prefix].sub!(old, new)
  end

  # ========================================================================= #
  # === empty?
  # ========================================================================= #
  def empty?
    @internal_hash[:prefix].empty?
  end

  # ========================================================================= #
  # === set_original_input
  # ========================================================================= #
  def set_original_input(i)
    @internal_hash[:original_input] = i.to_s.dup
  end

  # ========================================================================= #
  # === end_with?
  #
  # Determine whether our prefix ends with a specific string.
  # ========================================================================= #
  def end_with?(i)
    @internal_hash[:prefix].end_with? i
  end

  # ========================================================================= #
  # === include?
  #
  # Use this to query whether we include a string.
  # ========================================================================= #
  def include?(i)
    @internal_hash[:prefix].include? i
  end

  # ========================================================================= #
  # === set_program
  #
  # Use this method to set the instance variable @internal_hash[:program].
  #
  # It shall be kept like that. The input can contain a '-'.
  # ========================================================================= #
  def set_program(i = nil)
    case i
    # ======================================================================= #
    # === :traditional
    # ======================================================================= #
    when :traditional,
         :trad
      set_prefix(i)
      i = nil
    # ======================================================================= #
    # === :nontrad
    # ======================================================================= #
    when :nontrad,
         :ntrad
      we_will_use_an_appdir_prefix
    end
    @internal_hash[:program] = i # Can contain something like htop.
  end; alias assign set_program # === assign

  # ========================================================================= #
  # === program?
  # ========================================================================= #
  def program?
    @internal_hash[:program]
  end

  # ========================================================================= #
  # === clear_program
  # ========================================================================= #
  def clear_program
    @internal_hash[:program]
  end

  # ========================================================================= #
  # === eliminate_hyphens
  # ========================================================================= #
  def eliminate_hyphens
    program?.delete!('-')
    use_non_traditional_prefix # Also sync towards @internal_hash[:prefix] here.
  end

  # ========================================================================= #
  # === set_prefix
  #
  # This methods assigns to the instance variable @internal_hash[:prefix]
  # which keeps the prefix in use. The prefix instance variable is stored
  # in the variable @internal_hash[:prefix].
  #
  # Input should include the version, i.e. "htop-1.0" would be a proper
  # input. 
  # ========================================================================= #
  def set_prefix(
      i = program?
    )
    case i.to_s # case tag
    # ======================================================================= #
    # === gobolinux
    #
    # This entry point essentially means "make use of an AppDir-like prefix.
    # ======================================================================= #
    when 'gobolinux',
         'gobolinux_prefix',
         'ntrad',
         'false',
         'default',
         'app_dir_like',
         /^app(_|-)?dir/
      we_will_use_an_appdir_prefix
      i = return_non_traditional_prefix(
        program_name_and_program_version?
      ).to_s # defined in this file here.
      # ===================================================================== #
      # As of May 2014 we get rid of all '-'
      # ===================================================================== #
      i.delete!('-') if i.include?('-')
    # ======================================================================= #
    # === true
    # ======================================================================= #
    when 'true',
         't',
         'trad',
         'traditional',
         'tra',
         'def' # True means "usr" prefix.
      i = USR_PREFIX
    else # Else default to USR_PREFIX.
      # i = USR_PREFIX
    end
    assign_prefix(i) if i # Now, simply assign it here.
  end; alias prefix=                     set_prefix # === prefix=
       alias rebuild_prefix              set_prefix # === rebuild_prefix
       alias update                      set_prefix # === update
       alias set_new_prefix              set_prefix # === set_new_prefix
       alias use_this_as_the_new_prefix= set_prefix # === use_this_as_the_new_prefix=

  # ========================================================================= #
  # === program_name_and_program_version?
  # ========================================================================= #
  def program_name_and_program_version?
    "#{program?}-#{@program_version}"
  end

  # ========================================================================= #
  # === use_non_traditional_prefix
  #
  # This method will assign to a non-traditional (gobolinux) prefix.
  # ========================================================================= #
  def use_non_traditional_prefix
    set_prefix(:gobolinux)
  end; alias gobolinux_prefix                   use_non_traditional_prefix # === gobolinux_prefix
       alias enable_non_traditional_compilation use_non_traditional_prefix # === enable_non_traditional_compilation

  # ========================================================================= #
  # === assign_prefix
  #
  # Simply assign the prefix, without any further checking.
  # ========================================================================= #
  def assign_prefix(i)
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default,
         :default_prefix,
         nil
      i = DEFAULT_PREFIX
    end
    i = i.dup if i.frozen?
    i.strip!
    i << '/' unless i.end_with? '/'
    @internal_hash[:prefix] = i
  end; alias assign_prefix=      assign_prefix # === assign_prefix=
       alias use_this_prefix     assign_prefix # === use_this_prefix
       alias set_use_this_prefix assign_prefix # === set_use_this_prefix

  # ========================================================================= #
  # === strip!
  # ========================================================================= #
  def strip!
    @internal_hash[:prefix].strip!
  end

  # ========================================================================= #
  # === <<
  # ========================================================================= #
  def <<(i)
    @internal_hash[:prefix] << i
  end

  # ========================================================================= #
  # === prefix?
  #
  # This query-method will output the prefix to use.
  # ========================================================================= #
  def prefix?
    @internal_hash[:prefix].to_s
  end; alias query  prefix? # === query
       alias _      prefix? # === _
       alias text?  prefix? # === text?
       alias to_s   prefix? # === to_s
       alias to_str prefix? # === to_s

  # ========================================================================= #
  # === is_an_appdir_prefix?
  # ========================================================================= #
  def is_an_appdir_prefix?
    prefix?.start_with?(program_directory?)
  end; alias makes_use_of_an_appdir_prefix? is_an_appdir_prefix? # === makes_use_of_an_appdir_prefix?
       alias appdir?                        is_an_appdir_prefix? # === appdir?

  # ========================================================================= #
  # === set_program_version
  # ========================================================================= #
  def set_program_version(i = '1.0.0')
    @program_version = i
  end

  # ========================================================================= #
  # === use_appdir_prefix_for_this_program
  # ========================================================================= #
  def use_appdir_prefix_for_this_program(i)
    set_program(i)
    set_prefix(:ntrad)
  end

  # ========================================================================= #
  # === return_non_traditional_prefix
  #
  # This method will return a proper non-traditional prefix.
  # ========================================================================= #
  def return_non_traditional_prefix(
      i = program_name_and_program_version?
    )
    program_information = ::ProgramInformation::ProgramInformation.new(
      i,
      true,
      be_verbose?
    ) # bl $RUBY_COOKBOOKS/lib/cookbooks/program_information.rb
    name    = program_information.name.to_s.capitalize.delete('-_').capitalize
    version = program_information.version
    i = "#{programs_dir?}#{name}/#{version}/".dup # This has the version name
    # ======================================================================= #
    # Ensure that it will always have a trailing '/' token.
    # ======================================================================= #
    i << '/' unless i.end_with? '/'
    # ======================================================================= #
    # As of May 2014 we get rid of all '-'.
    # ======================================================================= #
    i.delete!('-') if i.include? '-'
    return i # This here should usually return a string like "/Programs/Htop/1.1.0"
  end; alias do_use_appdir_prefix_from_this_input return_non_traditional_prefix # === do_use_appdir_prefix_from_this_input

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    set_prefix
  end

  # ========================================================================= #
  # === RBT::Prefix.prefix?
  # ========================================================================= #
  def self.prefix?(i = nil)
    _ = RBT::Prefix.new(i)
    _.prefix? # Return value.
  end

end

# =========================================================================== #
# === @prefix
#
# This is for the toplevel prefix variable.
# =========================================================================== #
@prefix = nil

# =========================================================================== #
# === RBT.prefix?
# =========================================================================== #
def self.prefix?
  @prefix
end; self.instance_eval { alias last_prefix?      prefix? } # === RBT.last_prefix?
     self.instance_eval { alias last_prefix_used? prefix? } # === RBT.last_prefix_used?

# =========================================================================== #
# === RBT.set_prefix
#
# Keep track of the last prefix used via this toplevel method.
# =========================================================================== #
def self.set_prefix(i = '/usr/')
  RBT::Prefix.new(i)
end; self.instance_eval { alias set_last_prefix_used set_prefix } # === RBT.set_last_prefix_used
     self.instance_eval { alias prefix_object        set_prefix } # === RBT.prefix_object

end

if __FILE__ == $PROGRAM_NAME
  require 'colours/colours_e_autoinclude.rb'
  require 'cliner/autoinclude'
  e Colours.rev
  input = 'htop-1.2.4'
  prefix = RBT::Prefix.new(input)
  pp prefix
  prefix = RBT::Prefix.new
  prefix.set_prefix('def')
  e 'Now checking for: `'+sfancy(input)+'`.'
  prefix.set_program(input)
  cliner
  e 'Now setting to '+sfancy('Gobolinux-like prefix.')
  prefix.set_prefix(:gobolinux)
  e steelblue(prefix._)
  e prefix
  cliner
  prefix.set_prefix(true)
  pp prefix._
  e 'Now assigning to '+sfancy('htop-2.3.2')
  prefix.set_program 'htop-2.3.2'
  prefix.gobolinux_prefix
  pp prefix._
  cliner
  e royalblue('Now assigning to ICKI')
  prefix.assign_prefix 'ICKI'
  e steelblue(prefix._)
  pp prefix
  cliner
  e 'Next invoking '+Colours.orange('RBT.last_prefix_used?')
  e
  e RBT.last_prefix_used?
  e
end # prefix
