#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::RegisteredBinaries
#
# The task of this class is simple:
#
#   Iterate over all binaries and collect them, storing them into a
#   yaml file.
#
#   If however had an argument is passed, then we will try to find
#   out to which program this binary belongs to.
#
# =========================================================================== #
# require 'rbt/registered/registered_binaries.rb'
# RBT::Cookbooks::RegisteredBinaries.new
# =========================================================================== #
require 'rbt/registered/registered.rb'

module RBT

module Cookbooks

class RegisteredBinaries < Registered # === RBT::Cookbooks::RegisteredBinaries

  require 'rbt/requires/require_the_cookbook_class.rb'
  require 'rbt/toplevel_methods/all_binaries.rb'
  require 'rbt/requires/swift_version.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    if block_given?
      yielded = yield
      case yielded
      when :be_quiet
        set_be_quiet
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @hash_to_be_stored
    # ======================================================================= #
    @hash_to_be_stored = {} # This is the hash that will be stored.
    # ======================================================================= #
    # === @cookbook_dataset
    # ======================================================================= #
    @cookbook_dataset = nil
    # ======================================================================= #
    # === @this_program
    # ======================================================================= #
    @this_program = nil
  end

  # ========================================================================= #
  # === report_to_the_user_what_we_will_do
  # ========================================================================= #
  def report_to_the_user_what_we_will_do
    opne 'Obtaining Information about available binaries next.'
    opne 'This may take a while.'
    opne orange('Please be patient.')
  end

  # ========================================================================= #
  # === RBT::Cookbooks::RegisteredBinaries.location?
  # ========================================================================= #
  def self.location?
    opn; e FILE_REGISTERED_BINARIES
  end

  # ========================================================================= #
  # === save_yaml_file
  #
  # The method `save_yaml_file` will create the yaml file which lists our
  # binaries (in hash format). Right now, we need to manually update this
  # yaml file on our own.
  # ========================================================================= #
  def save_yaml_file
    into = RBT.file_registered_binaries
    what = YAML.dump(hash?)
    e sfancy('Storing yaml dataset in ')
    e '  '+sfile(into)
    write_what_into(what, into)
    if is_on_roebe?
      into = "#{ruby_src_dir_for_the_registered_yaml_files?}"\
             "#{File.basename(into)}"
      also_storing_into
      e "`#{sfile(into)}`"
      write_what_into(what, into)
    end
  end; alias create_yaml_file save_yaml_file # === create_yaml_file

  # ========================================================================= #
  # === RBT::Cookbooks::RegisteredBinaries.search_for_this_program
  # ========================================================================= #
  def self.search_for_this_program(i, &block)
    _ = ::RBT::Cookbooks::RegisteredBinaries.new(false, &block)
    _.set_this_program(i)
    _.search_for_this_program(i)
  end

  # ========================================================================= #
  # === RegisteredBinaries.opnn
  # ========================================================================= #
  def self.opnn
    Opn.opn(namespace: NAMESPACE)
  end

  # ========================================================================= #
  # === opnn
  # ========================================================================= #
  def opnn
    RegisteredBinaries.opnn
  end

  # ========================================================================= #
  # === search_for_this_program
  #
  # Use this method to search for a specific program.
  # ========================================================================= #
  def search_for_this_program(
      i = @this_program
    )
    be_verbose = be_verbose?
    i = i.to_s
    if i.include? '/'
      i = File.basename(i)
      opne "The input includes a #{simp('/')} token. We will "\
           "truncate this to the"
      opne 'last part only, as we only keep the names registered, '\
           'not the paths.'
      opne "Thus, our input will be #{sfancy(i)}#{rev}."
    end
    if be_verbose
      opne "#{rev}Searching for `#{sfancy(i)}#{rev}` now:"
    end
    # ======================================================================= #
    # Query whether it is already included or not.
    # ======================================================================= #
    if RBT.does_include? i
      if be_verbose
        opne "Yes, this appears to be already registered with the "\
             "same name #{simp(i)}#{rev}."
      end
      program_version = RBT.swift_return_version_of_this_program(i)
      if be_verbose
        opne "Its program version is: #{lightblue(program_version)}"
      end
      return i
    else
      hash = load_yaml(FILE_REGISTERED_BINARIES)
      if hash.keys.include?(i)
        result = hash[i]
        if be_verbose
          opne "#{rev}The binary called `#{sfancy(i)}#{rev}` was found."
          it_is_part_of_the_package(result)
          opne rev+'The currently registered version of '+
                tomato(result)+"#{rev} is: "+
                orchid(
                  return_program_version_of(result)
                )
        end
        return result
      else
        if be_verbose
          opne sfancy(i)+rev+' is '+swarn('NOT')+rev+' included. We could '\
               'not find any program'
          opne 'including this program.'
        end
      end
      false # Not included, thus we return false.
    end
  end

  # ========================================================================= #
  # === return_program_version_of
  # ========================================================================= #
  def return_program_version_of(i)
    return RBT.return_version_of_this_program(i)
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    report_to_the_user_what_we_will_do
    # ======================================================================= #
    # We could rescue the following code, but I think it is better to fail,
    # and then correct the error. But in case we wish to change the
    # behaviour again, the rescue-clause below was retained in the comments.
    # ======================================================================= #
    available_programs?.each { |program_name|
        _ = action(:SanitizeCookbook, program_name) { :fast }
        binaries = _.binaries? # This is an Array.
        unless binaries.empty?
          e program_name if SHOW_PROGRAM_NAME
          binaries.each {|entry|
            @hash_to_be_stored[entry] = program_name
          }
        end
      # rescue Exception => error
      #   opne 'An error happened for the program '+
      #        sfile(program_name)+
      #        '. (Error: '+error.class.to_s+')'
      #   pp error
      # end
    }
    save_yaml_file
  end

end; end

# =========================================================================== #
# === RBT.registered_binaries?
#
# Simply return an Array with all the binaries.
#
# Full usage example:
#
#   require 'rbt/registered/registered_binaries.rb'
#   RBT.registered_binaries?
#
# =========================================================================== #
def self.registered_binaries?
  ALL_BINARIES.keys.uniq.sort
end; self.instance_eval { alias registered_binaries registered_binaries? } # === RBT.registered_binaries

# =========================================================================== #
# === RBT.hash_registered_binaries?
#
# This method is similar to the method RBT.registered_binaries? - but
# rather than an Array we will return the original Hash.
# =========================================================================== #
def self.hash_registered_binaries?
  ALL_BINARIES
end

# =========================================================================== #
# === RBT.show_registered_binaries
# =========================================================================== #
def self.show_registered_binaries
  _ = ::RBT.registered_binaries?
  RegisteredBinaries.opnn; e 'These '+_.size.to_s+' binaries are registered:'
  e _.reject {|entry| entry.to_s.empty? }.join(', ').strip.chop
end

end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::Cookbooks::RegisteredBinaries.new(false)
  if ARGV.first
    argument = ARGV.first
    if argument.include? 'show'
      RBT.show_registered_binaries # regbin --show
    else # Pass the argument into the method search_for_this_program().
      _.search_for_this_program(argument)
    end
  else
    _.run
  end
end # regbin