#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::RegisteredLibraries
#
# The task of this class is simple:
#
#   Iterate over all libraries and collect them, and then store
#   them in a yaml file.
# =========================================================================== #
# require 'rbt/registered/registered_libraries.rb'
# RBT::Cookbooks::RegisteredLibraries.new
# =========================================================================== #
require 'rbt/registered/registered.rb'

module RBT

module Cookbooks

class RegisteredLibraries < RBT::Cookbooks::Registered

  require 'rbt/toplevel_methods/all_libraries.rb'

  # ========================================================================= #
  # === LOCATION_REGISTERED_LIBRARIES
  # ========================================================================= #
  LOCATION_REGISTERED_LIBRARIES = RBT.file_registered_libraries

  # ========================================================================= #
  # === SHOW_PROGRAM_NAME
  # ========================================================================= #
  SHOW_PROGRAM_NAME = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    case run_already
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @hash_to_be_stored
    # ======================================================================= #
    @hash_to_be_stored = {} # This is the hash that will be stored.
    # ======================================================================= #
    # === @query
    # ======================================================================= #
    @query = nil
    # ======================================================================= #
    # === @this_program
    # ======================================================================= #
    @this_program = nil
  end

  # ========================================================================= #
  # === report_to_the_user_what_we_will_do
  # ========================================================================= #
  def report_to_the_user_what_we_will_do
    opne 'Obtaining information about the '+
         steelblue('available libraries')+rev+' next.'
    opne 'This may take a while.'
    opne 'Please be patient.'
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    report_to_the_user_what_we_will_do
    use_this_key = :libraries # This is either a String or a Symbol.
    available_programs?.each { |program_name|
      @query = action(:SanitizeCookbook, program_name) { :fast }
      data = @query.return_dataset
      libraries = data[use_this_key] # This is an Array.
      unless libraries.empty?
        e program_name if SHOW_PROGRAM_NAME # ← This constant is defined in the file registered.rb
        libraries.each {|entry|
          @hash_to_be_stored[entry] = program_name
        }
      end
    }
    save_yaml_file
  end

  # ========================================================================= #
  # === search_for_this_program
  #
  # Use this method to search for a specific program.
  # ========================================================================= #
  def search_for_this_program(
      i = @this_program
    )
    i = i.to_s
    opne "Searching for #{sfancy(i)} now:"
    hash = load_yaml(File.location_registered_libraries)
    if hash.keys.include? i
      opne "The library called `#{sfancy(i)}` was found."
      it_is_part_of_the_package(hash[i])
    else
      opne p+' is '+swarn('NOT')+' included. We could not '\
              'find any program'
      opne 'including this program.'
    end
  end

  # ========================================================================= #
  # === save_yaml_file
  #
  # The method `save_yaml_file` will create the yaml file which lists our
  # libraries (in hash format). Right now, we need to manually update this
  # yaml file on our own.
  # ========================================================================= #
  def save_yaml_file
    what = YAML.dump(@hash_to_be_stored)
    into = LOCATION_REGISTERED_LIBRARIES
    e sfancy('Storing yaml dataset in ')
    e "  #{sfile(into)}"
    write_what_into(what, into)
    if is_on_roebe?
      into = "#{ruby_src_dir_for_the_registered_yaml_files?}"\
             "#{File.basename(into)}"
      e 'Also storing into:'
      e "  #{sfile(into)}"
      write_what_into(what, into)
    end
  end; alias create_yaml_file save_yaml_file # === create_yaml_file

end; end

# =========================================================================== #
# RBT.registered_libraries?
# =========================================================================== #
def self.registered_libraries?
  RBT::Cookbooks::RegisteredLibraries.new
end

end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::Cookbooks::RegisteredLibraries.new(:do_not_run_yet)
  if ARGV.first
    _.search_for_this_program(ARGV.first)
  else
    _.run
  end
end # reglib