#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Cookbooks::RegisteredM4Files
#
# The task of this class is simple:
#
#   Iterate over all .m4 files and collect them, and then
#   store them into a yaml file.
#
# =========================================================================== #
# require 'rbt/registered/registered_m4_files.rb'
# RBT::Cookbooks::RegisteredM4Files.new
# RBT.file_registered_m4_files
# =========================================================================== #
require 'rbt/registered/registered.rb'

module RBT

# =========================================================================== #
# === RBT.file_registered_m4_files
# =========================================================================== #
def self.file_registered_m4_files
  "#{RBT.yaml_directory_for_the_registered_yaml_files?}m4.yml"
end

module Cookbooks

class RegisteredM4Files < RBT::Cookbooks::Registered

  # ========================================================================= #
  # === LOCATION_REGISTERED_M4_FILES
  # ========================================================================= #
  LOCATION_REGISTERED_M4_FILES = RBT.file_registered_m4_files

  # ========================================================================= #
  # === SHOW_PROGRAM_NAME
  # ========================================================================= #
  SHOW_PROGRAM_NAME = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      run_already = true
    )
    reset
    case run_already
    when :do_not_run_yet
      run_already = false
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @hash_to_be_stored
    # ======================================================================= #
    @hash_to_be_stored = {} # This is the hash that will be stored.
    # ======================================================================= #
    # === @query
    # ======================================================================= #
    @query = nil
    # ======================================================================= #
    # === @try_to_use_the_expanded_cookbook_dataset_if_available
    # ======================================================================= #
    @try_to_use_the_expanded_cookbook_dataset_if_available = true
    @this_program = nil
  end

  # ========================================================================= #
  # === report_to_the_user_what_we_will_do
  # ========================================================================= #
  def report_to_the_user_what_we_will_do
    opne "Obtaining Information about available m4 "\
         "#{steelblue('.m4')} next."
    opne 'This may take a while.'
    opne 'Please be patient.'
  end

  # ========================================================================= #
  # === location_m4_files?
  # ========================================================================= #
  def location_m4_files?
    RBT.file_registered_m4_files
  end

  # ========================================================================= #
  # === return_m4_files_of_this_program
  # ========================================================================= #
  def return_m4_files_of_this_program(i)
    if @try_to_use_the_expanded_cookbook_dataset_if_available and
       does_this_expanded_cookbook_file_exist_for_this_program?(i)
      i = load_dataset_from_this_expanded_cookbook(i)
      headers = i['m4_files']
    else
      headers = action(:SanitizeCookbook, i) { :fast }.m4_files?
    end
    return headers
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    report_to_the_user_what_we_will_do
    available_programs?.each { |program_name|
      m4_files = return_m4_files_of_this_program(program_name)
      if m4_files.nil?
        e "Nil value for #{sfancy(program_name)}#{rev}."
        e 'Please correct this.'
        # exit
      else
        unless m4_files.empty?
          if SHOW_PROGRAM_NAME # ← This constant is defined in the file registered.rb
            program_name = File.basename(program_name).sub(/\.yml$/,'')
            e program_name+
              steelblue(
                ' ['+
                m4_files.join(',').strip+
                ']'
              )
          end
          m4_files.each {|entry|
            @hash_to_be_stored[entry] = program_name
          }
        end
      end
    }
    save_yaml_file
  end

  # ========================================================================= #
  # === save_yaml_file
  #
  # The method `save_yaml_file` will create the yaml file which lists our
  # m4 files (in hash format). Right now, we need to manually
  # update this yaml file on our own.
  # ========================================================================= #
  def save_yaml_file
    what = YAML.dump(@hash_to_be_stored)
    into = location_m4_files?
    e sfancy('Storing yaml dataset in ')
    e "  #{sfile(into)}"
    write_what_into(what, into)
    if is_on_roebe?
      into = "#{ruby_src_dir_for_the_registered_yaml_files?}"\
             "#{File.basename(into)}"
      e 'Also storing into:'
      e
      e "  #{sfile(into)}"
      e
      write_what_into(what, into)
    end
  end; alias create_yaml_file save_yaml_file # === create_yaml_file

  # ========================================================================= #
  # === search_for_this_program
  #
  # Use this method to search for a specific program.
  # ========================================================================= #
  def search_for_this_program(
      i = @this_program
    )
    i = i.to_s
    opne "Searching for #{sfancy(i)} now:"
    hash = load_yaml(location_registered_m4_files?)
    if hash.keys.include? i
      opne "The m4 file called `#{sfancy(i)}` was found."
      it_is_part_of_the_package(hash[i])
    else
      opne p+' is '+swarn('NOT')+' included. We could not '\
           'find any program'
      opne 'including this program.'
    end
  end

end; end

# =========================================================================== #
# === RBT.registered_m4_files?
# =========================================================================== #
def self.registered_m4_files?
  RBT::Cookbooks::RegisteredM4Files.new
end

# =========================================================================== #
# === RBT.is_this_a_registered_m4_file?
# =========================================================================== #
def self.is_this_a_registered_m4_file?(i = 'pkg.m4')
  if File.exist? file_registered_m4_files
    dataset = YAML.load_file(file_registered_m4_files)
    dataset.has_key?(i)
  else
    false
  end
end; self.instance_eval { alias is_this_m4_file_included? is_this_a_registered_m4_file? } # === RBT.is_this_m4_file_included?

# =========================================================================== #
# === RBT.return_the_associated_program_of_this_m4_file
#
# Invocation example:
#
#   RBT.return_the_associated_program_of_this_m4_file('pkg.m4') # => "libxau"
#
# =========================================================================== #
def self.return_the_associated_program_of_this_m4_file(i)
  YAML.load_file(file_registered_m4_files)[i]
end; self.instance_eval { alias return_the_associated_program_for_this_m4_file return_the_associated_program_of_this_m4_file } # === RBT.return_the_associated_program_for_this_m4_file

end

if __FILE__ == $PROGRAM_NAME
  _ = RBT::Cookbooks::RegisteredM4Files.new(:do_not_run_yet)
  if ARGV.first
    _.search_for_this_program(ARGV.first)
  else
    _.run
  end
end # regm4files