#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::Sinatra
#
# To subclass, try:
#
#   require 'rbt/sinatra/app.rb'
#   class Foobar < RBT::Base
#
# =========================================================================== #
# require 'rbt/sinatra/app.rb'
# RBT.run_app
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

require 'sinatra'
require 'open-uri'

class Sinatra < ::Sinatra::Base

  require 'json'
  require 'rbt/requires/require_word_wrap.rb'
  require 'rbt/toplevel_methods/available_programs.rb'
  require 'rbt/toplevel_methods/report_extended_information_about_this_program.rb'
  require 'rbt/url_action/url_action.rb'

  require 'rbt/sinatra/action_available_programs.rb'
  require 'rbt/sinatra/action_create.rb'
  require 'rbt/sinatra/action_view.rb'
  require 'rbt/sinatra/toplevel_methods.rb'

  begin
    require 'beautiful_url'
  rescue LoadError; end

  require 'colours'
  include Colours::E
  include Colours

  require 'html_tags'
  include HtmlTags::BaseModule

  # ========================================================================= #
  # === USE_THIS_PORT
  # ========================================================================= #
  USE_THIS_PORT = '5580'

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Web-API for the RBT project'

  # ========================================================================= #
  # === IS_ROEBE
  # ========================================================================= #
  IS_ROEBE = true

  set :port, USE_THIS_PORT

  # ========================================================================= #
  # Try to enable sessions:
  # ========================================================================= #
  enable :sessions

  # ========================================================================= #
  # === before                                                     (before tag)
  # ========================================================================= #
  before {
    content_type(:html, 'charset' => 'UTF-8')
  }

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize
    super()
    reset
    if IS_ROEBE
      require 'open'
      # ===================================================================== #
      # Tell us which port will be used:
      # ===================================================================== #
      target = "http://localhost:#{USE_THIS_PORT}/"
      e sfancy(target)
      Thread.new {
        sleep 1.0
        Open.in_browser(target)
      }
    end
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
  end

  # ========================================================================= #
  # === / Entry point
  #
  # This is the root of our web-application.
  # ========================================================================= #
  get('/') {
    return_main_index
  }

  # ========================================================================= #
  # === return_to_root_link
  # ========================================================================= #
  def return_to_root_link
    p {
      a('/', content: 'Home')
    }
  end

  # ========================================================================= #
  # === return_html_head_and_title_and_body
  # ========================================================================= #
  def return_html_head_and_title_and_body
    '<html><head><title>'+TITLE+'</title></head><body>'
  end

  # ========================================================================= #
  # === return_main_index
  #
  # This method will return the main "index" - the first "page" that
  # someone sees when using the sinatra-interface.
  # ========================================================================= #
  def return_main_index
    return_html_head_and_title_and_body+
    'This is the <b>www-interface</b> for the RBT '\
    'project, using <b>sinatra</b>.'+
    return_available_actions
  end

  # ========================================================================= #
  # === return_available_actions
  #
  # This method will list the available (registered) actions for
  # the web-interface.
  # ========================================================================= #
  def return_available_actions
    p('The following entry points are available:')+
    p(css_style: 'padding:10px; margin-left: 1em') {
      # ===================================================================== #
      # === /view
      # ===================================================================== #
      abr('/view',     content: '/view',   css_style: :bold)+
      # ===================================================================== #
      # === /create
      # ===================================================================== #
      abr('/create',   content: '/create', css_style: :bold)+
      # ===================================================================== #
      # === /available_programs
      # ===================================================================== #
      abr('/available_programs', content: '/available_programs', css_style: :bold)+
      # ===================================================================== #
      # === /install
      # ===================================================================== #
      abr('/install',  content: '/install', css_style: :bold) # /install
    }
  end

  # ========================================================================= #
  # === abr
  # ========================================================================= #
  def abr(i, hash)
    HtmlTags.a(i, hash)+br
  end

  # ========================================================================= #
  # === /install
  #
  # http://localhost:5580/install
  # ========================================================================= #
  get('/install') {
    route_to_this_action = '/install/'
    body = div(css_style: 'padding: 0.2em') {
      h1('<b>Install this program:</b>')+
      p(
        'This entry point can be used to install a program.',
        css_style: 'padding:0.55em; margin: 1px;'
      )+
      '<form id="view" style="margin-left:2em; margin-top:2px" action="'+route_to_this_action+'">
       <input type="text" name="user_input" style="border:3px solid slateblue; padding: 4px"><br>
       <input type="submit" name="user_input_submit" value="Display" style="2px dotted royalblue; '\
       'color: white; background-color: olive; font-size: 1.50em; margin:2px; padding: 4px">
       </form>'+
      return_to_root_link
    }
    return Cyberweb::HtmlTemplate[
      title: 'Install a program',
      body:  body
    ].to_s
  }

  # ========================================================================= #
  # === /install/*                                              (install tag)
  #
  # Usage example:
  #
  #   http://localhost:5580/install/htop
  #
  # ========================================================================= #
  get('/install/*'){
    use_these_params = request.path_info
    if params.has_key?('user_input')
      use_these_params = params.fetch('user_input')
    end
    result = ''.dup
    if use_these_params.is_a? Array
      use_these_params = use_these_params.flatten.join(' ').strip
    end
    result << RBT::UrlAction.new(use_these_params).to_s
    Cyberweb::HtmlTemplate[
      css_class: 'darkgreen',
      title: 'Installing the program '+use_these_params.to_s,
      body: result
    ].to_s
  }

end; end

if __FILE__ == $PROGRAM_NAME
  RBT.run_app
end