#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'
# pp RBT.get_all_directories_from('/home/x/src/')
# pp RBT.temp_directory?
# pp RBT.programs_directory?
# =========================================================================== #
module RBT

  require 'fileutils'
  require 'rbt/requires/require_colours_and_opn.rb'
  require 'rbt/constants/constants.rb'
  require 'rbt/constants/configuration_directory.rb'
  require 'rbt/toplevel_methods/convert_global_env.rb'
  require 'rbt/toplevel_methods/opn.rb'
  require 'rbt/toplevel_methods/rds.rb'
  require 'rbt/colours/rev_simp_sdir_sfancy_sfile_and_swarn.rb'

  # ========================================================================= #
  # === @cookbook_directory
  #
  # This variable will keep track where the individual cookbooks
  # can be found.
  # ========================================================================= #
  @cookbook_directory = 
    "#{RBT.project_yaml_directory?}cookbooks/"

  # ========================================================================= #
  # === RBT.cookbook_directory?
  #
  # Provide a pointer, through this query method, towards the directory
  # where the individual cookbooks are kept.
  # ========================================================================= #
  def self.cookbook_directory?
    @cookbook_directory
  end; self.instance_eval { alias cookbook_directory              cookbook_directory? } # === RBT.cookbook_directory
       self.instance_eval { alias cookbooks_directory?            cookbook_directory? } # === RBT.cookbooks_directory?
       self.instance_eval { alias cookbooks_dir?                  cookbook_directory? } # === RBT.cookbooks_dir?
       self.instance_eval { alias individual?                     cookbook_directory? } # === RBT.individual?
       self.instance_eval { alias individual_cookbooks?           cookbook_directory? } # === RBT.individual_cookbooks?
       self.instance_eval { alias individual_cookbooks_dir?       cookbook_directory? } # === RBT.individual_cookbooks_dir?
       self.instance_eval { alias individual_cookbooks_directory? cookbook_directory? } # === RBT.individual_cookbooks_directory?

  # ========================================================================= #
  # === RBT.set_cookbook_directory
  #
  # This setter-method can be used to determine where the individual
  # cookbooks can be found normally.
  # ========================================================================= #
  def self.set_cookbook_directory(i)
    if i # ← Assignment is only done if the user supplies a non-nil argument.
      unless i.end_with? '/'
        i = i.dup if i.frozen?
        i << '/'
      end
      @cookbook_directory = i
    end
  end

  # ========================================================================= #
  # === @temp_directory
  #
  # We set @temp_directory to an initial value, which may be changed at
  # a later time. This should come fairly early on in this .rb file.
  # ========================================================================= #
  @temp_directory = '/tmp/'

  # ========================================================================= #
  # === @programs_directory
  # ========================================================================= #
  @programs_directory = nil

  # ========================================================================= #
  # === RBT.set_programs_directory
  # ========================================================================= #
  def self.set_programs_directory(
      i = :use_value_from_the_configuration_file # This is hardcoded default, but can be changed by the user through later calls.
    )
    case i
    # ======================================================================= #
    # === :use_value_from_the_configuration_file
    # ======================================================================= #
    when :use_value_from_the_configuration_file
      use_this_file = RBT.file_programs_directory
      if File.exist? use_this_file
        i = YAML.load_file(use_this_file)
      else # else assume e. g. windows.
        i = '/home/Programs/'
      end
    end
    i = i.dup if i.frozen?
    # ======================================================================= #
    # Replace all $ with proper env-variables if they are included.
    # ======================================================================= #
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case we will use a hardcoded entry instead. It is
    # up to the user to resolve this problem, as it should not really
    # happen.
    # ======================================================================= #
    i = '/home/Programs/' if i.include? '$' # ← See above for the explanation.
    i = i.dup if i.frozen? # ← Need to do so again, e. g. in a .cgi environment context.
    i << '/' unless i.end_with? '/'
    i.squeeze!('/')
    @programs_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  set_programs_directory

  # ========================================================================= #
  # === RBT.programs_directory?
  #
  # Query the value of the toplevel @programs_directory instance variable.
  # ========================================================================= #
  def self.programs_directory?
    @programs_directory
  end; self.instance_eval { alias programs_dir?             programs_directory? } # === RBT.programs_dir?
       self.instance_eval { alias programs_dir              programs_directory? } # === RBT.programs_dir
       self.instance_eval { alias programs_directory?       programs_directory? } # === RBT.programs_directory?
       self.instance_eval { alias programs_directory        programs_directory? } # === RBT.programs_directory
       self.instance_eval { alias program_directory         programs_directory? } # === RBT.program_directory
       self.instance_eval { alias program_directory?        programs_directory? } # === RBT.program_directory?
       self.instance_eval { alias program_dir?              programs_directory? } # === RBT.program_dir?
       self.instance_eval { alias program_dir               programs_directory? } # === RBT.program_dir
       self.instance_eval { alias return_programs_directory programs_directory? } # === RBT.return_programs_directory

  # ========================================================================= #
  # === @system_directory
  # ========================================================================= #
  @system_directory = nil

  # ========================================================================= #
  # === RBT.system_directory?
  # ========================================================================= #
  def self.system_directory?
    @system_directory
  end; self.instance_eval { alias system_dir?             system_directory? } # === RBT.system_dir?
       self.instance_eval { alias return_system_directory system_directory? } # === RBT.return_system_directory

  # ========================================================================= #
  # === RBT.get_files_and_directories_from
  #
  # Simply combine RBT.get_all_directories_from and RBT.get_files_from
  # via this method here.
  # ========================================================================= #
  def self.get_files_and_directories_from(
      here = return_pwd
    )
    _ = get_all_directories_from(here)+
        get_all_files_from(here)
    return _.flatten
  end

  # ========================================================================= #
  # === RBT.chmod                                                 (chmod tag)
  #
  # Use this method to chmod a file or a directory. It also works with
  # several files/directories passed in as Array.
  # ========================================================================= #
  def self.chmod(
      this_file,
      permission = 0755
    )
    if this_file.is_a? Array
      this_file.each { |file| chmod(file, permission) }
    else
      File.chmod(permission.to_i, this_file)
    end
  end

  # ========================================================================= #
  # === RBT.change_directory
  # ========================================================================= #
  def self.change_directory(i)
    Dir.chdir(i)
  end; self.instance_eval { alias cd    change_directory } # === RBT.cd (cd tag)
       self.instance_eval { alias chdir change_directory } # === RBT.chdir

  # ========================================================================= #
  # === RBT.return_pwd
  #
  # This method will return the current working directory.
  # ========================================================================= #
  def self.return_pwd
    begin
      ("#{Dir.pwd}/").squeeze '/'
    rescue
      './'.dup # Assume this to be used in the current directory then.
    end
  end

  # ========================================================================= #
  # === RBT.create_log_directory
  #
  # This method will create the log directory, e. g. "/tmp/rbt/" or
  # wherever else the user has set this directory to be at.
  # ========================================================================= #
  def self.create_log_directory(
      _ = log_directory?
    )
    if _
      unless !File.directory?(_)
        opn; e "Creating the directory `#{sdir(_)}` now."
        FileUtils.mkdir_p(_)
      else
        opn; e "The directory at `#{sdir(_)}` already exists."
      end
    else
      opn; e 'No directory could be created as no input was given.'
    end
  end; self.instance_eval { alias create_log_dir create_log_directory } # === RBT.create_log_dir

  # ========================================================================= #
  # === RBT.return_all_archives_from_this_directory
  #
  # This method will return all archives from a given directory.
  # ========================================================================= #
  def self.return_all_archives_from_this_directory(i)
    i = i.first if i.is_a? Array
    i = i.dup if i.frozen?
    if i and File.directory?(i)
      i << '/' unless i.end_with? '/'
    end
    potential_matches = [] 
    ARRAY_REGISTERED_ARCHIVES.each {|file_format|
      potential_matches << Dir["#{i}*#{file_format}"]
    }
    potential_matches.flatten.compact
  end

  # ========================================================================= #
  # === RBT.get_all_directories_from
  #
  # This will return all directories.
  #
  # Invocation example:
  #
  #   RBT.get_all_directories_from('/home/x/src/')
  #
  # ========================================================================= #
  def self.get_all_directories_from(
      here             = return_pwd,
      return_full_path = false
    )
    here << '/' unless here.end_with? '/'
    _ = Dir["#{here}*"]
    _.select! {|entry| File.directory?(entry) }
    result = _.map {|entry| "#{entry}/" }
    if return_full_path
      result.map! {|line| File.absolute_path(line) }
    end
    return result.sort # Keep the result sorted.
  end; self.instance_eval { alias get_dir_listing       get_all_directories_from } # === RBT.get_dir_listing
       self.instance_eval { alias get_directory_listing get_all_directories_from } # === RBT.get_directory_listing
       self.instance_eval { alias get_directories_from  get_all_directories_from } # === RBT.get_directories_from

  # ========================================================================== #
  # === RBT.ensure_that_the_required_directories_exist
  #
  # This method ensures that the base-directory exists.
  #
  # This can also be invoked from the commandline, by issuing:
  #
  #   rbt --ensure-that-the-required-directories-exist
  #
  # ========================================================================== #
  def self.ensure_that_the_required_directories_exist(
      use_this_as_directory = RBT.store_into_this_directory?,
      be_verbose            = false
    )
    if block_given?
      yielded = yield
      case yielded
      when :be_verbose
        be_verbose = true
      when :be_quiet
        be_verbose = false
      end
    end
    if Dir.exist?(use_this_as_directory)
      if be_verbose
        opn; e "The directory at `#{use_this_as_directory}` "\
               "already exists."
      end
    else
      target = File.dirname(use_this_as_directory)
      if File.writable?(target)
        require 'fileutils'
        opn; e "Creating directory `#{use_this_as_directory}`."
        FileUtils.mkdir_p(use_this_as_directory)
      else
        e "Can not write into the directory "\
          "`#{use_this_as_directory}` as it is not writable. "\
          "Consider using"
        e 'another setting for temp_directory.yml perhaps.'
        e
        e 'This file can be found here:'
        e
        e "  #{file_temp_directory}"
        e
      end
    end
  end

  # ========================================================================= #
  # === RBT.remove_file
  #
  # Use this method to remove a file, but check if the file exists
  # before actually deleting it. Additionally, also check for proper
  # file permissions permissions as well.
  # ========================================================================= #
  def self.remove_file(
      file,
      report_action = false
    )
    file = file.dup if file.frozen?
    file.squeeze!('/') # We do not want double '/' entries.
    case report_action
    # ======================================================================= #
    # === :be_quiet
    # ======================================================================= #
    when :be_quiet
      report_action = false
    # ======================================================================= #
    # === :be_verbose
    # ======================================================================= #
    when :be_verbose
      report_action = true
    end
    can_we_delete_this_entry = false
    can_we_delete_this_entry = true if File.exist? file
    begin
      case File.ftype(file)
      when 'link'
        can_we_delete_this_entry = true
      end
    rescue Errno::ENOENT
      can_we_delete_this_entry = true # Failsafe.
    end
    # ======================================================================= #
    # We also have to handle symlinks, aka 'link', entries. These may no
    # longer exist, but we still will to delete such stray links, hence
    # why the above check has to be the way it is.
    # ======================================================================= #
    if can_we_delete_this_entry
      # ===================================================================== #
      # We need to check whether we can write to the file at hand.
      # ===================================================================== #
      base_directory = File.dirname(file)
      if File.writable? base_directory
        unless File.directory? file # Safeguard.
          if report_action
            e "Now removing the file `#{sfile(file)}`."
          end
          if File.exist? file
            File.delete(file) # Remove the file here.
          else
            if report_action
              e "No file exists at `#{sfile(file)}`."
            end
          end
        end
      else
        opnn(__FILE__)
        ewarn "Insufficient permission to remove file #{sfile(file)}"
      end
    else # Not sure if we should report this.
      if report_action
        opnn(__FILE__)
        ewarn "Trying to remove a file, but the file #{sfile(where)}"
        ewarn ' does not exist.'
      end    
    end
  end

  # ========================================================================= #
  # === RBT.create_directory
  #
  # Internal Wrapper to create a directory.
  #
  # We will also check if a Dir exists before actually creating it. This
  # is the method of choice for creating a directory. Use it for all the
  # RBT scripts, whenever you need to create a directory.
  # ========================================================================= #
  def self.create_directory(
      where         = '/Depot/Packages/',
      use_this_mode = 0755,
      be_verbose    = :be_quiet,
      do_use_opnn   = true,
      &block
    )
    _yielded = nil # Not currently in use as of Dec 2022.
    if block_given?
      _yielded = yield
    end
    case use_this_mode
    when :be_verbose
      be_verbose = :be_verbose
      use_this_mode = :default
    end
    if where.is_a? Array
      where.each {|entry|
        create_directory(
          entry, use_this_mode, be_verbose, do_use_opnn
        )
      }
    else
      if File.directory? where
        unless where.end_with? '/' # Append a /.
          where = where.dup if where.frozen?
          where << '/'
        end
      end
      case use_this_mode
      when :default # Default value is 0755.
        use_this_mode = 0755
      when Hash
        if use_this_mode.has_key? :verbosity
          use_this_mode = use_this_mode.delete(:verbosity)
        end
        use_this_mode = 0755
      when '', true
        use_this_mode = 0755
      when :be_verbose
        use_this_mode = 0755
        be_verbose = true
      end
      # ===================================================================== #
      # Handle special symbols given to be_verbose.
      # ===================================================================== #
      case be_verbose
      when Hash
        # =================================================================== #
        # === :verbosity
        # =================================================================== #
        if be_verbose.has_key? :verbosity
          be_verbose = be_verbose.delete(:verbosity)
        end
      # ===================================================================== #
      # === :be_quiet
      # ===================================================================== #
      when :be_quiet,
           :be_silent
        be_verbose = false
      end
      case do_use_opnn
      when :do_not_use_opn,
           :do_not_use_opnn
        do_use_opnn = false
      end
      if File.exist? where
        if be_verbose
          opnn if do_use_opnn
          e "Directory `#{sdir(where)}` already exists, thus"
          opnn if do_use_opnn
          e 'skipping creating a new one.'
        end
      else
        if where
          if be_verbose
            opnn if do_use_opnn
            e "Creating the directory `#{sdir(where)}` next."
          end
          unless File.directory? where
            options_hash = {
              mode: use_this_mode, verbose: false
            }
            # =============================================================== #
            # The variable where can be a String such as 
            # "/home/Programs/Tepl/Current/yaml/". We have
            # to keep in mind that the mkdir-part may fail,
            # so a rescue clause has to be used.
            # =============================================================== #
            begin
              FileUtils.mkdir_p(where, **options_hash)
            rescue Errno::EEXIST
              e "The directory `#{sdir(where)}` can not be created"
              e 'as some target, or an incorrect symlink, '\
                'already exists.' 
            end
            # if be_verbose
            #   opnn if do_use_opnn
            #   e "Can not write into the directory `#{sdir(where)}` "\
            #     "due to insufficient permissions."
            # end
          end
        else
          opnn if do_use_opnn; e 'No directory was given.'
        end
      end
    end
  end; self.instance_eval { alias mkdir   create_directory } # === RBT.mkdir
       self.instance_eval { alias mkdir_p create_directory } # === RBT.mkdir_p

  # ========================================================================= #
  # === RBT.report_where_the_cookbook_directory_can_be_found
  #
  # This method will report where the cookbook directory can be found.
  # ========================================================================= #
  def self.report_where_the_cookbook_directory_can_be_found
    opn(namespace: RBT::NAMESPACE[:namespace])
    e "#{rev}The cookbook directory can be found at this location:"
    e
    e Colours.sfancy("  #{RBT.individual_cookbooks_directory?}")+
      rev
    e
  end

  # ========================================================================= #
  # === RBT.temp_directory?
  #  
  # Preferentially use this method if you wish to find out the path
  # to the temporary directory in use by the RBT project.
  # ========================================================================= #
  def self.temp_directory?
    @temp_directory
  end; self.instance_eval { alias temp_directory temp_directory? } # === RBT.temp_directory
       self.instance_eval { alias temp_dir?      temp_directory? } # === RBT.temp_dir?
       self.instance_eval { alias rbt_temp_dir?  temp_directory? } # === RBT.rbt_temp_dir?

  # ========================================================================= #
  # === RBT.file_temp_directory
  #
  # This reader method will return the location to the file named
  # "temp_directory.yml", as a String.
  #
  # On my system, the method may return a String such as
  #
  #   "/Programs/Ruby/2.5.1/lib/ruby/site_ruby/2.5.0/rbt/yaml/configuration/temp_directory.yml"
  #
  # ========================================================================= #
  def self.file_temp_directory
     "#{RBT.configuration_directory?}"\
     "temp_directory.yml"
  end; self.instance_eval { alias file_temp_directory_yaml file_temp_directory } # === RBT.file_temp_directory_yaml

  # ========================================================================= #
  # === RBT.set_temp_directory
  # ========================================================================= #
  def self.set_temp_directory(
      i = '/tmp/'
    )
    i = convert_global_env(i) if i.include? '$'
    if i.include? '$'
      i = '/tmp/' # Use a default value in this case.
    end
    unless i.end_with? '/' # We must ensure that the last charcter is always a '/'.
      i = i.dup if i.frozen?
      i << '/' 
    end
    @temp_directory = i
  end; self.instance_eval { alias set_temp_dir set_temp_directory } # === RBT.set_temp_dir

  # ========================================================================= #
  # === RBT.permanently_set_temp_directory
  #
  # Use this method if you wish to permanently modify the temp directory,
  # that is, the directory we will extract to (or to a subdirectory in
  # that directory).
  # ========================================================================= #
  def self.permanently_set_temp_directory(i)
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default,
         :pwd,
         'pwd'
      i = return_pwd.dup # Return the current working directory here.
    # ======================================================================= #
    # === :home_directory
    # ======================================================================= #
    when :home_directory
      i = (ENV['HOME'].to_s+'/').squeeze('/')
    end
    # ======================================================================= #
    # Next, ensure trailing '/' character - unless the input includes
    # a '$' character.
    # ======================================================================= #
    unless i.include? '$'
      i << '/' unless i.end_with? '/'
    end
    opn; e "Setting the new temp directory to point towards `#{sdir(i)}`."
    # ======================================================================= #
    # Modify the yaml file next:
    # ======================================================================= #
    what = i
    into = RBT.file_temp_directory_yaml
    RBT.write_what_into(what, into)
    # ======================================================================= #
    # And also modify it on my home system, if IS_ROEBE is set to true.
    # ======================================================================= #
    if is_on_roebe?
      into = "#{RUBY_SRC_DIR_RBT_YAML_AT_HOME}configuration/"\
             "#{File.basename(into)}"
      RBT.write_what_into(what, into)
    end
    RBT.set_temp_directory(i) # And also sync it.
    unless File.directory? i
      opne 'Since that directory does not exist, it will be created now.'
      mkdir(i) { :be_quiet }
    end
    i # Also return the variable, so we can re-use it in other methods.
  end; self.instance_eval { alias set_permanently_extract_to permanently_set_temp_directory } # === RBT.set_permanently_extract_to
       self.instance_eval { alias permanently_extract_to     permanently_set_temp_directory } # === RBT.permanently_extract_to

  # ========================================================================= #
  # === RBT.set_system_directory
  # ========================================================================= #
  def self.set_system_directory(
      i = '/System/'
    )
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case, use a hardcoded entry instead.
    # ======================================================================= #
    i = '/System/' if i.include? '$' # ← See above for the explanation.
    i << '/' unless i.end_with? '/'
    i = i.dup if i.frozen? # ← Need to do so again, e. g. in a .cgi environment context.
    i.squeeze!('/')
    @system_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  set_system_directory

  # ========================================================================= #
  # === @sysbin_directory
  # ========================================================================= #
  @sysbin_directory = nil

  # ========================================================================= #
  # === RBT.set_sysbin_directory
  # ========================================================================= #
  def self.set_sysbin_directory(
      i = '/System/Index/bin/'
    )
    require 'rbt/toplevel_methods/convert_global_env.rb'
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case, use a hardcoded entry instead.
    # ======================================================================= #
    i = '/System/Index/bin/' if i.include? '$' # <- See above for the explanation.
    i << '/' unless i.end_with? '/'
    i = i.dup if i.frozen? # <- Need to do so again, e. g. in a .cgi environment context.
    i.squeeze!('/')
    @sysbin_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  set_sysbin_directory

  # ========================================================================= #
  # === RBT.sysbin_directory?
  # ========================================================================= #
  def self.sysbin_directory?
    @sysbin_directory
  end; self.instance_eval { alias sysbin_dir?             sysbin_directory? } # === RBT.sysbin_dir?
       self.instance_eval { alias return_sysbin_directory sysbin_directory? } # === RBT.return_sysbin_directory

  # ========================================================================= #
  # === @source_base_directory
  #
  # This variable keeps track as to where all source archives are usually
  # kept. This is thus a fairly important variable.
  #
  # We will set it to a default value on "startup". The user can
  # change this setting lateron.
  #
  # On my home system this defaults to /home/x/src/.
  # ========================================================================= #
  if ENV['USERS']
    @source_base_directory = ENV['USERS'].to_s+'/x/src/'
  else
    @source_base_directory = '/home/x/src/' # ← Hardcoded default value.
  end

  # ========================================================================= #
  # === RBT.file_source_directory
  # ========================================================================= #
  def self.file_source_directory
    "#{RBT.configuration_directory?}source_directory.yml"
  end

  # ========================================================================= #
  # === RBT.source_base_directory?
  #
  # This is a simple query-method for the source directory in use (called
  # "source base directory").
  #
  # The default value to it is "/home/x/SRC/" on my system - which may be
  # different on other computers, which is why this has to be settable
  # and modifiable by the user via the setter method called
  # RBT.set_source_base_directory().
  # ========================================================================= #
  def self.source_base_directory?
    @source_base_directory
  end; self.instance_eval { alias source_directory? source_base_directory? } # === RBT.source_directory?
       self.instance_eval { alias source_dir?       source_base_directory? } # === RBT.source_dir?
       self.instance_eval { alias src_dir?          source_base_directory? } # === RBT.src_dir?

  # ========================================================================= #
  # === RBT.set_source_base_directory
  #
  # Note that the environment variable called RBT_SOURCE_DIRECTORY can
  # be used to use another default value.
  # ========================================================================= #
  def self.set_source_base_directory(
      i = :default_but_consider_environment_variables # <- Preferred default. 
    )
    case i
    # ======================================================================= #
    # === :default_but_consider_environment_variables
    # ======================================================================= #
    when :default_but_consider_environment_variables
      # ===================================================================== #
      # Handle the environment-variable RBT_SOURCE_DIRECTORY first, as
      # it has a higher priority in this case.
      # ===================================================================== #
      if ENV.has_key? 'RBT_SOURCE_DIRECTORY'
        i = ENV['RBT_SOURCE_DIRECTORY'].to_s.dup
      else
        # =================================================================== #
        # else the user does not have this ENV-variable, so we use the
        # "regular" default.
        # =================================================================== #
        i = @source_base_directory
      end
    # ======================================================================= #
    # This is for the "regular" default. This one will always assign.
    # ======================================================================= #
    when :default, nil, ''
      i = @source_base_directory
    end
    i = i.dup if i.frozen?
    i = RBT.convert_global_env(i) if i.include? '$'
    i = @source_base_directory if i.include? '$' # <- Overrule missing environment variables.
    i << '/' unless i.end_with? '/'
    @source_base_directory = i
  end; self.instance_eval { alias set_source_directory set_source_base_directory } # === RBT.set_source_directory
       self.instance_eval { alias set_source_dir       set_source_base_directory } # === RBT.set_source_dir
       self.instance_eval { alias source_directory=    set_source_base_directory } # === RBT.source_directory=

  # ========================================================================= #
  # And set it at once to a default, for my home directory. This will
  # default to the path /home/x/src/.
  # ========================================================================= #
  set_source_base_directory(:default)

  # ========================================================================= #
  # === RBT.permanently_set_source_directory
  #
  # Use this method if you wish to permanently modify the source directory,
  # that is, the directory where our archives will be stored.
  #
  # Invocation example from the commandline:
  #
  #   rbt --set_source_dir=/opt
  #   rbt --permanently-set-base-directory-to=/opt
  #
  # ========================================================================= #
  def self.permanently_set_source_directory(i)
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    unless i.start_with? '/'
      i.prepend('/') # We require leading '/'.
    end
    # ======================================================================= #
    # Modify the yaml file next:
    # ======================================================================= #
    what = i
    into = RBT.file_source_directory
    write_what_into(what, into)
    if block_given?
      yielded = yield
      case yielded
      when :be_verbose
        e "We will permanently set to a new source "\
          "directory next (#{sdir(i)})"
      end
    end
    opn; e "Setting the new source directory to point towards `#{sdir(what)}`."
    opn; e "Storing into the file `#{sfile(into)}` next."
    # ======================================================================= #
    # And also modify it on my home system, if IS_ROEBE is true. This will
    # go right into the configuration/ subirectory.
    # ======================================================================= #
    if is_on_roebe?
      into = "#{RUBY_SRC_DIR_RBT}yaml/configuration/"\
             "#{File.basename(into)}"
      opn; e "Will also store into the file `#{sfile(into)}`."
      RBT.write_what_into(what, into)
    end
    RBT.set_source_base_directory(i) # And also sync it back.
    unless File.directory? i
      opne 'Since that directory does not exist, it will be created now.'
      mkdir(i) { :be_quiet }
    end
  end; self.instance_eval { alias permanently_set_path_to_this_source_directory permanently_set_source_directory } # === RBT.permanently_set_path_to_this_source_directory
       self.instance_eval { alias permanently_set_source_dir                    permanently_set_source_directory } # === RBT.permanently_set_source_dir
       self.instance_eval { alias permanently_set_source_dir=                   permanently_set_source_directory } # === RBT.permanently_set_source_dir=
       self.instance_eval { alias permanently_set_source_directory=             permanently_set_source_directory } # === RBT.permanently_set_source_directory=

  # ========================================================================= #
  # === @sysetc_directory
  # ========================================================================= #
  @sysetc_directory = nil

  # ========================================================================= #
  # === RBT.set_sysetc_directory
  # ========================================================================= #
  def self.set_sysetc_directory(
      i = '/System/Index/etc/'
    )
    require 'rbt/toplevel_methods/convert_global_env.rb'
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case, use a hardcoded entry instead.
    # ======================================================================= #
    i = '/System/Index/etc/' if i.include? '$' # <- See above for the explanation.
    i << '/' unless i.end_with? '/'
    i = i.dup if i.frozen? # <- Need to do so again, e. g. in a .cgi environment context.
    i.squeeze!('/')
    @sysetc_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  set_sysetc_directory

  # ========================================================================= #
  # === RBT.sysetc_directory?
  # ========================================================================= #
  def self.sysetc_directory?
    @sysetc_directory
  end; self.instance_eval { alias sysetc_dir?             sysetc_directory? } # === RBT.sysetc_dir?
       self.instance_eval { alias return_sysetc_directory sysetc_directory? } # === RBT.return_sysetc_directory

  # ========================================================================= #
  # === @log_directory
  #
  # Specify where the rbt-log directory should be.
  #
  # This instance-variable depends on @temp_directory.
  # ========================================================================= #
  @log_directory = "#{@temp_directory}rbt/"

  # ========================================================================= #
  # === RBT.log_directory?
  #
  # Query method for the @log_directory variable. Prefer to use this method
  # or any of its aliases rather than the constant called LOG_DIR.
  #
  # On my home system this usually defaults to "/home/Temp/rbt/".
  # ========================================================================= #
  def self.log_directory?
    @log_directory
  end; self.instance_eval { alias log_directory              log_directory? } # === RBT.log_directory
       self.instance_eval { alias rbt_temp_directory?        log_directory? } # === RBT.rbt_temp_directory?
       self.instance_eval { alias rbt_log_directory?         log_directory? } # === RBT.rbt_log_directory?
       self.instance_eval { alias rbt_log_dir?               log_directory? } # === RBT.rbt_log_dir?
       self.instance_eval { alias log_dir?                   log_directory? } # === RBT.log_dir?
       self.instance_eval { alias log_dir                    log_directory? } # === RBT.log_dir
       self.instance_eval { alias store_into_this_directory? log_directory? } # === RBT.store_into_this_directory?
       self.instance_eval { alias store_into_this_directory  log_directory? } # === RBT.store_into_this_directory
       self.instance_eval { alias store_where?               log_directory? } # === RBT.store_where?

  # ========================================================================= #
  # === RBT.set_log_directory
  #
  # This defaults to "/Depot/Temp/rbt/" on my system.
  # ========================================================================= #
  def self.set_log_directory(
      i = "#{RBT.temp_directory}rbt/"
    )
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    i << '/' unless i.end_with? '/'
    # ======================================================================= #
    # Next, we must ensure that the required directories exist.
    # ======================================================================= #
    i = i.dup if i.frozen? # <- Need to do so again, e. g. in a .cgi environment context.
    # ======================================================================= #
    # In the past we did ensure that the required directories exist,
    # whenever RBT.set_log_directory() is called. But this does not
    # make that much sense, and can lead to other problems, so it
    # was disabled here at 24.01.2019. Do note that the functionality
    # of ensuring that the required directories exist will still be
    # handled by RBT, but in other code parts instead.
    # ======================================================================= #
    # require 'rbt/toplevel_methods/ensure_that_the_required_directories_exist.rb'
    # ^^^ this used to be on top
    # ======================================================================= #
    # ensure_that_the_required_directories_exist(
    #   i
    # )
    @log_directory = i
  end

  RBT.set_log_directory # Set it up once.

  # ========================================================================= #
  # === RBT.store_the_last_configuration_options_in_this_file
  #
  # You can also use this alias-method for invoking the method:
  #
  #   RBT.store_the_last_configuration_options_into_which_file? # => "/Depot/Temp/rbt/last_configure_options_used.md"
  #
  # ========================================================================= #
  def self.store_the_last_configuration_options_in_this_file
    "#{RBT.store_into_this_directory?}"\
    "last_configure_options_used_through_class_show_configuration_options.md"
  end; self.instance_eval { alias store_the_last_configuration_options_into_which_file? store_the_last_configuration_options_in_this_file } # === RBT.store_the_last_configuration_options_into_which_file?
       self.instance_eval { alias file_last_configure_options                           store_the_last_configuration_options_in_this_file } # === RBT.file_last_configure_options

  # ========================================================================= #
  # === RBT.file_is_on_gobolinux
  #
  # This file may be generated by RBT if you issue any of the following
  # commands:
  #
  #   rbt --is-on-gobolinux
  #   rbt --isongobolinux
  #   rbt isongobolinux
  #
  # ========================================================================= #
  def self.file_is_on_gobolinux
    "#{RBT.log_directory?}is_on_gobolinux"
  end

  # ========================================================================= #
  # === RBT.file_storing_the_time_it_took_to_compile_programs
  #
  # Since as of August 2022 this is now a query-method.
  #
  # This method can be used to query where the yaml-"database" is kept
  # that contains our compile-times.
  # ========================================================================= #
  def self.file_storing_the_time_it_took_to_compile_programs 
    "#{RBT.log_directory?}database_storing_compile_times.yml"
  end

  # ========================================================================= #
  # === RBT.file_programs_version_url
  #
  # This is the path to the .md file that keeps track of the
  # programs-version and URL entries.
  # ========================================================================= #
  def self.file_programs_version_url
    "#{RBT.log_dir?}CookbookDirectory/programs_version_url.md"
  end

  # ========================================================================= #
  # === RBT.file_last_symlinked_program?
  # ========================================================================= #
  def self.file_last_symlinked_program?
    "#{RBT.log_directory?}last_symlinked_program.yml"
  end

  # ========================================================================= #
  # === RBT.file_last_compiled_program
  # ========================================================================= #
  def self.file_last_compiled_program
    "#{RBT.log_directory?}last_compiled_programs.yml"
  end

  # ========================================================================= #
  # === @packages_directory
  # ========================================================================= #
  @packages_directory = nil

  # ========================================================================= #
  # === RBT.set_packages_directory
  # ========================================================================= #
  def self.set_packages_directory(
      i = '/Depot/Packages/'
    )
    require 'rbt/toplevel_methods/convert_global_env.rb'
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case, use a hardcoded entry instead.
    # ======================================================================= #
    i = '/Depot/Packages/' if i.include? '$' # <- See above for the explanation.
    i << '/' unless i.end_with? '/'
    i = i.dup if i.frozen? # <- Need to do so again, e. g. in a .cgi environment context.
    i.squeeze!('/')
    @packages_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  set_packages_directory

  # ========================================================================= #
  # === RBT.packages_directory?
  # ========================================================================= #
  def self.packages_directory?
    @packages_directory
  end; self.instance_eval { alias packages_dir?             packages_directory? } # === RBT.packages_dir?
       self.instance_eval { alias return_packages_directory packages_directory? } # === RBT.return_packages_directory

  # ========================================================================= #
  # === @syslib_directory
  # ========================================================================= #
  @syslib_directory = nil

  require 'rbt/toplevel_methods/gobolinux.rb'
  # ========================================================================= #
  # === RBT.set_syslib_directory
  # ========================================================================= #
  def self.set_syslib_directory(
      i = :infer_which_path_to_use_for_syslib
    )
    require 'rbt/toplevel_methods/convert_global_env.rb'
    case i
    # ======================================================================= #
    # === :infer_which_path_to_use_for_syslib
    #
    # This here will automatically determine the target for the
    # syslib-directory.
    # ======================================================================= #
    when :infer_which_path_to_use_for_syslib
      if is_this_a_real_gobolinux_system?
        i = '/System/Index/lib/' # This is the path on a GoboLinux system.
      else
        i = '/usr/lib/'
      end
    end
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case, use a hardcoded entry instead.
    # ======================================================================= #
    i = '/System/Index/lib/' if i.include? '$' # ← See above for the explanation.
    i << '/' unless i.end_with? '/'
    i = i.dup if i.frozen? # ← Need to do so again, e. g. in a .cgi environment context.
    i.squeeze!('/')
    @syslib_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  set_syslib_directory

  # ========================================================================= #
  # === RBT.syslib_directory?
  # ========================================================================= #
  def self.syslib_directory?
    @syslib_directory
  end; self.instance_eval { alias syslib_dir?             syslib_directory? } # === RBT.syslib_dir?
       self.instance_eval { alias return_syslib_directory syslib_directory? } # === RBT.return_syslib_directory

  # ========================================================================= #
  # === @system_share_directory
  # ========================================================================= #
  @system_share_directory = nil

  # ========================================================================= #
  # === RBT.set_system_share_directory
  # ========================================================================= #
  def self.set_system_share_directory(
      i = '/System/Share/'
    )
    i = i.dup if i.frozen?
    require 'rbt/toplevel_methods/convert_global_env.rb'
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case, use a hardcoded entry instead.
    # ======================================================================= #
    i = '/System/Share/' if i.include? '$' # <- See above for the explanation.
    i << '/' unless i.end_with? '/'
    i = i.dup if i.frozen? # <- Need to do so again, e. g. in a .cgi environment context.
    i.squeeze!('/')
    @system_share_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  set_system_share_directory

  # ========================================================================= #
  # === RBT.system_share_directory?
  # ========================================================================= #
  def self.system_share_directory?
    @system_share_directory
  end; self.instance_eval { alias system_share_dir?             system_share_directory? } # === RBT.system_share_dir?
       self.instance_eval { alias sysshare_directory?           system_share_directory? } # === RBT.sysshare_directory?
       self.instance_eval { alias return_system_share_directory system_share_directory? } # === RBT.return_system_share_directory

  # ========================================================================= #
  # === @system_settings_directory
  # ========================================================================= #
  @system_settings_directory = nil

  # ========================================================================= #
  # === RBT.set_system_settings_directory
  # ========================================================================= #
  def self.set_system_settings_directory(
      i = '/System/Settings/'
    )
    i = i.dup if i.frozen?
    require 'rbt/toplevel_methods/convert_global_env.rb'
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case, use a hardcoded entry instead.
    # ======================================================================= #
    i = '/System/Settings/' if i.include? '$' # <- See above for the explanation.
    i << '/' unless i.end_with? '/'
    i = i.dup if i.frozen? # <- Need to do so again, e. g. in a .cgi environment context.
    i.squeeze!('/')
    @system_settings_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  set_system_settings_directory

  # ========================================================================= #
  # === RBT.system_settings_directory?
  # ========================================================================= #
  def self.system_settings_directory?
    @system_settings_directory
  end; self.instance_eval { alias system_settings_dir?             system_settings_directory? } # === RBT.system_settings_dir?
       self.instance_eval { alias return_system_settings_directory system_settings_directory? } # === RBT.return_system_settings_directory

  # ========================================================================= #
  # === @depot_directory
  # ========================================================================= #
  @depot_directory = nil

  # ========================================================================= #
  # === RBT.set_depot_directory
  # ========================================================================= #
  def self.set_depot_directory(
      i = '/Depot/'
    )
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case, use a hardcoded entry instead.
    # ======================================================================= #
    i = '/Depot/' if i.include? '$' # <- See above for the explanation.
    i << '/' unless i.end_with? '/'
    i = i.dup if i.frozen? # <- Need to do so again, e. g. in a .cgi environment context.
    i.squeeze!('/')
    @depot_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  set_depot_directory

  # ========================================================================= #
  # === RBT.depot_directory?
  # ========================================================================= #
  def self.depot_directory?
    @depot_directory
  end; self.instance_eval { alias depot_dir?             depot_directory? } # === RBT.depot_dir?
       self.instance_eval { alias return_depot_directory depot_directory? } # === RBT.return_depot_directory

  # ========================================================================= #
  # === @sysinclude_directory
  # ========================================================================= #
  @sysinclude_directory = nil

  # ========================================================================= #
  # === RBT.set_sysinclude_directory
  # ========================================================================= #
  def self.set_sysinclude_directory(
      i = '/System/Index/include/'
    )
    require 'rbt/toplevel_methods/convert_global_env.rb'
    i = i.dup if i.frozen?
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case, use a hardcoded entry instead.
    # ======================================================================= #
    i = '/System/Index/include/' if i.include? '$' # <- See above for the explanation.
    i = i.dup if i.frozen? # <- Need to do so again, e. g. in a .cgi environment context.
    i << '/' unless i.end_with? '/'
    i.squeeze!('/')
    @sysinclude_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  set_sysinclude_directory

  # ========================================================================= #
  # === RBT.sysinclude_directory?
  # ========================================================================= #
  def self.sysinclude_directory?
    @sysinclude_directory
  end; self.instance_eval { alias sysinclude_dir?             sysinclude_directory? } # === RBT.sysinclude_dir?
       self.instance_eval { alias return_sysinclude_directory sysinclude_directory? } # === RBT.return_sysinclude_directory

  # ========================================================================= #
  # === @pkgconfig_directory
  # ========================================================================= #
  @pkgconfig_directory = nil

  # ========================================================================= #
  # === RBT.set_pkgconfig_directory
  # ========================================================================= #
  def self.set_pkgconfig_directory(
      i = '/usr/lib/pkgconfig/'
    )
    i = i.dup if i.frozen?
    require 'rbt/toplevel_methods/convert_global_env.rb'
    i = convert_global_env(i) if i.include? '$'
    # ======================================================================= #
    # If there is still at the least one '$' entry then this variable does
    # not exist. In this case, use a hardcoded entry instead.
    # ======================================================================= #
    i = '/usr/lib/pkgconfig/' if i.include? '$' # ← See above for the explanation.
    i << '/' unless i.end_with? '/'
    i = i.dup if i.frozen? # ← Need to do so again, e. g. in a .cgi environment context.
    i.squeeze!('/')
    @pkgconfig_directory = i
  end

  # ========================================================================= #
  # Also set it at once, so that we have a proper start value.
  # ========================================================================= #
  set_pkgconfig_directory

  # ========================================================================= #
  # === RBT.pkgconfig_directory
  # ========================================================================= #
  def self.pkgconfig_directory?
    @pkgconfig_directory
  end; self.instance_eval { alias pkgconfig_dir?             pkgconfig_directory? } # === RBT.pkgconfig_dir?
       self.instance_eval { alias pkgconfig_directory?       pkgconfig_directory? } # === RBT.pkgconfig_directory?
       self.instance_eval { alias return_pkgconfig_directory pkgconfig_directory? } # === RBT.return_pkgconfig_directory

  # ========================================================================= #
  # === individual_cookbooks_directory?
  # ========================================================================= #
  def individual_cookbooks_directory?
    ::RBT.cookbook_directory?
  end; alias cookbooks_directory? individual_cookbooks_directory? # === cookbooks_directory?

  # ========================================================================= #
  # === RBT.get_all_files_from
  #
  # We will only obtain files through this method here.
  # ========================================================================= #
  def self.get_all_files_from(
      this_dir             = return_pwd,
      return_full_path     = true,
      include_hidden_files = false
    )
    this_dir = return_pwd if this_dir.nil?
    _ = this_dir.dup
    _ << '/*' unless _.end_with? '/*'
    _ = rds(_) # Make use of RBT:rds() here.
    _ = Dir[_].select {|entry| File.file?(entry) }
    return _
  end; self.instance_eval { alias get_files_from get_all_files_from } # === RBT.get_files_from

  # ========================================================================= #
  # === get_file_listing
  #
  # Get the file listing of a specific directory. Input will always become
  # a string. Use this method consistently when you want to get files from
  # a directory, for use in the RBT Project.
  #
  # Since September 2014 we will ONLY return real files.
  # ========================================================================= #
  def get_file_listing(
      a = return_pwd,
      b = true,
      c = false
    )
    RBT.get_all_files_from(a, b, c)
  end; alias get_all_files_from get_file_listing # === get_all_files_from
       alias get_files_from     get_file_listing # === get_files_from
       alias get_files          get_file_listing # === get_files

  # ======================================================================== #
  # === RBT.copy_recursively
  #
  # We can use a call to system(), or the pure ruby version, in order to
  # recursively copy files. But you may not enough information if the
  # pure ruby version is used.
  #
  # "noop" means "No Op" aka "No Operation". "No Op" is a dummy task
  # that always completes successfully. It is most often used in a
  # task whose role is to delete files.
  #
  # The method signature upstream is:
  #
  #    cp_r(src, dest, preserve: nil, noop: nil, verbose: nil, dereference_root: true, remove_destination: nil) (mod_func)
  #
  # ======================================================================== #
  def self.copy_recursively(
      what,
      where_to   = '/usr/',
      be_verbose = false
    )
    if what.is_a? String
      what << '.' unless what.end_with? '.'
    end
    begin
      ::FileUtils.cp_r(
        what,
        where_to
      )#,
      #  nil,       # for preserve
      #  nil,       # for noop
      #  be_verbose # for verbose
      #)
    rescue Exception => error
      pp error
    end
  end; self.instance_eval { alias rcp copy_recursively } # === RBT.rcp

# =========================================================================== #
# === RBT::SaveFile
#
# This file provides file-saving capabilities. This is already possible
# within ruby itself via File.open but I wanted to use a simpler API
# to memorize.
# =========================================================================== #
module SaveFile # === RBT::SaveFile

  # ========================================================================= #
  # === RBT::SaveFile.append_what_into
  # ========================================================================= #
  def self.append_what_into(what, into)
    File.open(into, 'a') { |file| file.write(what) }
  end

  # ========================================================================= #
  # === RBT::SaveFile.write_what_into
  #
  # This method can be used to write content into a file, via
  # ruby's File.open() functionality.
  #
  # The official documentation can be found here:
  #
  #   https://ruby-doc.org/core/File.html#method-c-open
  #
  # ========================================================================= #
  def self.write_what_into(
      what,
      into,
      use_these_permissions = 0755
    )
    case use_these_permissions
    when :default
      use_these_permissions = 0755
    end
    base_dir = File.dirname(into)
    RBT.mkdir(base_dir) unless File.exist? base_dir
    File.open(into, 'w+', use_these_permissions.to_i) { |file|
      file.write(what)
    }
    # RBT.e "No file exists at #{into}."
  end

  # ========================================================================= #
  # === RBT.write_what_into_via_unicode
  #
  # This method will specifically make use of the 'w:UTF-8' flag.
  # ========================================================================= #
  def self.write_what_into_via_unicode(
      what,
      into,
      use_these_permissions = 0755
    )
    case use_these_permissions
    when :default
      use_these_permissions = 0755
    end
    base_dir = File.dirname(into)
    RBT.mkdir(base_dir) unless File.exist? base_dir
    File.open(into, 'w:UTF-8', use_these_permissions.to_i) { |file|
      file.write(what)
    }
  end

end # end RBT::SaveFile

  # ========================================================================= #
  # === RBT.write_what_into
  # ========================================================================= #
  def self.write_what_into(
      what,
      into,
      use_these_permissions = :default
    )
    RBT::SaveFile.write_what_into(what, into, use_these_permissions)
  end

  # ========================================================================= #
  # === RBT.append_what_into
  # ========================================================================= #
  def self.append_what_into(what, into)
    RBT::SaveFile.append_what_into(what, into)
  end

  # ========================================================================= #
  # === RBT.write_what_into_via_unicode
  # ========================================================================= #
  def self.write_what_into_via_unicode(
      what, into, use_these_permissions = :default
    )
    RBT::SaveFile.write_what_into_via_unicode(what, into, use_these_permissions)
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e
  # ========================================================================= #
  # You may have to fine-tune from the available methods below,
  # to the methods that you may need.
  # ========================================================================= #
  RBT.create_directory(ARGV, :be_verbose)
  RBT.create_log_directory
  pp RBT.return_all_archives_from_this_directory(ARGV) # return_all_archives_from_this_directory /Users/x/SRC/fontconfig/
  pp RBT.get_all_directories_from('/home/x/src/')
  e
  e 'Next testing the save-file functionality:'
  e
  RBT::SaveFile.write_what_into('Hello world!', '/Depot/Temp/test.md')
  RBT::SaveFile.write_what_into('test', '/here.md')
  RBT::SaveFile.append_what_into('Hello world!', '/Depot/Temp/test.md')
  e
  e RBT.file_temp_directory
  e RBT.temp_directory?  
  e
  e 'The RBT-programs-directory can be found at '+RBT.programs_directory?
  e
  e 'The source directory (src) can be found at: '+RBT.source_dir? # e. g. "/home/x/src/"
  e
  e 'The RBT log-directory can be found at: '+RBT.log_directory?
  e RBT.store_the_last_configuration_options_into_which_file?
  e RBT.file_last_compiled_program
  e
  e 'The syslib-directory resides at: '+RBT.syslib_directory?
  e
  RBT.report_where_the_cookbook_directory_can_be_found
  pp RBT.sanitize_url1_entry(ARGV) if ARGV[0]
  e
  # RBT.copy_recursively(Dir['*'], '/home/x/Temp/', true)
end