#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/html.rb'
# RBT.create_html_page_for_these_programs(input)
# =========================================================================== #
module RBT

  require 'rbt/requires/require_the_cookbook_class.rb'
  require 'rbt/version/version.rb'
  require 'rbt/colours/colours.rb'
  require 'rbt/configuration/configuration.rb'
  require 'rbt/toplevel_methods/e_and_esystem.rb'
  require 'rbt/toplevel_methods/opn.rb'
  require 'rbt/toplevel_methods/files_and_directories_related_methods.rb'
  require 'rbt/time/time.rb'

  # ========================================================================= #
  # === RBT.create_programs_version_html_file
  #
  # Commandline invocation example:
  #
  #   rbt --create_programs_version_html_file
  #
  # The default input file can be found at:
  #
  #   /Depot/Temp/rbt/CookbookDirectory/programs_version_url.txt
  #
  # ========================================================================= #
  def self.create_programs_version_html_file(
      use_this_file =
        RBT.file_programs_version
    )
    updated_when = "#{RBT.return_date} #{RBT.return_time}"
    # ======================================================================= #
    # Generate a .html file.
    # ======================================================================= #
    if File.exist? use_this_file
      if use_this_file.end_with? '.yml'
        dataset = YAML.load_file(use_this_file)
        _ = ''.dup
        dataset.each_pair {|key, value|
          padded_key = ("#{key}:").ljust(50)
          _ << "  <b>#{padded_key} #{value}</b>\n"
        }
        dataset = _
      end
      dataset = '<html><title>Programs of Cookbooks Version: '+RBT.version?+
                ' Last updated: '+updated_when+
                '</title><body>
                Last update: <b>'+updated_when+'</b><br>
                <pre style="font-size: 1.4em">'+
                dataset+'</pre></body></html>'
      # ======================================================================= #
      # Store that file next:
      # ======================================================================= #
      use_this_file = "#{RBT.temp_directory?}programs_version.html"
      opn; e "Generating a new file at "\
             "`#{::Colours.sfile(use_this_file)}#{::Colours.rev}`."
      write_what_into(
        dataset, use_this_file
      ) # Store it here.
    else
      no_file_exists_at(use_this_file)
    end
  end

  # ========================================================================= #
  # === @html_page_content
  # ========================================================================= #
  @html_page_content = ''.dup

  # ========================================================================= #
  # === RBT.create_html_page_for_these_programs
  #
  # This method works on an Array as its input. This Array should be
  # about programs that you want to see an auto-generated webpage
  # containing some information about this program.
  # ========================================================================= #
  def self.create_html_page_for_these_programs(array = ARGV)
    if array.is_a?(String) and
       array.start_with?(':')
      array = array.to_s.delete(':').to_sym
    end
    if array.is_a? Symbol
      require 'rbt/utility_scripts/search_for_tags/search_for_tags.rb'
      # ===================================================================== #
      # Handle special cases such as KDE5.
      # ===================================================================== #
      case array # case tag
      # ===================================================================== #
      # === cookbooks --create_html_page_for_these_programs=:kde5
      # ===================================================================== #
      when :kde, :kde5
        array = RBT.return_all_programs_that_match_this_tag(:kde5)
      # ===================================================================== #
      # === cookbooks --create_html_page_for_these_programs=:kde5_toolchain
      # ===================================================================== #
      when :kde5_toolchain
        # =================================================================== #
        # This variant here should be equivalent to LFS/BLFS, in the proper
        # order. This thus requires a yaml file.
        # =================================================================== #
        dataset = load_yaml(RBT.file_cookbook_chained_programs)
        array = dataset['kde5_foundation']+
                dataset['kde5_plasma']+
                dataset['kde5_applications']+
                dataset['kde5_porting_aids']
        if array.is_a? String # Well ... :)
          array.delete!(' ')
          array = array.split(',')
        end
      end
    end
    if array.is_a? Array
      if array.first.is_a?(String) and array.first.include?(',') # <- Input such as ["php,kio,ruby"]
        array = array.first.split(',')
      end
      array.flatten!
      if array.size > 20
        opn; e 'More than 20 entries were given (exactly '+
               sfancy(array.size.to_s)+'). This may take a while.'
      end
      @html_page_content = "<html>".dup
      begin
        require 'cyberweb'
        @html_page_content << Cyberweb.return_unicode_meta
      rescue LoadError; end
      @html_page_content << "<head><title>HTML Pages for programs</title>"\
                            "<style>pre { font-size: 1.4em; }</style>"\
                            "</head><body><hr>\n".dup # Reset again.
      array.each_with_index {|this_program, index| index += 1
        # =================================================================== #
        # For each program, we do the same actions:
        #
        #   (1) Obtain the dataset via Cookbooks::Cookbook.new() OR
        #       from the expanded directory containing all yaml files.
        #   (2) 
        # =================================================================== #
        if File.directory?(RBT.directory_expanded_cookbooks) and
           File.exist?(target_file = "#{RBT.directory_expanded_cookbooks}#{this_program}.yml")
          dataset = load_yaml(target_file)
          remote_url  = dataset['url1'.to_sym]
          description = dataset['raw_description'.to_sym]
          licence     = dataset['licence'.to_sym]
          homepage    = dataset['homepage'.to_sym].to_s
          binaries    = dataset['binaries'.to_sym]
          libraries   = dataset['libraries'.to_sym]
        else
          dataset = action(:SanitizeCookbook, this_program)
          remote_url  = dataset.url1?
          description = dataset.raw_description?
          licence     = dataset.licence?
          homepage    = dataset.homepage?
          binaries    = dataset.binaries?
          libraries   = dataset.binaries?
        end
        if licence.is_a? Array
          licence = licence.join(', ')
        end
        @html_page_content << "<p>\n<h3 style=\"color:darkgreen\"><b style=\"color:darkblue\">"\
                              "#{index})</b> #{this_program}</h3><br>\n"
        @html_page_content <<
          "  <span style=\"margin-left: 1em; font-weight:bold; color: darkblue\">wget #{remote_url}<span><br><br>\n"
        unless description.to_s.empty? # <- Ignore empty descriptions.
          @html_page_content << "<pre>#{description}</pre>\n"
        end
        # =================================================================== #
        # Add the binaries next:
        # =================================================================== #
        if binaries and binaries.is_a?(Array) and !binaries.empty?
          @html_page_content << "Binaries:<br><br>\n"
          binaries.each {|this_binary|
            @html_page_content << "<span style=\"margin-left:2em\">→ #{this_binary}<span><br>\n"
          }
          @html_page_content << "<br>\n"
        end
        # =================================================================== #
        # Add the libraries next:
        # =================================================================== #
        if libraries and libraries.is_a?(Array) and !libraries.empty?
          @html_page_content << "Libraries:<br><br>\n"
          libraries.each {|this_binary|
            @html_page_content << "<span style=\"margin-left:2em\">→ #{this_binary}<span><br>\n"
          }
          @html_page_content << "<br>\n"
        end
        @html_page_content << "licence: #{licence}<br><br>\n"
        @html_page_content << "homepage: <a href=\"#{homepage}\">#{homepage}</a><br>\n"
        @html_page_content << "</p><br><hr>\n"
      }
      # ===================================================================== #
      # Next, store that HTML page somewhere.
      # ===================================================================== #
      what = @html_page_content
      into = "#{log_dir?}programs.html"
      opn; e "Storing these #{sfancy(array.size)} #{rev}programs into the file:"
      e
      e "  #{sfile(into)}"
      e
      write_what_into(what, into)
      @html_page_content = ''.dup # Clear it again here.
    else
      opn; e 'This method requires an Array as input.'
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  RBT.create_programs_version_html_file
  input = ARGV
  # input = :kde5 # For debugging purposes.
  RBT.create_html_page_for_these_programs(input)
end # create_programs_version_html_file
    # createhtmlpagefortheseprograms solid phonon plasmaframework kwin