#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/swift_return_version_of_this_program.rb'
# RBT.swift_return_version_of_this_program(:htop) # => "2.0.2"
# =========================================================================== #
module RBT

  require 'rbt/requires/require_yaml.rb'
  require 'rbt/errors/file_does_not_exist.rb'

  # ========================================================================= #
  # === RBT.swift_return_version_of_this_program
  #
  # This method here is different from RBT.return_version_of_this_program()
  #
  # Rather than instantiate a new RBT::Cookbooks::SanitizeCookbook object,
  # we just query the version from a yaml file directly. This is obviously
  # much faster, as we don't have to do any sanitize-operation and other
  # method calls related to this.
  #
  # The obvious drawback is that we may have to manually update that yaml
  # file and keep it up-to-date - but the advantage by far outweighs this
  # disadvantage. This method here will be much faster than
  # RBT.return_version_of_this_program() - and ideally, you should
  # use it, rather than the other method.
  #
  # If the program in question can not be found, this method will return
  # nil.
  #
  # Invocation examples:
  #
  #   RBT.swift_return_version_of_this_program(:ruby) # => "2.3.1"
  #   RBT.swift_return_version_of_this_program(:htop) # => "2.0.2"
  #   RBT.swift_return_version_of_this_program(:awk)  # => "2.0.2"
  #
  # ========================================================================= #
  def self.swift_return_version_of_this_program(
      this_program = :htop
    )
    require 'rbt/toplevel_methods/available_programs.rb'
    require 'rbt/toplevel_methods/does_include.rb'
    this_program = this_program.to_s
    # ======================================================================= #
    # Next, try to pass it against a list of aliases, if it has not
    # been yet "findable".
    # ======================================================================= #
    if !RBT.does_include?(this_program, :ignore_aliases)
      require 'rbt/aliases/aliases.rb'
      possible_alternative_name = RBT.find_cookbook_alias_for(this_program)
      if possible_alternative_name
        if possible_alternative_name != this_program
          this_program = possible_alternative_name
        end
      end
    end
    # ======================================================================= #
    # Load up the file that contains the programs-version information:
    #
    # This may reside at a location such as this one here:
    #
    #   /Programs/Ruby/2.6.4/lib/ruby/site_ruby/2.6.0/rbt/yaml/programs_version.yml
    #
    # The old code until June 2020 was:
    #
    #   _ = RBT.file_programs_version
    #
    # On 11.06.2020 this was changed, to use the "bigfile" instead.
    #
    # ======================================================================= #
    _ = RBT.file_programs_version
    _ = RBT.file_available_programs_versions
    if File.exist? _ # This is e. g. "rbt/yaml/programs_version/available_programs_versions.md"
      dataset = File.readlines(_) # Read in the file next.
      selection = dataset.select {|entry|
        entry.start_with? "#{this_program} "
      }
      if selection and !selection.empty?
        selection = selection.first
        splitted = selection.split(' ')
        program_version = splitted[1].sub(/.src$/,'')
        return program_version
      end 
    else
      e "#{rev}No file called `#{sfile(_)}#{rev}` exists."
      raise FileDoesNotExist, "#{_} does not exist."
    end
    return nil # Default return value here.
  end; self.instance_eval { alias swift_return_version         swift_return_version_of_this_program } # === RBT.swift_return_version
       self.instance_eval { alias return_version_of            swift_return_version_of_this_program } # === RBT.return_version_of
       self.instance_eval { alias swift_return_version_of      swift_return_version_of_this_program } # === RBT.swift_return_version_of
       self.instance_eval { alias swift_return_program_version swift_return_version_of_this_program } # === RBT.swift_return_program_version

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  if ARGV.empty?
    e 'Awk version:     '+RBT.swift_return_version_of_this_program(:awk).to_s # => "2.0.2"
    e 'Glibc version:   '+RBT.swift_return_version_of_this_program(:glibc).to_s
    e 'Atk version:     '+RBT.swift_return_version_of_this_program(:atk).to_s
    e 'Blinken version: '+RBT.swift_return_version_of_this_program(:blinken).to_s
    e 'Blinken version: '+RBT.swift_return_version_of_this_program(:blinken).to_s
  else
    puts RBT.swift_return_version_of_this_program(ARGV.first.to_sym).to_s
  end
end # swiftreturnversionofthisprogram gettext
    # swiftreturnversionofthisprogram ninja