#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'rbt/toplevel_methods/toplevel_methods.rb'
# RBT.set_xorg_buffer
# RBT.uchroot_compile(ARGV) # uchroot nasm bison
# RBT.return_string_showing_the_last_updated_programs
# =========================================================================== #
module RBT

  # ========================================================================= #
  # === RBT::N_CHARACTERS_PER_LINE
  #
  # Default value to be used for word-wrap splitting.
  # ========================================================================= #
  N_CHARACTERS_PER_LINE = 78

  # ========================================================================= #
  # === @show_names
  #
  # Whether to show the name of the class at hand (through Opn) or
  # whether to not do so. This can be modified via a method call
  # towards RBT.set_show_names().
  # ========================================================================= #
  @show_names = true

  # ========================================================================= #
  # === RBT.set_show_names
  # ========================================================================= #
  def self.set_show_names(i = true)
    @show_names = i
  end

  # ========================================================================= #
  # === RBT.show_names?
  # ========================================================================= #
  def self.show_names?
    @show_names
  end

  # ========================================================================= #
  # === RBT.do_not_show_names
  # ========================================================================= #
  def self.do_not_show_names
    @show_names = false
  end

  # ========================================================================= #
  # === RBT.depends_on_cmake?
  #
  # This method will only query the original cookbook-dataset, whether
  # the project at hand depends on cmake or whether it does not.
  # ========================================================================= #
  def self.depends_on_cmake?(i = nil)
    i and i.include? 'cmake'
  end; self.instance_eval { alias program_uses_cmake? depends_on_cmake? } # === RBT.program_uses_cmake?

  # ========================================================================= #
  # === RBT.shasum
  # ========================================================================= #
  def self.shasum(i)
    result = `sha256sum #{i}`.strip
    if result.include?(' ')
      result = result.split(' ').first.to_s
    end
    return result
  end

  # ========================================================================= #
  # === RBT.depends_on_python?
  #
  # Whether python is registered as a dependency.
  # ========================================================================= #
  def self.depends_on_python?(i = nil)
    i and i.include? 'python'
  end

  # ========================================================================= #
  # === RBT.depends_on_ninja?
  # ========================================================================= #
  def self.depends_on_ninja?(i = nil)
    i and i.include? 'ninja'
  end

  # ========================================================================= #
  # === RBT.depends_on_waf?
  # ========================================================================= #
  def self.depends_on_waf?(i = nil)
    i and i.include? 'waf'
  end

  # ========================================================================= #
  # === RBT.return_string_showing_the_last_updated_programs
  #
  # This method will return a String showing the last updated programs.
  # ========================================================================= #
  def self.return_string_showing_the_last_updated_programs
    _ = ''.dup
    into = "#{rbt_log_dir?}these_programs_were_updated_copy.yml"
    if is_on_roebe? and
      File.exist?("#{::RBT::RUBY_SRC_DIR}rbt/lib/rbt/yaml/these_programs_were_updated.yml")
      # ===================================================================== #
      # In this case prefer my file at home.
      # ===================================================================== #
      into = "#{::RBT::RUBY_SRC_DIR}rbt/lib/rbt/yaml/these_programs_were_updated_copy.yml"
    end
    if File.exist? into
      dataset = File.readlines(into)
      dataset.each {|line|
        # =================================================================== #
        # A line may look like this:
        #
        #   - libhandy                       # program_version: 0.84.0 17.07.2020-12:08:21
        #
        # =================================================================== #
        splitted = line.split('#').map(&:strip)
        name_of_the_program = splitted.first.delete('-').strip
        program_version     = splitted.last.strip.sub(/program_version:/,'').
                              strip.split(' ').first.to_s
        this_date = line.split(' ').last
        _ << "#{name_of_the_program} was updated "\
             "to program version #{program_version} "\
             "(on: #{this_date})\n"
      }
      return _
    else
      no_file_exists_at(into)
    end
  end

  # ========================================================================= #
  # === RBT.do_show_the_classname
  # ========================================================================= #
  def self.do_show_the_classname
    @do_show_the_classname = true
  end

  # ========================================================================= #
  # === And invoke it at once, for the default value.
  # ========================================================================= #
  RBT.do_show_the_classname

  # ========================================================================= #
  # === RBT.do_not_show_the_classname
  # ========================================================================= #
  def self.do_not_show_the_classname
    @do_show_the_classname = false
  end

  # ========================================================================= #
  # === RBT.show_the_classname?
  #
  # Query method over as to whether we should show the classname.
  # ========================================================================= #
  def self.show_the_classname?
    @do_show_the_classname
  end; self.instance_eval { alias show_the_classname show_the_classname? } # === RBT.show_the_classname

  # ========================================================================= #
  # === RBT.do_not_show_names?
  #
  # This is the reverse complement to RBT.show_the_classname?.
  # ========================================================================= #
  def self.do_not_show_names?
    !RBT.show_the_classname?
  end

  # ========================================================================= #
  # === RBT.sanitize_this_licence
  #
  # This method will take the given input and output a valid licence if
  # possible. So, for instance, the input "gpl3" should be turned into
  # "GPLv3". The idea here is to have one unified method that will
  # be able to consistently enforce valid licence names.
  # ========================================================================= #
  def self.sanitize_this_licence(i)
    i = i.dup # Work on a copy, just in case.
    case i.downcase
    # ======================================================================= #
    # === GNU LGPL
    # ======================================================================= #
    when 'gnu lgpl' # This one should ideally be changed by the user.
      i = 'LGPL variant (should be specified clearly)'
    # ======================================================================= #
    # === LGPL
    #
    # Should have a version.
    # ======================================================================= #
    when 'lgpl' # This one should ideally be changed by the user.
      i = 'LGPL variant (should be specified clearly)' 
    # ======================================================================= #
    # === gpl3
    # ======================================================================= #
    when 'gpl3',
         'gplv3'
      i = 'GPLv3'
    # ======================================================================= #
    # === gpl2
    # ======================================================================= #
    when 'gpl2',
         'gplv2'
      i = 'GPLv2'
    # ======================================================================= #
    # === GNU GPLv2
    # ======================================================================= #
    when 'gnu gplv2'
      i = 'GPLv2 (should be GPLv2 only, not GNU GPLv2)'
    when ''
      # pass through
    # else
    #  i = 'Unknown licence'
    end
    return i
  end; self.instance_eval { alias sanitize_licence sanitize_this_licence } # === RBT.sanitize_licence

  # ========================================================================= #
  # === RBT.statistics?
  #
  # This method will return an Array, showcasing how many programs were
  # registered at certain dates (time stamps).
  #
  # This will be based on reading in the content of the file called
  # 'cookbook_statistics.md'. The extracted rbt gem will have this
  # .md file, should you need it.
  # ========================================================================= #
  def self.statistics?(
      this_file = '/home/x/programming/ruby/src/rbt/doc/statistics/cookbook_statistics.md'
    )
    result = []
    if File.exist? this_file
      _ = File.readlines(this_file).select {|line| line.include? '→' }
      _.map! {|entry|
        # ===================================================================== #
        # Sort first by date, then by n programs.
        #
        # See: https://rubular.com/r/5O9frs0hmuqI4Q
        # ===================================================================== #
        entry =~ /→\s*(\d{1,5})(.+)(\d{2}\.\d{2}\.\d{4})/
        entry = [$3.to_s.dup, $1.to_s.dup.to_i]
        entry
      }
      result = _
    end
    return result
  end; self.instance_eval { alias statistics statistics? } # === RBT.statistics

  # ========================================================================= #
  # === RBT.no_file_exists_at
  # ========================================================================= #
  def self.no_file_exists_at(i)
    e "#{rev}No file exists at #{sfile(i)}#{rev}."
  end

  # ========================================================================= #
  # === RBT.does_this_file_exist?
  #
  # This method will check whether a given file exists or whether it does
  # not.
  #
  # The second argument can be used to determine whether we shall use
  # a file at a particular position on the filesystem, e. g. like
  # '../include/' that can be passed. This would check for the file
  # at the directory below the current one, within the include/
  # subdirectory.
  # ========================================================================= #
  def self.does_this_file_exist?(
      i                          = FILE_SCONS,
      prepend_this_for_the_check = nil
    )
    if i.is_a? Array
      i = i.first
    end
    i = i.to_s unless i.is_a? String # We need a String past this point.
    if prepend_this_for_the_check
      unless prepend_this_for_the_check.is_a? String
        if prepend_this_for_the_check == true
          # ================================================================= #
          # true values are converted into '../' strings.
          # ================================================================= #
          prepend_this_for_the_check = '../'.dup
        end
      end
      i = i.dup.prepend(
        prepend_this_for_the_check.dup
      )
    end
    File.exist? i
  end

  # ========================================================================= #
  # === RBT.meson_build_file_exists?
  #
  # When we check for the file called "meson.build", we must also
  # keep in mind that we may use a build directory. This is the
  # reason why the following code uses a variable.
  # ========================================================================= #
  def self.meson_build_file_exists?(
      check_for_this_file        = 'meson.build',
      prepend_this_for_the_check = nil
    )
    case check_for_this_file
    when :default
      check_for_this_file = 'meson.build'
    end
    does_this_file_exist?( # Call the above method again here.
      check_for_this_file, prepend_this_for_the_check
    )
  end

  # ========================================================================= #
  # === RBT.author_name?
  # ========================================================================= #
  def self.author_name?
    'Robert A. Heiler'
  end

  # ========================================================================= #
  # === RBT.email?
  # ========================================================================= #
  def self.email?
    begin
      require 'rbt/configuration/configuration.rb'
    rescue LoadError; end
    RBT.configuration?.email.to_s
  end

  # ========================================================================= #
  # === RBT.dependency_chain
  #
  # This method will calculate the full dependency chain of all
  # given programs.
  #
  # Usage example:
  #
  #   dependencies = RBT.dependency_chain(%w( php ruby python )) # => ["httpd", "libxml2 >= 2.6.31", "mysql", "sqlite3", and more ...
  #
  # ========================================================================= #
  def self.dependency_chain(of_this_program = ARGV)
    array = []
    of_this_program = [of_this_program].flatten.compact
    # ======================================================================= #
    # Always work on an Array past this point.
    # ======================================================================= #
    of_this_program.each {|this_program|
      target_file = RBT.path_to_expanded_directory?+this_program+'.yml'
      if File.exist? target_file
        dataset = YAML.load_file(target_file)
        deps_on = dataset[:required_deps_on]
        array << deps_on
      end
    }
    # ======================================================================= #
    # Now we finished the first iteration. Add another iteration now.
    # ======================================================================= #
    new_array = []
    array.flatten!
    array.each {|inner_entry|
      if inner_entry.include? ' >'
        inner_entry = inner_entry.split(' >').first
      end
      inner_entry = RBT.path_to_expanded_directory?+inner_entry+'.yml'
      if File.exist? inner_entry
        dataset = YAML.load_file(inner_entry)
        deps_on = dataset[:required_deps_on]
        new_array << deps_on
      end
    }
    new_array.flatten!
    array << new_array
    array.flatten!
    array.uniq!
    return array
  end

  # ========================================================================= #
  # === RBT.is_this_version_higher_than_that_version
  # ========================================================================= #
  def self.is_this_version_higher_than_that_version?(
      a, b, report_all_errors_to_the_user = true
    )
    case report_all_errors_to_the_user
    # ======================================================================= #
    # === :do_not_report_any_errors
    # ======================================================================= #
    when :do_not_report_any_errors
      report_all_errors_to_the_user = false
    end
    begin
      a = Gem::Version.new(a)
      b = Gem::Version.new(b)
      a > b
    rescue Exception => error
      if report_all_errors_to_the_user
        e '-' * 80
        pp error
        e '-' * 80
        pp a
        pp b
        e '-' * 80
      end
    end
  end

  # ========================================================================= #
  # === RBT.wrap_at                 (Begrenzung tag, limit tag, boundary tag)
  #
  # Wraps at a specified position. The second argument denotes this limit.
  #
  # Specific examples:
  #
  #   puts "abc abc abdc abc abc abc abc abc abc abc abcdef abc abc abc abc abc abc abc abc abc ".wrap_at(20)
  #   puts RBT.wrap_at(("x"*200), 30)
  #   puts RBT.word_wrap(("x"*200), 50)
  #   ("x"*200).wrap_at(30,true,5)
  #
  # ========================================================================= #  
  def self.wrap_at(
      this_string, # ← This must be the input-String.
      my_limit    = N_CHARACTERS_PER_LINE,
      add_newline = true,
      add_padding = nil
    )
    case my_limit
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      my_limit = N_CHARACTERS_PER_LINE
    end
    begin
      add_padding = add_padding.to_i unless add_padding.nil?
      if add_newline
        _ = "\\1\\3\n" # newline at end
        unless add_padding.nil?
          _ = _.linsert(' ' * add_padding)
        end
        return this_string.gsub(
          /(.{1,#{my_limit}})( +|$)\n?|(.{#{my_limit}})/,_
        )
      else
        _ = "\\1\\3"
        unless add_padding.nil?
          _ = _.linsert(' ' * add_padding)
        end
        return this_string.gsub(
          /(.{1,#{my_limit}})( +|$)\n?|(.{#{my_limit}})/,
          _
        ) # No newline at the end.
      end
    rescue Exception => error
      puts error
    end
  end; self.instance_eval { alias split_at  wrap_at } # === RBT.split_at
       self.instance_eval { alias word_wrap wrap_at } # === RBT.word_wrap

  # ========================================================================= #
  # === RBT.host_cpu?
  # ========================================================================= #
  def self.host_cpu?
    HOST_CPU
  end

  # ========================================================================= #
  # === RBT.consider_changing_the_tab_title                       (shell tag)
  #
  # Use this method to change the title of a tab in a Terminal like Bash.
  # ========================================================================= # 
  def self.consider_changing_the_tab_title(
      i                             = return_program_name_then_hyphen_then_the_program_version,
      shall_we_rename_the_tab_title = rename_tab_title?
    )
    case shall_we_rename_the_tab_title
    # ======================================================================= #
    # === :do_rename_the_tab
    # ======================================================================= #
    when :do_rename_the_tab,
         :do_rename,
         :default,
         nil
      shall_we_rename_the_tab_title = true
    end
    # ======================================================================= #
    # First check whether we ought to rename the tab title or not.
    # ======================================================================= #
    if shall_we_rename_the_tab_title
      case i
      # ===================================================================== #
      # === :default
      # ===================================================================== #
      when :default
        # This is the current hardcoded default. May change at a later time.
        i = return_program_name_then_hyphen_then_the_program_version
      # ===================================================================== #
      # === :clear
      # ===================================================================== #
      when :clear
        i = '.'
      end
      # ===================================================================== #
      # Query whether Roebe is available or not.
      # ===================================================================== #
      if Object.const_defined?(:Roebe) and
         shall_we_rename_the_tab_title # Only use it if the user enabled this.
        if Roebe.respond_to? :rename_konsole_title_to
          # ================================================================= #
          # Rename the konsole-title here.
          # ================================================================= #
          Roebe.rename_konsole_title_to(i, :be_quiet)
        else
          e "The namespace #{sfancy('Roebe')} does not respond to the method"
          e 'rename_konsole_title_to()'
          e 'Either the functionality is missing - or something has not'
          e 'been properly required. Either way, we can not rename the'
          e 'konsole tab for now.'
        end
      end
    end unless i.empty?
  end

  # ========================================================================= #
  # === RBT.return_meson_configure_options_of_this_program
  # ========================================================================= #
  def self.return_meson_configure_options_of_this_program(i)
    require 'rbt/requires/require_sanitize_cookbook.rb'
    action(:SanitizeCookbook, i).meson_options?
  end

  # ========================================================================= #
  # === RBT.install_this_rubygem
  #
  # This method can be used to install a rubygem (.gem) archive.
  #
  # In August 2022 the code was a bit changed - the old code made
  # use of RBT::Compile. During the rewrite of the RBT project in
  # August 2022 the method was simplified, to just use a direct
  # "gem install" command.
  #
  # We also now use the --local flag, but we may have to adapt
  # this in the future, if this method here should also install
  # remote gems. For now we only support installing local .gem
  # files though.
  #
  # Usage example:
  #
  #   RBT.install_this_rubygem('/tmp/thor-1.2.0.gem')
  #
  # ========================================================================= #
  def self.install_this_rubygem(
      use_this_local_path,
      be_verbose = true
    )
    unless RBT.const_defined?(:Installer)
      require 'rbt/requires/require_the_installer_class.rb'
    end
    # ======================================================================= #
    # The old code up until August 2022:
    #
    #   compile_object = RBT::Action::Installer.new(:do_not_run_yet)
    #   compile_object.determine_the_dataset_for_this_program(i)
    #   compile_object.set_program_path(compile_object.cookbook_dataset_program_path?)
    #   compile_object.install_this_ruby_gem(:default, :do_not_exit)
    #
    # ======================================================================= #
    cmd = "gem install --local #{use_this_local_path}"
    _ = RBT.file_ruby_gem_install_options
    if File.exist? _
      cmd = cmd.dup
      cmd << " #{File.read(_)}"
      cmd.strip!
    end
    if be_verbose
      esystem cmd
    else
      system cmd
    end
  end; self.instance_eval { alias install_this_ruby_gem install_this_rubygem } # === RBT.install_this_ruby_gem

  # ========================================================================= #
  # === RBT.create_simple_version_file
  #
  # This method has been specifically created to allow the Java bindings
  # in the RBT project to compile stuff.
  # ========================================================================= #
  def self.create_simple_version_file
    hash = {}
    target_directory = RBT.base_dir_to_store_expanded_cookbooks?
    if File.directory? target_directory
      all_files = Dir[target_directory+'*.yml']
      all_files.each {|this_file|
        dataset = YAML.load_file(this_file)
        program_name = dataset[:program_name]
        program_path = dataset[:program_path]
        hash[program_name] = program_path
      }
      what = hash.map {|key, value| "#{key.downcase}: #{value}\n" }.join
      into = "#{log_dir?}simple_version_file.md"
      e "#{rev}Storing into `#{sfile(into)}`."
      write_what_into(what, into)
    else
      e "No directory at #{sdir(target_directory)}."
    end
  end

  # ========================================================================= #
  # === RBT.favicon?
  #
  # This is currently hardcoded. Users of RBT will not need this image,
  # so it is ok to use a hardcoded path.
  # ========================================================================= #
  def self.favicon?
    '/home/x/programming/ruby/src/rbt/img/'\
    'COOKBOOKS_LOGO.png'
  end

  # ========================================================================= #
  # === RBT.namespace?
  # ========================================================================= #
  def self.namespace?
    inspect
  end

  # ========================================================================= #
  # === RBT.is_on_windows?
  #
  # This method will return true if we are on windows; and false otherwise.
  # ========================================================================= #
  def self.is_on_windows?
    return true if RUBY_PLATFORM.include? 'win'
    return false
  end

  # ========================================================================= #
  # === RBT.stderr
  #
  # By default, "puts" in ruby writes to STDOUT. We can use STDERR.puts
  # to send to the error-stream specifically.
  #
  # The best practice is to log debugging information, errors, warnings,
  # status and so forth to STDERR; actual program output should go
  # towards STDOUT.
  # ========================================================================= #
  def self.stderr(
      i                 = '',
      use_puts_or_print = :puts
    )
    if i.is_a? Array
      i = i.join
    end
    case use_puts_or_print
    # ======================================================================= #
    # === :puts
    # ======================================================================= #
    when :puts
      $stderr.puts i
    # ======================================================================= #
    # === :print
    # ======================================================================= #
    when :print,
         :use_print
      $stderr.print i
    else
      e 'Unknown input at line 35 in the method RBT.stderr().'
    end
  end

  # =========================================================================== #
  # === RBT.show_tutorial
  #
  # This will show a tutorial to the user, instructing them how to use
  # the Cookbooks project.
  #
  # Usage example:
  #
  #   rbt --tutorial
  #   cookbooks --tutorial
  #
  # =========================================================================== #
  def self.show_tutorial(
      try_to_use_colours = true
    )
    require 'io/console'
    display_this_message = "\n[Press any key to continue; q to exit early]"
    # ========================================================================= #
    # The file for the short tutorial can be seen in 
    # "tutorial/short_tutorial.md".
    # ========================================================================= #
    _ = RBT.file_short_tutorial?
    if File.exist? _
      dataset = File.readlines(_) # Display the file next.
      chunks = dataset.each_slice(20).to_a
      counter = 0
      e rev # And a newline to start with. 
      loop {
        content_that_will_be_displayed = chunks[counter]
        if try_to_use_colours
          content_that_will_be_displayed.map! {|line|
            # ================================================================= #
            # Next colourize anything that is indented with 4 leading spaces.
            # ================================================================= #
            line.gsub(/^(    .+)$/, lightgreen('\1'))
          }
        end
        e content_that_will_be_displayed
        # ===================================================================== #
        # Next, display the "press any key to continue" message.
        # ===================================================================== #
        if try_to_use_colours
          display_this_message = mediumseagreen(display_this_message)
        end
        e display_this_message
        case STDIN.getch
        when 'q'
          exit
        else
          counter += 1
          exit if counter >= chunks.size
        end
      }
    else
      e "#{rev}No file exists at #{sfile(_)}"
    end
  end

  # ========================================================================= #
  # === RBT.does_this_file_exist_and_is_it_a_file?
  #
  # This method will add an additional File.file? check.
  # ========================================================================= #
  def self.does_this_file_exist_and_is_it_a_file?(i)
    i = i.to_s
    does_this_file_exist?(i) && File.file?(i)
  end

  # ========================================================================= #
  # === RBT.is_xsel_available?
  # ========================================================================= #
  def self.is_xsel_available?
    result = false
    _ = ENV['PATH']
    if _.include? ':'
      _ = _.split(':')
    end
    _ = [_]
    _.flatten.each {|entry|
      if File.exist?(entry+'xsel')
        result = true
      end
    }
    return result
  end

  # ========================================================================= #
  # === RBT.store_this_expanded_dataset_into_that_yaml_file
  # ========================================================================= #
  def self.store_this_expanded_dataset_into_that_yaml_file(
      expanded_dataset,
      yaml_file
    )
    write_what_into(expanded_dataset, yaml_file)
    # ===================================================================== #
    # Since as of May 2020, on roebe systems, we will also store this.
    # ===================================================================== #
    if is_on_roebe?
      yaml_file = "#{RUBY_SRC_DIR_RBT_YAML}expanded_cookbooks/#{File.basename(yaml_file)}"
      write_what_into(expanded_dataset, yaml_file)
    end
  end

  # ========================================================================= #
  # === RBT.symlink_appdir_libraries_into_usr_lib_directory
  #  
  # The following method is very specialized - it will symlink all
  # entries under lib/* into the /usr/lib/ hierarchy.
  #
  # This will be invoked as part of the postinstallation-steps, but
  # only if we make use of an AppDir layout, and only if we are
  # on a roebe-system (aka my home directory normally).
  # ========================================================================= #
  def self.symlink_appdir_libraries_into_usr_lib_directory(
      from_this_appdir_library_target
    )
    e "#{rev}Now obtaining all entries from the directory `"\
      "#{sdir(from_this_appdir_library_target)}#{rev}`:"
    entries = Dir["#{from_this_appdir_library_target}*"]
    entries.each {|this_library|
      symlink(
        this_library,
        "/usr/lib/#{File.basename(this_library)}"
      )
    }
  end

  # ========================================================================= #
  # === RBT.return_cmake_options_of_this_program
  # ========================================================================= #
  def self.return_cmake_options_of_this_program(i)
    require 'rbt/requires/require_sanitize_cookbook.rb'
    action(:SanitizeCookbook, i).cmake_options?
  end

  # ========================================================================= #
  # === RBT.return_base_directory_of_this_program
  #
  # This method will return the base-directory for a given program,
  # such as "/home/x/src/htop/" for the program htop.
  #
  # A String is returned from this method.
  # ========================================================================= #
  def self.return_base_directory_of_this_program(i)
    i = i.first if i.is_a? Array
    return "#{RBT.source_directory?}#{File.basename(i).delete('_')}/"
  end

  # ========================================================================= #
  # === RBT.set_xorg_buffer
  #
  # This method depends on the external gem called xorg_buffer.
  # ========================================================================= #
  def self.set_xorg_buffer(i, notify_if_xsel_is_not_installed = true)
    begin
      require 'xorg_buffer' unless Object.const_defined? :XorgBuffer
    rescue LoadError; end
    if Object.const_defined? :XorgBuffer
      if is_xsel_available?
        XorgBuffer.set_xorg_buffer(i) { :be_quiet }
      elsif notify_if_xsel_is_not_installed
        e 'xsel is not installed - the xorg-buffer can not be used.'
      end
    end
  end

  # ========================================================================= #
  # === RBT.sanitize_url1_entry
  #
  # This method will sanitize a url1 entry from a cookbook .yml file.
  #
  # Note that this code is currently (May 2020) a bit duplicated, because
  # another file also handles this. In the long run we should unify both
  # code parts, but for the time being I had to move on and just finish
  # the variant here.
  # ========================================================================= #
  def self.sanitize_url1_entry(
      i, hash = {}
    )
    # ======================================================================= #
    # We must allow for symbols or strings as key.
    # ======================================================================= #
    use_this_as_the_key_for_url1 = :url1
    if hash and hash.has_key?('url1') # Check whether we have a String here, nonetheless.
      use_this_as_the_key_for_url1 = 'url1'
    end
    url = hash[use_this_as_the_key_for_url1]
    i = i.first if i.is_a? Array
    case i # case tag
    # ======================================================================= #
    # === USE_URL_DOWNCASE
    #
    # Use the URL as program name but downcase it.
    # ======================================================================= #
    when 'USE_URL_DOWNCASE',
         'USE_DOWNCASED'
      i = remove_archive_from_the_end(File.basename(url.downcase))
    # ======================================================================= #
    # === FIRST_UNDERSCORE_IS_HYPHEN
    # ======================================================================= #
    when 'FIRST_UNDERSCORE_IS_HYPHEN',
         'URL_REPLACE_UNDERSCORE'
      splitted = url.split('/')
      splitted[-1] = splitted[-1].tr('_','-') # Replace the '_' with a '-'.
      i = File.basename(splitted.join('/'))
    # ======================================================================= #
    # === PROGRAM_NAME_AND_PROGRAM_VERSION_WITHOUT_ALPHABET
    # ======================================================================= #
    when 'PROGRAM_NAME_AND_PROGRAM_VERSION_WITHOUT_ALPHABET'
      i = remove_archive_from_the_end(
        File.basename(url).dup
      ) # Example: brotli-v1.0.7.tar.gz
      splitted = i.split('-')
      splitted[1].gsub!(/[A-Za-z]/,'')
      i = splitted.join('-')
    # ======================================================================= #
    # === USE_URL_REPLACE_UNDERSCORE
    #
    # This variant is for entries such as "LMDB_0.9.23.tar.gz".
    # ======================================================================= #
    when 'USE_URL_REPLACE_UNDERSCORE',
         'REPLACE_UNDERSCORES',
         'URL_CONVERT_UNDERSCORE'
      i = remove_archive_from_the_end(
            File.basename(url).dup
          ).
          tr('_','-')
    # ======================================================================= #
    # === FIRST_UNDERSCORE_AS_HYPHEN_AND_OTHER_UNDERSCORES_AS_DOTS
    # ======================================================================= #
    when 'FIRST_UNDERSCORE_AS_HYPHEN_AND_OTHER_UNDERSCORES_AS_DOTS'
      i = remove_archive_from_the_end(
            File.basename(url).dup
          ). # Example: boost_1_72_0.tar.bz2
          sub(/_/,'-').
          tr('_','.')
    # ======================================================================= #
    # === USE_URL_LAST_HYPHEN_IS_DOT
    # ======================================================================= #
    when 'USE_URL_LAST_HYPHEN_IS_DOT'
      i = remove_archive_from_the_end(File.basename(url).dup) # Example: ImageMagick-7.0.9-25
      i = i.reverse.sub('-','.').reverse
    # ======================================================================= #
    # === FIRST_DOT_AS_HYPHEN
    # ======================================================================= #
    when 'FIRST_DOT_AS_HYPHEN'
      i = remove_archive_from_the_end(File.basename(url).dup).sub(/\./,'-')
    # ======================================================================= #
    # === USE_URL_MIDDLE_HYPHENS
    # ======================================================================= #
    when 'USE_URL_MIDDLE_HYPHENS'
      i = remove_archive_from_the_end(File.basename(url).dup)
      i.sub!(/_/,'-')
    else
      if i.include? '-PROGRAM_VERSION'
        i.sub!(/PROGRAM_VERSION/,
        remove_archive_from_the_end( File.basename(url).dup).
            delete('v')
        )
      end
    end
    return i
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  require 'colours/colours_e_autoinclude.rb'
  require 'verbose_truth/autoinclude'
  e RBT.wrap_at(('x' * 140), 50)
  e RBT.author_name?
  e RBT.wrap_at(('x' * 140), 50)
  e 'Testing statistics next:'
  e
  pp RBT.statistics
  e
  e RBT.wrap_at(('x' * 140), 50)
  e
  RBT.stderr(ARGV)
  RBT.uchroot_compile(ARGV) # uchroot nasm bison
  RBT.show_tutorial
  e RBT.does_this_file_exist?(ARGV) # rb does_this_file_exist.rb /Depot/foo.md
  RBT.fix_chown_root_entries_in_this_file # fixchownrootentriesinthisfile
  e RBT.return_base_directory_of_this_program(ARGV) # rbtreturnbasedirectoryofthisprogram htop
  e 'Is the program given by '+ARGV.first+' included? '+
    sfancy(
      VerboseTruth[RBT.program_is_included?(ARGV)]
    ) # program_is_included? gtk
end # ruby test.rb > stdout 2> stderr
