#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::ChainCompile
#
# This class will allow us to batch-compile several programs one after
# the other.
#
# See the bottom of this file for a few more usage examples.
#
# Simple usage example:
#
#   RBT::ChainCompile.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/chain_compile.rb'
# RBT.chain_compile(ARGV)
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class ChainCompile < Base # === RBT::ChainCompile

  require 'rbt/toplevel_methods/try_to_find_shortcut_for_a_chain_compiled_program.rb'

  # ========================================================================= #
  # === ARRAY_DEFAULT_COMPILE_THESE_PROGRAMS
  # ========================================================================= #
  ARRAY_DEFAULT_COMPILE_THESE_PROGRAMS = %w( htop m4 )

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = ARRAY_DEFAULT_COMPILE_THESE_PROGRAMS,
      run_already = true
    )
    reset
    # ======================================================================= #
    # === Handle blocks next
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      when :require_compile_base
        require_compile_base
      end
    end
    menu(i) # Intercept here for the commandline arguments.
    set_compile_these_programs(i)
    run if run_already
  end
 
  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @array_help_options
    # ======================================================================= #
    @array_help_options = []
  end

  # ========================================================================= #
  # === require_compile_base
  #
  # We need to have a separate require-step, in order to avoid circular
  # require warnings.
  # ========================================================================= #
  def require_compile_base
    # ======================================================================= #
    # === @array_help_options
    # ======================================================================= #
    @array_help_options = RBT::ARRAY_HELP_OPTIONS
  end; alias prepare require_compile_base # === prepare

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === Request help
      #
      # Iff the user requested help-feedback, then we will show a Hash.
      #
      # Invocation example:
      #
      #   chain_compile --help
      #
      # ===================================================================== #
      when '?', *@array_help_options
        e 'Available entry points are:'
        e
        RBT.return_chained_programs.keys.each {|this_key| # Invoke via: chain_compile "?"
          e steelblue("  #{this_key}")
        }
        e
        exit
      end
    end
  end

  # ========================================================================= #
  # === set_compile_these_programs
  #
  # Use this method to assign to @compile_these_programs. After that, we
  # will sanitize the input.
  #
  # If you pass in ?, then we will return the available chained programs.
  # ========================================================================= #
  def set_compile_these_programs(
      i = ARRAY_DEFAULT_COMPILE_THESE_PROGRAMS
    )
    i = ARRAY_DEFAULT_COMPILE_THESE_PROGRAMS if i.nil?
    i = ARRAY_DEFAULT_COMPILE_THESE_PROGRAMS if i.empty?
    if i.is_a? String
      i = i.split(',') if i.include? ','
    end
    i = [i] unless i.is_a? Array
    @compile_these_programs = i
    sanitize_compile_these_programs
  end

  # ========================================================================= #
  # === compile_these_programs?
  # ========================================================================= #
  def compile_these_programs? 
    @compile_these_programs
  end; alias input? compile_these_programs? # === input?

  # ========================================================================= #
  # === notify_the_user_which_programs_we_will_compile
  #
  # This method will notify the user which programs we are about to
  # compile next.
  # ========================================================================= #
  def notify_the_user_which_programs_we_will_compile(
      i = input?
    )
    n_programs = i.size.to_s
    opne "#{rev}We will compile these `#{sfancy(n_programs)}#{rev}` programs:"
    _ = simp(
      i.join(', ').strip
    )
    _.chop! if _.end_with?(',')
    opne "  #{_}"
  end; alias show_compile_chain notify_the_user_which_programs_we_will_compile # === show_compile_chain

  # ========================================================================= #
  # === start_batch_compilation                                   (batch tag)
  # ========================================================================= #
  def start_batch_compilation
    notify_the_user_which_programs_we_will_compile
    # ======================================================================= #
    # The input has to be sanitized past this point here.
    # ======================================================================= #
    input?.each {|this_program|
      opne "#{rev}Now compiling `#{simp(this_program)}#{rev}`."
      do_compile_this_program(this_program)
    }
  end

  # ========================================================================= #
  # === do_compile_this_program
  # ========================================================================= #
  def do_compile_this_program(i)
    action(:Installer, i)
  end

  # ========================================================================= #
  # === sanitize_compile_these_programs
  # ========================================================================= #
  def sanitize_compile_these_programs(i = input?)
    i.flatten! # Just in case.
    i.map! {|entry|
      entry = entry.dup if entry.frozen?
      entry.strip! # Don't want whitespace there.
      RBT.try_to_find_shortcut_for_a_chain_compiled_program(entry)
    }
    # ======================================================================= #
    # Next, perform a separate conversion.
    # ======================================================================= #
    i.map! {|entry|
      _ = RBT.return_chained_programs # Get a handler here.
      entry = _[entry] if _.has_key? entry
      if entry.include?(',') and entry.is_a?(String)
        entry = entry.split(',')
      end
      if entry.is_a? Array
        entry.map!(&:strip)
        entry.reject!(&:empty?)
      end
      entry
    }
    i.flatten! if i.is_a? Array
  end; alias sanitize_input sanitize_compile_these_programs # === sanitize_input

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    start_batch_compilation
  end

  # ========================================================================= #
  # === RBT::ChainCompile[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end

# =========================================================================== #
# === RBT.chain_compile
# =========================================================================== #
def self.chain_compile(i = ARGV)
  RBT::ChainCompile.new(i) { :require_compile_base }
end

end

if __FILE__ == $PROGRAM_NAME
  RBT.chain_compile(ARGV)
end # chaincompile ?
    # chaincompile htop php
    # chaincompile mypython
    # chaincompile 'htop,m4,waf,ruby'
    # chaincompile games