#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::MissingHomepages
#
# This class will report which entries from the individual .yml cookbook
# files lack a homepage entry.
#
# Usage example:
#
#   RBT::MissingHomepages.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/missing_homepages.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class MissingHomepages < Base # === RBT::MissingHomepages

  require 'rbt/cookbooks/homepage/homepage.rb'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = nil,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @array_these_programs_lack_a_homepage_entry
    # ======================================================================= #
    @array_these_programs_lack_a_homepage_entry = []
    # ======================================================================= #
    # === @obtain_how
    # ======================================================================= #
    @obtain_how = :use_raw_cookbook
  end

  # ========================================================================= #
  # === notify_the_user_that_this_program_has_no_homepage_entry
  # ========================================================================= #
  def notify_the_user_that_this_program_has_no_homepage_entry(
      this_program
    )
    opne "#{rev}Missing homepage entry "\
         "for `#{steelblue(this_program)}`#{rev}."
  end

  # ========================================================================= #
  # === do_process_the_available_programs
  #
  # Since as of April 2021, this method allows the user to operate via
  # two different modes.
  # ========================================================================= #
  def do_process_the_available_programs
    available_programs = available_programs?
    opne "Working over the available programs next. "\
         "#{seagreen('This may take a while.')}"
    available_programs.each {|this_program|
      case @obtain_how
      # ===================================================================== #
      # === :use_raw_cookbook
      # ===================================================================== #
      when :use_raw_cookbook
        remote_homepage = action(:raw_cookbook, this_program) { :fast }.homepage?.to_s
      # ===================================================================== #
      # === :use_a_rbt_toplevel_method
      # ===================================================================== #
      when :use_a_rbt_toplevel_method
        remote_homepage = RBT.return_homepage_of?(this_program)
      end
      if remote_homepage.nil? or remote_homepage.empty?
        @array_these_programs_lack_a_homepage_entry << this_program
        # =================================================================== #
        # We have to report at once because it takes too long otherwise.
        # =================================================================== #
        notify_the_user_that_this_program_has_no_homepage_entry(this_program)
      end
    }
    do_process_the_available_programs
  end

  # ========================================================================= #
  # === report_how_many_programs_lack_a_homepage_entry
  # ========================================================================= #
  def report_how_many_programs_lack_a_homepage_entry
    _ = @array_these_programs_lack_a_homepage_entry
    if _.size > 1
      opne 'A total of '+steelblue(_.size.to_s)+
           ' programs lack a proper homepage entry right now.'
    else
      opne "All registered programs have a proper homepage entry."\
           "#{return_coloured_cheering_person}"
    end
  end

  # ========================================================================= #
  # === report                                                   (report tag)
  # ========================================================================= #
  def report(
      i = @array_these_programs_lack_a_homepage_entry
    )
    if i.empty?
      opne 'Every registered program has a '\
           'homepage entry. This is good!'
    else
      i.each {|this_program|
        notify_the_user_that_this_program_has_no_homepage_entry
      }
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    do_process_the_available_programs
    report_how_many_programs_lack_a_homepage_entry
  end

  # ========================================================================= #
  # === RBT::MissingHomepages[]
  # ========================================================================= #
  def self.[](i = ARGV, &block)
    new(i, &block)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::MissingHomepages.new(ARGV)
end # missinghomepages