#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SymlinkPkgconfigFiles
#
# This class will symlink all .pc files, from an AppDir, into the
# /usr/lib/pkgconfig/ hierarchy.
#
# Usage example:
#
#   RBT::SymlinkPkgconfigFiles.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/symlink_pkgconfig_files.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class SymlinkPkgconfigFiles < Base # === RBT::SymlinkPkgconfigFiles

  # ========================================================================= #
  # === SYMLINK_INTO_THIS_DIRECTORY
  #
  # This constant designates where to symlink into.
  # ========================================================================= #
  SYMLINK_INTO_THIS_DIRECTORY =
    '/usr/lib/pkgconfig/'

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      use_this_directory = ARGV,
      run_already        = true
    )
    reset
    set_target_directory(
      use_this_directory
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === set_target_directory
  #
  # This is the target directory that must contain the .pc files.
  # ========================================================================= #
  def set_target_directory(
      i = return_pwd
    )
    i = i.first if i.is_a? Array
    i = return_pwd if i.nil?
    i = i.dup if i.frozen?
    i << '/' unless i.end_with? '/'
    unless i.include? 'lib/pkgconfig/'
      i << 'lib/pkgconfig/'
    end
    @target_directory = i
  end

  # ========================================================================= #
  # === target_directory?
  # ========================================================================= #
  def target_directory?
    @target_directory
  end; alias target_dir? target_directory? # === target_dir?
       alias target?     target_directory? # === target?

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    appdir_target_with_pc_files = target?
    target_dir = SYMLINK_INTO_THIS_DIRECTORY
    # ===================================================================== #
    # Only continue here if the target directory exists.
    # ===================================================================== #
    if File.directory? appdir_target_with_pc_files
      opne "#{rev}Now attempting to symlink all #{slateblue('.pc files')} "\
           "#{rev}from"
      opne "`#{sdir(appdir_target_with_pc_files)}"\
           "#{rev}` into `#{sdir(target_dir)}#{rev}`."
      symlink_all_files_from_this_directory_to_that_directory(
        appdir_target_with_pc_files,
        target_dir
      ) { :delete_target_file_if_it_exists }
    end
  end

  # ========================================================================= #
  # === RBT::SymlinkPkgconfigFiles[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::SymlinkPkgconfigFiles.new(ARGV)
end # symlinkpkgconfigfiles $MY_PROGRAMS/Ruby/Current/