#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::SystemCompilePossibilities
#
# The job of this class is to gather the compile-possibilities that the
# given system has. This can then be made available to other classes.
# =========================================================================== #
require 'rbt/base/base.rb'
  
module RBT

class SystemCompilePossibilities < Base

  require 'rbt/requires/swift_version.rb'

  # ========================================================================= #
  # === TRY_TO_COMPILE_THE_PROGRAM_IF_IT_IS_OUTDATED
  #
  # If the following constant is set to true, then this class will also
  # try to compile the program at hand if it is outdated.
  # ========================================================================= #
  TRY_TO_COMPILE_THE_PROGRAM_IF_IT_IS_OUTDATED = true

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
  end

  # ========================================================================= #
  # === check_gmp_version
  # ========================================================================= #
  def check_gmp_version(i = :gmp)
    local_version = EnvironmentInformation.return_version_of_gmp?
    e "GMP Version: #{sfancy(local_version)}"
    # ======================================================================= #
    # Next check if we could update the GCC version:
    # ======================================================================= #
    current_version_is = RBT.swift_return_version_of_this_program(i)
    if local_version < current_version_is
      consider_compiling_this_program(i)
    end
  end

  # ========================================================================= #
  # === check_gcc_version
  # ========================================================================= #
  def check_gcc_version(i = :gcc)
    local_version = EnvironmentInformation.return_version_of_gcc?
    e "GCC Version: #{sfancy(local_version)}"
    # ======================================================================= #
    # Next check if we could update the GCC version:
    # ======================================================================= #
    current_version_is = RBT.swift_return_version_of_this_program(i)
    if local_version < current_version_is
      consider_compiling_this_program(i)
    end
  end

  # ========================================================================= #
  # === check_sed_version
  # ========================================================================= #
  def check_sed_version(i = :sed)
    local_version = EnvironmentInformation.return_version_of_sed?
    e 'Sed Version: '+sfancy(local_version)
    current_version_is = RBT.swift_return_version_of_this_program(i)
    if local_version < current_version_is
      consider_compiling_this_program(i)
    end
  end

  # ========================================================================= #
  # === consider_compiling_this_program
  #
  # This method will honour a constant, before it will attempt to compile
  # anything.
  # ========================================================================= #
  def consider_compiling_this_program(i)
    if TRY_TO_COMPILE_THE_PROGRAM_IF_IT_IS_OUTDATED
      opne "#{rev}Next trying to compile `#{sfancy(i.to_s)}#{rev}`."
      action(:Installer, i)
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    require 'environment_information'
    check_sed_version # (1)
    check_gcc_version # (2)
    check_gmp_version # (3)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::SystemCompilePossibilities.new(ARGV)
end # systemcompilepossibilities