#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === RBT::VersionSwitcher
#
# This class allows you to switch to another program version.
#
# It depends on a GoboLinux-like layout, that is, a /Programs hierarchy.
#
# Usage example:
#
#   RBT:VersionSwitcher.new(ARGV)
#
# =========================================================================== #
# require 'rbt/utility_scripts/version_switcher.rb'
# =========================================================================== #
require 'rbt/base/base.rb'

module RBT

class VersionSwitcher < Base # === RBT::VersionSwitcher

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    # ======================================================================= #
    # === Handle blocks given to this class
    # ======================================================================= #
    if block_given?
      yielded = yield
      case yielded
      # ===================================================================== #
      # === :be_quiet
      # ===================================================================== #
      when :be_quiet
        be_quiet
      # ===================================================================== #
      # === :be_quiet_and_do_not_registered
      # ===================================================================== #
      when :be_quiet_and_do_not_registered
        be_quiet
        @do_not_register = true
      end
    end
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    infer_the_namespace
    # ======================================================================= #
    # === @do_not_register
    #
    # By default, register into the yaml database.
    # ======================================================================= #
    @do_not_register = false
    # ======================================================================= #
    # === @array_available_program_versions
    #
    # The following Array will keep track of the possible python versions.
    # ======================================================================= #
    @array_available_program_versions = []
    # ======================================================================= #
    # === @program_version
    #
    # If the user wants a program version then this variable can be of help.
    # ======================================================================= #
    @program_version = nil
  end

  # ========================================================================= #
  # === program_version?
  # ========================================================================= #
  def program_version?
    @program_version
  end

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i)
    i = [i].flatten.compact # <- We need an Array.
    @internal_hash[:commandline_arguments] = i
    # ======================================================================= #
    # We will assume that the first argument given ^^^ is the
    # name of the program at hand.
    # ======================================================================= #
    program_name = @internal_hash[:commandline_arguments].first
    set_program_name(program_name)
    if @internal_hash[:commandline_arguments].size > 1
      set_program_version(@internal_hash[:commandline_arguments][1])
    end
  end

  # ========================================================================= #
  # === set_program_version
  # ========================================================================= #
  def set_program_version(i)
    @program_version = i
  end

  # ========================================================================= #
  # === set_program_name
  # ========================================================================= #
  def set_program_name(
      i = return_pwd
    )
    i = i.join(' ').strip if i.is_a? Array
    case i
    when :default, nil
      i = return_pwd
    end
    i = i.to_s.dup
    i = File.basename(i) if i.include? '/'
    i.capitalize!
    @program_name = i
    menu(i)
  end

  # ========================================================================= #
  # === program_name?
  # ========================================================================= #
  def program_name?
    @program_name
  end

  # ========================================================================= #
  # === determine_the_available_program_versions
  # ========================================================================= #
  def determine_the_available_program_versions
    target = "#{programs_dir?}#{program_name?}/*"
    @array_available_program_versions = Dir[target].select {|entry|
      File.directory?(entry)
    }.select {|line|
      line =~ /\d+/ # Only obtain numbers.
    }
  end

  # ========================================================================= #
  # === show_help                                                  (help tag)
  #
  # Invocation example:
  #
  #   version_switcher --help
  #
  # ========================================================================= #
  def show_help
    opne 'These options are available for class RBT::VersionSwitcher:'
    e
    e '  --info # Show some information about what this class is doing.'
    e
  end

  # ========================================================================= #
  # === report_the_current_version_in_use
  #
  # This method will simply report the current program version in use.
  # ========================================================================= #
  def report_the_current_version_in_use
    _ = return_current_program_version
    if _.nil?
      opne 'No current program version could be determined.'
    else
      opne rev+'The current program version for the program '+
           sfancy(program_name?.downcase)+
           rev+' in use is: '+
           sfancy(_)
    end
  end

  # ========================================================================= #
  # === return_path_to_current
  # ========================================================================= #
  def return_path_to_current
    "#{programs_dir?}#{program_name?}/Current"
  end; alias path_to_current return_path_to_current # === path_to_current

  # ========================================================================= #
  # === return_current_program_version
  # ========================================================================= #
  def return_current_program_version
    target = return_path_to_current
    if File.exist? target
      readlink = File.readlink(target)
      return return_program_version_of(readlink)
    else
      nil
    end
  end

  # ========================================================================= #
  # === return_program_version_of
  # ========================================================================= #
  def return_program_version_of(
      i = File.readlink(path_to_current)
    )
    i = File.basename(i) if i.include? '/'
    return i
  end

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(i)
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === version_switcher --info
      # ===================================================================== #
      when /info/
        require 'class_docu_shower'
        ClassDocuShower.new(__FILE__)
        exit
      # ===================================================================== #
      # === version_switcher --help
      # ===================================================================== #
      when /help/
        show_help
        exit
      end
    end
  end

  # ========================================================================= #
  # === to_current
  # ========================================================================= #
  def to_current(
      the_other_version
    )
    orev "Now switching to the version `#{sfile(the_other_version)}#{rev}`."
    if @do_not_register
      action(:ToCurrent, the_other_version) { :do_not_register }
    else
      action(:ToCurrent, the_other_version)
    end
  end

  # ========================================================================= #
  # === try_to_pick_that_particular_program_version
  # ========================================================================= #
  def try_to_pick_that_particular_program_version(i)
    i = File.basename(i)
    target = "#{programs_directory?}#{program_name?}/#{i}/"
    to_current(target)
  end

  # ========================================================================= #
  # === try_to_report_which_program_versions_appear_to_be_available
  # ========================================================================= #
  def try_to_report_which_program_versions_appear_to_be_available
    # ======================================================================= #
    # Now we have all available program versions. We must first present to
    # the user which versions are available.
    # ======================================================================= #
    _ = @array_available_program_versions
    unless _.empty?
      orev 'These program versions appear to be available:'
      e
      sorted = _.sort_by {|entry| entry.to_s } # Better to keep it sorted.
      sorted.each {|entry|
        version = File.basename(entry).to_s
        efancy " #{version}"
      }; e
      report_the_current_version_in_use
      if program_version?
        # =================================================================== #
        # In this case, try to pick that particular program version.
        # =================================================================== #
        try_to_pick_that_particular_program_version(program_version?)
        return
      end
      # ===================================================================== #
      # Next, switch to the other version; this works only if we have
      # exactly two elements.
      # ===================================================================== #
      if _.size == 2
        the_other_version = _.reject {|line|
          line.include? return_current_program_version
        }
        the_other_version = the_other_version.first if the_other_version.is_a? Array
        to_current(the_other_version)
      end
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    # ======================================================================= #
    # First, obtain all available program versions.
    # ======================================================================= #
    determine_the_available_program_versions
    try_to_report_which_program_versions_appear_to_be_available
  end

  # ========================================================================= #
  # === RBT::VersionSwitcher[]
  # ========================================================================= #
  def self.[](i = ARGV)
    new(i)
  end

end; end

if __FILE__ == $PROGRAM_NAME
  RBT::VersionSwitcher.new(ARGV)
end # vswitcher python
    # vswitcher python 3.7.0
    # vswitcher python 2.7.16